#ifndef __TYPES__

#pragma pack(1)

#ifdef _MSVC_INTEL
#define _littleEndian
#endif



#ifdef _MSVC_INTEL
typedef char i8;     // signed 8-bit integer
typedef unsigned char ui8;
typedef _int16 i16;  // signed 16-bit integer
typedef unsigned _int16 ui16;
typedef int i32;  // signed 32-bit integer
typedef unsigned int ui32;
typedef _int64 i64; //signed 64-bit integer
typedef i8 * pnt;
typedef ui8 * upnt;
#else
typedef char i8;     // signed 8-bit integer
typedef unsigned char ui8;
typedef short i16;  // signed 16-bit integer
typedef unsigned short ui16;
typedef long i32;  // signed 32-bit integer
typedef unsigned long ui32;
typedef long long i64; //signed 64-bit integer
typedef i8 * pnt;
typedef ui8 * upnt;
#endif

void _Assert(bool, char *, i32);

#ifdef _DEBUG
#define ASSERT(value) _Assert((value),__FILE__,__LINE__)
#else
#define ASSERT(valze) ;
#endif




// **********  How to make small integers from big integers

// The cast operators such as (i16) should not appear in the code!

#define I8   i8
#define UI8  ui8   // Use these when converting to a smaller
#define I16  i16   // type and you either:
#define UI16 ui16  //  - Don't care if it fits
                   //  - You can prove it fits



#ifdef _DEBUGCAST //Use these when you want to make
                  //a smaller integer and you would like
                  //us to check that it will fit.
extern i32 dbgcstint;
i8  _sb(i32 i, i32 line, char *file);              
#define sb(i) (_sb((i),__LINE__,__FILE__))
ui8 _ub(i32 i, i32 line, char *file);
#define ub(i) (_ub((i),__LINE__,__FILE__))
i16 _sw(i32 i, i32 line, char *file);
#define sw(i) (_sw((i),__LINE__,__FILE__))
ui16 _uw(i32 i, i32 line, char *file);
#define uw(i) (_uw((i),__LINE__,__FILE__))
#else
#define sb(x) ((i8)(x))     //If no _DEBUGCAST then we simply
#define ub(x) ((ui8)(x))    //assume that all is well.
#define sw(x) ((i16)(x))
#define uw(x) ((ui16)(x))
#endif

#ifdef _littleEndian
#define LE16(W)((ui16)((((ui16)(W))*0x10001)>>8))
i32 LE32(i32 lng);
#else
#define LE16(x) (x)
#define LE32(x) (x)
#endif

typedef ui8 CELLFLAG; 
    // For roomtype = roomSTONE
    //    bit 0 = west??
    //    bit 1 = south??
    //    bit 2 = east??
    //    bit 3 = north??
    // For roomtype = roomDOOR
    //    bits 0-2 = door type (0, 1, or 5 == allow passage)
    //                  (I have yet to see the value of 1!)
    //                  (0, 4, and 5 are all I have seen).      
    //              00 = open 
    //              04 = closed
    //              05 = door has been bashed open.
    //    bit 3 = N/S door (direction of travel through door)
    // For roomtype = roomSTAIRS
    //    bit 2 (0x04) = stairs up
    // For roomtype = roomPIT
    //    bit 0 means ??? When set things don't fall?
    //    if (0x08) then pit is open (you will fall through)
    // For roomtype = roomTELEPORTER
    //    bit 3 (0x08) means teleporter is active
    // For roomtype = roomFALSEWALL
    //    bit 0 (0x1) means wall is open to party
    //    bit 2 (0x4) means wall is invisible and open to party 
    // 0x08   active  (like teleporter)
    // bits 5-7 room type (see enum ROOMTYPE)


enum RNVAL
{
  RNeof             =0xfffe,
  RNnul             =0xffff,
  RNff80            =0xff80, //fireball
  RNff81            =0xff81,
  RNff82            =0xff82,
  RNDispellMissile  =0xff83, //dispell - Damage non-Material
  RNZoSpell         =0xff84, //open door?
  RNff86            =0xff86,
  RNff87            =0xff87, //Poison Cloud
  RNffa8            =0xffa8,
  RNffe4            =0xffe4,
  RNempty           =0x0000
};

#ifdef _MSVC_INTEL
struct RNBF
{
  unsigned short m_idx:10; // Record number within database
  unsigned short m_db:4;   // Database number
  unsigned short m_pos:2;  // Position within room
};

struct RNRP
{
  unsigned short m_rid:14;
  unsigned short m_pos:2;
};
#endif
#ifdef _APPLECOMPILER
struct RNBF
{
  unsigned short m_pos:2;  // Position within room
  unsigned short m_db:4;   // Database number
  unsigned short m_idx:10; // Record number within database
};

struct RNRP
{
  unsigned short m_pos:2;
  unsigned short m_rid:14;
};
#endif

class RN // Record Name
{
  friend class TIMER;
  friend class DBCOMMON;
  friend class DB4;
  friend class DB9;
  friend class DB14;
  friend void DumpDB3(FILE *, RN, i32, i32, i32, CELLFLAG);
//  friend i16 TAG0093d4(RN);
  void swap(void) {d.ui=LE16(d.ui);};
private:
  union {
    ui16 ui;
    RNBF bf;
    RNRP rp;
  } d;
public:
  RN(void) {d.ui = 0xffff;};
  RN(ui32 pos, ui32 db, ui32 idx)
  {
    ASSERT((pos<4) && (db<16) && (idx<1024));
    d.bf.m_pos=pos;
    d.bf.m_db=db;
    d.bf.m_idx=idx;
  };
  RN(RNVAL i) { d.ui = (ui16)i;};
  void ConstructFromInteger(i32 i) {d.ui = (UI16)(i);};
  void LittleEndian(void) {d.ui = LE16(d.ui);};
  ui8 db(void) const {return (UI8)d.bf.m_db;};
  ui8 pos(void) const {return (UI8)d.bf.m_pos;};
  RN pos(ui32 p) {d.bf.m_pos=p; return *this;};
  ui16 idx(void) const {return d.bf.m_idx;};
  ui32 rid(void) const {return d.rp.m_rid;};
  bool operator == (RN rn) const {return d.ui==rn.d.ui;};
  bool operator != (RN rn) const {return d.ui!=rn.d.ui;};
  bool operator == (RNVAL i) const {return d.ui==i;};
  bool operator >= (RNVAL i) const {return d.ui>=i;};
  bool operator <  (RNVAL i) const {return d.ui< i;};
  bool operator != (RNVAL i) const {return d.ui!=i;};
  ui16 ConvertToInteger(void) const {return d.ui;}; //USE SPARINGLY!
};

#endif //__TYPES__