#include "stdafx.h"

#include <stdio.h>

#include "UI.h"

#include "Objects.h"
#include "Dispatch.h"
#include "CSB.h"
#include "data.h"

extern unsigned int numRandomCalls;
extern char PlayfileName[80];
extern char szCSBVersion[];
extern i32 VBLMultiplier;

#define MaxTraceFiles 100
FILE *TraceFile=NULL;
void CloseTraceFile(void)
{
  fclose(TraceFile);
  TraceFile = NULL;
}

bool OpenTraceFile(void)
{//Return true if file successfully.
  i32 i;
  bool result = false;
  for (i=0; i<MaxTraceFiles; i++)
  {
    char filename[100];
    sprintf(filename,"TRACE%03d.txt",i);
    TraceFile = fopen(filename,"r");
    if (TraceFile == NULL)
    {
      char  msg[100];;
      TraceFile = fopen(filename,"w");
      if (TraceFile == NULL)
      {
        sprintf(msg,"Cannot open file %s for Trace",filename);
        UI_MessageBox(msg,
             "",MESSAGE_OK);
      }
      else
      {
        result = true;
      };
      break;
    }
    else
    {
      fclose (TraceFile);
    };
  };
  if (i==MaxTraceFiles)
      UI_MessageBox("Too many Trace files",
              "",MESSAGE_OK);
  return result;
};

i32 millisecondsPerVBL = 17;
i32 VBLperTimer = 10;
bool TimerTraceActive=false;
bool AttackTraceActive = false;;
i32 GameMode=0;
//i32 clockSpeed = 10;
bool BeginRecordOK = false;
bool RepeatGame = false;
bool GameIsComplete;
bool ItemsRemainingOK = false;
bool gameFrozen = false;
i32 DrawViewportCount = 0;
DOESNOTEXIST DoesNotExist;
bool DiskMenuNeeded;
i32  MostRecentlyAdjustedSkills[2] = {0,0}; //Just for Player display
i32  LatestSkillValues[2] = {0,0}; //Just for Player display.
i32  AdjustedSkillNumber = 0;
bool AutoEnlarge = false;

SPEEDTABLE speedTable[SPEED_NUMSPEED];
SPEEDS gameSpeed = SPEED_NORMAL;
bool playerClock = false;
bool extraTicks = false;

SDLIST::~SDLIST(void)
{
  SD *temp;
  while (psd != NULL)
  {
    temp = psd;
    psd = psd->pnext;
    free(temp);
  };
}
SD *SDLIST::newSD(char *name,
                  i32 minimum,
                  float level,
                  float y_intercept,
                  float slope)
{
  SD *temp;
  if (strlen(name) > 15) return NULL;
  temp = new SD;
  temp->pnext = psd;
  psd = temp;
  strcpy(temp->name,name);
  temp->minimum = minimum;
  temp->level = level;
  temp->y_intercept = y_intercept;
  temp->slope = slope;
  return temp;
}

SD *SDLIST::First(void)
{
  return psd;
}

SD *SDLIST::Next(SD *ent)
{
  return ent->pnext;
}

i32 AddSD(char *name,
          i32 min,
          float level,
          float y_intercept,
          float slope)
{
  if(sdList.newSD(name,min,level,y_intercept,slope)==NULL) return 1;
  return 0;
}

// The list of Smart Discards from the config.txt file
SDLIST sdList;
SDENT sdTable[199];




DATABASES db;

//*********************************************************
// Object Database class member functions
//*********************************************************

i32 dbEntrySizes[16] =
{
  sizeof (DB0),
  sizeof (DB1),
  sizeof (DB2),
  sizeof (DB3),
  sizeof (DB4),
  sizeof (DB5),
  sizeof (DB6),
  sizeof (DB7),
  sizeof (DB8),
  sizeof (DB9),
  sizeof (DB10),
  sizeof (DB11),
  sizeof (DB12),
  sizeof (DB13),
  sizeof (DB14),
  sizeof (DB15)
};

char *dbNames[16] =
{
  "Doors",        //0
  "Teleporters",  //1
  "Scrolls",      //2
  "Actuators",    //3
  "Monsters",     //4
  "Weapons",      //5
  "Clothes",      //6
  "Scrolls",      //7
  "potions",      //8
  "chests",       //9
  "food, misc",   //10
  "??????",       //11
  "??????",       //12
  "??????",       //13
  "Missiles",     //14
  "Clouds"        //15
};

DATABASES::DATABASES (void)
{
  i32 i;
  for (i=0; i<16; i++)
  {
    m_Address[i] = NULL;
    m_numEnt[i] = 0;
  };
}

DATABASES::~DATABASES (void)
{
  i32 i;
  for (i=0; i<16; i++) DeAllocate(i);
}

const RN DBCOMMON::link(void)
{
  return m_link;
}

void DBCOMMON::link(RN rn)
{
  m_link = rn;
}

DBCOMMON *DATABASES::GetCommonAddress(RN object)
{
  i32 dbNum, index;
  dbNum = object.db();
  index = object.idx();
  ASSERT(index < m_numEnt[dbNum]);
  return (DBCOMMON *)
       (((char *)m_Address[dbNum]) + index * dbEntrySizes[dbNum]);
}

i32 DATABASES::NumEntry(DBTYPE dbNum)
{
  return m_numEnt[dbNum];
}

void DATABASES::Allocate(i32 dbNum, i32 numEnt)
{
  DeAllocate(dbNum); // In case something is there already.
  ASSERT((dbNum & 0xfffffff0) == 0);
  dbNum &= 15;
  ASSERT(numEnt <= ((dbNum==15) ? 768 : 1024));
  m_numEnt[dbNum] = numEnt;
  if (numEnt > 0)
  {
    m_Address[dbNum] = (DBCOMMON *)malloc(dbEntrySizes[dbNum] * numEnt);
    if (m_Address[dbNum]==NULL)
    {
      UI_MessageBox("Cannot allocate memory",NULL,MESSAGE_OK);
      throw (0xbad);
    };
    pnt p = (pnt)m_Address[dbNum];
    for (i32 i=0; i<m_numEnt[dbNum]; i++)
    {
      *(RN *)p = RNnul; // Mark entry as empty
      p += dbEntrySizes[dbNum];
    }
  }
  else
  {
    m_Address[dbNum] = NULL;
  };
}

void DATABASES::DeAllocate(i32 dbNum)
{
  ASSERT((dbNum & 0xfffffff0) == 0);
  dbNum &= 15;
  if (m_Address[dbNum] != NULL) free(m_Address[dbNum]);
  m_Address[dbNum] = NULL;
  m_numEnt[dbNum] = 0;
}

RN DATABASES::Enlarge(DBTYPE dbType)
{
  i32 max, iNew, i;
  i32 result;
  DBCOMMON *pC;
  char msg[1000];
  ASSERT( (dbType & 0xfffffff0) == 0);
  if (m_noEnlarge & (1 << dbType)) return RNnul;
  max = (dbType==15) ? 767 : 1023;
  if (m_numEnt[dbType] >= max)
  {
    return RNnul;//Already as big as possible.
  };

  sprintf(msg,
             "Database Number %d (containing %s)\n"
             "has become full.  There is a rather arbitrary\n"
             "limit of %d items in this database.  The\n"
             "original Atari program handled this by\n"
             "discarding items at random.  It is possible\n"
             "to increase the size to %d but of course this\n"
             "will be 'non-standard'.\n"
             "\n"
             "Do you want me to enlarge the database?"
             ,dbType
             ,dbNames[dbType]
             ,m_numEnt[dbType]
             ,max);
  if (AutoEnlarge) result = MESSAGE_IDYES;
  else result = UI_MessageBox(msg,NULL,MESSAGE_YESNO);
  if (result == MESSAGE_IDNO)
  {
    m_noEnlarge |= 1 << dbType;
    return RNnul;
  };
  m_Address[dbType] =
     (DBCOMMON *)realloc(m_Address[dbType], max*dbEntrySizes[dbType]);
  if (m_Address[dbType] == NULL)
  {
    UI_MessageBox("Cannot allocate memory",NULL,MESSAGE_OK);
    die(0);
  };
  d.dungeonDatIndex[dbType+6] = sw(max);
  // We need to clear all the entries.
  iNew = m_numEnt[dbType]; // First entry to clear
  m_numEnt[dbType] = max;
  for (i=iNew; i<max; i++)
  {
    pC = GetCommonAddress(RN(0,dbType,i));
    memset((char *)pC,0,dbEntrySizes[dbType]);
    pC->link(RNnul);
  };
  pC = GetCommonAddress(RN(0,dbType,iNew));
  pC->link(RNeof);
  return RN(0,dbType,iNew);
}

bool DATABASES::IsDBType(DBCOMMON *pDB, DBTYPE type)
{
  if (pDB < m_Address[type]) return false;
  if (pDB > m_Address[type] + dbEntrySizes[type] * m_numEnt[type]) return false;
  return true;
}

// TAG00a0b0
RN DATABASES::FindEmptyDBEntry(DBTYPE dbType, bool important)
{//(RN)
  // important means to assign even if database is
  // nearly full.  Only used to assign fallen hero's bones
  // as far as I know.
  DBCOMMON *entryAddr;
  RN result;
  i32 j;
  i32 DBsize, entrySize;
  ASSERT((dbType&0x7fff) < 16);
  DBsize = db.m_numEnt[dbType]; // # entries
  if (!important) DBsize -=3;
  j = DBsize;
  entrySize = dbEntrySizes[dbType];
  entryAddr = db.m_Address[dbType];
  for (;;)
  {
    if (entryAddr->link() == RNnul)
    {
      result = RN(0,dbType,DBsize-j);
      break; // if entry is empty
    };
    j--;
    if (j == 0)
    {
      return RNnul;
    };
    entryAddr = (DBCOMMON *)((pnt)entryAddr + entrySize);
  } ;
  ClearMemory((pnt)entryAddr, entrySize);
  entryAddr->link(RNeof);
  return result;
}

void DATABASES::swapDOOR(void)
{
  DB0 *pDB0;
  pDB0 = m_Address[dbDOOR]->CastToDB0();
  for (i32 i=0; i<m_numEnt[dbDOOR]; i++)
  {
    pDB0[i].swap();
  };
}

void DATABASES::swapTELEPORTER(void)
{
  DB1 *pDB1;
  pDB1 = m_Address[dbTELEPORTER]->CastToDB1();
  for (i32 i=0; i<m_numEnt[dbTELEPORTER]; i++)
  {
    pDB1[i].swap();
  };
}

void DATABASES::swapTEXT(void)
{
  DB2 *pDB2;
  pDB2 = m_Address[dbTEXT]->CastToDB2();
  for (i32 i=0; i<m_numEnt[dbTEXT]; i++)
  {
    pDB2[i].swap();
  };
}

void DATABASES::swapACTUATOR(void)
{
  DB3 *pDB3;
  pDB3 = m_Address[dbACTUATOR]->CastToDB3();
  for (i32 i=0; i<m_numEnt[dbACTUATOR]; i++)
  {
    pDB3[i].swap();
  };
}

void DATABASES::swapMONSTER(void)
{
  DB4 *pDB4;
  pDB4 = m_Address[dbMONSTER]->CastToDB4();
  for (i32 i=0; i<m_numEnt[dbMONSTER]; i++)
  {
    pDB4[i].swap();
  };
}

void DATABASES::swapWEAPON(void)
{
  DB5 *pDB5;
  pDB5 = m_Address[dbWEAPON]->CastToDB5();
  for (i32 i=0; i<m_numEnt[dbWEAPON]; i++)
  {
    pDB5[i].swap();
  };
}

void DATABASES::swapCLOTHING(void)
{
  DB6 *pDB6;
  pDB6 = m_Address[dbCLOTHING]->CastToDB6();
  for (i32 i=0; i<m_numEnt[dbCLOTHING]; i++)
  {
    pDB6[i].swap();
  };
}

void DATABASES::swapSCROLL(void)
{
  DB7 *pDB7;
  pDB7 = m_Address[dbSCROLL]->CastToDB7();
  for (i32 i=0; i<m_numEnt[dbSCROLL]; i++)
  {
    pDB7[i].swap();
  };
}

void DATABASES::swapPOTION(void)
{
  DB8 *pDB8;
  pDB8 = m_Address[dbPOTION]->CastToDB8();
  for (i32 i=0; i<m_numEnt[dbPOTION]; i++)
  {
    pDB8[i].swap();
  };
}

void DATABASES::swapCHEST(void)
{
  DB9 *pDB9;
  pDB9 = m_Address[dbCHEST]->CastToDB9();
  for (i32 i=0; i<m_numEnt[dbCHEST]; i++)
  {
    pDB9[i].swap();
  };
}

void DATABASES::swapMISC(void)
{
  DB10 *pDB10;
  pDB10 = m_Address[dbMISC]->CastToDB10();
  for (i32 i=0; i<m_numEnt[dbMISC]; i++)
  {
    pDB10[i].swap();
  };
}

void DATABASES::swapMISSILE(void)
{
  DB14 *pDB14;
  pDB14 = m_Address[dbMISSILE]->CastToDB14();
  for (i32 i=0; i<m_numEnt[dbMISSILE]; i++)
  {
    pDB14[i].swap();
  };
}

void DATABASES::swapCLOUD(void)
{
  DB15 *pDB15;
  pDB15 = m_Address[dbCLOUD]->CastToDB15();
  for (i32 i=0; i<m_numEnt[dbCLOUD]; i++)
  {
    pDB15[i].swap();
  };
}




void DATABASES::swap(DBTYPE dbType)
{
  switch (dbType)
  {
    // We need to littleEndian some of the words in these
    // miscellaneous databases.
  case dbDOOR:       swapDOOR();       break;
  case dbTELEPORTER: swapTELEPORTER(); break;
  case dbTEXT:       swapTEXT();       break;
  case dbACTUATOR:   swapACTUATOR();   break;
  case dbMONSTER:    swapMONSTER();    break;
  case dbWEAPON:     swapWEAPON();     break;
  case dbCLOTHING:   swapCLOTHING();   break;
  case dbSCROLL:     swapSCROLL();     break;
  case dbPOTION:     swapPOTION();     break;
  case dbCHEST:      swapCHEST();      break;
  case dbMISC:       swapMISC();       break;
  case 11:
  case 12:
  case 13:                             break;
  case dbMISSILE:    swapMISSILE();    break;
  case dbCLOUD  :    swapCLOUD();      break;
  default: ASSERT(0);
  };
}


bool DBCOMMON::IsDBType(DBTYPE dbtype)
{
  return db.IsDBType(this, dbtype);
}

DB0 *DBCOMMON::CastToDB0(void)
{
  ASSERT(IsDBType(dbDOOR));
  return (DB0 *)this;
}

DB1 *DBCOMMON::CastToDB1(void)
{
  ASSERT(IsDBType(dbTELEPORTER));
  return (DB1 *)this;
}

DB2 *DBCOMMON::CastToDB2(void)
{
  ASSERT(IsDBType(dbTEXT));
  return (DB2 *)this;
}

DB3 *DBCOMMON::CastToDB3(void)
{
  ASSERT(IsDBType(dbACTUATOR));
  return (DB3 *)this;
}

DB4 *DBCOMMON::CastToDB4(void)
{
  ASSERT(IsDBType(dbMONSTER));
  return (DB4 *)this;
}

DB5 *DBCOMMON::CastToDB5(void)
{
  ASSERT(IsDBType(dbWEAPON));
  return (DB5 *)this;
}

DB6 *DBCOMMON::CastToDB6(void)
{
  ASSERT(IsDBType(dbCLOTHING));
  return (DB6 *)this;
}

DB7 *DBCOMMON::CastToDB7(void)
{
  ASSERT(IsDBType(dbSCROLL));
  return (DB7 *)this;
}

DB8 *DBCOMMON::CastToDB8(void)
{
  ASSERT(IsDBType(dbPOTION));
  return (DB8 *)this;
}

DB9 *DBCOMMON::CastToDB9(void)
{
  ASSERT(IsDBType(dbCHEST));
  return (DB9 *)this;
}

DB10 *DBCOMMON::CastToDB10(void)
{
  ASSERT(IsDBType(dbMISC));
  return (DB10 *)this;
}

DB14 *DBCOMMON::CastToDB14(void)
{
  ASSERT(IsDBType(dbMISSILE));
  return (DB14 *)this;
}

DB15 *DBCOMMON::CastToDB15(void)
{
  ASSERT(IsDBType(dbCLOUD));
  return (DB15 *)this;
}

void DBCOMMON::swapLink(void)
{
  m_link.swap();
}

void DB0::swap(void)
{
  swapLink();
  word2 = LE16(word2);
}

void DB1::swap(void)
{
  swapLink();
  word2 = LE16(word2); // x, y, flags
  word4 = LE16(word4); // level
}

void DB2::swap(void)
{
  swapLink();
  word2 = LE16(word2); // x, y, flags
}

void DB3::swap(void)
{
  swapLink();
  word2 = LE16(word2); // x, y, flags
  word4 = LE16(word4);
  word6 = LE16(word6);
}

void DB4::swap(void)
{
  swapLink();
  possession2.swap();
  hitPoints6[0]  = LE16(hitPoints6[0]);
  hitPoints6[1]  = LE16(hitPoints6[1]);
  hitPoints6[2]  = LE16(hitPoints6[2]);
  hitPoints6[3]  = LE16(hitPoints6[3]);
  word14 = LE16(word14);
}

void DB4::Clear(void)
{
  m_link = RNeof;
  possession2 = RNeof;
  uByte4 = 0;
  uByte5 = 0;
  hitPoints6[0] = 0;
  hitPoints6[1] = 0;
  hitPoints6[2] = 0;
  hitPoints6[3] = 0;
  word14 = 0;
}

void DB5::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB5::Clear(void)
{
  m_link = RNeof;
  word2 = 0;
}

void DB6::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB6::Clear(void)
{
  m_link = RNeof;
  word2 = 0;
}


void DB7::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB8::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB9::swap(void)
{
  swapLink();
  m_contents.swap();
  m_word4 = LE16(m_word4);
}

void DB10::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB10::Clear(void)
{
  m_link = RNeof;
  word2 = 0;
}

void DB14::swap(void)
{
  swapLink();
  m_obj2.swap();
  m_timerIndex  = LE16(m_timerIndex);
}

void DB14::Clear(void)
{
  m_link = RNeof;
  m_obj2 = RNeof;
  m_energyRemaining = 0;
  m_energyRemaining2 = 0;
  m_timerIndex = 0;
}

void DB15::swap(void)
{
  swapLink();
  word2  = LE16(word2);
}

void DB15::Clear(void)
{
  m_link = RNeof;
  word2 = 0;
}

//*********************************************************
// End of class DATABASES member functions
//*********************************************************

void ATTRIBUTE::Current(i32 value)
{
  ubCurrent = ub(value);
}

void ATTRIBUTE::Maximum(i32 value)
{
  ubMaximum = ub(value);
}

void ATTRIBUTE::Minimum(i32 value)
{
  ubMinimum = ub(value);
}

ui8 ATTRIBUTE::Maximum(void)
{
  if (ubMaximum > 240) return 0; //It went negative
  return ubMaximum;
}

ui8 ATTRIBUTE::Minimum(void)
{
  if (ubMinimum > 240) return 0; //It went negative
  return ubMinimum;
}

ui8 ATTRIBUTE::Current(void)
{
  if (ubCurrent > 240) return 0; //It went negative
  return ubCurrent;
}

void ATTRIBUTE::IncrMinimum(i32 incr)
{
  ubMinimum = (UI8)(ubMinimum + incr);// It can go negative!
}

void ATTRIBUTE::IncrMaximum(i32 incr)
{
  ubMaximum = ub(ubMaximum + incr);
}

void ATTRIBUTE::IncrCurrent(i32 incr)
{
  ubCurrent = ub(ubCurrent + incr);
}


void RECORDFILE::Open(void)
{
  for (i32 i=0; i<100; i++)
  {
    char filename[100];
    sprintf(filename,"Record%03d.log",i);
    m_file = fopen(filename,"r");
    if (m_file == NULL)
    {
      char msg[300];
      m_file = fopen(filename,"w");
      if (m_file == NULL)
      {
        sprintf(msg,"Cannot open file %s for Recording",filename);
        UI_MessageBox(msg,
                      NULL,MESSAGE_OK);
      };
      fprintf(m_file,"#%s\n",szCSBVersion);
      fprintf(m_file,"#                  0093 --> Freeze Game\n");
      fprintf(m_file,"#                  0094 --> un-Freeze Game\n");
      fprintf(m_file,"#                  00c9 --> Resume (at prison door)\n");
      fprintf(m_file,"#                  008c --> Select Disk option from inventory\n");
      fprintf(m_file,"#                  00d2 --> First Menu option\n");
      fprintf(m_file,"#                  00d3 --> Second Menu Option\n");
      fprintf(m_file,"#                  00d4 --> Third Menu Option\n");
      fprintf(m_file,"#                  0003 --> Move Forward\n");
      fprintf(m_file,"#                  0001 --> Turn Left\n");
      fprintf(m_file,"#                  0002 --> Turn Right\n");
      fprintf(m_file,"#                  0006 --> Slide Left\n");
      fprintf(m_file,"#                  0053 --> Select Inventory Screen\n");
      fprintf(m_file,"#                  000b --> Leave Inventory Screen\n");
      return;
    }
    else
    {
      fclose(m_file);
    };
  };
  UI_MessageBox("Too many Record files",
             "",MESSAGE_OK);
};

void RECORDFILE::Close(void)
{
  if (m_file!=NULL) fclose(m_file);
  m_file=NULL;
}

void RECORDFILE::Record(MouseQueueEnt *ent)
{
  if (m_file==NULL) return;
  if (((d.RandomNumber & 0x3651) == 0) && (d.Time>100))
  {
    fprintf(m_file, "%08x 0000 %04x 2221 %08x\n",
                   d.Time, gameSpeed, d.RandomNumber);
    for (int i=0; i<=gameSpeed; i++) STRandom();
  };
  fprintf(m_file, "%08x %04x %04x %04x %08x\n",
                   d.Time, ent->x, ent->y, ent->num, d.RandomNumber);
  fflush(m_file);
}

RECORDFILE RecordFile;
bool RecordfileIsOpen(void) {return RecordFile.IsOpen();};
void RecordfileOpen(bool open)
{
  if (open) RecordFile.Open();
  else RecordFile.Close();
}


void PLAYFILE::Open(void)
{
  char versionLine[200];
  m_file = fopen(PlayfileName,"r");
  if (m_file == NULL)
  {
    char msg[200];
    sprintf(msg,"Cannot open Playfile \"%s\"",PlayfileName);
    UI_MessageBox(msg,"",MESSAGE_OK);
  };
  fgets(versionLine,199,m_file);
  if (strlen(versionLine)!=0) versionLine[strlen(versionLine)-1] = 0;
  if (strcmp(versionLine+1, szCSBVersion) != 0)
  {
    char msg[200];
    sprintf(msg,"Playback file is not the correct version");
    UI_MessageBox(msg,"",MESSAGE_OK);
  };
};

void PLAYFILE::Close(void)
{
  if (m_file!=NULL) fclose(m_file);
  m_file=NULL;
}

bool PLAYFILE::Play(MouseQueueEnt *ent)
{
  static char line[80];
  unsigned int deltatime;
  int numfield;
  if (m_file==NULL) return false;
  while (m_time == -1)
  { // Must read another record from file.
    int len;
    if (fgets(line,80,m_file)==NULL) return false;
    len = strlen(line);
    if ( (len>0) && (line[len-1] == '\n'))
    {
      len--;
      line[len] = 0;
    };
    if (line[0]== '#')
    {
      if (strcmp(line,"#NoSpeedLimit")==0)
      {
        NoSpeedLimit = 2000000000;
        continue;
      };
      if (strcmp(line,"#SpeedLimit") == 0)
      {
        NoSpeedLimit = 0;
        VBLMultiplier = 1;
        continue;
      };
      if (strcmp(line,"#AutoEnlarge") == 0)
      {
        AutoEnlarge = true;
        continue;
      };
      if (strcmp(line,"#FastVBL") == 0)
      {
        VBLMultiplier = 4;
        continue;
      };
      if (strcmp(line,"#TimerTraceOn") == 0)
      {
        if (TimerTraceActive) continue; // Ignore it
        if (AttackTraceActive || OpenTraceFile())
        {
          TimerTraceActive = true;
        };
        continue;
      };
      if (strcmp(line,"#TimerTraceOff") == 0)
      {
        if (!TimerTraceActive) continue; //Ignore it
        TimerTraceActive = false;
        if (!AttackTraceActive)
        {
          CloseTraceFile();
        };
        continue;
      };
      continue; //ignore it
    };
    if (line[0] == '+') // Increments rather than absolute values
    {
      numfield = sscanf(line+1,"%x %x %x %x %x\n",
                           &deltatime, &m_x, &m_y, &m_num, &m_ran);
      if (numfield == 3) // x and y not needed
      {
        m_ran = m_y;
        m_num = m_x;
        m_x = 0;
        m_y = 0;
      };
      m_time = m_oldTime + deltatime;
    }
    else
    {
      sscanf(line,"%08x %04x %04x %04x %08x\n",
                     &m_time, &m_x, &m_y, &m_num, &m_ran);
    };
  };
  ASSERT( (m_time >= d.Time) || (m_time == 0) );
  if (m_time > d.Time) return false;
  if (    (m_time!=0)
       && (m_ran!=d.RandomNumber)
       && (m_ran!=numRandomCalls-m_oldCallCount)
       && (m_num != 0x2222)  //Cycle random number generator
       && (m_num != 0xd7)    //Party died???
      )
  {
    int answer;
	  answer = UI_MessageBox("Sequence/timing error\nShould we quit?",
		                       "Playback error",
					                 MESSAGE_YESNO);
    if (answer == MESSAGE_IDYES) throw(0x7dbc);
  };
  if (m_num == 0xd7) // Game over???
  {
    m_time = 0x7fffffff;
    return false;
  };
  ent->x = sw(m_x);
  ent->y = sw(m_y);
  ent->num = sw(m_num);
  m_oldTime = m_time;
  m_oldCallCount = numRandomCalls;
  m_time = -1;
  if (m_num == 0x2222)
  {
    i32 count;
    count = m_ran;
    while (count > 0)
    {
      STRandom();
      count--;
    };
    m_num = 0;
  };
  if (m_num == 0x2221)
  {
    for (int i=0; i<=ent->y; i++) STRandom();
    return  Play(ent);
  };
  if ((m_num == 0x2223) || (m_num == 0x2224) )
  { // What is this for?  Be patient and I will explain.
    // The recording files often contain game-save operations.
    // Saving a game modifies the game's state.  Unfortunately, that is
    // what the original Atari program did.
    // I want to remove these game-save operations from the recording
    // to make them 'cleaner'.  So I had to provide a substitute
    // operation that would change the game's state in the same way that
    // a save operation would change it.  The 2223 operation is designed
    // to do just that.  See example below.
    //
    // The 2224 function is used to splice two Recordnn.log files
    // together.  Since the save/restore cleared a few things
    // the 2224 function clears the same things.
    TAG00bd40(); //Remove the Active Monster (ITEM16) entries
    ProcessMonstersOnLevel();
    SmartDiscard(true); // Initialize Smart Discard Processing.

    if (m_num == 0x2224)
    {
      if (d.AttackingCharacterOrdinal != 0)
      {
        d.AttackingCharacterOrdinal = 0;
        d.Word20252 = 1;
        TAG01b098();
      };
    };

 // Here is an example of two record files pasted together.
 // 000330fe  0136 0037 0002 f2b7762c          file 1
 // 00033127  0065 005c 0053 4bca7f8a          file 1 select inventory
 // ##00033131  00ae 0027 008c 5ac82e95        file 1 select disk icon
 // ##00033131  008c 0070 00d3 5ac82e95        file 1 select save & quit
 // ##00033131  0091 006b 00d3 5ac82e95        file 1 select game2
 // #
 // #
 // #
 // #CSB for Windows Version 7.3
 // #                  0093 --> Freeze Game
 // #                  0094 --> un-Freeze Game
 // #                  00c9 --> Resume (at prison door)
 // #                  008c --> Select Disk option from inventory
 // #                  00d2 --> First Menu option
 // #                  00d3 --> Second Menu Option
 // #                  00d4 --> Third Menu Option
 // #                  0003 --> Move Forward
 // #                  0001 --> Turn Left
 // #                  0002 --> Turn Right
 // #                  0006 --> Slide Left
 // #                  0053 --> Select Inventory Screen
 // #                  000b --> Leave Inventory Screen
 // ##00000000  00fd 0056 00c9 00000cd9      file 2 resume
 // ##00000000  00e9 0071 00d3 00000cd9      file 2 select game 2
 // ##00033131  00c9 006e 00d2 7454166b      file 2 play
 // 00033131  00c9 006e 2223 5ac82e95        I inserted this to do what
 //                                           the save/reload does
 // 00000000  0000 0000 000b 00000000        I inserted this to get out
 //                                           of inventory mode
 // 00033149  00c9 006e 0003 44e9eb83        file 2 move
 // 00033155  00c9 006e 0005 802596e0        file 2 move
 // 0003315e  004f 0021 0053 4626f1c2        etc.
    return Play(ent);

  };

  return true;
}

void PLAYFILE::Backspace(MouseQueueEnt *
#ifdef _DEBUG
                         ent
#endif
                         )
{
  if (m_file==NULL) return;
  ASSERT (m_time == -1);
  ASSERT (m_x == ent->x);
  ASSERT (m_y == ent->y);
  ASSERT (m_num == ent->num);
  m_time = d.Time;
}


PLAYFILE PlayFile;
bool PlayfileIsOpen(void) {return PlayFile.IsOpen();};
void PlayfileOpen(bool open)
{
  if (open) PlayFile.Open();
  else PlayFile.Close();
}


bool verticalIntEnabled = false;
i32 NoSpeedLimit= 0;
i32 VBLMultiplier = 1;
bool NoClock=false;
i32 NumberFloppyDrives = 1;
i32 dbgcstint;
i32 GameTime;

i32 keyboardMode=1; // 1 while adventuring
                    // 2 while ReIncarnation

i32 data000b2c; // Contains A4 for use by interrupt handler
i32 data000b30; // Contains A5 for use by interrupt handler

DIRECTION dirNORTH = 0;
DIRECTION dirEAST  = 1;
DIRECTION dirSOUTH = 2;
DIRECTION dirWEST  = 3;

char *tracePrefix = "                                      ";
char *initialTracePrefix(bool trace)
{
  return trace?tracePrefix+strlen(tracePrefix)-2:NULL;
}
char *nextTracePrefix(char *trace)
{
  return trace==NULL?NULL:trace-2;
}