#include "stdafx.h"

#include <stdio.h>
#include <time.h>

#include "UI.h"

#include "Objects.h"
#include "Dispatch.h"
#include "CSB.h"
#include "Data.h"

void TraceTimer(TIMER *pTimer, i32 index, char *msg);


//i32 D0,D1,D2,D3,D4,D5,D6,D7;
//pnt A0,A1,A2,A3,A4,A5,A6,A7;

extern bool PlaybackCommandOption;
extern bool GameIsComplete;
extern bool RepeatGame;
extern i32 VBLMultiplier;
#ifdef GraphicsDebug
FILE *GrphDbg = NULL;
#endif

pnt firstMemoryBlock=NULL; // Debug list of allocated blocks

DBank d;

void _MessageBox(char *msg)
{
  UI_MessageBox(msg,NULL,MESSAGE_OK);
}

void Terminate(void)
{
  abort();
};

void die(i32 errorNum, char *msg) // TAG000efc
{
  char message[300];
  if ((msg==NULL)||(strlen(msg)==0))
  {
    switch(errorNum)
    {
    case 41:  msg="Cannot open GRAPHICS.DAT"; break;
    case 141: msg="Too many files open"; break;
    default:  msg="SystemError";
    };
  };
  sprintf(message,"System Error  %d\n%s",errorNum,msg);
  UI_MessageBox(message,NULL,MESSAGE_OK);
  UI_Die();
}

void _Assert(bool value, char *program, i32 line)
{
  i32 n;
  if (value==0)
  {
    char msg[1000];
    sprintf(msg,
        "Assertion failure\n"
        "Line %d\n"
        "Program %s\nShould we quit?",  line, program);
    n = UI_MessageBox(msg,NULL,MESSAGE_YESNO);
    if (n == MESSAGE_IDYES) die (0,NULL);
  };
}

struct PRINTENTRY
{
  PRINTENTRY *m_next;
  i32         m_color;
  i32         m_type;
  char        m_text[1];
};

class PRINTQUEUE
{
private:
  PRINTENTRY *m_firstEnt;
public:
  PRINTQUEUE(void) {m_firstEnt = NULL;};
  ~PRINTQUEUE(void)
  {
    while (m_firstEnt != NULL)
    {
      PRINTENTRY *temp = m_firstEnt;
      m_firstEnt = temp->m_next;
      free (temp);
    };
  };
  void Queue(i16 color, i16 type, char *text);
  PRINTENTRY *GetEntry(void);
};

void PRINTQUEUE::Queue(i16 color, i16 type, char *text)
{
  i32 len;
  PRINTENTRY *temp, *next;
  len = strlen(text);
  len += sizeof (PRINTENTRY) + 10;// Just to feel safe.
  temp = (PRINTENTRY *)malloc (len);
  if (temp == NULL)
  {
    die(0x7dea3,"Cannot allocate Printline Memory");
  };
  temp->m_next = NULL;
  temp->m_color = color;
  temp->m_type = type;
  strcpy(temp->m_text,text);
  if (m_firstEnt ==NULL) m_firstEnt = temp;
  else
  {
    next = m_firstEnt;
    while (next->m_next != NULL) next = next->m_next;
    next->m_next = temp; // Add new entry at end of queue.
  };
}

PRINTENTRY *PRINTQUEUE::GetEntry(void)
{
  PRINTENTRY *temp;
  if (m_firstEnt == NULL) return NULL;
  temp = m_firstEnt;
  m_firstEnt = temp->m_next;
  return temp;
}

PRINTQUEUE printQueue;


bool verifyRectPos(RectPos *r)
{
  ui32 x1, x2, y1, y2;
  if (d.UseByteCoordinates)
  {
    x1=r->b.x1;
    x2=r->b.x2;
    y1=r->b.y1;
    y2=r->b.y2;
  }
  else
  {
    x1=r->w.x1;
    x2=r->w.x2;
    y1=r->w.y1;
    y2=r->w.y2;
  };
  if (x1>x2) return false;
  if (y1>y2) return false;
  if (x2>319) return false;
  if (y2>199) return false;
  return true;
}

#ifdef _MSVC_INTEL
i32 LE32(i32 lng)
{
  return   ((lng >> 24)&0xff)
         | ((lng >> 8)&0xff00)
         | ((lng << 8)&0xff0000)
         | ((lng <<24)&0xff000000);
}
#endif


RN& RNGear(pnt p)
{
  return *((RN *)(p));
}

pnt& pntGear(pnt p)
{
  return *((pnt *)(p));
}

upnt& upntGear(pnt p)
{
  return *((upnt *)(p));
}

void SwapWordsInButtonList(btn *pButton)
{
  for (;pButton->word0 != 0; pButton++)
  {
    pButton->word0  = LE16(pButton->word0);
    pButton->xMin   = LE16(pButton->xMin);
    pButton->xMax   = LE16(pButton->xMax);
    pButton->yMin   = LE16(pButton->yMin);
    pButton->yMax   = LE16(pButton->yMax);
    pButton->button = LE16(pButton->button);
  };
}

i32 programDescriptor[] = {
  0,      // Program Header Address
  0,      // Last Allocated address +1
  0,      // Start of program I-Bank
  0,      // Length of program I-Bank
  0,      // Start of initialized data
  0x05ec, // Length of initialized data
  0,      // Start of uninitialized data
  0x5b70  // Length of uninitialized data
};



i8 LScreenBase[320][210]; // Extra room at end for ???

void SetButtonPointers(void)
{
  d.pButn18970[0][0] = d.Buttons18624;
  d.pButn18970[0][1] = d.Buttons18660;
  d.pButn18970[0][2] = d.Buttons18708;
  d.pButn18970[0][3] = d.Buttons18768;
  d.pButn18970[1][0] = d.Buttons18792;
  d.pButn18970[1][1] = d.Buttons18828;
  d.pButn18970[1][2] = d.Buttons18876;
  d.pButn18970[1][3] = d.Buttons18936;
}


void InitializeDBank(void) // TAG00332a
{
  ASSERT(sizeof (d) == 23408);
  d.Word1834=512;
  d.Word1844=512;
  d.Word1870 = 1;
  d.Pointer2066[0] = &d.Byte1982;
  d.Pointer2066[1] = (pnt)&d.Word1838;
  d.Pointer2066[2] = (pnt)&d.Word1870;
  d.Pointer2066[3] = (pnt)&d.Word1844;
  d.Pointer2526[0] = d.Word2050;
  d.Pointer2526[1] = d.Word2210;
  d.Pointer2526[2] = d.Word2318;
  d.Word7250  = -1;
  d.Word7252  = -1;
  d.Word7254  = 1;
  d.LoadedLevel = -1;
  d.Word11684 = 1;
  d.clockTick = 1;
  d.Word11748 = -1;
  d.LogicalScreenBase=&LScreenBase[0][0];
  srand( (unsigned)time( NULL ) );
  d.RandomNumber=rand();
  d.TextAlreadyScrolledCount = -1;
  d.Long12950 = -200;
  d.Pointer12988[0] = &d.Byte23114;
  d.Pointer12988[1] = &d.Byte11850;
  d.Pointer12988[2] = &d.Byte22676;
  d.Pointer12988[3] = &d.Byte16550;
  d.Pointer12988[4] = &d.Byte13118;
  d.Pointer12988[5] = &d.Byte16702;
  d.ITEM16QueLen = 0;
  d.Word13148 = -1;
  d.HandChar = -1; // owner of cursor
  d.Pointer16596[0] = "FIGHTER";
  d.Pointer16596[1] = "NINJA";
  d.Pointer16596[2] = "PRIEST";
  d.Pointer16596[3] = "WIZARD";
  d.Long16600 = -505;
  d.WpnChestOrScroll = 0;
  d.objOpenChest = RNnul;
  d.Pointer16770[2]  = "NEOPHYTE";
  d.Pointer16770[3]  = "NOVICE";
  d.Pointer16770[4]  = "APPRENTICE";
  d.Pointer16770[5]  = "JOURNEYMAN";
  d.Pointer16770[6]  = "CRAFTSMAN";
  d.Pointer16770[7]  = "ARTISAN";
  d.Pointer16770[8]  = "ADEPT";
  d.Pointer16770[9]  = "EXPERT";
  d.Pointer16770[10] = "` MASTER";
  d.Pointer16770[11] = "a MASTER";
  d.Pointer16770[12] = "b MASTER";
  d.Pointer16770[13] = "c MASTER";
  d.Pointer16770[14] = "d MASTER";
  d.Pointer16770[15] = "e MASTER";
  d.Pointer16770[16] = "ARCHMASTER";
  d.Pointer16778  = &d.Byte10638;
  d.pwPointer16774  = &d.Word918;
  d.Pointer16770[0]  = (pnt)&d.Long16600;
  d.Pointer16770[1]  = (pnt)&d.Word1834;
  d.DirectionNames[0] = "NORTH";
  d.DirectionNames[1] = "EAST";
  d.DirectionNames[2] = "SOUTH";
  d.DirectionNames[3] = "WEST";
  d.Pointer16822[0] = "L";
  d.Pointer16822[1] = "STRENGTH";
  d.Pointer16822[2] = "DEXTERITY";
  d.Pointer16822[3] = "WISDOM";
  d.Pointer16822[4] = "VITALITY";
  d.Pointer16822[5] = "ANTI-MAGIC";
  d.Pointer16822[6] = "ANTI-FIRE";
  d.MouseQEnd = 4;
  d.MouseInterlock = 1;
  SetButtonPointers();
  d.MagicCaster = -1;
  d.Pointer22812 = "GAME LOADED, READY TO PLAY.";
  d.Pointer22816 = "CAN'T MODIFY CHAOS STRIKES BACK DISK!";
  d.Pointer22820 = "THAT'S NOT THE MASTER DISK!";
  d.Pointer22824 = "CAN'T FIND SAVED GAME!";
  d.Pointer22828 = "UNABLE TO SAVE GAME!";
  d.Pointer22832 = "UNABLE TO FORMAT DISK!";
  d.Pointer22836 = "THAT'S THE CHAOS STRIKES BACK DISK!";
  d.Pointer22840 = "THAT'S A GAME SAVE DISK!";
  d.Pointer22844 = "THAT DISK IS WRITE-PROTECTED!";
  d.Pointer22848 = "THAT DISK IS UNREADABLE";
  d.Pointer22852 = "THAT'S NOT THE SAME GAME";
  d.Pointer22856 = "SAVED GAME DAMAGED!";
  d.Pointer22860 = "PUT THE GAME SAVE DISK IN ~";
  d.Pointer22864 = "PUT THE CHAOS STRIKES BACK DISK IN ~";
  d.Pointer22868 = "PUT A BLANK DISK IN ~";
  d.Pointer22872 = "SAVING GAME . . .";
  d.Pointer22876 = "LOADING GAME . . .";
  d.Pointer22880 = "FORMATTING DISK . . .";
  d.Pointer22884 = "FORMAT DISK ANYWAY?";
  d.Pointer22888 = "THERE IS NO DISK IN ~!";
  d.Pointer22892 = "LOAD SAVED GAME";
  d.Pointer22896 = "SAVE AND PLAY";
  d.Pointer22900 = "SAVE AND QUIT";
  d.Pointer22904 = "FORMAT FLOPPY";
  d.Pointer22908 = "OK";
  d.Pointer22912 = "CANCEL";
  strcpy(&d.Byte23008,"A:\\csbgame.dat");
  strcpy(&d.Byte23024,"A:\\csbgame.bak");
  strcpy(&d.Byte23030,"A:\\F");
  d.UseByteCoordinates = 0;
  d.Pointer23040 = (pnt)0x484;
  d.CurrentSound = -1;
  d.Word23046 = 0;
  d.Word23118 = 1;
  d.NewCursorShape = 1;
  d.Word23150 = 0;
  d.Byte23202[0] = 2;
  d.Byte23202[1] = 17;
  d.Byte23202[2] = 2;
  d.Byte23202[3] = 17;
  d.Byte23206[0] = 0;
  d.Byte23206[1] = 15;
  d.Byte23206[2] = 0;
  d.Byte23206[3] = 15;
  d.wRectPos23214.w.x1 = 2;
  d.wRectPos23214.w.x2 = 20;
  d.wRectPos23214.w.y1 = 2;
  d.wRectPos23214.w.y2 = 15;
  d.wRectPos23222.w.x1 = 0;
  d.wRectPos23222.w.x2 = 18;
  d.wRectPos23222.w.y1 = 0;
  d.wRectPos23222.w.y2 = 13;
  d.Word23224 = 0;
  d.Word23226 = 0;
  d.Word23228 = 0;
  d.ClusterInCache = -1; //(23276)
  d.Word23366 = 4096;
  d.RightZeroMask[0] = -1;
  d.RightZeroMask[1] = -2;
  d.RightZeroMask[2] = -4;
  d.RightZeroMask[3] = -8;
  d.RightZeroMask[4] = -16;
  d.RightZeroMask[5] = -32;
  d.RightZeroMask[6] = -64;
  d.RightZeroMask[7] = -128;
  d.RightZeroMask[8] = 0;
  d.RightOneMask[0] = 0;
  d.RightOneMask[1] = 1;
  d.RightOneMask[2] = 3;
  d.RightOneMask[3] = 7;
  d.RightOneMask[4] = 15;
  d.RightOneMask[5] = 31;
  d.RightOneMask[6] = 63;
  d.RightOneMask[7] = 127;
  d.RightOneMask[8] = -1;
  d.LZWBitNumber = 0;
  d.Word23406 = 0;
}

void SetSupervisorMode(void) {}

void ClearSupervisorMode(void) {}

void HangIfZero(i16 P1)
{
  if (P1 == 0)
    die(0,"HangIfZero");
}

void DisableCursor(void) {}

i32 GetAvailMemory(void)
{
  return 1000000;//510000;//1000000;
}

void jdisint(i16)
{ // Disable some interrupts.
  HopefullyNotNeeded();
}


//   TAG00068e
RESTARTABLE _MainLoop(const CSB_UI_MESSAGE * /*msg*/)
{//(void)
  static dReg D0, D1;
  static PRINTENTRY *pPrintEntry;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(4)
    RESTART(6)
    RESTART(7)
    RESTART(8)
    RESTART(9)
    RESTART(10)
  END_RESTARTMAP
  // We discard any message information.  At least for now.
tag000698: //We get here for each tick of the clock.
  if (d.PartySleeping == 0) d.Speed 
       = sw(speedTable[gameSpeed].vblPerTick); // Delay vblOccurred for 10 VBLs
  d.vblCount = 0;
  if (d.Word11748 == -1) goto tag0006d0;
tag0006a4:
  LoadPartyLevel(_1_,d.Word11748); // Load graphics for level
  MoveObject(RN(RNnul), -1, 0, d.partyX, d.partyY);
  d.Word11748 = -1;
  TAG01953a();
tag0006d0:

  ProcessTimers(_6_);
  if (GameIsComplete) RETURN;
  if (d.Word11748 != -1) goto tag0006a4;
  if (d.SelectedCharacterOrdinal != 0) goto tag000730;
  if (d.PartySleeping != 0) goto tag000730;
     DrawViewport(d.partyFacing, d.partyX, d.partyY);
  if (d.Word11744 != 0)
  {
    d.Word11744=0;
    STHideCursor(); //TAG002fd2
    CreateObjectCursor(d.Pointer16572);
    STShowCursor(); //TAG003026
  };
  if (d.Word11746 != 0)
  {
    d.Word11746=0;
    d.NewCursorShape=1;
    STHideCursor(); //TAG002fd2
    STShowCursor(); //TAG003026
  };
tag000730:
  WaitForButtonFlash(); // If any are active.
  TAG002336(); //counted
  TAG016e54(); //counted
  if (d.Word11696 == 0)
  {
    d.Time++;
    SmartDiscard(false);
    GameTime = d.Time;
    if ((d.Time&511)==0) // about every 85 seconds
    {
      NinetySecondUpdate(); //TAG018124
    };
    if (d.uByte13271 != 0) d.uByte13271--;
    TAG01b29a(_4_);
    D0L = d.Time;
    D0L &= d.PartySleeping ? 15 : 63;
    if (D0L==0) TenSecondUpdate();
    D0L=d.Word11710;
    if (D0L!=0) d.Word11710--;
    D0L=d.Word11712;
    if (D0L != 0) d.Word11712--;
    TAG0018f6(_10_); // Remove text when its time is up.
    D0L=d.WatchdogTime;
    D1L=d.Time;
    if (D0L < D1L)
    {
      die(60, "Watchdog Timer failure"); //die (error number)
    };
    d.clockTick=0;
tag0007b4:
    goto tag0007c8;
tag0007b6: // A UIM_KEYDOWN message was received.
    D0L=UI_DIRECT_CONIN(); // Wait for key
    //D0L = msg->p1;
    ASSERT(D0L != -1);
    TAG0196da(D0L);
tag0007c8:
    pPrintEntry = printQueue.GetEntry();
    if (pPrintEntry != NULL)
    {
      PrintLines(_2_,pPrintEntry->m_color,pPrintEntry->m_text);
      free (pPrintEntry);
    };
    WAITFORMESSAGE(_8_);//Any message!;//pumper();
    {
      //SYSTEMTIME systime;
      //GetSystemTime(&systime);
    };
    D0W=(i16)UI_CONSTAT();
    if (D0W != 0) goto tag0007b6;
    if (d.Word11758 == 0) goto tag0007ea;
    d.Word11756=0;
    d.Word11758=0;
    TAG019264();    goto tag0007fc;
tag0007ea:
    if (d.Word11762 != 0)
    {
      d.Word11760=0;
      d.Word11762=0;
      TAG019018();
    };
tag0007fc:
    HandleMouseEvents(_7_,1);  //TAG01a7b2
    if (DiskMenuNeeded) DisplayDiskMenu(_9_);
    if (d.clockTick    == 0) WaitForButtonFlash(); // If any active
    if (d.clockTick    == 0) goto tag0007b4; // Usually 0
    if (d.ClockRunning == 0) goto tag0007b4; // Usually 1
    if ((d.Time&63)    == 0) // about every 10 seconds
    {
      D0W = (i16)(d.Word11722 & (d.Word11724 ^ 0xffff));
      if (D0W != 0)
      {
        TAG02076e();
        TAG0207cc();
      };
    };
    goto tag000698;
  };
  RETURN;
}

//*********************************************************
//
//*********************************************************
//   TAG000850
RESTARTABLE _LoadPartyLevel(const i32 level)
{//(void)
  static dReg D0, D1;
  //i32  LOCAL_1;
  //i8   LOCAL_2[6];
  static TIMER LOCALa;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
  END_RESTARTMAP
  TAG00bd40();
  SetPartyLevel(level);
  TAG00390e(_2_);
  if (d.Time > d.Long16600+500)
  {
    d.Long16600=d.Time+1;
    LOCALa.function = 22;
    D0L=d.Long16600+2;
    D1L=level;
    D1L >>= 24;
    D0L |= D1L;
    LOCALa.time = D0L;
    SetTimer(&LOCALa);
    TAG01daa8(_1_,0,0);
////0008b4 41fa 097e                LEA      $+2432 (=0x001234),A0
////0008b8 d1fc 0002 0000           ADD.L    #131072,A0
////0008be 4e90                     JSR      (A0)
    HopefullyNotNeeded();
    d.Word19218 = 12777; // Seems in middle of graphic data
    d.Word918 = 136;
    d.Word1834 = 555;
  };
  ProcessMonstersOnLevel();
  SelectPaletteForLightLevel();
  RETURN;
}


void MemMove(pnt src, pnt dest, i32 byteCount) // TAG0009dc
{
  memmove(dest, src, byteCount);
}

void ClearMemory(i8 *dest,i32 numByte) // TAG000a84
{
  memset(dest,0,numByte);
}

void fillWithByte(i8 *addr,i16 num,i8 value,i16 spacing) //TAG000ac0
{
  do
  {
    *addr = value;
    addr+=spacing;
  } while (--num!=0);
}

void fillMemory(i16 *pwAddress, i32 num, i16 value, i16 spacing) //TAG000af6
{ // num words,  spacing apart
  spacing /= 2; // Must be word spacing
  do
  {
    *pwAddress = value;
    pwAddress += spacing;
  } while ((--num) != 0);
}

void TAG000bb0(dReg& D0, dReg& D4, dReg& D5) // called by VBL handler
{  // Expects something in D0, D5, D4
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//000bb0 4a6c a544                TST.W    -23228(A4)
//000bb4 6a10                     BPL      $+18 (=0x000bc6)
  if (d.Word23228 < 0)
  {
//000bb6 322c d238                MOVE.W   -11720(A4),D1
//000bba 836c d236                OR.W     D1,-11722(A4)
    d.Word11722 |= d.Word11720;
//000bbe 426c d238                CLR.W    -11720(A4)
//000bc2 426c a544                CLR.W    -23228(A4)
    d.Word11720 = 0;
    d.Word23228 = 0;
  };
//000bc6 4a6c a5a0                TST.W    -23136(A4)
//000bca 6602                     BNE      $+4 (=0x000bce)
//000bcc 4e75                     RTS
  if (d.Word23136 == 0) return;
//000bce 4a40                     TST.W    D0
//000bd0 6660                     BNE      $+98 (=0x000c32)
  if (D0W != 0)goto tag000c32;
//000bd2 4a6c cd8c                TST.W    -12916(A4)
//000bd6 6d06                     BLT      $+8 (=0x000bde)
  if (d.TextAlreadyScrolledCount < 0) goto tag000bde;
//000bd8 0c45 0097                CMP.W    #151,D5
//000bdc 6454                     BCC      $+86 (=0x000c32)
  if ((ui16)D5W >= 151) goto tag000c32;
//000bde 4a6c d222                TST.W    -11742(A4)
//000be2 6740                     BEQ      $+66 (=0x000c24)
tag000bde:
  if (d.ViewportUpdated == 0) goto tag000c24;
//000be4 0c45 00a9                CMP.W    #169,D5
//000be8 6c3a                     BGE      $+60 (=0x000c24)
  if (D5W >= 169) goto tag000c24;
//000bea 0c44 00e0                CMP.W    #224,D4
//000bee 6c1c                     BGE      $+30 (=0x000c0c)
  if (D4W >= 224) goto tag000c0c;
//000bf0 4a6c a596                TST.W    -23146(A4)
//000bf4 670e                     BEQ      $+16 (=0x000c04)
  if (d.Word23146 == 0) goto tag000c04;
//000bf6 4a6c a592                TST.W    -23150(A4)
//000bfa 6708                     BEQ      $+10 (=0x000c04)
  if (d.Word23150 == 0) goto tag000c04;
//000bfc 0c45 0017                CMP.W    #23,D5
//000c00 6f22                     BLE      $+36 (=0x000c24)
  if (D5W <= 23) goto tag000c24;
  goto tag000c32;
//000c02 602e                     BRA      $+48 (=0x000c32)
tag000c04:
//000c04 0c45 0010                CMP.W    #16,D5
//000c08 6d1a                     BLT      $+28 (=0x000c24)
  if (D5W < 16) goto tag000c24;
//000c0a 6026                     BRA      $+40 (=0x000c32)
  goto tag000c32;
//000c0c 4a6c a596                TST.W    -23146(A4)
//000c10 6712                     BEQ      $+20 (=0x000c24)
tag000c0c:
  if (d.Word23146 == 0) goto tag000c24;
//000c12 0c45 001c                CMP.W    #28,D5
//000c16 6e0c                     BGT      $+14 (=0x000c24)
  if (D5W > 28) goto tag000c24;
//000c18 0c45 0017                CMP.W    #23,D5
//000c1c 6f06                     BLE      $+8 (=0x000c24)
  if (D5W <= 23) goto tag000c24;
//000c1e 0c44 00e8                CMP.W    #232,D4
//000c22 6d0e                     BLT      $+16 (=0x000c32)
  if (D4W < 232) goto tag000c32;
//000c24 4a6c d202                TST.W    -11774(A4)
//000c28 6706                     BEQ      $+8 (=0x000c30)
tag000c24:
  if (d.FlashButnActive == 0) goto tag000c30;
//000c2a 0c44 00c0                CMP.W    #192,D4
//000c2e 6c02                     BGE      $+4 (=0x000c32)
  if (D4W >= 192) goto tag000c32;
tag000c30:
  return;
//000c30 4e75                     RTS
tag000c32:
//000c32 7001                     MOVEQ.L  #1,D0
  D0L = 1;
//000c34 4eba 1c64                JSR      $+7270 (=0x00289a)
  RemoveCursor(); //TAG00289a();
//000c38 4e75                     RTS
}

ui32 VBLCount = 0;
ui32 VBLInterruptCount = 0;
void vblInterrupt(void) // TAG000c3a
{
// Vertical Blank Handler
//000c3a 48e7 fc8c                MOVEM.L  D012345A045,-(A7)
  dReg D0, D1, D2, D3, D4, D5;
  aReg A0;
  static i32 Count=0;
  VBLInterruptCount++;
  //SaveRegs(0xfc8c);
  //i32 *A4,*A5;
  //VBLCount++; // Counted by function that calls us.
//000c3e 287a feec                MOVE.L   $-274 (=0x000b2c),A4
//000c42 2a7a feec                MOVE.L   $-274 (=0x000b30),A5
  // A4 = data000b2c; // both start at zero
  // A5 = data000b30;
//000c46 4a6c d222                TST.W    -11742(A4)
//000c4a 6706                     BEQ      $+8 (=0x000c52)
  if (d.ViewportUpdated)
  {
//000c4c 396c d224 d226           MOVE.W   -11740(A4),-11738(A4)
    d.DynamicPaletteSwitching = d.Word11740;
  }
//000c52 4a6c d226                TST.W    -11738(A4)
//000c56 6724                     BEQ      $+38 (=0x000c7c)
  if (d.DynamicPaletteSwitching != 0)
  {
//000c58 426c d12c                CLR.W    -11988(A4)
    d.Word11988 = 0;
//000c5c 307c fa00                MOVE.W   #64000,A0  ; fffffa00
    A0 = (pnt)0xfffffa00;
//000c60 4228 001b                CLR.B    27(A0)     ; Timer B control
//000c64 117c 001f 0021           MOVE.B   #31,33(A0) ; Timer B data
//000c6a 117c 0008 001b           MOVE.B   #8,27(A0)  ; Timer B control
//000c70 0028 0001 0013           OR.B     #1,19(A0)  ; Interrupt Mask A
//000c76 0028 0001 0007           OR.B     #1,7(A0)   ; Interrupt Enable A
  };

//000c7c 4a6c d12a                TST.W    -11990(A4)
//000c80 670a                     BEQ      $+12 (=0x000c8c)
  if (d.Word11990 != 0)
  {
    return;
//000c82 526c d12a                ADDQ.W   #1,-11990(A4)
//000c86 4cdf 313f                MOVEM.L  (A7)+,D012345A045
//000c8a 4e73                     RTE
  };
//000c8c 526c d12a                ADDQ.W   #1,-11990(A4)
  d.Word11990++;
//                            Loop:  ; From below.
loop:
//000c90 40c0                     MOVE     SR,D0
//000c92 0240 f8ff                AND.W    #63743,D0
//000c96 0040 0300                OR.W     #768,D0
//000c9a 46c0                     MOVE     D0,SR
//000c9c 4a6c a59e                TST.W    -23138(A4)
//000ca0 6600 01d6                BNE      $+472 (=0x000e78)
  Instrumentation(icntVBLLoop);
  if (d.Word23138 == 0)
  {
//000ca4 7000                     MOVEQ.L  #0,D0
//000ca6 262c a5ac                MOVE.L   -23124(A4),D3
    D0L = 0;
    D3H1 = d.NewMouseY;//Word23124;
    D3W = d.NewMouseX;//Word23122;
//000caa 2943 a5a8                MOVE.L   D3,-23128(A4)
    if (!PlayFile.IsOpen())
    {
      d.CurMouseY = D3H1;//Word23128 = D3U;
      d.CurMouseX = D3W;//Word23126 = D3W;
    };
    D2L = D3L;
//000cae 2403                     MOVE.L   D3,D2
//000cb0 4842                     SWAP     D2
    SWAP(D2);
//000cb2 382c a590                MOVE.W   -23152(A4),D4
//000cb6 3a2c a58e                MOVE.W   -23154(A4),D5
    D4W = d.CurCursorX;
    D5W = d.CurCursorY;
//000cba 4a6c a598                TST.W    -23144(A4)
//000cbe 6718                     BEQ      $+26 (=0x000cd8)
    if (d.Word23144 != 0)
    {
//000cc0 0c43 0112                CMP.W    #274,D3
//000cc4 6d06                     BLT      $+8 (=0x000ccc)
//000cc6 0c42 001c                CMP.W    #28,D2
//000cca 6f0c                     BLE      $+14 (=0x000cd8)
      if ((D3W < 274) || (D2W > 28))
      {
//000ccc 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
        //SaveRegs(0xf0c0);
//000cd0 4eba 1b46                JSR      $+6984 (=0x002818)
        TAG002818();
//000cd4 4cdf 030f                MOVEM.L  (A7)+,D0123A01
        //RestoreRegs(0x030f);
      };
    };
//000cd8 302c a5a4                MOVE.W   -23132(A4),D0
//000cdc 426c a5a4                CLR.W    -23132(A4)
    D0W = d.Word23132;
    d.Word23132 = 0;
//000ce0 4a6c a5a6                TST.W    -23130(A4)
//000ce4 6714                     BEQ      $+22 (=0x000cfa)
//000ce6 4a6c a59a                TST.W    -23142(A4)
//000cea 660e                     BNE      $+16 (=0x000cfa)
//000cec b843                     CMP.W    D3,D4
//000cee 660a                     BNE      $+12 (=0x000cfa)
//000cf0 ba42                     CMP.W    D2,D5
//000cf2 6606                     BNE      $+8 (=0x000cfa)
    if (   (d.Word23130 != 0)
        && (d.NewCursorShape == 0)
        && (D3W == D4W)
        && (D2W == D5W) )
    {
//000cf4 7001                     MOVEQ.L  #1,D0
//000cf6 426c a5a6                CLR.W    -23130(A4)
      D0L = 1;
      d.Word23130 = 0;
    };
//000cfa 4eba feb4                JSR      $-330 (=0x000bb0)
    TAG000bb0(D0, D4, D5); // Expects something in D0, D5, D4//counted
    // May set D0 to 1;
//000cfe 2f08                     MOVE.L   A0,-(A7)
//000d00 41ec f8c6                LEA      -1850(A4),A0
//000d04 4a68 ffec                TST.W    -20(A0)
//000d08 6616                     BNE      $+24 (=0x000d20)
//000d0a 4a6c d21a                TST.W    -11750(A4)
//000d0e 6610                     BNE      $+18 (=0x000d20)
    if ((d.Word1870 == 0) && (d.Word11750 == 0))
    {
//000d10 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
      //SaveRegs(0xf0c0);
//000d14 4ead 01fe                JSR      510(A5)    (=0x00456c)
      TAG00456c();
//000d18 4cdf 030f                MOVEM.L  (A7)+,D0123A01
      //RestoreRegs(0x030f);
//000d1c 5a6c d21a                ADDQ.W   #5,-11750(A4)
        d.Word11750+=5;
    };
//000d20 205f                     MOVE.L   (A7)+,A0
//000d22 4a40                     TST.W    D0
//000d24 670c                     BEQ      $+14 (=0x000d32)
//000d26 0c42 000f                CMP.W    #15,D2
//000d2a 6206                     BHI      $+8 (=0x000d32)
    if ((D0W != 0) && ((ui16)D2W <= 15))
    {
//000d2c 4eba 1f70                JSR      $+8050 (=0x002c9e)
      DrawCursor();
      D0L = 0;
//000d30 7000                     MOVEQ.L  #0,D0
    };
//000d32 4a6c cd8c                TST.W    -12916(A4)
//000d36 6d34                     BLT      $+54 (=0x000d6c)
    if (d.TextAlreadyScrolledCount >= 0)
    {
//000d38 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
      //SaveRegs(0xf0c0);
//000d3c 323c 001b                MOVE.W   #27,D1
//000d40 926c cd8c                SUB.W    -12916(A4),D1
      D1W = (i16)(27 - d.TextAlreadyScrolledCount);
//000d44 c2fc 00a0                MULU     #160,D1
      D1L = (ui16)D1W * (ui16)160;
//000d48 3f01                     MOVE.W   D1,-(A7)
//000d4a 222c d132                MOVE.L   -11982(A4),D1
//000d4e 0681 0000 6b80           ADD.L    #27520,D1
//000d54 2f01                     MOVE.L   D1,-(A7)
//000d56 0681 0000 00a0           ADD.L    #160,D1
//000d5c 2f01                     MOVE.L   D1,-(A7)
//000d5e 4eba fc7c                JSR      $-898 (=0x0009dc)
//000d62 defc 000a                ADD.W    #10,A7
      MemMove(d.LogicalScreenBase+27520+160, // Up one line
              d.LogicalScreenBase+27520,
              D1W);
      Invalidate();
//000d66 4cdf 030f                MOVEM.L  (A7)+,D0123A01
      //RestoreRegs(0x030f);
//000d6a 600a                     BRA      $+12 (=0x000d76)
    }
    else
    {
//000d6c 4a6c cd8a                TST.W    -12918(A4)
//000d70 674c                     BEQ      $+78 (=0x000dbe)
      if (d.Word12918 == 0) goto tag000dbe;
//000d72 426c cd8a                CLR.W    -12918(A4)
      d.Word12918 = 0;
    };
//000d76 7207                     MOVEQ.L  #7,D1
    D1L = 7;
//000d78 4a6c cd8c                TST.W    -12916(A4)
//000d7c 6d14                     BLT      $+22 (=0x000d92)
    if (d.TextAlreadyScrolledCount >= 0)
    {
//000d7e 526c cd8c                ADDQ.W   #1,-12916(A4)
      d.TextAlreadyScrolledCount++;
//000d82 322c cd8c                MOVE.W   -12916(A4),D1
      D1W = d.TextAlreadyScrolledCount;
//000d86 0c41 0007                CMP.W    #7,D1
//000d8a 6606                     BNE      $+8 (=0x000d92)
      if (D1W == 7) d.TextAlreadyScrolledCount = -1;
//000d8c 397c ffff cd8c           MOVE.W   #65535,-12916(A4)
    };
//000d92 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
    //SaveRegs(0xf0c0);
//000d96 3401                     MOVE.W   D1,D2
//000d98 c2fc 00a0                MULU     #160,D1
    D2W = D1W;
    D1L = (ui16)D1W * (ui16)160;
//000d9c 3f01                     MOVE.W   D1,-(A7)
//000d9e 4442                     NEG.W    D2
    D2W = (i16)-D2W;
//000da0 0642 00c8                ADD.W    #200,D2
    D2W += 200;
//000da4 c4fc 00a0                MULU     #160,D2
    D2L = (ui16)D2W * (ui16)160;
//000da8 d4ac d132                ADD.L    -11982(A4),D2
//000dac 2f02                     MOVE.L   D2,-(A7)
//000dae 2f2c cd86                MOVE.L   -12922(A4),-(A7)
//000db2 4eba fc28                JSR      $-982 (=0x0009dc)
//000db6 defc 000a                ADD.W    #10,A7
    MemMove(d.newTextLine,
            d.LogicalScreenBase + D2L,
            D1W);
//000dba 4cdf 030f                MOVEM.L  (A7)+,D0123A01
    //RestoreRegs(0x030f);
tag000dbe:
//000dbe 4a6c d222                TST.W    -11742(A4)
//000dc2 6760                     BEQ      $+98 (=0x000e24)
    if (d.ViewportUpdated != 0)
    {
      dReg D0;
      pnt A0,A1;
//000dc4 426c d222                CLR.W    -11742(A4)
      d.ViewportUpdated = 0;
//000dc8 48e7 fff8                MOVEM.L  D01234567A01234,-(A7)
//000dcc 4a6c d1fe                TST.W    -11778(A4)
//000dd0 671a                     BEQ      $+28 (=0x000dec)
      if (d.Word11778 != 0)
      {
//000dd2 426c d1fe                CLR.W    -11778(A4)
        d.Word11778 = 0;
//000dd6 41ec fdd8                LEA      -552(A4),A0
        A0 = (pnt)&d.Palette552[d.CurrentPalette]; // palette address
//000dda 302c d24e                MOVE.W   -11698(A4),D0
//000dde eb48                     LSL.W    #5,D0
//000de0 d0c0                     ADD.W    D0,A0
//000de2 4cd0 00ff                MOVEM.L  (A0),D01234567
//000de6 48ec 00ff d156           MOVEM.L  D01234567,-11946(A4)
        memmove((pnt)&d.Palette11946,A0,32); // set current palette
      };
//000dec 206c d25e                MOVE.L   -11682(A4),A0
      A0 = d.pViewportBMP;
//000df0 226c d132                MOVE.L   -11982(A4),A1
//000df4 d2fc 14a0                ADD.W    #5280,A1
      A1 = d.LogicalScreenBase+5280;
//000df8 303c 0087                MOVE.W   #135,D0
      Instrumentation(icntCopyViewportToScreen);
      if ((VBLMultiplier==1)|| (((Count++)%3)==0))
      for (D0W=0; D0W <136; D0W++)
      {
//000dfc 4cd8 1cfe                MOVEM.L  (A0)+,D1234567A234
//000e00 48d1 1cfe                MOVEM.L  D1234567A234,(A1)
//000e04 4cd8 1cfe                MOVEM.L  (A0)+,D1234567A234
//000e08 48e9 1cfe 0028           MOVEM.L  D1234567A234,40(A1)
//000e0e 4cd8 04fe                MOVEM.L  (A0)+,D1234567A2
//000e12 48e9 04fe 0050           MOVEM.L  D1234567A2,80(A1)
        memmove(A1,A0,112);
        A0 += 112;
        A1 += 160;
//000e18 43e9 00a0                LEA      160(A1),A1
//000e1c 51c8 ffde                DBF      D0,$-32 (=0x000dfc)
      };
//000e20 4cdf 1fff                MOVEM.L  (A7)+,D01234567A01234
    };
//000e24 4a6c d202                TST.W    -11774(A4)
//000e28 6726                     BEQ      $+40 (=0x000e50)
    if (d.FlashButnActive != 0)
    {
//000e2a 426c d202                CLR.W    -11774(A4)
      d.FlashButnActive = 0;
//000e2e 466c d200                NOT.W    -11776(A4)
      d.Word11776 ^= 0xffff;
//000e32 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
      //SaveRegs(0xf0c0);
//000e36 3f2c d204                MOVE.W   -11772(A4),-(A7)
//000e3a 3f2c d206                MOVE.W   -11770(A4),-(A7)
//000e3e 3f2c d208                MOVE.W   -11768(A4),-(A7)
//000e42 3f2c d20a                MOVE.W   -11766(A4),-(A7)
//000e46 4eba fad4                JSR      $-1322 (=0x00091c)
//000e4a 508f                     ADDQ.L   #0,A7
      TraceFlash("vblFlashButn from 000e46");
      vblFlashButn(d.FlashButnX1,
                   d.FlashButnX2,
                   d.FlashButnY1,
                   d.FlashButnY2);
//000e4c 4cdf 030f                MOVEM.L  (A7)+,D0123A01
      //RestoreRegs(0x030f);
    };
//000e50 4a40                     TST.W    D0
//000e52 6704                     BEQ      $+6 (=0x000e58)
    if (D0W != 0)
    {
//000e54 4eba 1e48                JSR      $+7754 (=0x002c9e)
      DrawCursor(); // put cursor in LogBase...save old pixels//counted
    };
//000e58 4a6c a5a6                TST.W    -23130(A4)
//000e5c 671a                     BEQ      $+28 (=0x000e78)
    if (d.Word23130 != 0)
    {
//000e5e 426c a5a6                CLR.W    -23130(A4)
      d.Word23130 = 0;
//000e62 397c 0001 a5a4           MOVE.W   #1,-23132(A4)
//000e68 426c a59a                CLR.W    -23142(A4)
      d.Word23132 = 1;
      d.NewCursorShape = 0;
//000e6c 48e7 f0c0                MOVEM.L  D0123A01,-(A7)
      //SaveRegs(0xf0c0);
//000e70 4eba 1aec                JSR      $+6894 (=0x00295e)
      CreateCursorBitmap(); // TAG00295e
//000e74 4cdf 030f                MOVEM.L  (A7)+,D0123A01
      //RestoreRegs(0x030f);
    };
  };
//000e78 4a6c a544                TST.W    -23228(A4)
//000e7c 6f0c                     BLE      $+14 (=0x000e8a)
  if (d.Word23228 > 0)
  {
//000e7e 206c a540                MOVE.L   -23232(A4),A0
    //A0 = d.pFunc23232;
//000e82 2f09                     MOVE.L   A1,-(A7)
//000e84 4e71                     NOP
//000e86 225f                     MOVE.L   (A7)+,A1
//000e88 600c                     BRA      $+14 (=0x000e96)
  }
  else
  {
//000e8a 4a6c a546                TST.W    -23226(A4)
//000e8e 6706                     BEQ      $+8 (=0x000e96)
    if (d.Word23226 != 0)
    {
//000e90 206c a53c                MOVE.L   -23236(A4),A0
      //A0 = d.pFunc23236;
//000e94 4e71                     NOP
    };
  };
//000e96 526c d232                ADDQ.W   #1,-11726(A4)
  d.vblCount += 1;
//000e9a 302c d232                MOVE.W   -11726(A4),D0
  //D0W = d.vblCount;
//000e9e b06c d230                CMP.W    -11728(A4),D0
//000ea2 6306                     BLS      $+8 (=0x000eaa)
  if ((d.vblCount >= d.Speed) || (NoSpeedLimit!=0))
  {
    if (NoSpeedLimit!=0)
    {
      if (d.Time >= NoSpeedLimit)
      {
        NoSpeedLimit = 0;
        VBLMultiplier = 1;
      };
    };
//000ea4 397c 0001 d228           MOVE.W   #1,-11736(A4)
    d.clockTick = 1;
  }
  else
  {
    if (   playerClock
        && (d.vblCount > 10)
        && !mouseQueueIsEmpty() 
        && !PlayFile.IsOpen() )
    {
      d.clockTick = 1;
    };
  }
//000eaa 4e44                     TRAP     #4   ; Original vertical blan handler
//000eac 40c0                     MOVE     SR,D0
//000eae 0240 f8ff                AND.W    #63743,D0
//000eb2 0040 0400                OR.W     #1024,D0
//000eb6 46c0                     MOVE     D0,SR
//000eb8 536c d12a                SUBQ.W   #1,-11990(A4)
  d.Word11990 -= 1;
//000ebc 6600 fdd2                BNE      $-556 (=0x000c90)
  if (d.Word11990 != 0) goto loop;

//000ec0 4cdf 313f                MOVEM.L  (A7)+,D012345A045
  //RestoreRegs(0x313f);
//000ec4 4e73                     RTE
  return;
}



void TAG000ec6(void)
{
//////;;;;;;;;;;;;;;;;;;;;;;;;;;;; Set some user interrupt vectors
////000ec6 4e56 0000                LINK     A6,#0
////000eca 41fa fc68                LEA      $-918 (=0x000b34),A0
////000ece 21c8 0114                MOVE.L   A0,276   TRAP  #37
////
////000ed2 41fa fc74                LEA      $-906 (=0x000b48),A0
////000ed6 20b8 0118                MOVE.L   280,(A0)  save TRAP 38 vector
////000eda 41fa fc5a                LEA      $-932 (=0x000b36),A0
////000ede 21c8 0118                MOVE.L   A0,280   TRAP  #38
////
////000ee2 41fa fc68                LEA      $-918 (=0x000b4c),A0
////000ee6 21c8 0120                MOVE.L   A0,288   # TRAP 40
////000eea 21f8 0070 0090           MOVE.L   112,144  TRAP#4 = Vertical Blank
////                                   TRAP #4 now goes to vertical blank code.
////000ef0 41fa fd48                LEA      $-694 (=0x000c3a),A0
////000ef4 21c8 0070                MOVE.L   A0,112   Vertical Blank
  verticalIntEnabled = true;
////000ef8 4e5e                     UNLK     A6
////000efa 4e75                     RTS
}

void SystemError(i16 errorNum)
{
  die(errorNum, "");
}

//   TAG000f7e
void BLT2Screen(i8 *P1,RectPos *dstPos,i16 P3,i16 P4)
{   //             8       12     16     18
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0f7e 4e56 0000                LINK     A6,#0
//000f82 303c 0000                MOVE.W   #0,D0
//000f86 3940 a608                MOVE.W   D0,-23032(A4)
  d.UseByteCoordinates = 0;
  ASSERT(verifyRectPos(dstPos));
//000f8a 3f2e 0012                MOVE.W   18(A6),-(A7)
//000f8e 3f3c 00a0                MOVE.W   #160,-(A7)
//000f92 3f2e 0010                MOVE.W   16(A6),-(A7)
//000f96 4267                     CLR.W    -(A7)
//000f98 4267                     CLR.W    -(A7)
//000f9a 2f2e 000c                MOVE.L   12(A6),-(A7)
//000f9e 2f2c d132                MOVE.L   -11982(A4),-(A7)
//000fa2 2f2e 0008                MOVE.L   8(A6),-(A7)
//000fa6 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//000faa defc 0016                ADD.W    #22,A7
  TAG0088b2(P1,
            d.LogicalScreenBase,
            dstPos,
            0,   // src x
            0,   // src y
            P3,  // src bytes/line
            160, // dst bytes/line
            P4);
//000fae 4e5e                     UNLK     A6
//000fb0 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG000fb2
RESTARTABLE _VBLDelay(const i32 n) //void
{ // Pause for n vertical blanks
  static i32 i;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//000fb2 4e56 0000                LINK     A6,#0
  for (i=0; i<n; i++)
  {
//000fb6 600a                     BRA      $+12 (=0x000fc2)
//000fb8 3f3c 0025                MOVE.W   #37,-(A7)
//000fbc 4eba 0d2a                JSR      $+3372 (=0x001ce8)
//000fc0 548f                     ADDQ.L   #2,A7
    wvbl(_1_);
//000fc2 302e 0008                MOVE.W   8(A6),D0
//000fc6 536e 0008                SUBQ.W   #1,8(A6)
//000fca 4a40                     TST.W    D0
//000fcc 66ea                     BNE      $-20 (=0x000fb8)
  };
//000fce 4e5e                     UNLK     A6
//000fd0 4e75                     RTS
  RETURN;
}

/*
i16 Abs(i32 x)
{
  if (x>=0) return (i16)x; else return (i16)-x;
}

i32 Min(i32 n, i32 m)
{
  return n<m?n:m;
}


//  TAG000ff8
i16 Max(i16 x, i16 y)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//000ff8 4e56 0000                LINK     A6,#0
//000ffc 302e 0008                MOVE.W   8(A6),D0
//001000 b06e 000a                CMP.W    10(A6),D0
//001004 6e04                     BGT      $+6 (=0x00100a)
  if (x <= y) return y;
  else return x;
//001006 302e 000a                MOVE.W   10(A6),D0
//00100a 4e5e                     UNLK     A6
//00100c 4e75                     RTS
}

  */
//  TAG00100e
i16 ApplyLimits(i32 min,i32 x,i32 max)
{
  if      (x <= min) x = min;
  else if (x >  max) x = max;
  return (i16)x;
}

unsigned int numRandomCalls = 0;

i32 STRandom(void) //TAG001030
{
  dReg D0;
  numRandomCalls++;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001030 4e56 0000                LINK     A6,#0
//001034 2f2c d12e                MOVE.L   -11986(A4),-(A7)
//001038 2f3c bb40 e62d           MOVE.L   #-1153374675,-(A7)
//00103e 4eba 2118                JSR      $+8474 (=0x003158)
  D0L = d.RandomNumber * ui32(0xbb40e62d) + 11;
//001042 700b                     MOVEQ.L  #11,D0
//001044 d09f                     ADD.L    (A7)+,D0
//001046 2940 d12e                MOVE.L   D0,-11986(A4)
  d.RandomNumber = D0L;
//00104a e088                     LSR.L    #8,D0
  D0L >>= 8;
  D0L &= 0xffffff;
  return D0L;
//00104c 4e5e                     UNLK     A6
//00104e 4e75                     RTS
}

i32 STRandom(i32 num)
{
  return (STRandom() & 0xffff) % num; // To be exactly like Atari
}

//  TAG001050
ui8 STRandomBool(void)
{ // Zero or One
  numRandomCalls++;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001050 4e56 0000                LINK     A6,#0
//001054 2f2c d12e                MOVE.L   -11986(A4),-(A7)
//001058 2f3c bb40 e62d           MOVE.L   #-1153374675,-(A7)
//00105e 4eba 20f8                JSR      $+8442 (=0x003158)
//001062 700b                     MOVEQ.L  #11,D0
//001064 d09f                     ADD.L    (A7)+,D0
  d.RandomNumber = d.RandomNumber * (ui32)0xbb40e62d + 11;
//001066 2940 d12e                MOVE.L   D0,-11986(A4)
//00106a e088                     LSR.L    #8,D0
//00106c c07c 0001                AND.W    #1,D0
  return (UI8)((d.RandomNumber>>8) & 1);
//001070 4e5e                     UNLK     A6
//001072 4e75                     RTS
}

//  TAG001074
ui8 STRandom0_3(void)
{
  numRandomCalls++;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001074 4e56 0000                LINK     A6,#0
//001078 2f2c d12e                MOVE.L   -11986(A4),-(A7)
//00107c 2f3c bb40 e62d           MOVE.L   #-1153374675,-(A7)
//001082 4eba 20d4                JSR      $+8406 (=0x003158)
//001086 700b                     MOVEQ.L  #11,D0
//001088 d09f                     ADD.L    (A7)+,D0
  d.RandomNumber = d.RandomNumber * (ui32)0xbb40e62d + 11;
//00108a 2940 d12e                MOVE.L   D0,-11986(A4)
//00108e e088                     LSR.L    #8,D0
//001090 c07c 0003                AND.W    #3,D0
  return (UI8)((d.RandomNumber>>8) & 3);
//001094 4e5e                     UNLK     A6
//001096 4e75                     RTS
}

void TAG0010ae(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0010ae 4e56 0000                LINK     A6,#0
  dReg D7;
  aReg A2, A3;
  //i32 saveD7=D7;pnt saveA2=A2,saveA3=A3;
//0010b2 48e7 0130                MOVEM.L  D7A23,-(A7)
//0010b6 3f3c 0001                MOVE.W   #1,-(A7)
//0010ba 2f3c 0000 0080           MOVE.L   #128,-(A7)
//0010c0 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0010c4 5c8f                     ADDQ.L   #6,A7
  d.Pointer16572 = allocateMemory(128,1);
//0010c6 2940 bf44                MOVE.L   D0,-16572(A4)
//0010ca 2f3c 022c 0001           MOVE.L   #36438017,-(A7)
//0010d0 4ead 05dc                JSR      1500(A5)    (=0x022d7a)
//0010d4 548f                     ADDQ.L   #2,A7
//0010d6 d07c 00c7                ADD.W    #199,D0
//0010da c0bc 0000 ffff           AND.L    #65535,D0
//0010e0 2f00                     MOVE.L   D0,-(A7)
//0010e2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0010e6 5c8f                     ADDQ.L   #6,A7
  A3 = allocateMemory((GetGraphicExpSize(0x22c)+199)&0xffff,1);
//0010e8 2640                     MOVE.L   D0,A3
//0010ea 4267                     CLR.W    -(A7)
//0010ec 2f3c 0000 0ae2           MOVE.L   #2786,-(A7)
//0010f2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0010f6 5c8f                     ADDQ.L   #6,A7
  A2 = allocateMemory(2786,0);
//0010f8 2440                     MOVE.L   D0,A2
//0010fa 2f0a                     MOVE.L   A2,-(A7)
//0010fc 3f3c 822c                MOVE.W   #33324,-(A7)
//001100 4ead 0606                JSR      1542(A5)    (=0x022b86)
//001104 5c8f                     ADDQ.L   #6,A7
  ReadAndExpandGraphic(33324,A2,0,0);
//001106 4247                     CLR.W    D7
//001108 6028                     BRA      $+42 (=0x001132)
  for (D7W=0;D7W<199;D7W++)
  {
//00110a 3007                     MOVE.W   D7,D0
//00110c e580                     ASL.L    #2,D0
//00110e 41ec ce0e                LEA      -12786(A4),A0
//001112 d0c0                     ADD.W    D0,A0
//001114 208b                     MOVE.L   A3,(A0)
    d.ObjectNames[D7W] = A3;
//001116 6002                     BRA      $+4 (=0x00111a)
    while ((*A2 & 128)   == 0)
    {
//001118 16da                     MOVE.B   (A2)+,(A3)+
      *(A3++) = *(A2++);
//00111a 4240                     CLR.W    D0
//00111c 1012                     MOVE.B   (A2),D0
//00111e c07c 0080                AND.W    #128,D0
//001122 67f4                     BEQ      $-10 (=0x001118)
    };
//001124 4240                     CLR.W    D0
//001126 101a                     MOVE.B   (A2)+,D0
//001128 c07c 007f                AND.W    #127,D0
//00112c 16c0                     MOVE.B   D0,(A3)+
    *(A3++) = (i8)(*(A2++)&0x7f);
    *(A3++) = 0;
//00112e 421b                     CLR.B    (A3)+
//001130 5247                     ADDQ.W   #1,D7

//001132 0c47 00c7                CMP.W    #199,D7
//001136 65d2                     BCS      $-44 (=0x00110a)
  };
  BuildSmartDiscardTable();
//001138 2f3c 0000 0ae2           MOVE.L   #2786,-(A7)
//00113e 4ead 05be                JSR      1470(A5)    (=0x0217d8)
//001142 588f                     ADDQ.L   #4,A7
  FreeTempMemory(2786);
  //D7=saveD7;A2=saveA2;A3=saveA3;
//001144 4cdf 0c80                MOVEM.L  (A7)+,D7A23
//001148 4e5e                     UNLK     A6
//00114a 4e75                     RTS
}

OBJECTTYPE GetBasicObjectType(RN object)
{
  dReg D0, D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00114c 4e56 0000                LINK     A6,#0
//001150 3f07                     MOVE.W   D7,-(A7)
//001152 0c6e ffff 0008           CMP.W    #65535,8(A6)
//001158 6606                     BNE      $+8 (=0x001160)
  if (object == RNnul)
  {
//00115a 303c ffff                MOVE.W   #65535,D0
//00115e 6022                     BRA      $+36 (=0x001182)
    D0W = -1;
  }
  else
  {
//001160 3f2e 0008                MOVE.W   8(A6),-(A7)
//001164 4ead 0306                JSR      774(A5)    (=0x009308)
//001168 548f                     ADDQ.L   #2,A7
//00116a 3e00                     MOVE.W   D0,D7
    D7W = GetObjectIndex(object);//TAG009308
//00116c 0c47 ffff                CMP.W    #65535,D7
//001170 670e                     BEQ      $+16 (=0x001180)
    if (D7W != -1)
    {
//001172 3007                     MOVE.W   D7,D0
//001174 c1fc 0006                MULS     #6,D0
//001178 41ec df42                LEA      -8382(A4),A0
//00117c d0c0                     ADD.W    D0,A0
//00117e 3e10                     MOVE.W   (A0),D7
      D7W = (i16)d.ObjDesc[D7W].objectType();
    };
//001180 3007                     MOVE.W   D7,D0
    D0W = D7W;
  };
//001182 3e1f                     MOVE.W   (A7)+,D7
//001184 4e5e                     UNLK     A6
//001186 4e75                     RTS
  return (OBJECTTYPE)(D0W);
}

//         TAG001534
OBJECTTYPE ObjectAtLocation(i32 bodyLocation)
{ // Object type in nth (0 to 7) hand at top of screen.
  // Or object in nth position of clothing/backpack (8 to 37)
  //dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001534 4e56 0000                LINK     A6,#0
//001538 302e 0008                MOVE.W   8(A6),D0
  //D0W = 6*index;
//00153c c1fc 0006                MULS     #6,D0
//001540 41ec fc70                LEA      -912(A4),A0
//001544 d0c0                     ADD.W    D0,A0
  return d.IconDisplay[bodyLocation].objectType();
//001546 3010                     MOVE.W   (A0),D0
//001548 4e5e                     UNLK     A6
//00154a 4e75                     RTS
  //return D0W;
}

//   TAG001836
void SetPrintPosition(i16 column, i16 row)
{
  if (column <   0) column = 0;
  if (column >= 53) column = 52;
  if (row <   0) row = 0;
  if (row >=  4) row = 3;
  d.PrintColumn = column;
  d.PrintRow = row;
}


RESTARTABLE _TAG0018f6(void)
{//(void)
  static dReg D0, D5, D6;
  static RectPos LOCAL_8;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0018f6 4e56 fff8                LINK     A6,#-8
//0018fa 48e7 0700                MOVEM.L  D567,-(A7)
//0018fe 426c a608                CLR.W    -23032(A4)
  d.UseByteCoordinates = 0;
//001902 426e fff8                CLR.W    -8(A6)
  LOCAL_8.w.x1 = 0;
//001906 3d7c 013f fffa           MOVE.W   #319,-6(A6)
  LOCAL_8.w.x2 = 319;
//00190c 4246                     CLR.W    D6
//00190e 6074                     BRA      $+118 (=0x001984)
  for (D6W=0; D6W<4; D6W++)
  {
//001910 3a06                     MOVE.W   D6,D5
//001912 e585                     ASL.L    #2,D5
//001914 41ec cd6e                LEA      -12946(A4),A0
//001918 d0c5                     ADD.W    D5,A0
//00191a 2a10                     MOVE.L   (A0),D5
    D5L = d.TextTimeout[D6W];
//00191c 0c85 ffff ffff           CMP.L    #-1,D5
//001922 670a                     BEQ      $+12 (=0x00192e)
    if (D5L == -1) continue;
//001924 2005                     MOVE.L   D5,D0
//001926 222c d23a                MOVE.L   -11718(A4),D1
//00192a b081                     CMP.L    D1,D0
//00192c 6f02                     BLE      $+4 (=0x001930)
//00192e 6052                     BRA      $+84 (=0x001982)
    if (D5L > d.Time) continue;
//001930 303c 00ac                MOVE.W   #172,D0
//001934 3206                     MOVE.W   D6,D1
//001936 c2fc 0007                MULU     #7,D1
//00193a d041                     ADD.W    D1,D0
    D0W = (i16)(172 + D6W*7);
//00193c 3d40 fffc                MOVE.W   D0,-4(A6)
    LOCAL_8.w.y1 = D0W;
//001940 5c40                     ADDQ.W   #6,D0
//001942 3d40 fffe                MOVE.W   D0,-2(A6)
    LOCAL_8.w.y2 = (i16)(D0W +6);
//001946 0c6c 0000 cd8c           CMP.W    #0,-12916(A4)
//00194c 6cf8                     BGE      $-6 (=0x001946)
      while (   (d.TextAlreadyScrolledCount >= 0)
             || (d.Word12918 !=0) )
      {
        wvbl(_1_);//NotImplemented(0x194e);//pumper();
      };
//00194e 302c cd8a                MOVE.W   -12918(A4),D0
//001952 66f2                     BNE      $-12 (=0x001946)
//001954 4eba 167c                JSR      $+5758 (=0x002fd2)
    STHideCursor();//TAG002fd2
//001958 3f3c 00a0                MOVE.W   #160,-(A7)
//00195c 4267                     CLR.W    -(A7)
//00195e 486e fff8                PEA      -8(A6)
//001962 2f2c d132                MOVE.L   -11982(A4),-(A7)
//001966 4ead 01ce                JSR      462(A5)    (=0x008e60)
//00196a defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase, (RectPos *)&LOCAL_8, 0, 160);
//00196e 4eba 16b6                JSR      $+5816 (=0x003026)
    STShowCursor(); //TAG003026
//001972 3006                     MOVE.W   D6,D0
//001974 e580                     ASL.L    #2,D0
//001976 41ec cd6e                LEA      -12946(A4),A0
//00197a d0c0                     ADD.W    D0,A0
//00197c 20bc ffff ffff           MOVE.L   #-1,(A0)
    d.TextTimeout[D6W] = -1;
//001982 5246                     ADDQ.W   #1,D6
//
//001984 0c46 0004                CMP.W    #4,D6
//001988 6586                     BCS      $-120 (=0x001910)
  };
//00198a 4cdf 00e0                MOVEM.L  (A7)+,D567
//00198e 4e5e                     UNLK     A6
//001990 4e75                     RTS
  RETURN;
}

RESTARTABLE _TAG001992(void)
{//(void)
  static dReg D7;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001992 4e56 0000                LINK     A6,#0
//001996 3f07                     MOVE.W   D7,-(A7)
//001998 0c6c 0003 cd80           CMP.W    #3,-12928(A4)
//00199e 664e                     BNE      $+80 (=0x0019ee)
  if (d.PrintRow == 3)
  {
//0019a0 0c6c 0000 cd8c           CMP.W    #0,-12916(A4)
//0019a6 6cf8                     BGE      $-6 (=0x0019a0)
//0019a8 302c cd8a                MOVE.W   -12918(A4),D0
//0019ac 66f2                     BNE      $-12 (=0x0019a0)
    while (   (d.TextAlreadyScrolledCount>=0)
      || (d.Word12918!=0) )
    { //NotImplemented(0x19ae);};//pumper();};
      wvbl(_1_);
    };
//0019ae 3f3c 0460                MOVE.W   #1120,-(A7)
//0019b2 2f2c cd86                MOVE.L   -12922(A4),-(A7)
//0019b6 4eba f0cc                JSR      $-3890 (=0x000a84)
//0019ba 5c8f                     ADDQ.L   #6,A7
    ClearMemory(d.newTextLine, 1120);
//0019bc 426c cd8c                CLR.W    -12916(A4)
    d.TextAlreadyScrolledCount = 0; //Start new line into scroll area
//0019c0 4247                     CLR.W    D7
//0019c2 601a                     BRA      $+28 (=0x0019de)
    for (D7W=0; D7W<3; D7W++)
    {
//0019c4 3007                     MOVE.W   D7,D0
//0019c6 e580                     ASL.L    #2,D0
//0019c8 41ec cd72                LEA      -12942(A4),A0
//0019cc d0c0                     ADD.W    D0,A0
//0019ce 2010                     MOVE.L   (A0),D0
      d.TextTimeout[D7W] = d.TextTimeout[D7W+1];
//0019d0 3207                     MOVE.W   D7,D1
//0019d2 e581                     ASL.L    #2,D1
//0019d4 41ec cd6e                LEA      -12946(A4),A0
//0019d8 d0c1                     ADD.W    D1,A0
//0019da 2080                     MOVE.L   D0,(A0)
//0019dc 5247                     ADDQ.W   #1,D7
//0019de 0c47 0003                CMP.W    #3,D7
//0019e2 65e0                     BCS      $-30 (=0x0019c4)
    };

//0019e4 297c ffff ffff cd7a      MOVE.L   #-1,-12934(A4)
//0019ec 6004                     BRA      $+6 (=0x0019f2)
    d.TextTimeout[3] = -1;
    RETURN;
  };
//0019ee 526c cd80                ADDQ.W   #1,-12928(A4)
  d.PrintRow++;
//0019f2 3e1f                     MOVE.W   (A7)+,D7
//0019f4 4e5e                     UNLK     A6
//0019f6 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************

//   TAG0019f8
void Printf(i32 color, pnt text)
{
  dReg D0, D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0019f8 4e56 0000                LINK     A6,#0
//0019fc 48e7 0110                MOVEM.L  D7A3,-(A7)
//001a00 266e 000a                MOVE.L   10(A6),A3
//001a04 2f0b                     MOVE.L   A3,-(A7)
//001a06 4eba 18b2                JSR      $+6324 (=0x0032ba)
//001a0a 588f                     ADDQ.L   #4,A7
//001a0c 3e00                     MOVE.W   D0,D7
  D7W = StrLen(text);
//001a0e 0c6c ffff cd8c           CMP.W    #65535,-12916(A4)
//001a14 6638                     BNE      $+58 (=0x001a4e)
  if (d.TextAlreadyScrolledCount == -1)
  {
//001a16 302c cd8a                MOVE.W   -12918(A4),D0
//001a1a 66fa                     BNE      $-4 (=0x001a16)
    while (d.Word12918 != 0) {};
//001a1c 4eba 15b4                JSR      $+5558 (=0x002fd2)
    STHideCursor();
//001a20 2f0b                     MOVE.L   A3,-(A7)
//001a22 4267                     CLR.W    -(A7)
//001a24 3f2e 0008                MOVE.W   8(A6),-(A7)
//001a28 302c cd80                MOVE.W   -12928(A4),D0
//001a2c c1fc 0007                MULS     #7,D0
//001a30 d07c 00b1                ADD.W    #177,D0
//001a34 3f00                     MOVE.W   D0,-(A7)
//001a36 302c cd7e                MOVE.W   -12930(A4),D0
//001a3a c1fc 0006                MULS     #6,D0
//001a3e 3f00                     MOVE.W   D0,-(A7)
//001a40 4eba 0200                JSR      $+514 (=0x001c42)
//001a44 defc 000c                ADD.W    #12,A7
    TextOutToScreen(d.PrintColumn*6, d.PrintRow*7+177, color, 0, text);
//001a48 4eba 15dc                JSR      $+5598 (=0x003026)
    STShowCursor();
//001a4c 6034                     BRA      $+54 (=0x001a82)
  }
  else
  {
//001a4e 2f0b                     MOVE.L   A3,-(A7)
//001a50 4267                     CLR.W    -(A7)
//001a52 3f2e 0008                MOVE.W   8(A6),-(A7)
//001a56 3f3c 0005                MOVE.W   #5,-(A7)
//001a5a 302c cd7e                MOVE.W   -12930(A4),D0
//001a5e c1fc 0006                MULS     #6,D0
//001a62 3f00                     MOVE.W   D0,-(A7)
//001a64 3f3c 00a0                MOVE.W   #160,-(A7)
//001a68 2f2c cd86                MOVE.L   -12922(A4),-(A7)
//001a6c 4eba fade                JSR      $-1312 (=0x00154c)
//001a70 defc 0012                ADD.W    #18,A7
    TextOut(d.newTextLine, 160, d.PrintColumn*6, 5, color, 0, text);
//001a74 0c6c ffff cd8c           CMP.W    #65535,-12916(A4)
//001a7a 6606                     BNE      $+8 (=0x001a82)
    if (d.TextAlreadyScrolledCount == -1)
    {
//001a7c 397c 0001 cd8a           MOVE.W   #1,-12918(A4)
      d.Word12918 = 1;
    };
  };
//001a82 3007                     MOVE.W   D7,D0
//001a84 d16c cd7e                ADD.W    D0,-12930(A4)
  d.PrintColumn = (i16)(d.PrintColumn + D7W);
//001a88 202c d23a                MOVE.L   -11718(A4),D0
//001a8c d0bc 0000 0046           ADD.L    #70,D0
  D0L = d.Time + 70; // about 10 seconds
//001a92 322c cd80                MOVE.W   -12928(A4),D1
//001a96 e581                     ASL.L    #2,D1
//001a98 41ec cd6e                LEA      -12946(A4),A0
//001a9c d0c1                     ADD.W    D1,A0
//001a9e 2080                     MOVE.L   D0,(A0)
  d.TextTimeout[d.PrintRow] = D0L;
//001aa0 4cdf 0880                MOVEM.L  (A7)+,D7A3
//001aa4 4e5e                     UNLK     A6
//001aa6 4e75                     RTS
}

//*********************************************************
//PrintLines needs to be restartable.  And that is very
//inconvenient.  So what do we do?  We change everyone
//to call QuePrintLines instead!  Then when it is convenient
//for us to call a restartable function we will take the
//lines out of the queue and call PrintLines.  We will probably
//do this in MainLoop.
//*********************************************************
void QuePrintLines(i16 color, pnt text)
{
  printQueue.Queue(color,1,text);  //There.  That was easy.
}

//*********************************************************
//
//*********************************************************
//   TAG001aa8
RESTARTABLE _PrintLines(const i32 color, pnt Text)
{//(void)
  static dReg D0, D6;
  static i8  LOCAL_56[54];
  static i16 LOCAL_2;
  static pnt text;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
  END_RESTARTMAP
  text = Text;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001aa8 4e56 ffc8                LINK     A6,#-56
//001aac 48e7 0310                MOVEM.L  D67A3,-(A7)
//001ab0 3e2e 0008                MOVE.W   8(A6),D7
  //D7W = P1;
  //A3 = P2;
//001ab4 266e 000a                MOVE.L   10(A6),A3
//001ab8 6000 00a2                BRA      $+164 (=0x001b5c)

  while (*text != 0)
  {
//001abc 1013                     MOVE.B   (A3),D0
    D0W = (i16)sb(*text);
//001abe 4880                     EXT.W    D0
//001ac0 b07c 000a                CMP.W    #10,D0
    if (D0W == 10) // Linefeed
//001ac4 661c                     BNE      $+30 (=0x001ae2)
    {
//001ac6 528b                     ADDQ.L   #1,A3
      text++;
//001ac8 0c6c 0000 cd7e           CMP.W    #0,-12930(A4)
//001ace 6608                     BNE      $+10 (=0x001ad8)
//001ad0 0c6c 0000 cd80           CMP.W    #0,-12928(A4)
//001ad6 6708                     BEQ      $+10 (=0x001ae0)
      if ( (d.PrintColumn!=0) || (d.PrintRow!=0) )
      {
//001ad8 426c cd7e                CLR.W    -12930(A4)
//001adc 4eba feb4                JSR      $-330 (=0x001992)
        d.PrintColumn = 0;
        TAG001992(_2_); // scroll timeouts for the 4 lines
      };
//001ae0 607a                     BRA      $+124 (=0x001b5c)
    }
    else
    {
//001ae2 1013                     MOVE.B   (A3),D0
//001ae4 4880                     EXT.W    D0
      D0W = (i16)sb(*text);
//001ae6 b07c 0020                CMP.W    #32,D0
//001aea 661e                     BNE      $+32 (=0x001b0a)
      if (D0W == 32)
      {
//001aec 528b                     ADDQ.L   #1,A3
        text++;
//001aee 0c6c 0035 cd7e           CMP.W    #53,-12930(A4)
//001af4 6712                     BEQ      $+20 (=0x001b08)
        if (d.PrintColumn != 53)
        {
//001af6 3d7c 2000 fffe           MOVE.W   #8192,-2(A6)
          LOCAL_2 = 0x0020; // A single blank
//001afc 486e fffe                PEA      -2(A6)
//001b00 3f07                     MOVE.W   D7,-(A7)
//001b02 4eba fef4                JSR      $-266 (=0x0019f8)
//001b06 5c8f                     ADDQ.L   #6,A7
          Printf(color, (pnt)&LOCAL_2);
        };
//001b08 6052                     BRA      $+84 (=0x001b5c)
      }
      else
      {
//001b0a 4246                     CLR.W    D6
        D6W = 0;
        do
        {
//001b0c 3006                     MOVE.W   D6,D0
//001b0e 5246                     ADDQ.W   #1,D6
//001b10 41ee ffc8                LEA      -56(A6),A0
//001b14 d0c0                     ADD.W    D0,A0
//001b16 109b                     MOVE.B   (A3)+,(A0)
          LOCAL_56[D6W++] = *(text++);

//001b18 1013                     MOVE.B   (A3),D0
//001b1a 6714                     BEQ      $+22 (=0x001b30)

//001b1c 1013                     MOVE.B   (A3),D0
//001b1e 4880                     EXT.W    D0
//001b20 b07c 0020                CMP.W    #32,D0
//001b24 670a                     BEQ      $+12 (=0x001b30)
//001b26 1013                     MOVE.B   (A3),D0
//001b28 4880                     EXT.W    D0
//001b2a b07c 000a                CMP.W    #10,D0
//001b2e 66dc                     BNE      $-34 (=0x001b0c)
        } while ((*text!=0) && (*text!=32) && (*text!=10));
//001b30 3006                     MOVE.W   D6,D0
//001b32 41ee ffc8                LEA      -56(A6),A0
//001b36 d0c0                     ADD.W    D0,A0
//001b38 4210                     CLR.B    (A0)
        LOCAL_56[D6W] = 0;
//001b3a 302c cd7e                MOVE.W   -12930(A4),D0
//001b3e d046                     ADD.W    D6,D0
//001b40 0c40 0035                CMP.W    #53,D0
//001b44 630a                     BLS      $+12 (=0x001b50)
        if (d.PrintColumn+D6W > 53)
        {
//001b46 397c 0002 cd7e           MOVE.W   #2,-12930(A4)
          d.PrintColumn = 2;
//001b4c 4eba fe44                JSR      $-442 (=0x001992)
          TAG001992(_1_);
        };
//001b50 486e ffc8                PEA      -56(A6)
//001b54 3f07                     MOVE.W   D7,-(A7)
//001b56 4eba fea0                JSR      $-350 (=0x0019f8)
//001b5a 5c8f                     ADDQ.L   #6,A7
        Printf(color, LOCAL_56);
      };
    };
//001b5c 1013                     MOVE.B   (A3),D0
//001b5e 6600 ff5c                BNE      $-162 (=0x001abc)
  };
//001b62 4cdf 08c0                MOVEM.L  (A7)+,D67A3
//001b66 4e5e                     UNLK     A6
  RETURN;
}

//*********************************************************
//
//*********************************************************
//   TAG001c02
void PrintLinefeed(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001c02 4e56 0000                LINK     A6,#0
//001c06 486c f8da                PEA      -1830(A4)
//001c0a 4267                     CLR.W    -(A7)
//001c0c 4eba fe9a                JSR      $-356 (=0x001aa8)
//001c10 5c8f                     ADDQ.L   #6,A7
  QuePrintLines(0, d.Byte1830);
//001c12 4e5e                     UNLK     A6
//001c14 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG001c42
void TextOutToScreen(i32 xPixel,i32 yPixel,i32 color,i32 P4,pnt P5)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001c42 4e56 0000                LINK     A6,#0
//001c46 2f2e 0010                MOVE.L   16(A6),-(A7)
//001c4a 3f2e 000e                MOVE.W   14(A6),-(A7)
//001c4e 3f2e 000c                MOVE.W   12(A6),-(A7)
//001c52 3f2e 000a                MOVE.W   10(A6),-(A7)
//001c56 3f2e 0008                MOVE.W   8(A6),-(A7)
//001c5a 3f3c 00a0                MOVE.W   #160,-(A7)
//001c5e 2f2c d132                MOVE.L   -11982(A4),-(A7)
//001c62 4eba f8e8                JSR      $-1814 (=0x00154c)
//001c66 defc 0012                ADD.W    #18,A7
  TextOut(d.LogicalScreenBase,
          160,
          xPixel,
          yPixel,
          color,
          P4,
          P5);
//001c6a 4e5e                     UNLK     A6
//001c6c 4e75                     RTS
}

void TAG001c6e(void)
{
  dReg D7;
  //i16 saveD7 = D7W;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//001c6e 4e56 0000                LINK     A6,#0
//001c72 3f07                     MOVE.W   D7,-(A7)
//001c74 3f3c 0003                MOVE.W   #3,-(A7)
//001c78 4267                     CLR.W    -(A7)
//001c7a 4eba fbba                JSR      $-1092 (=0x001836)
//001c7e 588f                     ADDQ.L   #4,A7
  SetPrintPosition(0, 3);
//001c80 3f3c 0001                MOVE.W   #1,-(A7)
//001c84 2f3c 0000 0460           MOVE.L   #1120,-(A7)
//001c8a 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//001c8e 5c8f                     ADDQ.L   #6,A7
  d.newTextLine = allocateMemory(1120,1);
//001c90 2940 cd86                MOVE.L   D0,-12922(A4)
//001c94 3f3c 0001                MOVE.W   #1,-(A7)
//001c98 2f3c 0000 0300           MOVE.L   #768,-(A7)
//001c9e 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//001ca2 5c8f                     ADDQ.L   #6,A7
  d.Pointer12926 = allocateMemory(768,1);
//001ca4 2940 cd82                MOVE.L   D0,-12926(A4)
//001ca8 2f2c cd82                MOVE.L   -12926(A4),-(A7)
//001cac 3f3c 822d                MOVE.W   #33325,-(A7)
//001cb0 4ead 0606                JSR      1542(A5)    (=0x022b86)
//001cb4 5c8f                     ADDQ.L   #6,A7
  ReadAndExpandGraphic(33325,d.Pointer12926,0,0);
//001cb6 4247                     CLR.W    D7
  for (D7W = 0; D7W<4; D7W++)
  {
//001cb8 6012                     BRA      $+20 (=0x001ccc)
//001cba 3007                     MOVE.W   D7,D0
//001cbc e580                     ASL.L    #2,D0
//001cbe 41ec cd6e                LEA      -12946(A4),A0
//001cc2 d0c0                     ADD.W    D0,A0
//001cc4 20bc ffff ffff           MOVE.L   #-1,(A0)
    d.TextTimeout[D7W] = -1;
//001cca 5247                     ADDQ.W   #1,D7

//001ccc 0c47 0004                CMP.W    #4,D7
//001cd0 65e8                     BCS      $-22 (=0x001cba)
  };
//001cd2 3e1f                     MOVE.W   (A7)+,D7
//001cd4 4e5e                     UNLK     A6
//001cd6 4e75                     RTS
  //D7W = saveD7;
}



void TAG0020ca(void)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0020ca 4e56 0000                LINK     A6,#0

//0020ce 4eba 0094                JSR      $+150 (=0x002164)
  KeyclickOff();
//0020d2 3f3c 008b                MOVE.W   #139,-(A7)
//0020d6 4267                     CLR.W    -(A7)
//0020d8 3f3c 001c                MOVE.W   #28,-(A7)
//0020dc 4eba fc0a                JSR      $-1012 (=0x001ce8)   TRAP 14
//0020e0 5c8f                     ADDQ.L   #6,A7
  giaccess(0,0x80 + 11);
//0020e2 3f3c 008c                MOVE.W   #140,-(A7)
//0020e6 4267                     CLR.W    -(A7)
//0020e8 3f3c 001c                MOVE.W   #28,-(A7)
//0020ec 4eba fbfa                JSR      $-1028 (=0x001ce8)   TRAP 14
//0020f0 5c8f                     ADDQ.L   #6,A7
  giaccess(0, 0x80 + 12);
//0020f2 3f3c 0007                MOVE.W   #7,-(A7)
//0020f6 4267                     CLR.W    -(A7)
//0020f8 3f3c 001c                MOVE.W   #28,-(A7)
//0020fc 4eba fbea                JSR      $-1044 (=0x001ce8)   TRAP 14
//002100 5c8f                     ADDQ.L   #6,A7
  D0W = (i16)(giaccess(0, 7) &0xff);
//002102 c07c 00ff                AND.W    #255,D0
//002106 3940 a604                MOVE.W   D0,-23036(A4)
  d.Word23036 = D0W;
//00210a 3f3c 0087                MOVE.W   #135,-(A7)
//00210e 302c a604                MOVE.W   -23036(A4),D0
//002112 807c 003f                OR.W     #63,D0
//002116 3f00                     MOVE.W   D0,-(A7)
//002118 3f3c 001c                MOVE.W   #28,-(A7)
//00211c 4eba fbca                JSR      $-1076 (=0x001ce8)   TRAP 14
//002120 5c8f                     ADDQ.L   #6,A7
  giaccess(d.Word23036 | 0x3f, 0x80 + 7);
//002122 3f3c 0008                MOVE.W   #8,-(A7)
//002126 4eba fcee                JSR      $-784 (=0x001e16)
//00212a 548f                     ADDQ.L   #2,A7
  TAG001e16(8);
//00212c 2f3c 001a 000d           MOVE.L   #1703949,-(A7)
//002132 4eba fbb4                JSR      $-1098 (=0x001ce8)   TRAP 14
//002136 588f                     ADDQ.L   #4,A7
  jdisint(13);
//002138 41fa fd16                LEA      $-744 (=0x001e50),A0
//00213c 2008                     MOVE.L   A0,D0
//00213e 2f00                     MOVE.L   D0,-(A7)
//002140 2f3c 0001 0070           MOVE.L   #65648,-(A7)
//002146 4267                     CLR.W    -(A7)
//002148 3f3c 001f                MOVE.W   #31,-(A7)
//00214c 4eba fb9a                JSR      $-1124 (=0x001ce8)   TRAP 14
//002150 defc 000c                ADD.W    #12,A7
  xbtimer(0, 1, 112, TAG001e50);
//002154 2f3c 001a 000d           MOVE.L   #1703949,-(A7)
//00215a 4eba fb8c                JSR      $-1138 (=0x001ce8)   TRAP 14
//00215e 588f                     ADDQ.L   #4,A7
  jdisint(13);
//0021604e5e                     UNLK     A6
//002162 4e75                     RTS
}

void KeyclickOff(void) //TAG002164
{
  aReg A0;
  pnt LOCAL_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002164 4e56 fffc                LINK     A6,#-4
//002168 42a7                     CLR.L    -(A7)
//00216a 3f3c 0020                MOVE.W   #32,-(A7)
//00216e 4eba fb88                JSR      $-1142 (=0x001cf8)
//002172 5c8f                     ADDQ.L   #6,A7
  LOCAL_4 = dosound(0); // Stop any existing sound
//002174 2d40 fffc                MOVE.L   D0,-4(A6)
//002178 206c a600                MOVE.L   -23040(A4),A0
  A0 = d.Pointer23040;
//00217c 1950 a5fe                MOVE.B   (A0),-23042(A4)
   ////  d.Byte23042 = *A0; // save old keyclick state
//002180 206c a600                MOVE.L   -23040(A4),A0
//002184 0210 00fe                AND.B    #-2,(A0)
//002188 2f2e fffc                MOVE.L   -4(A6),-(A7)
   ////  *A0 &= 2; // keyclick off.
//00218c 3f3c 0020                MOVE.W   #32,-(A7)
//002190 4eba fb66                JSR      $-1176 (=0x001cf8)
//002194 5c8f                     ADDQ.L   #6,A7
  dosound(LOCAL_4); // Restart sound
//002196 4e5e                     UNLK     A6
//002198 4e75                     RTS
}

//   TAG00219a
void QueueSound(i32 soundNum,i32 mapX,i32 mapY,i32 deltaTime)
{
  dReg D0, D1, D7;
  SOUND *psA3;
  i16 LOCAL_12;
  TIMER LOCAL_10;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00219a 4e56 fff4                LINK     A6,#-12
//00219e 48e7 0110                MOVEM.L  D7A3,-(A7)
//0021a2 302e 000e                MOVE.W   14(A6),D0
//0021a6 670e                     BEQ      $+16 (=0x0021b6)
  if (deltaTime)
  {
//0021a8 302c d746                MOVE.W   -10426(A4),D0
//0021ac b06c d244                CMP.W    -11708(A4),D0
//0021b0 6704                     BEQ      $+6 (=0x0021b6)
//0021b2 6000 017a                BRA      $+380 (=0x00232e)
    if (d.LoadedLevel != d.partyLevel) return;
  };
//0021b6 302e 0008                MOVE.W   8(A6),D0
//0021ba e780                     ASL.L    #3,D0
//0021bc 41ec f914                LEA      -1772(A4),A0
//0021c0 d0c0                     ADD.W    D0,A0
//0021c2 41d0                     LEA      (A0),A0
//0021c4 2648                     MOVE.L   A0,A3
  psA3 = &d.sound1772[soundNum];
//0021c6 0c6e 0001 000e           CMP.W    #1,14(A6)
//0021cc 6356                     BLS      $+88 (=0x002224)
  if (deltaTime > 1)
  {
//0021ce 202c d23a                MOVE.L   -11718(A4),D0
    D0L = d.Time;
//0021d2 362e 000e                MOVE.W   14(A6),D3
//0021d6 c6bc 0000 ffff           AND.L    #65535,D3
//0021dc d083                     ADD.L    D3,D0
    D0L += deltaTime & 0xffff;
    D0L--;
//0021de 5380                     SUBQ.L   #1,D0
//0021e0 322c d746                MOVE.W   -10426(A4),D1
//0021e4 48c1                     EXT.L    D1
//0021e6 263c 0000 0018           MOVE.L   #24,D3
//0021ec e7a1                     ASL.L    D3,D1
//0021ee 8081                     OR.L     D1,D0
    D0L |= d.LoadedLevel<<24;
//0021f0 2d40 fff6                MOVE.L   D0,-10(A6)
    LOCAL_10.time = D0L;
//0021f4 1d7c 0014 fffa           MOVE.B   #20,-6(A6)
    LOCAL_10.function = 20;
//0021fa 1d6b 0004 fffb           MOVE.B   4(A3),-5(A6)
//002200 3d6e 0008 fffe           MOVE.W   8(A6),-2(A6)
    LOCAL_10.uByte5 = psA3->byte4;
    LOCAL_10.word8() = sw(soundNum);
//002206 302e 000a                MOVE.W   10(A6),D0
//00220a 1d40 fffc                MOVE.B   D0,-4(A6)
    LOCAL_10.uByte6 = (i8)mapX;
//00220e 302e 000c                MOVE.W   12(A6),D0
//002212 1d40 fffd                MOVE.B   D0,-3(A6)
    LOCAL_10.uByte7 = (i8)mapY;
//002216 486e fff6                PEA      -10(A6)
//00221a 4ead 0366                JSR      870(A5)    (=0x00ffbe)
//00221e 588f                     ADDQ.L   #4,A7
    SetTimer(&LOCAL_10);
//002220 6000 010c                BRA      $+270 (=0x00232e)
    return;
  };
//002224 302e 000a                MOVE.W   10(A6),D0
//002228 906c d24a                SUB.W    -11702(A4),D0
//00222c 3e00                     MOVE.W   D0,D7
  D7W = (i16)(mapX - d.partyX);
//00222e 4a40                     TST.W    D0
//002230 6c06                     BGE      $+8 (=0x002238)
  if (D7W < 0)
  {
//002232 3007                     MOVE.W   D7,D0
//002234 4440                     NEG.W    D0
    D0W = (i16)-D7W;
//002236 6002                     BRA      $+4 (=0x00223a)
  }
  else
  {
//002238 3007                     MOVE.W   D7,D0
    D0W = D7W;
  };
//00223a 322e 000c                MOVE.W   12(A6),D1
//00223e 926c d248                SUB.W    -11704(A4),D1
//002242 3e01                     MOVE.W   D1,D7
  D7W = (i16)(mapY- d.partyY);
//002244 4a41                     TST.W    D1
//002246 6c06                     BGE      $+8 (=0x00224e)
  if (D7W < 0)
  {
//002248 3207                     MOVE.W   D7,D1
//00224a 4441                     NEG.W    D1
//00224c 6002                     BRA      $+4 (=0x002250)
    D1W = (i16)-D7W;
  }
  else
  {
//00224e 3207                     MOVE.W   D7,D1
    D1W = D7W;
  };
//002250 d041                     ADD.W    D1,D0
//002252 3e00                     MOVE.W   D0,D7
  D7W = (i16)(D0W + D1W); //Total orthoganal distance
//002254 3007                     MOVE.W   D7,D0
//002256 4241                     CLR.W    D1
//002258 122b 0006                MOVE.B   6(A3),D1
//00225c b041                     CMP.W    D1,D0
//00225e 6c08                     BGE      $+10 (=0x002268)
  if (D7W < (UI8)(psA3->byte6))
  {
//002260 3d7c 0001 fff4           MOVE.W   #1,-12(A6)
    LOCAL_12 = 1;
//002266 6016                     BRA      $+24 (=0x00227e)
  }
  else
  {
//002268 3007                     MOVE.W   D7,D0
//00226a 4241                     CLR.W    D1
//00226c 122b 0007                MOVE.B   7(A3),D1
//002270 b041                     CMP.W    D1,D0
//002272 6c06                     BGE      $+8 (=0x00227a)
    if (D7W >= (UI8)(psA3->byte7)) return;
    LOCAL_12 = 0;
//002274 426e fff4                CLR.W    -12(A6)
//002278 6004                     BRA      $+6 (=0x00227e)
//00227a 6000 00b2                BRA      $+180 (=0x00232e)
  };
//00227e 526c a5b4                ADDQ.W   #1,-23116(A4)
  d.Word23116++;
//002282 302c a5b4                MOVE.W   -23116(A4),D0
//002286 0c40 0058                CMP.W    #88,D0
//00228a 6534                     BCS      $+54 (=0x0022c0)
//00228c 302c a544                MOVE.W   -23228(A4),D0
//002290 662e                     BNE      $+48 (=0x0022c0)
  if ( (d.Word23116>=88) && (d.Word23228==0) )
  {
//002292 426c a5b4                CLR.W    -23116(A4)
    d.Word23116 = 0;
//002296 566c dd44                ADDQ.W   #3,-8892(A4)
    d.Word8892 += 3;
//00229a 006c 0002 a506           OR.W     #2,-23290(A4)
    d.CacheInvalid |= 2;
//0022a0 202c a508                MOVE.L   -23288(A4),D0
//0022a4 d0bc 0000 0200           ADD.L    #512,D0
//0022aa 2f00                     MOVE.L   D0,-(A7)
//0022ac 4ead 0576                JSR      1398(A5)    (=0x02134a)
//0022b0 588f                     ADDQ.L   #4,A7
    TAG02134a(d.ClusterCache + 512);
//0022b2 302c dd44                MOVE.W   -8892(A4),D0
//0022b6 e540                     ASL.W    #2,D0
//0022b8 3940 f9cc                MOVE.W   D0,-1588(A4)
    d.Word1588 = (i16)(d.Word8892<<2);
//0022bc c16c d218                AND.W    D0,-11752(A4)
    d.Word11752 &= d.Word1588;
  };
//0022c0 302e 000e                MOVE.W   14(A6),D0
//0022c4 6624                     BNE      $+38 (=0x0022ea)
  if (deltaTime == 0)
  {
//0022c6 3f2e fff4                MOVE.W   -12(A6),-(A7)
//0022ca 4240                     CLR.W    D0
//0022cc 102b 0003                MOVE.B   3(A3),D0
//0022d0 3f00                     MOVE.W   D0,-(A7)
//0022d2 3013                     MOVE.W   (A3),D0
//0022d4 807c 8000                OR.W     #32768,D0
//0022d8 3f00                     MOVE.W   D0,-(A7)
//0022da 4ead 061e                JSR      1566(A5)    (=0x022a92)
//0022de 548f                     ADDQ.L   #2,A7
//0022e0 2f00                     MOVE.L   D0,-(A7)
//0022e2 4eba fa32                JSR      $-1484 (=0x001d16)
//0022e6 508f                     ADDQ.L   #0,A7
    StartSound(GetGraphicAddress(psA3->word0|0x8000),// Start Sound
               (UI8)(psA3->byte3),
               LOCAL_12);
//0022e8 6044                     BRA      $+70 (=0x00232e)
    return;
  };
//0022ea 0c6c ffff a5fc           CMP.W    #65535,-23044(A4)
//0022f0 6730                     BEQ      $+50 (=0x002322)
//0022f2 302e fff4                MOVE.W   -12(A6),D0
//0022f6 b06c a5fa                CMP.W    -23046(A4),D0
//0022fa 6226                     BHI      $+40 (=0x002322)
  if ( (d.CurrentSound!=-1) && (LOCAL_12 <= d.Word23046) )
  {
//0022fc 302e fff4                MOVE.W   -12(A6),D0
//002300 b06c a5fa                CMP.W    -23046(A4),D0
//002304 6628                     BNE      $+42 (=0x00232e)
    if (LOCAL_12 == d.Word23046) return;
//002306 102b 0004                MOVE.B   4(A3),D0
//00230a 322c a5fc                MOVE.W   -23044(A4),D1
//00230e e781                     ASL.L    #3,D1
//002310 41ec f918                LEA      -1768(A4),A0
//002314 d0c1                     ADD.W    D1,A0
//002316 4241                     CLR.W    D1
//002318 1210                     MOVE.B   (A0),D1
    D1W = (UI8)(d.sound1772[d.CurrentSound].byte4);
//00231a c07c 00ff                AND.W    #255,D0
    D0W = (i16)(psA3->byte4 & 0xff);
//00231e b041                     CMP.W    D1,D0
//002320 6f0c                     BLE      $+14 (=0x00232e)
    if (D0W <= D1W) return;
  };
//002322 396e 0008 a5fc           MOVE.W   8(A6),-23044(A4)
  d.CurrentSound = sw(soundNum);
//002328 396e fff4 a5fa           MOVE.W   -12(A6),-23046(A4)
  d.Word23046 = LOCAL_12;
//00232e 4cdf 0880                MOVEM.L  (A7)+,D7A3
//002332 4e5e                     UNLK     A6
//002334 4e75                     RTS
}

void TAG002336(void)
{
  SOUND *psA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002336 4e56 0000                LINK     A6,#0
//00233a 2f0b                     MOVE.L   A3,-(A7)
//00233c 0c6c ffff a5fc           CMP.W    #65535,-23044(A4)
//002342 6766                     BEQ      $+104 (=0x0023aa)
  Instrumentation(icntTAG002336);
  if (d.CurrentSound != -1)
  {
//002344 302c a5fc                MOVE.W   -23044(A4),D0
//002348 e780                     ASL.L    #3,D0
//00234a 41ec f914                LEA      -1772(A4),A0
//00234e d0c0                     ADD.W    D0,A0
//002350 41d0                     LEA      (A0),A0
//002352 2648                     MOVE.L   A0,A3
    psA3 = &d.sound1772[d.CurrentSound];
//002354 3f2c a5fa                MOVE.W   -23046(A4),-(A7)
//002358 4240                     CLR.W    D0
//00235a 102b 0003                MOVE.B   3(A3),D0
//00235e 3f00                     MOVE.W   D0,-(A7)
//002360 3013                     MOVE.W   (A3),D0
//002362 807c 8000                OR.W     #32768,D0
//002366 3f00                     MOVE.W   D0,-(A7)
//002368 4ead 061e                JSR      1566(A5)    (=0x022a92)
//00236c 548f                     ADDQ.L   #2,A7
//00236e 2f00                     MOVE.L   D0,-(A7)
//002370 4eba f9a4                JSR      $-1626 (=0x001d16)
//002374 508f                     ADDQ.L   #0,A7
    StartSound(GetGraphicAddress(psA3->word0|0x8000),
               (UI8)(psA3->byte3),
               d.Word23046);
    //playSound(psA3->word0);
//002376 302c dd44                MOVE.W   -8892(A4),D0
//00237a c07c 0001                AND.W    #1,D0
//00237e 6720                     BEQ      $+34 (=0x0023a0)
    if (d.Word8892 & 1)
    {
//002380 302c a544                MOVE.W   -23228(A4),D0
//002384 661a                     BNE      $+28 (=0x0023a0)
      if (d.Word23228 == 0)
      {
//002386 302c d234                MOVE.W   -11724(A4),D0
//00238a c07c 0004                AND.W    #4,D0
//00238e 670c                     BEQ      $+14 (=0x00239c)
        if (d.Word11724 & 4)
        {
//002390 026c fffb d234           AND.W    #65531,-11724(A4)
          d.Word11724 &= 0xfffb;
//002396 026c fffb d236           AND.W    #65531,-11722(A4)
          d.Word11722 &= 0xfffb;
        };
//00239c 526c dd44                ADDQ.W   #1,-8892(A4)
        d.Word8892++;
      };
    };
//0023a0 397c ffff a5fc           MOVE.W   #65535,-23044(A4)
    d.CurrentSound = -1;
//0023a6 426c a5fa                CLR.W    -23046(A4)
    d.Word23046 = 0;
  };
//0023aa 265f                     MOVE.L   (A7)+,A3
//0023ac 4e5e                     UNLK     A6
//0023ae 4e75                     RTS
}

void TAG0023b0(void)
{
  //aReg A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0023b0 4e56 0000                LINK     A6,#0
  //pnt saveA3 = A3;
//0023b4 2f0b                     MOVE.L   A3,-(A7)
//0023b6 3f3c 0001                MOVE.W   #1,-(A7)
//0023ba 2f3c 0000 0120           MOVE.L   #288,-(A7)
//0023c0 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0023c4 5c8f                     ADDQ.L   #6,A7
//0023c6 2940 a56a                MOVE.L   D0,-23190(A4)
  d.Pointer23190 = allocateMemory(288, 1);
//0023ca 3f3c 0001                MOVE.W   #1,-(A7)
//0023ce 2f3c 0000 0120           MOVE.L   #288,-(A7)
//0023d4 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0023d8 5c8f                     ADDQ.L   #6,A7
//0023da 2940 a56e                MOVE.L   D0,-23186(A4)
  d.Pointer23186 = allocateMemory(288,1);
//0023de 3f3c 0001                MOVE.W   #1,-(A7)
//0023e2 2f3c 0000 0100           MOVE.L   #256,-(A7)
//0023e8 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0023ec 5c8f                     ADDQ.L   #6,A7
//0023ee 2940 a572                MOVE.L   D0,-23182(A4)
  d.Pointer23182 = allocateMemory(256, 1);
//0023f2 3f3c 0001                MOVE.W   #1,-(A7)
//0023f6 2f3c 0000 0100           MOVE.L   #256,-(A7)
//0023fc 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//002400 5c8f                     ADDQ.L   #6,A7
//002402 2940 a576                MOVE.L   D0,-23178(A4)
  d.Pointer23178 = allocateMemory(256, 1);
//002406 3f3c 0001                MOVE.W   #1,-(A7)
//00240a 2f3c 0000 01b0           MOVE.L   #432,-(A7)
//002410 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//002414 5c8f                     ADDQ.L   #6,A7
//002416 2940 a57a                MOVE.L   D0,-23174(A4)
  d.CursorBitmap = allocateMemory(432, 1);
//00241a 3f3c 0001                MOVE.W   #1,-(A7)
//00241e 2f3c 0000 01b0           MOVE.L   #432,-(A7)
//002424 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//002428 5c8f                     ADDQ.L   #6,A7
//00242a 2940 a562                MOVE.L   D0,-23198(A4)
  d.SavedCursorPixels = allocateMemory(18*24, 1);
//00242e 397c 00fb a5ae           MOVE.W   #251,-23122(A4)
//002434 397c 0033 a5ac           MOVE.W   #51,-23124(A4)
  d.NewMouseX = 251;
  d.NewMouseY = 51;
//00243a 3f3c 0022                MOVE.W   #34,-(A7)
//00243e 4eba f8a8                JSR      $-1878 (=0x001ce8)
//002442 548f                     ADDQ.L   #2,A7
//002444 2640                     MOVE.L   D0,A3
  // kbdvbase  A3 = TAG001c8(34);
//002446 41fa 0aac                LEA      $+2734 (=0x002ef4),A0

//00244a 2008                     MOVE.L   A0,D0
//00244c 2740 0010                MOVE.L   D0,16(A3)
  // Set Mouse Vector to 002ef4
  //A3 = saveA3;
//002450 265f                     MOVE.L   (A7)+,A3
//002452 4e5e                     UNLK     A6
//002454 4e75                     RTS
}

//   TAG002456
void SetCursorShape(i16 P1)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002456 4e56 0000                LINK     A6,#0
//00245a 4eba 0b76                JSR      $+2936 (=0x002fd2)
  STHideCursor();
//00245e 426c a596                CLR.W    -23146(A4)
  d.Word23146 = 0;
//002462 0c6e 0001 0008           CMP.W    #1,8(A6)
//002468 57c0                     SEQ      D0
//00246a c07c 0001                AND.W    #1,D0
  D0W = (i16)((P1==1) ? 1 : 0);
//00246e 3940 a594                MOVE.W   D0,-23148(A4)
  d.CursorShape = D0W;
//002472 397c 0001 a59a           MOVE.W   #1,-23142(A4)
  d.NewCursorShape = 1;
//002478 4eba 0bac                JSR      $+2990 (=0x003026)
  STShowCursor();
//00247c 4e5e                     UNLK     A6
//00247e 4e75                     RTS
}

//   TAG002480
void CreateObjectCursor(pnt P1)
{
  aReg A2, A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002480 4e56 0000                LINK     A6,#0
//002484 48e7 0030                MOVEM.L  A23,-(A7)
//002488 266e 0008                MOVE.L   8(A6),A3
  A3 = P1;
//00248c 4eba 0b44                JSR      $+2886 (=0x002fd2)
  STHideCursor();
//002490 397c 0001 a596           MOVE.W   #1,-23146(A4)
  d.Word23146 = 1;
//002496 426c a594                CLR.W    -23148(A4)
  d.CursorShape = 0;
//00249a 397c 0001 a59a           MOVE.W   #1,-23142(A4)
  d.NewCursorShape = 1;
//0024a0 397c 0001 a608           MOVE.W   #1,-23032(A4)
  d.UseByteCoordinates = 1;
//0024a6 246c a56a                MOVE.L   -23190(A4),A2
  A2 = d.Pointer23190;
//0024aa 2f3c 000c 0024           MOVE.L   #786468,-(A7)
//0024b0 2f0a                     MOVE.L   A2,-(A7)
//0024b2 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//0024b6 508f                     ADDQ.L   #0,A7
  Flood(A2, 12, 36);
//0024b8 486c fa9a                PEA      -1382(A4)
//0024bc 2f3c 0010 0010           MOVE.L   #1048592,-(A7)
//0024c2 2f3c 0010 0010           MOVE.L   #1048592,-(A7)
//0024c8 2f2c a562                MOVE.L   -23198(A4),-(A7)
//0024cc 2f0b                     MOVE.L   A3,-(A7)
//0024ce 4ead 01da                JSR      474(A5)    (=0x008498)
//0024d2 defc 0014                ADD.W    #20,A7
  ShrinkBLT(A3,
            d.SavedCursorPixels,
            16,
            16,
            16,
            16,
            d.Byte1382);
//0024d6 2f3c 0010 ffff           MOVE.L   #1114111,-(A7)
//0024dc 3f3c 0008                MOVE.W   #8,-(A7)
//0024e0 4267                     CLR.W    -(A7)
//0024e2 4267                     CLR.W    -(A7)
//0024e4 486c a55e                PEA      -23202(A4)
//0024e8 2f0a                     MOVE.L   A2,-(A7)
//0024ea 2f2c a562                MOVE.L   -23198(A4),-(A7)
//0024ee 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//0024f2 defc 0016                ADD.W    #22,A7
  TAG0088b2(d.SavedCursorPixels,
            A2,
            (RectPos *)d.Byte23202,
            0,
            0,
            8,
            16,
            -1);
//0024f6 3f3c 0120                MOVE.W   #288,-(A7)
//0024fa 2f2c a56e                MOVE.L   -23186(A4),-(A7)
//0024fe 2f0a                     MOVE.L   A2,-(A7)
//002500 4eba e4da                JSR      $-6948 (=0x0009dc)
//002504 defc 000a                ADD.W    #10,A7
  MemMove(A2, d.Pointer23186, 288);
//002508 486c faaa                PEA      -1366(A4)
//00250c 2f3c 0010 0010           MOVE.L   #1048592,-(A7)
//002512 2f3c 0010 0010           MOVE.L   #1048592,-(A7)
//002518 2f2c a562                MOVE.L   -23198(A4),-(A7)
//00251c 2f0b                     MOVE.L   A3,-(A7)
//00251e 4ead 01da                JSR      474(A5)    (=0x008498)
//002522 defc 0014                ADD.W    #20,A7
  ShrinkBLT(A3,
            d.SavedCursorPixels,
            16,
            16,
            16,
            16,
            d.Byte1366);
//002526 2f3c 0010 000c           MOVE.L   #1048588,-(A7)
//00252c 3f3c 0008                MOVE.W   #8,-(A7)
//002530 4267                     CLR.W    -(A7)
//002532 4267                     CLR.W    -(A7)
//002534 486c a55a                PEA      -23206(A4)
//002538 2f0a                     MOVE.L   A2,-(A7)
//00253a 2f2c a562                MOVE.L   -23198(A4),-(A7)
//00253e 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002542 defc 0016                ADD.W    #22,A7
  TAG0088b2(d.SavedCursorPixels,
            A2,
            (RectPos *)d.Byte23206,
            0,
            0,
            8,
            16,
            12);
//002546 2f3c 0010 000c           MOVE.L   #1048588,-(A7)
//00254c 3f3c 0008                MOVE.W   #8,-(A7)
//002550 4267                     CLR.W    -(A7)
//002552 4267                     CLR.W    -(A7)
//002554 486c a55a                PEA      -23206(A4)
//002558 2f2c a56e                MOVE.L   -23186(A4),-(A7)
//00255c 2f0b                     MOVE.L   A3,-(A7)
//00255e 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002562 defc 0016                ADD.W    #22,A7
  TAG0088b2(A3,
            d.Pointer23186,
            (RectPos *)d.Byte23206,
            0,
            0,
            8,
            16,
            12);
//002566 4eba 0abe                JSR      $+2752 (=0x003026)
  STShowCursor();
//00256a 4cdf 0c00                MOVEM.L  (A7)+,A23
//00256e 4e5e                     UNLK     A6
//002570 4e75                     RTS
}

void TAG002572(void)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002572 4e56 0000                LINK     A6,#0
//002576 302c bf3e                MOVE.W   -16578(A4),D0
//00257a 671c                     BEQ      $+30 (=0x002598)
  if (d.EmptyHanded)
  {
//00257c 0c6c ffff bf48           CMP.W    #65535,-16568(A4)
//002582 6606                     BNE      $+8 (=0x00258a)
    if (d.HandChar == -1) // owner of cursor
    {
//002584 303c 0000                MOVE.W   #0,D0
      D0L = 0;
//002588 6004                     BRA      $+6 (=0x00258e)
    }
    else
    {
//00258a 303c 0001                MOVE.W   #1,D0
      D0L = 1;
    };
//00258e 3f00                     MOVE.W   D0,-(A7)
//002590 4eba fec4                JSR      $-314 (=0x002456)
//002594 548f                     ADDQ.L   #2,A7
    SetCursorShape(D0W);

//002596 600a                     BRA      $+12 (=0x0025a2)
  }
  else
  {
//002598 2f2c bf44                MOVE.L   -16572(A4),-(A7)
//00259c 4eba fee2                JSR      $-284 (=0x002480)
//0025a0 588f                     ADDQ.L   #4,A7
    CreateObjectCursor(d.Pointer16572);
  };
//0025a2 4e5e                     UNLK     A6
//0025a4 4e75                     RTS
}

void TAG0025a6(i32 P1)
{//(void)
  dReg D0, D4, D5, D6, D7;
  aReg A2;
  RectPos *rectPosA3;
  CHARDESC *pcA0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0025a6 4e56 0000                LINK     A6,#0
//0025aa 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//0025ae 3e2e 0008                MOVE.W   8(A6),D7
  D7W = sw(P1);
//0025b2 302c a598                MOVE.W   -23144(A4),D0
//0025b6 6600 015c                BNE      $+350 (=0x002714)
  if (d.Word23144 == 0)
  {
//0025ba 3007                     MOVE.W   D7,D0
//0025bc d06c d246                ADD.W    -11706(A4),D0
//0025c0 c07c 0003                AND.W    #3,D0
//0025c4 3f00                     MOVE.W   D0,-(A7)
//0025c6 4ead 03fc                JSR      1020(A5)    (=0x01474e)
//0025ca 548f                     ADDQ.L   #2,A7
//0025cc 0c40 ffff                CMP.W    #65535,D0
    D0W = (i16)CharacterAtPosition((d.partyFacing+D7W) & 3);
//0025d0 6604                     BNE      $+6 (=0x0025d6)
//0025d2 6000 023c                BRA      $+574 (=0x002810)
    if (D0W == -1) return;
//0025d6 397c 0001 a59a           MOVE.W   #1,-23142(A4)
    d.NewCursorShape = 1;
//0025dc 4eba 09f4                JSR      $+2550 (=0x002fd2)
    STHideCursor();
//0025e0 426c a608                CLR.W    -23032(A4)
    d.UseByteCoordinates = 0;
//0025e4 246c a572                MOVE.L   -23182(A4),A2
    A2 = d.Pointer23182;
//0025e8 2f3c 000c 0020           MOVE.L   #786464,-(A7)
//0025ee 2f0a                     MOVE.L   A2,-(A7)
//0025f0 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//0025f4 508f                     ADDQ.L   #0,A7
    Flood(A2, 12, 32);
//0025f6 3007                     MOVE.W   D7,D0
//0025f8 e540                     ASL.W    #2,D0
//0025fa e380                     ASL.L    #1,D0
//0025fc 41ec fa52                LEA      -1454(A4),A0
//002600 d0c0                     ADD.W    D0,A0
//002602 41d0                     LEA      (A0),A0
//002604 2648                     MOVE.L   A0,A3
    rectPosA3 = &d.wRectPos1454[D7W];
//002606 4267                     CLR.W    -(A7)
//002608 2f3c 00a0 0010           MOVE.L   #10485776,-(A7)
//00260e 200b                     MOVE.L   A3,D0
//002610 5880                     ADDQ.L   #4,D0
//002612 2040                     MOVE.L   D0,A0
//002614 3f10                     MOVE.W   (A0),-(A7)
//002616 3f13                     MOVE.W   (A3),-(A7)
//002618 486c a552                PEA      -23214(A4)
//00261c 2f0a                     MOVE.L   A2,-(A7)
//00261e 2f2c d132                MOVE.L   -11982(A4),-(A7)
//002622 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002626 defc 0016                ADD.W    #22,A7
    TAG0088b2(d.LogicalScreenBase,
              A2,
              &d.wRectPos23214,
              rectPosA3->w.x1,
              rectPosA3->w.y1,
              160,
              16,
              0);
//00262a 486c fa9a                PEA      -1382(A4)
//00262e 2f3c 0020 0010           MOVE.L   #2097168,-(A7)
//002634 2f3c 0020 0010           MOVE.L   #2097168,-(A7)
//00263a 2f0a                     MOVE.L   A2,-(A7)
//00263c 2f0a                     MOVE.L   A2,-(A7)
//00263e 4ead 01da                JSR      474(A5)    (=0x008498)
//002642 defc 0014                ADD.W    #20,A7
    ShrinkBLT(A2, A2, 32, 16,32,16,d.Byte1382);
//002646 3f3c 0100                MOVE.W   #256,-(A7)
//00264a 2f2c a576                MOVE.L   -23178(A4),-(A7)
//00264e 2f0a                     MOVE.L   A2,-(A7)
//002650 4eba e38a                JSR      $-7284 (=0x0009dc)
//002654 defc 000a                ADD.W    #10,A7
    MemMove(A2, d.Pointer23178, 256);
//002658 4267                     CLR.W    -(A7)
//00265a 2f3c 00a0 0010           MOVE.L   #10485776,-(A7)
//002660 200b                     MOVE.L   A3,D0
//002662 5880                     ADDQ.L   #4,D0
//002664 2040                     MOVE.L   D0,A0
//002666 3f10                     MOVE.W   (A0),-(A7)
//002668 3f13                     MOVE.W   (A3),-(A7)
//00266a 486c a54a                PEA      -23222(A4)
//00266e 2f2c a576                MOVE.L   -23178(A4),-(A7)
//002672 2f2c d132                MOVE.L   -11982(A4),-(A7)
//002676 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//00267a defc 0016                ADD.W    #22,A7
    TAG0088b2(d.LogicalScreenBase,
              d.Pointer23178,
              &d.wRectPos23222,
              rectPosA3->w.x1,
              rectPosA3->w.y1,
              160,
              16,
              0);
//00267e 246c a562                MOVE.L   -23198(A4),A2
    A2 = d.SavedCursorPixels;
//002682 2f3c 000c 0020           MOVE.L   #786464,-(A7)
//002688 2f0a                     MOVE.L   A2,-(A7)
//00268a 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//00268e 508f                     ADDQ.L   #0,A7
    Flood(A2, 12, 32);
//002690 4267                     CLR.W    -(A7)
//002692 2f3c 00a0 0010           MOVE.L   #10485776,-(A7)
//002698 200b                     MOVE.L   A3,D0
//00269a 5880                     ADDQ.L   #4,D0
//00269c 2040                     MOVE.L   D0,A0
//00269e 3f10                     MOVE.W   (A0),-(A7)
//0026a0 3f13                     MOVE.W   (A3),-(A7)
//0026a2 486c a54a                PEA      -23222(A4)
//0026a6 2f0a                     MOVE.L   A2,-(A7)
//0026a8 2f2c d132                MOVE.L   -11982(A4),-(A7)
//0026ac 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//0026b0 defc 0016                ADD.W    #22,A7
    TAG0088b2(d.LogicalScreenBase,
              A2,
              &d.wRectPos23222,
              rectPosA3->w.x1,
              rectPosA3->w.y1,
              160,
              16,
              0);
//0026b4 486c faaa                PEA      -1366(A4)
//0026b8 2f3c 0020 000e           MOVE.L   #2097166,-(A7)
//0026be 2f3c 0020 000e           MOVE.L   #2097166,-(A7)
//0026c4 2f0a                     MOVE.L   A2,-(A7)
//0026c6 2f0a                     MOVE.L   A2,-(A7)
//0026c8 4ead 01da                JSR      474(A5)    (=0x008498)
//0026cc defc 0014                ADD.W    #20,A7
    ShrinkBLT(A2, A2, 32, 14, 32, 14, d.Byte1366);
//0026d0 2f3c 0010 000c           MOVE.L   #1048588,-(A7)
//0026d6 3f3c 0010                MOVE.W   #16,-(A7)
//0026da 4267                     CLR.W    -(A7)
//0026dc 4267                     CLR.W    -(A7)
//0026de 486c a54a                PEA      -23222(A4)
//0026e2 2f2c a572                MOVE.L   -23182(A4),-(A7)
//0026e6 2f0a                     MOVE.L   A2,-(A7)
//0026e8 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//0026ec defc 0016                ADD.W    #22,A7
    TAG0088b2(A2,
              d.Pointer23182,
              &d.wRectPos23222,
              0,
              0,
              16,
              16,
              12);
//0026f0 3f3c 00a0                MOVE.W   #160,-(A7)
//0026f4 4267                     CLR.W    -(A7)
//0026f6 2f0b                     MOVE.L   A3,-(A7)
//0026f8 2f2c d132                MOVE.L   -11982(A4),-(A7)
//0026fc 4ead 01ce                JSR      462(A5)    (=0x008e60)
//002700 defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,
                  rectPosA3,
                  0,
                  160);
//002704 4eba 0920                JSR      $+2338 (=0x003026)
    STShowCursor();
//002708 3007                     MOVE.W   D7,D0
//00270a 5240                     ADDQ.W   #1,D0
//00270c 3940 a598                MOVE.W   D0,-23144(A4)
    d.Word23144 = (i16)(D7W + 1);
//002710 6000 00fe                BRA      $+256 (=0x002810)
  }
  else
  {
//002714 397c 0001 a59a           MOVE.W   #1,-23142(A4)
//00271a 3c2c a598                MOVE.W   -23144(A4),D6
//00271e 5346                     SUBQ.W   #1,D6
//002720 426c a598                CLR.W    -23144(A4)
    d.NewCursorShape = 1;
    D6W = (i16)(d.Word23144-1);
    d.Word23144 = 0;
//002724 3006                     MOVE.W   D6,D0
//002726 d06c d246                ADD.W    -11706(A4),D0
//00272a c07c 0003                AND.W    #3,D0
//00272e 3f00                     MOVE.W   D0,-(A7)
//002730 4ead 03fc                JSR      1020(A5)    (=0x01474e)
//002734 548f                     ADDQ.L   #2,A7
//002736 3800                     MOVE.W   D0,D4
    D4W = (i16)CharacterAtPosition((d.partyFacing + D6W) & 3);
//002738 302c d246                MOVE.W   -11706(A4),D0
//00273c 3204                     MOVE.W   D4,D1
//00273e c3fc 0320                MULS     #800,D1
//002742 41ec bfba                LEA      -16454(A4),A0
//002746 d0c1                     ADD.W    D1,A0
    pcA0 = &d.CH16482[D4W];
//002748 1080                     MOVE.B   D0,(A0)
    pcA0->facing = (i8)d.partyFacing;
//00274a 3006                     MOVE.W   D6,D0
//00274c b047                     CMP.W    D7,D0
//00274e 661c                     BNE      $+30 (=0x00276c)
    if (D6W == D7W)
    {
//002750 3004                     MOVE.W   D4,D0
//002752 c1fc 0320                MULS     #800,D0
//002756 41ec bfce                LEA      -16434(A4),A0
//00275a d0c0                     ADD.W    D0,A0
      pcA0 = &d.CH16482[D4W];
//00275c 0050 0400                OR.W     #1024,(A0)
      pcA0->word48 |= 0x400;
//002760 3f04                     MOVE.W   D4,-(A7)
//002762 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//002766 548f                     ADDQ.L   #2,A7
      DrawCharacterState(D4W);
//002768 6000 00a6                BRA      $+168 (=0x002810)
    }
    else
    {
//00276c 3007                     MOVE.W   D7,D0
//00276e d06c d246                ADD.W    -11706(A4),D0
//002772 c07c 0003                AND.W    #3,D0
//002776 3f00                     MOVE.W   D0,-(A7)
//002778 4ead 03fc                JSR      1020(A5)    (=0x01474e)
//00277c 548f                     ADDQ.L   #2,A7
//00277e 3a00                     MOVE.W   D0,D5
      D5W = (i16)CharacterAtPosition((D7W + d.partyFacing) & 3);
//002780 4eba 0850                JSR      $+2130 (=0x002fd2)
      STHideCursor();
//002784 4a45                     TST.W    D5
//002786 6d32                     BLT      $+52 (=0x0027ba)
      if (D5W >= 0)
      {
//002788 3006                     MOVE.W   D6,D0
//00278a d06c d246                ADD.W    -11706(A4),D0
//00278e c07c 0003                AND.W    #3,D0
//002792 3205                     MOVE.W   D5,D1
//002794 c3fc 0320                MULS     #800,D1
//002798 41ec bfbb                LEA      -16453(A4),A0
//00279c d0c1                     ADD.W    D1,A0
        pcA0 = &d.CH16482[D5W];
//00279e 1080                     MOVE.B   D0,(A0)
        pcA0->position = ub((D6W + d.partyFacing) & 3);
//0027a0 3005                     MOVE.W   D5,D0
//0027a2 c1fc 0320                MULS     #800,D0
//0027a6 41ec bfce                LEA      -16434(A4),A0
//0027aa d0c0                     ADD.W    D0,A0
//0027ac 0050 0400                OR.W     #1024,(A0)
        d.CH16482[D5W].word48 |= 0x400;
//0027b0 3f05                     MOVE.W   D5,-(A7)
//0027b2 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//0027b6 548f                     ADDQ.L   #2,A7
        DrawCharacterState(D5W);
//0027b8 6022                     BRA      $+36 (=0x0027dc)
      }
      else
      {
//0027ba 3f3c 00a0                MOVE.W   #160,-(A7)
//0027be 4267                     CLR.W    -(A7)
//0027c0 3006                     MOVE.W   D6,D0
//0027c2 e540                     ASL.W    #2,D0
//0027c4 e380                     ASL.L    #1,D0
//0027c6 41ec fa52                LEA      -1454(A4),A0
//0027ca d0c0                     ADD.W    D0,A0
//0027cc 41d0                     LEA      (A0),A0
//0027ce 2f08                     MOVE.L   A0,-(A7)
//0027d0 2f2c d132                MOVE.L   -11982(A4),-(A7)
//0027d4 4ead 01ce                JSR      462(A5)    (=0x008e60)
//0027d8 defc 000c                ADD.W    #12,A7
        FillRectangle(d.LogicalScreenBase,
                      &d.wRectPos1454[D6W],
                      0,
                      160);
      };
//0027dc 3007                     MOVE.W   D7,D0
//0027de d06c d246                ADD.W    -11706(A4),D0
//0027e2 c07c 0003                AND.W    #3,D0
//0027e6 3204                     MOVE.W   D4,D1
//0027e8 c3fc 0320                MULS     #800,D1
//0027ec 41ec bfbb                LEA      -16453(A4),A0
//0027f0 d0c1                     ADD.W    D1,A0
//0027f2 1080                     MOVE.B   D0,(A0)
      d.CH16482[D4W].position = (UI8)((D7W+d.partyFacing)&3);
//0027f4 3004                     MOVE.W   D4,D0
//0027f6 c1fc 0320                MULS     #800,D0
//0027fa 41ec bfce                LEA      -16434(A4),A0
//0027fe d0c0                     ADD.W    D0,A0
//002800 0050 0400                OR.W     #1024,(A0)
      d.CH16482[D4W].word48 |= 0x400;
//002804 3f04                     MOVE.W   D4,-(A7)
//002806 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//00280a 548f                     ADDQ.L   #2,A7
      DrawCharacterState(D4W);
//00280c 4eba 0818                JSR      $+2074 (=0x003026)
      STShowCursor();
    };
  };
//002810 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//002814 4e5e                     UNLK     A6
//002816 4e75                     RTS
}

void TAG002818(void) // called by VBL handler
{// Mouse cursor has moved out of the champion alignment area of the screen.
 // ( The upper right corner )
  dReg D6, D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002818 4e56 0000                LINK     A6,#0
//00281c 48e7 0300                MOVEM.L  D67,-(A7)
//002820 3c2c a598                MOVE.W   -23144(A4),D6
//002824 5346                     SUBQ.W   #1,D6
//002826 426c a598                CLR.W    -23144(A4)
  D6W = (i16)(d.Word23144-1);
  d.Word23144 = 0;
  {
    MouseQueueEnt ent;
    ent.x = 0xffff;
    ent.y = 0xffff;
    ent.num = 0xffff;
    RecordFile.Record(&ent);
  };
//00282a 397c 0001 a59a           MOVE.W   #1,-23142(A4)
  d.NewCursorShape = 1;
//002830 526c a5b2                ADDQ.W   #1,-23118(A4)
  d.Word23118++;
//002834 0c6c 0001 a5b2           CMP.W    #1,-23118(A4)
//00283a 661a                     BNE      $+28 (=0x002856)
  if (d.Word23118 == 1)
  {
//00283c 426c a5a2                CLR.W    -23134(A4)
//002840 426c a5a6                CLR.W    -23130(A4)
//002844 426c a5a4                CLR.W    -23132(A4)
    d.Word23134 = 0;
    d.Word23130 = 0;
    d.Word23132 = 0;
//002848 4a6c a5a0                TST.W    -23136(A4)
//00284c 6708                     BEQ      $+10 (=0x002856)
    if (d.Word23136)
    {
//00284e 426c a5a0                CLR.W    -23136(A4)
      d.Word23136 = 0;
//002852 4eba 0046                JSR      $+72 (=0x00289a)
      RemoveCursor();
    };
  };
//002856 3e2c a608                MOVE.W   -23032(A4),D7
  D7W = d.UseByteCoordinates;
//00285a 2f3c 0010 000c           MOVE.L   #1048588,-(A7)
//002860 3006                     MOVE.W   D6,D0
//002862 e540                     ASL.W    #2,D0
//002864 e380                     ASL.L    #1,D0
//002866 41ec fa52                LEA      -1454(A4),A0
//00286a d0c0                     ADD.W    D0,A0
//00286c 41d0                     LEA      (A0),A0
//00286e 2f08                     MOVE.L   A0,-(A7)
//002870 2f2c a576                MOVE.L   -23178(A4),-(A7)
//002874 4eba e708                JSR      $-6390 (=0x000f7e)
//002878 defc 000c                ADD.W    #12,A7
  BLT2Screen(d.Pointer23178,
             &d.wRectPos1454[D6W],
             16,
             12);
//00287c 3947 a608                MOVE.W   D7,-23032(A4)
  d.UseByteCoordinates = D7W;
//002880 536c a5b2                SUBQ.W   #1,-23118(A4)
  d.Word23118--;
//002884 660c                     BNE      $+14 (=0x002892)
  if (d.Word23118 == 0)
  {
//002886 397c 0001 a5a6           MOVE.W   #1,-23130(A4)
    d.Word23130 = 1;
    d.Word23134 = 1;
//00288c 397c 0001 a5a2           MOVE.W   #1,-23134(A4)
  };
//002892 4cdf 00c0                MOVEM.L  (A7)+,D67
//002896 4e5e                     UNLK     A6
//002898 4e75                     RTS
}

//   TAG00289a
void RemoveCursor(void) // called by VBL handler
{ // restore area under cursor
  Instrumentation(icntRemoveCursor);
  for (i32 i=0; i<18; i++)
  {
    memmove(d.ScreenCursorAddress+160*i,
            d.SavedCursorPixels+24*i,
            24);
  };
}

void CreateCursorBitmap(void) // TAG00295e
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00295e 4e56 fff4                LINK     A6,#-12
  dReg D0, D1, D4, D5, D6, D7;
  aReg A2, A3;
  //i32 saveD4=D4, saveD5=D5, saveD6=D6, saveD7=D7;
  //pnt saveA2=A2, saveA3=A3;
  pnt LOCAL_12;
  i16 LOCAL_8;
  i16 LOCAL_6;
  RectPos rect;
//  i8  LOCAL_4;
//  i8  LOCAL_3;
//  i8  LOCAL_2;
//  i8  LOCAL_1;
//002962 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//002966 296c a57a a57e           MOVE.L   -23174(A4),-23170(A4)
  Instrumentation(icntCreateCursorBitmap);
  d.Pointer23170 = d.CursorBitmap;
//00296c 3d6c a608 fffa           MOVE.W   -23032(A4),-6(A6)
  LOCAL_6 = d.UseByteCoordinates;
//002972 397c 0001 a608           MOVE.W   #1,-23032(A4)
  d.UseByteCoordinates = 1;
//002978 2e2c a5a8                MOVE.L   -23128(A4),D7
  D7H1 = d.CurMouseY;//Word23128;
  D7H2 = d.CurMouseX;//Word23126;
//00297c 2c07                     MOVE.L   D7,D6
//00297e 4846                     SWAP     D6
  D6H1 = D7H2;
  D6H2 = D7H1;
//002980 3947 a588                MOVE.W   D7,-23160(A4)
//002984 3946 a586                MOVE.W   D6,-23162(A4)
  d.NewCursorX = D7W;
  d.NewCursorY = D6W;
//002988 4a6c a598                TST.W    -23144(A4)
//00298c 6600 0146                BNE      $+328 (=0x002ad4)
  if (d.Word23144 != 0) goto tag002ad4;
//002990 47ec fb3a                LEA      -1222(A4),A3
  A3 = d.Byte1222; // Arror cursor
//002994 3d7c 0006 fff8           MOVE.W   #6,-8(A6)
  LOCAL_8 = 6;
//00299a 397c 0000 a592           MOVE.W   #0,-23150(A4)
  d.Word23150 = 0;
//0029a0 0c46 00a9                CMP.W    #169,D6
//0029a4 6c52                     BGE      $+84 (=0x0029f8)
  if (D6W >= 169) goto tag0029f8;
//0029a6 0c47 0112                CMP.W    #274,D7
//0029aa 6c4c                     BGE      $+78 (=0x0029f8)
  if (D7W >= 274) goto tag0029f8;
//0029ac 0c46 001c                CMP.W    #28,D6
//0029b0 6f08                     BLE      $+10 (=0x0029ba)
  if (D6W <= 28) goto tag0029ba;
//0029b2 0c47 00e0                CMP.W    #224,D7
//0029b6 6c40                     BGE      $+66 (=0x0029f8)
  if (D7W >= 224) goto tag0029f8;
//0029b8 6026                     BRA      $+40 (=0x0029e0)
  goto tag0029e0;
tag0029ba:
//0029ba 7800                     MOVEQ.L  #0,D4
//0029bc 3807                     MOVE.W   D7,D4
  D4H2 = (i16)(D7UW / 69);
  D4H1 = (i16)(D7UW % 69);
//0029be 88fc 0045                DIVU     #69,D4
  D5W = D4W;  // Which character
//0029c2 3a04                     MOVE.W   D4,D5
//0029c4 ba6c d24c                CMP.W    -11700(A4),D5
//0029c8 6c16                     BGE      $+24 (=0x0029e0)
  if (D5W >= d.NumCharacter) goto tag0029e0;
//0029ca 4844                     SWAP     D4
  SWAP(D4); // pixel within character
//0029cc 0c44 002a                CMP.W    #42,D4
//0029d0 6e0e                     BGT      $+16 (=0x0029e0)
  if (D4W > 42) goto tag0029e0;
//0029d2 5245                     ADDQ.W   #1,D5
  D5W++; // index to ordinal character number
//0029d4 ba6c bf1a                CMP.W    -16614(A4),D5
//0029d8 671e                     BEQ      $+32 (=0x0029f8)
  if (D5W == d.SelectedCharacterOrdinal) goto tag0029f8;
//0029da 0c46 0006                CMP.W    #6,D6
//0029de 6f18                     BLE      $+26 (=0x0029f8)
  if (D6W <= 6) goto tag0029f8;
//0029e0 4a6c a596                TST.W    -23146(A4)
//0029e4 6600 00a4                BNE      $+166 (=0x002a8a)
tag0029e0:
  if (d.Word23146 != 0) goto tag002a8a;
//0029e8 4a6c a594                TST.W    -23148(A4)
//0029ec 670a                     BEQ      $+12 (=0x0029f8)
  if (d.CursorShape == 0) goto tag0029f8;
//0029ee 47ec faba                LEA      -1350(A4),A3
  A3 = d.Byte1350;
//0029f2 3d7c 0000 fff8           MOVE.W   #0,-8(A6)
  LOCAL_8 = 0;
//0029f8 397c 000f a584           MOVE.W   #15,-23164(A4)
//0029fe 426c a58c                CLR.W    -23156(A4)
//002a02 426c a58a                CLR.W    -23158(A4)
tag0029f8:
  d.CursorHeight = 15;
  d.Word23156 = 0;
  d.Word23158 = 0;
//002a06 0247 000f                AND.W    #15,D7
  D7W &= 15; // x offset in word
//002a0a 6714                     BEQ      $+22 (=0x002a20)
  if (D7W == 0) goto tag002a20;
//002a0c 397c 0002 a582           MOVE.W   #2,-23166(A4)
  d.CursorWidth = 2; // 32 pixels
//002a12 be6e fff8                CMP.W    -8(A6),D7
//002a16 6e16                     BGT      $+24 (=0x002a2e)
  if (D7W > LOCAL_8) goto tag002a2e;
//002a18 397c 0001 a582           MOVE.W   #1,-23166(A4)
  d.CursorWidth = 1; // 16 pixels
//002a1e 600e                     BRA      $+16 (=0x002a2e)
  goto tag002a2e;
tag002a20:
//002a20 397c 0001 a582           MOVE.W   #1,-23166(A4)
  d.CursorWidth = 1; // 16 pixels
//002a26 294b a57e                MOVE.L   A3,-23170(A4)
  d.Pointer23170 = A3;
//002a2a 6000 0260                BRA      $+610 (=0x002c8c)
  goto tag002c8c;
tag002a2e:
//002a2e 302c a582                MOVE.W   -23166(A4),D0
//002a32 e940                     ASL.W    #4,D0
  D0W = (i16)(d.CursorWidth << 4); // convert to pixels
//002a34 3f00                     MOVE.W   D0,-(A7)
//002a36 3f3c 000c                MOVE.W   #12,-(A7)
//002a3a 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002a3e 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//002a42 508f                     ADDQ.L   #0,A7
  Flood(d.CursorBitmap,12,D0W); // Flood with color 12
//002a44 2007                     MOVE.L   D7,D0
  D0 = D7; // 1st pixel offset within word
//002a46 1d40 fffc                MOVE.B   D0,-4(A6)
  rect.b.x1 = D0UB; // first pixel x
//002a4a c07c 00ff                AND.W    #255,D0
//002a4e d07c 000f                ADD.W    #15,D0
  D0W = (i16)((D0W & 0xff) + 15); // last pixel x
//002a52 1d40 fffd                MOVE.B   D0,-3(A6)
  rect.b.x2 = D0UB; // last pixel x
//002a56 422e fffe                CLR.B    -2(A6)
  rect.b.y1 = 0;
//002a5a 1d7c 000f ffff           MOVE.B   #15,-1(A6)
  rect.b.y2 = 15;
//002a60 3f3c ffff                MOVE.W   #65535,-(A7)
//002a64 302c a582                MOVE.W   -23166(A4),D0
//002a68 e740                     ASL.W    #3,D0
//002a6a 3f00                     MOVE.W   D0,-(A7)
//002a6c 3f3c 0008                MOVE.W   #8,-(A7)
//002a70 4267                     CLR.W    -(A7)
//002a72 4267                     CLR.W    -(A7)
//002a74 486e fffc                PEA      -4(A6)
//002a78 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002a7c 2f0b                     MOVE.L   A3,-(A7)
//002a7e 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002a82 defc 0016                ADD.W    #22,A7
  TAG0088b2(A3,              // src
            d.CursorBitmap,  // dst
            &rect,           // rectPos
            0,               // srcXOffset
            0,               // srcYOffset
            8,               // src bytes/line
            (d.CursorWidth<<3), // dst bytes/line = pixels/2
            -1);
//002a86 6000 0204                BRA      $+518 (=0x002c8c)
  goto tag002c8c;
tag002a8a:
//002a8a 397c 0001 a592           MOVE.W   #1,-23150(A4)
//002a90 397c 0011 a584           MOVE.W   #17,-23164(A4)
//002a96 397c 0002 a582           MOVE.W   #2,-23166(A4)
//002a9c 397c 0008 a58c           MOVE.W   #8,-23156(A4)
//002aa2 397c 0008 a58a           MOVE.W   #8,-23158(A4)
  d.Word23150 = 1;
  d.CursorHeight = 17;
  d.CursorWidth = 2; // 32 pixels
  d.Word23156 = 8;
  d.Word23158 = 8;
//002aa8 7a12                     MOVEQ.L  #18,D5
//002aaa 7811                     MOVEQ.L  #17,D4
  D5L = 18;
  D4L = 17;
//002aac 266c a56a                MOVE.L   -23190(A4),A3
//002ab0 246c a56e                MOVE.L   -23186(A4),A2
  A3 = d.Pointer23190;
  A2 = d.Pointer23186;
//002ab4 2d4a fff4                MOVE.L   A2,-12(A6)
  LOCAL_12 = A2;
//002ab8 5147                     SUBQ.W   #0,D7
//002aba 0247 000f                AND.W    #15,D7
  D7W = (i16)((D7W - 8) & 15);
//002abe 0c47 000f                CMP.W    #15,D7
//002ac2 6606                     BNE      $+8 (=0x002aca)
  if (D7W != 15) goto tag002aca;
//002ac4 397c 0003 a582           MOVE.W   #3,-23166(A4)
  d.CursorWidth = 3; // 48 pixels
//002aca 0446 0013                SUB.W    #19,D6
tag002aca:
  D6W -= 19;
//002ace 6f00 0154                BLE      $+342 (=0x002c24)
  if (D6W <= 0) goto tag002c24;
//002ad2 6048                     BRA      $+74 (=0x002b1c)
  goto tag002b1c;
tag002ad4:
//002ad4 397c 000f a584           MOVE.W   #15,-23164(A4)
//002ada 397c 0002 a582           MOVE.W   #2,-23166(A4)
//002ae0 397c 0009 a58c           MOVE.W   #9,-23156(A4)
//002ae6 397c 0007 a58a           MOVE.W   #7,-23158(A4)
//002aec 3a3c 0010                MOVE.W   #16,D5
//002af0 383c 0014                MOVE.W   #20,D4
  d.CursorHeight = 15;
  d.CursorWidth = 2; // 16 pixels
  d.Word23156 = 9;
  d.Word23158 = 7;
  D5W = 16;
  D4W = 20;
//002af4 266c a572                MOVE.L   -23182(A4),A3
//002af8 246c a576                MOVE.L   -23178(A4),A2
  A3 = d.Pointer23182;
  A2 = d.Pointer23178;
//002afc 2d4a fff4                MOVE.L   A2,-12(A6)
  LOCAL_12 = A2;
//002b00 0447 0009                SUB.W    #9,D7
//002b04 0247 000f                AND.W    #15,D7
//002b08 0c47 000c                CMP.W    #12,D7
//002b0c 6d06                     BLT      $+8 (=0x002b14)
  D7W = (I16)((D7W - 9) & 15);
  if (D7W < 12) goto tag002b14;
//002b0e 397c 0003 a582           MOVE.W   #3,-23166(A4)
  d.CursorWidth = 3; // 48 pixels
//002b14 0446 0014                SUB.W    #20,D6
//002b18 6f00 010a                BLE      $+268 (=0x002c24)
tag002b14:
  D6W -= 20;
  if (D6W <= 0) goto tag002c24;
//002b1c 4a6c d226                TST.W    -11738(A4)
//002b20 6700 0102                BEQ      $+260 (=0x002c24)
tag002b1c:
  if (d.DynamicPaletteSwitching == 0) goto tag002c24;
//002b24 bc45                     CMP.W    D5,D6
//002b26 6d08                     BLT      $+10 (=0x002b30)
  if (D6W < D5W) goto tag002b30;
//002b28 2d4b fff4                MOVE.L   A3,-12(A6)
  LOCAL_12 = A3;
//002b2c 6000 00f6                BRA      $+248 (=0x002c24)
  goto tag002c24;
//002b30 2007                     MOVE.L   D7,D0
//002b32 4a40                     TST.W    D0
tag002b30:
  D0 = D7;
  if (D0W != 0) goto tag002b86;
//002b34 6650                     BNE      $+82 (=0x002b86)
//002b36 3e05                     MOVE.W   D5,D7
//002b38 2206                     MOVE.L   D6,D1
//002b3a 9e41                     SUB.W    D1,D7
  D7W = D5W;
  D1 = D6;
  D7W = sw((D7W - D1W) << 4);
//002b3c e947                     ASL.W    #4,D7
//002b3e cebc 0000 ffff           AND.L    #65535,D7
  D7L &= 0xffff;
//002b44 2007                     MOVE.L   D7,D0
  D0 = D7;
//002b46 3f00                     MOVE.W   D0,-(A7)
//002b48 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002b4c 2f0a                     MOVE.L   A2,-(A7)
//002b4e 4eba de8c                JSR      $-8562 (=0x0009dc)
//002b52 defc 000a                ADD.W    #10,A7
  MemMove(A2, d.CursorBitmap, D0W);
//002b56 2006                     MOVE.L   D6,D0
//002b58 e940                     ASL.W    #4,D0
//002b5a 3f00                     MOVE.W   D0,-(A7)
  {
    i16 num = sw(D6W<<4);
//002b5c 202c a57a                MOVE.L   -23174(A4),D0
    pnt dst = d.CursorBitmap + D7UW;
//002b60 2207                     MOVE.L   D7,D1
//002b62 c2bc 0000 ffff           AND.L    #65535,D1
//002b68 d081                     ADD.L    D1,D0
//002b6a 2f00                     MOVE.L   D0,-(A7)
//002b6c 200b                     MOVE.L   A3,D0
//002b6e 2207                     MOVE.L   D7,D1
//002b70 c2bc 0000 ffff           AND.L    #65535,D1
//002b76 d081                     ADD.L    D1,D0
//002b78 2f00                     MOVE.L   D0,-(A7)
    pnt src = A3 + D7UW;
//002b7a 4eba de60                JSR      $-8606 (=0x0009dc)
//002b7e defc 000a                ADD.W    #10,A7
    MemMove(src,dst,num);
  };
//002b82 6000 009e                BRA      $+160 (=0x002c22)
  goto tag002c22;
//002b86 302c a582                MOVE.W   -23166(A4),D0
//002b8a c0c5                     MULU     D5,D0
tag002b86:
  D0L = d.CursorWidth * D5UW;
//002b8c 3f00                     MOVE.W   D0,-(A7)
//002b8e 3f3c 000c                MOVE.W   #12,-(A7)
//002b92 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002b96 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//002b9a 508f                     ADDQ.L   #0,A7
  Flood(d.CursorBitmap,12,D0W); // Flood with color 12
//002b9c 3005                     MOVE.W   D5,D0
  D0W = D5W;
//002b9e 2206                     MOVE.L   D6,D1
  D1 = D6;
//002ba0 9041                     SUB.W    D1,D0
  D0W = sw(D0W - D1W);
//002ba2 c0bc 0000 ffff           AND.L    #65535,D0
  D0L &= 0xffff;
//002ba8 2c00                     MOVE.L   D0,D6
  D6 = D0;
//002baa 2007                     MOVE.L   D7,D0
  D0 = D7;
//002bac 1d40 fffc                MOVE.B   D0,-4(A6)
  rect.b.x1 = D0UB;
//002bb0 c07c 00ff                AND.W    #255,D0
  D0W &= 0xff;
//002bb4 d044                     ADD.W    D4,D0
  D0W = sw(D0W + D4W);
//002bb6 1d40 fffd                MOVE.B   D0,-3(A6)
  rect.b.x2 = D0B;
//002bba 422e fffe                CLR.B    -2(A6)
  rect.b.y1 = 0;
//002bbe 2006                     MOVE.L   D6,D0
  D0 = D6;
//002bc0 5340                     SUBQ.W   #1,D0
  D0W--;
//002bc2 1d40 ffff                MOVE.B   D0,-1(A6)
  rect.b.y2 = D0B;
//002bc6 3f3c ffff                MOVE.W   #65535,-(A7)
//002bca 302c a582                MOVE.W   -23166(A4),D0
//002bce e740                     ASL.W    #3,D0
//002bd0 3f00                     MOVE.W   D0,-(A7)
//002bd2 3f3c 0010                MOVE.W   #16,-(A7)
//002bd6 4267                     CLR.W    -(A7)
//002bd8 4267                     CLR.W    -(A7)
//002bda 486e fffc                PEA      -4(A6)
//002bde 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002be2 2f0a                     MOVE.L   A2,-(A7)
//002be4 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002be8 defc 0016                ADD.W    #22,A7
  TAG0088b2(A2,d.CursorBitmap,&rect,0,0,16,d.CursorWidth<<3,-1);
//002bec 2006                     MOVE.L   D6,D0
  D0 = D6;
//002bee 1d40 fffe                MOVE.B   D0,-2(A6)
  rect.b.y1 = D0B;
//002bf2 3005                     MOVE.W   D5,D0
  D0W = D5W;
//002bf4 5340                     SUBQ.W   #1,D0
  D0W--;
//002bf6 1d40 ffff                MOVE.B   D0,-1(A6)
  rect.b.y2 = D0B;
//002bfa 3f3c ffff                MOVE.W   #65535,-(A7)
//002bfe 302c a582                MOVE.W   -23166(A4),D0
//002c02 e740                     ASL.W    #3,D0
//002c04 3f00                     MOVE.W   D0,-(A7)
//002c06 3f3c 0010                MOVE.W   #16,-(A7)
//002c0a 2006                     MOVE.L   D6,D0
//002c0c 3f00                     MOVE.W   D0,-(A7)
//002c0e 4267                     CLR.W    -(A7)
//002c10 486e fffc                PEA      -4(A6)
//002c14 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002c18 2f0b                     MOVE.L   A3,-(A7)
//002c1a 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002c1e defc 0016                ADD.W    #22,A7
  TAG0088b2(A3,d.CursorBitmap,&rect,0,D6W,16,d.CursorWidth<<3,-1);
//002c22 6068                     BRA      $+106 (=0x002c8c)
tag002c22:
  goto tag002c8c;
tag002c24:
//002c24 2007                     MOVE.L   D7,D0
//002c26 4a40                     TST.W    D0
//002c28 6608                     BNE      $+10 (=0x002c32)
  D0 = D7;
  if (D0W != 0) goto tag002c32;
//002c2a 296e fff4 a57e           MOVE.L   -12(A6),-23170(A4)
  d.Pointer23170 = LOCAL_12;
//002c30 605a                     BRA      $+92 (=0x002c8c)
  goto tag002c8c;
tag002c32:
//002c32 2007                     MOVE.L   D7,D0
//002c34 1d40 fffc                MOVE.B   D0,-4(A6)
  D0 = D7;
  rect.b.x1 = D0B;
//002c38 c07c 00ff                AND.W    #255,D0
//002c3c d044                     ADD.W    D4,D0
  D0W &= 0xff;
  D0W = sw(D0W + D4W);
//002c3e 1d40 fffd                MOVE.B   D0,-3(A6)
//002c42 422e fffe                CLR.B    -2(A6)
  rect.b.x2 = D0B;
  rect.b.y1 = 0;
//002c46 3005                     MOVE.W   D5,D0
//002c48 5340                     SUBQ.W   #1,D0
//002c4a 1d40 ffff                MOVE.B   D0,-1(A6)
  D0W = sw(D5W - 1);
  rect.b.y2 = D0B;
//002c4e 302c a582                MOVE.W   -23166(A4),D0
//002c52 c0c5                     MULU     D5,D0
//002c54 3f00                     MOVE.W   D0,-(A7)
  D0W = sw(d.CursorWidth * (ui16)D5W);
//002c56 3f3c 000c                MOVE.W   #12,-(A7)
//002c5a 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002c5e 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//002c62 508f                     ADDQ.L   #0,A7
  Flood(d.CursorBitmap,12,D0W); // Flood with color 12 = transparent
//002c64 3f3c ffff                MOVE.W   #65535,-(A7)
//002c68 302c a582                MOVE.W   -23166(A4),D0
//002c6c e740                     ASL.W    #3,D0
//002c6e 3f00                     MOVE.W   D0,-(A7)
//002c70 3f3c 0010                MOVE.W   #16,-(A7)
//002c74 4267                     CLR.W    -(A7)
//002c76 4267                     CLR.W    -(A7)
//002c78 486e fffc                PEA      -4(A6)
//002c7c 2f2c a57a                MOVE.L   -23174(A4),-(A7)
//002c80 2f2e fff4                MOVE.L   -12(A6),-(A7)
//002c84 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//002c88 defc 0016                ADD.W    #22,A7
  TAG0088b2(LOCAL_12,d.CursorBitmap,&rect,0,0,16,d.CursorWidth<<3,-1);
//002c8c 396e fffa a608           MOVE.W   -6(A6),-23032(A4)
tag002c8c:
  d.UseByteCoordinates = LOCAL_6;
//002c92 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//002c96 4e5e                     UNLK     A6
  //D4=saveD4;D5=saveD5;D6=saveD6;D7=saveD7;
  //A2=saveA2;A3=saveA3;
//002c98 4e75                     RTS
}


//*********************************************************
//
//*********************************************************
//   TAG002c9a
void DrawCursor(void) // called by VBL handler
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//002c9a 4e56 0000                LINK     A6,#0
//002c9e 48e7 ffe0                MOVEM.L  D01234567A012,-(A7)
  dReg D0,D1,D2,D3,D4,D5,D6,D7;
  pnt A0,A1,A2;
  Instrumentation(icntPlaceCursor);
  d.Word23136 = 1;

  D6W = d.NewCursorX;
  D7W = d.NewCursorY;
  if (!PlayFile.IsOpen())
  {
    d.CurCursorX = D6W;
    d.CurCursorY = D7W;
  };
  D2L = 0; // Left of screen Flag
  D6W = sw(D6W - d.Word23156);
  if (D6W < 0)
  {
    D6L = 0;
    D1L = 0;
    D2L = 1; // Left of screen flag
    goto tag002cd6;
  };
  if (D6W >= 272) // Room for widest cursor = 48 pixels
  {
    D1L = 136;  // The last 24 bytes on the line
  }
  else
  {
    D1W = D6W;
tag002cd6:
    D1W = (I16)((D1W >> 1) & 0x7ff8);
  };
    SWAP(D2);
    D7W = sw(D7W - d.Word23158);
  if (D7W < 0)
  {
    D2W = sw(-D7W);
    D7W = 0;
  }
  else
  {
    if (D7W >= 182)
    {
      D1W +=29120;
    }
    else
    {
      D0W = sw(D7W << 5);
      D1W = sw(D1W + D0W);
      D0W = sw(D0W + D0W);
      D0W = sw(D0W + D0W);
      D1W = sw(D1W + D0W); // D1 += 160*D7W;
    };
  };
  A2 = (pnt)D2L; // save flag
  A0 = d.LogicalScreenBase + D1W;
  d.ScreenCursorAddress = A0;
  A1 = d.SavedCursorPixels;
  for (i32 ii=0; ii<18; ii++)
  { // save pixels under cursor
    memmove(A1+24*ii,A0+160*ii,24);
  };
  D0W = d.CursorHeight;
  D1W = d.CursorWidth;
  A1 =  d.Pointer23170; // CurCursorBitmap
  D2W = (i16)A2;
  if (D2W != 0)
  {
    D0W = sw(D0W - D2W);
    D2L = D2UW * D1UW;
    D2W <<= 3;
    A1 += D2W;
  }
  else
  {
    // CursorHeight + cursorY - 200
    D3W = sw(D0W + D7W - 200); //
    if (D3W >= 0) // if partly offscreen
    {
      D0W = sw(D0W - D3W); // reduce height to fit
    };
  };
  D2L = (i32)A2;
  SWAP(D2);
  if (D2W != 0) // if left of screen
  {
tag002df8:
    D1W -= 2;
    if (D1W == 0)
    {
      A2 = (pnt)0x002eb2;
      D5W = 152;
      goto tag002e42;
    };
    A2 = (pnt)0x002e86;
    D5W = 144;
    goto tag002e42;
  }
  else
  {
    // cursorWidth + cursorX - 336
    // Assme offset might be as great as 15
    D3W = sw((D1W<<4)+D6W-336);
    // D3W = how far it spills over the right margin
    if (D3W >= 0)
    {
      A1 -= 8; //
      goto tag002df8;
    };
    D1W -= 2;
    if (D1W < 0)
    {
      A2 = (pnt)0x002eb4;
      D5W = 152;
      goto tag002e42;
    }
    else
    {
      if (D1W == 0)
      {
        A2 = (pnt)0x002e88;
        D5W = 144;
        goto tag002e42;
      };
      A2 = (pnt)0x002e5c;
      D5W = 136;
    };
  };
tag002e42:
  D6W = (I16)((D6W >> 1)&0x7ff8);
  D7W <<= 5;
  D6W = sw(D6W + D7W);
  D7W = sw(D7W + D7W);
  D7W = sw(D7W + D7W);
  D6W = sw(D6W + D7W); // D6W += 5*(D7W*32);
  A0 = d.LogicalScreenBase + D6W;
  D7W = D0W;

tag002e5a:
  switch((i32)A2)
  {
  case 0x002e5c: goto tag002e5c;
  case 0x002e86: goto tag002e86;
  case 0x002e88: goto tag002e88;
  case 0x002eb4: goto tag002eb4;
  case 0x002eb2: goto tag002eb2;
  default: NotImplemented(0x2e5a);
  };
tag002e5c:
//002e5c 4c91 000f                MOVEM.W  (A1),D0123
  D0W = LE16(wordGear(A1+0));
  D1W = LE16(wordGear(A1+2));
  D2W = LE16(wordGear(A1+4));
  D3W = LE16(wordGear(A1+6));
//002e60 3802                     MOVE.W   D2,D4
//002e62 c843                     AND.W    D3,D4
  D4W = (UI16)(((D2W & D3W) ^ 0xffff) | D0W | D1W);
//002e64 4644                     NOT.W    D4
//002e66 8840                     OR.W     D0,D4
//002e68 8841                     OR.W     D1,D4
//002e6a 4840                     SWAP     D0
  //SWAP(D0);
  D0H1 = D0H2;
//002e6c 3001                     MOVE.W   D1,D0
  D0W = D1W;
//002e6e 4842                     SWAP     D2
  //SWAP(D2);
  D2H1 = D2H2;
//002e70 3403                     MOVE.W   D3,D2
  D2W = D3W;
//002e72 3604                     MOVE.W   D4,D3
  D3W = D4W;
//002e74 4844                     SWAP     D4
  //SWAP(D4);
  D4H1 = D4H2;
//002e76 3803                     MOVE.W   D3,D4
  D4W = D3W;
//002e78 c084                     AND.L    D4,D0
  D0L &= D4L;
//002e7a c484                     AND.L    D4,D2
  D2L &= D4L;
//002e7c 4684                     NOT.L    D4
  D4L ^= 0xffffffff;
//002e7e c990                     AND.L    D4,(A0)
  longGear(A0) &= LE32(D4L);
//002e80 8198                     OR.L     D0,(A0)+
  longGear(A0) |= LE32(D0L);
  A0 += 4;
//002e82 c990                     AND.L    D4,(A0)
  longGear(A0) &= LE32(D4L);
//002e84 8598                     OR.L     D2,(A0)+
  longGear(A0) |= LE32(D2L);
  A0 += 4;
tag002e86:
  A1 += 8;
tag002e88:
  D0W = LE16(wordGear(A1));
  D1W = LE16(wordGear(A1+2));
  D2W = LE16(wordGear(A1+4));
  D3W = LE16(wordGear(A1+6)); // 16 pixels colors D3->D0
  D4W = (UI16)(((D2W & D3W)^0xffff)|D0W|D1W); //0 where color=12 (transparent)
  //SWAP(D0); //D0 = (D0UW&0xffff)|((D0W&0xffff)<<16);
  D0H1 = D0H2;
  D0W = D1W;
  //SWAP(D2); //D2 = (D2U&0xffff)|((D2W&0xffff)<<16);
  D2H1 = D2H2;
  D2W = D3W;
  D3W = D4W;
  //SWAP(D4); //D4 = (D4U&0xffff)|((D4W&0xffff)<<16);
  D4H1 = D4H2;
  D4W = D3W;
  D0L &= D4L;
  D2L &= D4L;
  D4L ^= -1;
  wordGear(A0+0) &= LE16(D4H1);
  wordGear(A0+2) &= LE16(D4H2);
  wordGear(A0+0) |= LE16(D0H1);
  wordGear(A0+2) |= LE16(D0H2);
  A0 += 4;
  wordGear(A0+0) &= LE16(D4H1);
  wordGear(A0+2) &= LE16(D4H2);
  wordGear(A0+0) |= LE16(D2H1);
  wordGear(A0+2) |= LE16(D2H2);
  A0 +=4;
tag002eb2:
  A1 += 8;
tag002eb4:
  D0W = LE16(wordGear(A1));
  D1W = LE16(wordGear(A1+2));
  D2W = LE16(wordGear(A1+4));
  D3W = LE16(wordGear(A1+6));
  A1 += 8;
  D4W = (UI16)(((D2W & D3W) ^ 0xffff)|D0W|D1W);
  //SWAP(D0); // = (D0UW&0xffff)|((D0W&0xffff)<<16);
  D0H1 = D0H2;
  D0W = D1W;
  D2H1 = D2W;
  D2W = D3W;
  D3W = D4W;
  D4H1 = D4W;
  D4W = D3W;
  D2L &= D4L;
  D0L &= D4L;
  D4L ^= -1;
  wordGear(A0+0) &= LE16(D4H1);
  wordGear(A0+2) &= LE16(D4W);
  wordGear(A0+0) |= LE16(D0H1);
  wordGear(A0+2) |= LE16(D0W);
  A0 += 4;
  wordGear(A0+0) &= LE16(D4H1);
  wordGear(A0+2) &= LE16(D4W);
  wordGear(A0+0) |= LE16(D2H1);
  wordGear(A0+2) |= LE16(D2W);
  A0 += 4;
  A0 += D5W;
  D7W--;
  if (D7W >= 0)  goto tag002e5a;
}


void TAG00306c(void)
{
  NotImplemented(0x306c);
}

/*
void TAG0030e6(i32 divisor,
               i32 dividend,
               i32 *quotient,
               i32 *remainder)
{
  dReg D0, D1, D2, D3;
  i32 LOCAL_2;
  i32 carry;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0030e6 4e56 fffe                LINK     A6,#-2
//0030ea 48e7 f000                MOVEM.L  D0123,-(A7)
//0030ee 242e 0008                MOVE.L   8(A6),D2
  D2L = divisor;
//0030f2 660a                     BNE      $+12 (=0x0030fe)
  if (divisor == 0)
//0030f4 80fc 0000                DIVU     #0,D0
//0030f8 4280                     CLR.L    D0
//0030fa 4281                     CLR.L    D1
//0030fc 604a                     BRA      $+76 (=0x003148)
  if (quotient != NULL) *quotient = 0;
  if (remainder != NULL) *remainder = 0;
  return;
//0030fe 222e 000c                MOVE.L   12(A6),D1
  D1L = dividend;
//003102 426e fffe                CLR.W    -2(A6)
  LOCAL_2 = 0;
//003106 4a81                     TST.L    D1
//003108 6c06                     BGE      $+8 (=0x003110)
  if (D1L < 0)
  {
//00310a 566e fffe                ADDQ.W   #3,-2(A6)
//00310e 4481                     NEG.L    D1
    LOCAL_2 += 3;
    D1L = -D1L;
  };
//003110 4a2                     TST.L    D2
//003112 6c06                     BGE      $+8 (=0x00311a)
  if (D2L < 0)
  {
//003114 526e fffe                ADDQ.W   #1,-2(A6)
//003118 4482                     NEG.L    D2
    LOCAL_2 += 1;
    D2L = -D2L;
  };
//00311a 7601                     MOVEQ.L  #1,D3
  D3L = 1;
  for (;;)
  {
//00311c b481                     CMP.L    D1,D2
//00311e 6406                     BCC      $+8 (=0x003126)
    if (D2L >= D1L) break;
//003120 e382                     ASL.L    #1,D2
//003122 e383                     ASL.L    #1,D3
    D2L <<= 1;
    D3L <<= 1;
//003124 60f6                     BRA      $-8 (=0x00311c)
  };
//003126 4280                     CLR.L    D0
  D0L = 0;
  do
  {
//003128 b481                     CMP.L    D1,D2
//00312a 6204                     BHI      $+6 (=0x003130)
    if (D2L <= D1L)
    {
//00312c 8083                     OR.L     D3,D0
      D0L |= D3L;
//00312e 9282                     SUB.L    D2,D1
      D1L -= D2L;
    };
//003130 e28a                     LSR.L    #1,D2
//003132 e28b                     LSR.L    #1,D3
    D2L >>= 1;
    carry = D3L & 1;
    D3L >>= 1;
//003134 64f2                     BCC      $-12 (=0x003128)
  } while (carry==0);
  while (carry==0);
//003136 0c6e 0003 fffe           CMP.W    #3,-2(A6)
//00313c 6d02                     BLT      $+4 (=0x003140)
  if (LOCAL_2>=3)
  {
//00313e 4481                     NEG.L    D1
    D1L = -D1L;
  };
//003140 e2ee fffe                ASR      -2(A6)
//003144 6402                     BCC      $+4 (=0x003148)
  if (D2L & 1)
  {
//003146 4480                     NEG.L    D0
    D0L = -D0L;
  };
//003148 2d40 0008                MOVE.L   D0,8(A6)
  if (quotient != NULL) *quotient = D0L;
  if (remainder != NULL) *remainder = D1L;
//00314c 2d41 000c                MOVE.L   D1,12(A6)
//003150 4cdf 000f                MOVEM.L  (A7)+,D0123
//003154 4e5e                     UNLK     A6
//003156 4e75                     RTS
}
*/

void MoveWords(aReg& A0, aReg A1, i32 numM1) // 0031c4
{ // Uses A0, A1, and D0
  // Move D0+1 words from (A1) to (A0) and increment A0
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0031c4 4e56 0000                LINK     A6,#0
//  dReg D0;
//  aReg A0, A1;
  //pnt saveA1=A1;
//0031c8 2f09                     MOVE.L   A1,-(A7)
//0031ca 30d9                     MOVE.W   (A1)+,(A0)+
//0031cc 51c8 fffc                DBF      D0,$-2 (=0x0031ca)
  memcpy (A0, A1, 2*(numM1+1));
  A0 += 2*(numM1+1);

  //while (numM1 >= 0)
  //{
  //  wordGear(A0) = wordGear(A1);
  //  A0 += 2;
  //  A1 += 2;
  //  numM1--;
  //}
//0031d0 225f                     MOVE.L   (A7)+,A1
//0031d2 4e5e                     UNLK     A6
//0031d4 4e75                     RTS
  //A1 = saveA1;
}


/*
//   TAG0031d6
void SearchWordNE(dReg *D0,i16 **A0,i16 *A1)
{ // Search two word strings for 'Not Equal'. Update D0, A0
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0031d6 4e56 0000                LINK     A6,#0
//0031da 2f09                     MOVE.L   A1,-(A7)
//0031dc b149                     CMPM.W   (A1)+,(A0)+
  do
  {
//0031de 56c8 fffc                DBNE     D0,$-2 (=0x0031dc)
    if (*((*A0)++) != *(A1++)) break;
  } while (--(D0->W) != -1);
//0031e2 225f                     MOVE.L   (A7)+,A1
//0031e4 4e5e                     UNLK     A6
//0031e6 4e75                     RTS
}
*/


//  TAG0031e8
pnt StrCat(pnt msg1, pnt msg2)
{
  aReg A2, A3;
  pnt LOCAL_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0031e8 4e56 fffc                LINK     A6,#-4
//0031ec 48e7 0030                MOVEM.L  A23,-(A7)
//0031f0 266e 0008                MOVE.L   8(A6),A3
//0031f4 246e 000c                MOVE.L   12(A6),A2
//0031f8 2d4b fffc                MOVE.L   A3,-4(A6)
  A3 = msg1;
  A2 = msg2;
  LOCAL_4 = A3;
//0031fc 6000 0002                BRA      $+4 (=0x003200)
//003200 101b                     MOVE.B   (A3)+,D0
//003202 6600 fffc                BNE      $-2 (=0x003200)
  while ( *(A3++) != 0) {};
  A3--;
//003206 538b                     SUBQ.L   #1,A3
//003208 6000 0002                BRA      $+4 (=0x00320c)
//00320c 16da                     MOVE.B   (A2)+,(A3)+
//00320e 6600 fffc                BNE      $-2 (=0x00320c)
  while ( (*(A3++) = *(A2++)) != 0) {};
//003212 202e fffc                MOVE.L   -4(A6),D0
//003216 6000 0002                BRA      $+4 (=0x00321a)
//00321a 4cdf 0c00                MOVEM.L  (A7)+,A23
//00321e 4e5e                     UNLK     A6
//003220 4e75                     RTS
  return LOCAL_4;
}

//  TAG003222
i16 StrCmp(pnt s1, pnt s2)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//003222 4e56 0000                LINK     A6,#0
//003226 48e7 0030                MOVEM.L  A23,-(A7)
//00322a 266e 0008                MOVE.L   8(A6),A3
//00322e 246e 000c                MOVE.L   12(A6),A2
//003232 6000 0006                BRA      $+8 (=0x00323a)
  while ( (*s1!=0) && (*s1 ==*s2))
  {
    s1++;
    s2++;
  };
//003236 528b                     ADDQ.L   #1,A3
//003238 528a                     ADDQ.L   #1,A2
//
//00323a 1013                     MOVE.B   (A3),D0
//00323c 6700 0010                BEQ      $+18 (=0x00324e)
//
//003240 1013                     MOVE.B   (A3),D0
//003242 1212                     MOVE.B   (A2),D1
//003244 4881                     EXT.W    D1
//003246 4880                     EXT.W    D0
//003248 b041                     CMP.W    D1,D0
//00324a 6700 ffea                BEQ      $-20 (=0x003236)

//00324e 1013                     MOVE.B   (A3),D0
//003250 4880                     EXT.W    D0
//003252 1612                     MOVE.B   (A2),D3
//003254 4883                     EXT.W    D3
//003256 9043                     SUB.W    D3,D0
//003258 6000 0002                BRA      $+4 (=0x00325c)
  return sw((UI8)(*s1) - (UI8)(*s2));
//00325c 4cdf 0c00                MOVEM.L  (A7)+,A23
//003260 4e5e                     UNLK     A6
//003262 4e75                     RTS
}


//  TAG00327c
pnt StrCpy(pnt dest, pnt src, i16 maxChar)
{
  dReg D0, D7;
  aReg A2, A3;
  pnt LOCAL_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00327c 4e56 fffc                LINK     A6,#-4
//003280 48e7 0130                MOVEM.L  D7A23,-(A7)
//003284 266e 0008                MOVE.L   8(A6),A3
  A3 = dest;
  A2 = src;
  D7W = maxChar;
//003288 246e 000c                MOVE.L   12(A6),A2
//00328c 3e2e 0010                MOVE.W   16(A6),D7
//003290 2d4b fffc                MOVE.L   A3,-4(A6)
//003294 6000 0002                BRA      $+4 (=0x003298)
  LOCAL_4 = A3;
  do
  {
//003298 3007                     MOVE.W   D7,D0
    D0W = D7W;
    D7W--;
//00329a 5347                     SUBQ.W   #1,D7
//00329c 4a40                     TST.W    D0
//00329e 6700 000a                BEQ      $+12 (=0x0032aa)
    if (D0B == 0) break;
//0032a2 101a                     MOVE.B   (A2)+,D0
    D0B = *(A2++);
    *(A3++) = D0B;
//0032a4 16c0                     MOVE.B   D0,(A3)+
//0032a6 6600 fff0                BNE      $-14 (=0x003298)
  } while (D0B != 0);
//0032aa 202e fffc                MOVE.L   -4(A6),D0
//0032ae 6000 0002                BRA      $+4 (=0x0032b2)
  return LOCAL_4;
//0032b2 4cdf 0c80                MOVEM.L  (A7)+,D7A23
//0032b6 4e5e                     UNLK     A6
//0032b8 4e75                     RTS

}

//  TAG0032ba
i16 StrLen(pnt string)
{
  aReg A2, A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0032ba 4e56 0000                LINK     A6,#0
//0032be 48e7 0030                MOVEM.L  A23,-(A7)
//0032c2 266e 0008                MOVE.L   8(A6),A3
  A3 = string;
//0032c6 244b                     MOVE.L   A3,A2
  A2 = A3;
  while (*A3 != 0)
  {
//0032c8 6000 0004                BRA      $+6 (=0x0032ce)
//0032cc 528b                     ADDQ.L   #1,A3
    A3++;
//0032ce 1013                     MOVE.B   (A3),D0
//0032d0 6600 fffa                BNE      $-4 (=0x0032cc)
  };
//0032d4 200b                     MOVE.L   A3,D0
//0032d6 908a                     SUB.L    A2,D0
  return sw(A3-A2);
//0032d8 6000 0002                BRA      $+4 (=0x0032dc)
//0032dc 4cdf 0c00                MOVEM.L  (A7)+,A23
//0032e0 4e5e                     UNLK     A6
//0032e2 4e75                     RTS
}

//  TAG0032e4
pnt StrChr(pnt P1,i16 P2)
{ // Find character in string.  Returns address or NULL
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0032e4 4e56 0000                LINK     A6,#0
//0032e8 48e7 0310                MOVEM.L  D67A3,-(A7)
  dReg D0, D6, D7;
  aReg A3;
//0032ec 266e 0008                MOVE.L   8(A6),A3
//0032f0 3e2e 000c                MOVE.W   12(A6),D7
  A3 = P1;
  D7W = P2;
//0032f4 6000 0004                BRA      $+6 (=0x0032fa)
  goto tag0032fa;
tag0032f8:
//0032f8 528b                     ADDQ.L   #1,A3
//0032fa 1013                     MOVE.B   (A3),D0
  A3 += 1;
tag0032fa:
  D0B = *A3;
//0032fc 1c00                     MOVE.B   D0,D6
  D6B = D0B;
//0032fe 6700 000c                BEQ      $+14 (=0x00330c)
  if (D6B == 0) goto tag00330c;
//003302 1006                     MOVE.B   D6,D0
//003304 4880                     EXT.W    D0
//003306 b047                     CMP.W    D7,D0
//003308 6600 ffee                BNE      $-16 (=0x0032f8)
  D0W = D6UB;
  if (D0W != D7W) goto tag0032f8;
//00330c 1006                     MOVE.B   D6,D0
//00330e 6700 0008                BEQ      $+10 (=0x003318)
tag00330c:
  D0B = D6B;
  if (D0B == 0) goto tag003318;
//003312 200b                     MOVE.L   A3,D0
//003314 6000 0008                BRA      $+10 (=0x00331e)
  return A3;
//003318 203c 0000 0000           MOVE.L   #0,D0
tag003318:
  return NULL;
//00331e 6000 0002                BRA      $+4 (=0x003322)
//003322 4cdf 08c0                MOVEM.L  (A7)+,D67A3
//003326 4e5e                     UNLK     A6
//003328 4e75                     RTS
}
//*********************************************************
//
//*********************************************************
//   TAG003796
void ModifyPalettes(i32 whichColor,i32 P2)
{
//003796 4e56 0000                LINK     A6,#0
//00379a 48e7 0700                MOVEM.L  D567,-(A7)
  dReg D0, D5, D6;
//00379e 3e2e 0008                MOVE.W   8(A6),D7
//0037a2 3c2e 000a                MOVE.W   10(A6),D6
//0037a6 4245                     CLR.W    D5
  //D7W = whichColor;
  D6W = sw(P2);
  for (D5W=0; D5W<6; D5W++)
  {
//0037a8 602e                    BRA      $+48 (=0x0037d8)
//0037aa 3005                     MOVE.W   D5,D0
//0037ac 3206                     MOVE.W   D6,D1
//0037ae c3fc 000e                MULS     #14,D1
//0037b2 41ec e63c                LEA      -6596(A4),A0
//0037b6 d0c1                     ADD.W    D1,A0
//0037b8 e380                     ASL.L    #1,D0
//0037ba 41d0                     LEA      (A0),A0
//0037bc d0c0                     ADD.W    D0,A0
//0037be 3010                     MOVE.W   (A0),D0
    D0W = LE16(d.Word6596[D6W][D5W]);
//0037c0 3207                     MOVE.W   D7,D1
//0037c2 3405                     MOVE.W   D5,D2
//0037c4 c5fc 0020                MULS     #32,D2
//0037c8 41ec fdd8                LEA      -552(A4),A0
//0037cc d488                     ADD.L    A0,D2
//0037ce e381                     ASL.L    #1,D1
//0037d0 2042                     MOVE.L   D2,A0
//0037d2 d0c1                     ADD.W    D1,A0
//0037d4 3080                     MOVE.W   D0,(A0)
    ASSERT(D0W <= 0x777);
    d.Palette552[D5W].color[whichColor] = D0W;
//0037d6 5245                     ADDQ.W   #1,D5
//
//0037d8 0c45 0006                CMP.W    #6,D5
//0037dc 6dcc                     BLT      $-50 (=0x0037aa)
  };
//0037de 3006                     MOVE.W   D6,D0
//0037e0 c1fc 000e                MULS     #14,D0
//0037e4 41ec e648                LEA      -6584(A4),A0
//0037e8 d0c0                     ADD.W    D0,A0
//0037ea 1010                     MOVE.B   (A0),D0
  D0W = d.Word6596[D6W][6]; // first byte
//0037ec 41ec e61c                LEA      -6628(A4),A0
//0037f0 d0c7                     ADD.W    D7,A0
//0037f2 c07c 00ff                AND.W    #255,D0
//0037f6 1080                     MOVE.B   D0,(A0)
  d.Byte6628[whichColor] = D0B;
//0037f8 3006                     MOVE.W   D6,D0
//0037fa c1fc 000e                MULS     #14,D0
//0037fe 41ec e649                LEA      -6583(A4),A0
//003802 d0c0                     ADD.W    D0,A0
//003804 1010                     MOVE.B   (A0),D0
  //D0B = d.Word6596[D6W][6]; // least significant byte
//003806 41ec e62c                LEA      -6612(A4),A0
//00380a d0c7                     ADD.W    D7,A0
//00380c c07c 00ff                AND.W    #255,D0
//003810 1080                     MOVE.B   D0,(A0)
  d.Byte6612[whichColor] = sb(D0W>>8);
//003812 4cdf 00e0                MOVEM.L  (A7)+,D567
//003816 4e5e                     UNLK     A6
//003818 4e75                     RTS

}

void TAG00381a(i16 P1)
{
  dReg D0, D7;
  //i16 saveD7 = D7W;
  D7W = P1;
  D0W = d.Word7250;
  if (D7W != D0W)
  {
    d.Word7250 = D7W;
    D0L = D7W * 2 + 75;
    D7W = D0W;
    ReadAndExpandGraphic(D7W, d.Pointer2070, 0,0);
    ReadAndExpandGraphic(D7W+1, d.Pointer2074, 0,0);
  };
  //D7W = saveD7;
}

void TAG00386a(i16 P1)
{
  dReg D0, D6, D7;
  aReg A0, A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00386a 4e56 0000                LINK     A6,#0
//00386e 48e7 0310                MOVEM.L  D67A3,-(A7)
  //i32 saveD6=D6, saveD7=D7;
  //pnt saveA3=A3;
//003872 3e2e 0008                MOVE.W   8(A6),D7
//003876 302c e3ac                MOVE.W   -7252(A4),D0
  D7W = P1;
  D0W = d.Word7252;
//00387a b047                     CMP.W    D7,D0
//00387c 6608                     BNE      $+10 (=0x003886)

//00387e 302c a7c8                MOVE.W   -22584(A4),D0
//003882 6700 0082                BEQ      $+132 (=0x003906)
  if ( (D7W != D0W) || (d.Word22584 != 0) )
  {
//003886 3947 e3ac                MOVE.W   D7,-7252(A4)
//00388a 3007                     MOVE.W   D7,D0
    d.Word7252 = D7W;
    D7W = sw(D7W * 13 + 77);
//00388c c1fc 000d                MULS     #13,D0
//003890 d07c 004d                ADD.W    #77,D0
//003894 3e00                     MOVE.W   D0,D7
//003896 41ec f7a6                LEA      -2138(A4),A0
//00389a 2648                     MOVE.L   A0,A3
    A3 = A0 = (pnt)&d.Pointer2138;
//00389c 3c07                     MOVE.W   D7,D6
    D6W = sw(D7W + 13);
//00389e dc7c 000d                ADD.W    #13,D6
//0038a2 6014                     BRA      $+22 (=0x0038b8)
    while (D6W > D7W)
    {
//0038a4 4267                     CLR.W    -(A7)
//0038a6 4267                     CLR.W    -(A7)
//0038a8 2f1b                     MOVE.L   (A3)+,-(A7)
//0038aa 3007                     MOVE.W   D7,D0
//0038ac 5247                     ADDQ.W   #1,D7
//0038ae 3f00                     MOVE.W   D0,-(A7)
//0038b0 4ead 0606                JSR      1542(A5)    (=0x022b86)
//0038b4 defc 000a                ADD.W    #10,A7
      ReadAndExpandGraphic(D7W, pntGear(A3), 0, 0);
      A3 += 4;
      D7W++;
    };
//0038b8 3007                     MOVE.W   D7,D0
//0038ba b046                     CMP.W    D6,D0
//0038bc 6de6                     BLT      $-24 (=0x0038a4)

//0038be 102c f3c3                MOVE.B   -3133(A4),D0
//0038c2 c07c 00ff                AND.W    #255,D0
//0038c6 3f00                     MOVE.W   D0,-(A7)
//0038c8 102c f3c2                MOVE.B   -3134(A4),D0
//0038cc c07c 00ff                AND.W    #255,D0
//0038d0 3f00                     MOVE.W   D0,-(A7)
//0038d2 2f2c f7a2                MOVE.L   -2142(A4),-(A7)
//0038d6 2f2c f7aa                MOVE.L   -2134(A4),-(A7)
//0038da 4eba 0a6e                JSR      $+2672 (=0x00434a)
//0038de defc 000c                ADD.W    #12,A7
    MakeMirror(d.Pointer2134,
               d.Pointer2142,
               (UI8)(d.Byte3134),
               (UI8)(d.Byte3133));
//0038e2 102c f463                MOVE.B   -2973(A4),D0
//0038e6 c07c 00ff                AND.W    #255,D0
//0038ea 3f00                     MOVE.W   D0,-(A7)
//0038ec 102c f462                MOVE.B   -2974(A4),D0
//0038f0 c07c 00ff                AND.W    #255,D0
//0038f4 3f00                     MOVE.W   D0,-(A7)
//0038f6 2f2c f7da                MOVE.L   -2086(A4),-(A7)
//0038fa 2f2c f7d6                MOVE.L   -2090(A4),-(A7)
//0038fe 4eba 0a4a                JSR      $+2636 (=0x00434a)
//003902 defc 000c                ADD.W    #12,A7
    MakeMirror(d.Pointer2090,
               d.Pointer2086,
               (UI8)(d.Byte2978[4]),
               (UI8)(d.Byte2978[5]));
  };
//003906 4cdf 08c0                MOVEM.L  (A7)+,D67A3
//00390a 4e5e                     UNLK     A6
  //D6=saveD6;D7=saveD7;A3=saveA3;
//00390c 4e75                     RTS
}
//*********************************************************
//
//*********************************************************
//   TAG0042da
void MarkViewportUpdated(i16 MultiplePalettes)
{ //(void)
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0042da 4e56 0000                LINK     A6,#0
//0042de 0c6e 0002 0008           CMP.W    #2,8(A6)
//0042e4 6608                     BNE      $+10 (=0x0042ee)
  if (MultiplePalettes == 2)
  {
//0042e6 396c d226 d224           MOVE.W   -11738(A4),-11740(A4)
    d.Word11740 = d.DynamicPaletteSwitching;
//0042ec 6006                     BRA      $+8 (=0x0042f4)
  }
  else
  {
//0042ee 396e 0008 d224           MOVE.W   8(A6),-11740(A4)
    //if ((MultiplePalettes==0) && (d.Word11740!=0))
    //  memmove((pnt)palette,(pnt)&d.Palette11978,32);
    d.Word11740 = MultiplePalettes;
  };
//0042f4 397c 0001 d222           MOVE.W   #1,-11742(A4)
  d.ViewportUpdated = 1; //Force VBL to copy Viewport to Screen
//0042fa 3f3c 0025                MOVE.W   #37,-(A7)
//0042fe 4ead 0138                JSR      312(A5)    (=0x001ce8)
//004302 548f                     ADDQ.L   #2,A7
  vblInterrupt();// No increment of VBLCount!//wvbl(_1_);
//004304 4e5e                     UNLK     A6
//004306 4e75                     RTS
}

void MakeMirror(pnt src, pnt dst, i16 width, i16 height) //TAG00434a
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00434a 4e56 0000                LINK     A6,#0
//00434e 302e 0010                MOVE.W   16(A6),D0
  D0L = (ui16)width * (ui16)height;
//004352 c0ee 0012                MULU     18(A6),D0
//004356 3f00                     MOVE.W   D0,-(A7)
//004358 2f2e 000c                MOVE.L   12(A6),-(A7)
//00435c 2f2e 0008                MOVE.L   8(A6),-(A7)
//004360 4ead 0006                JSR      6(A5)    (=0x0009dc)
//004364 defc 000a                ADD.W    #10,A7
  MemMove(src,dst,D0W);
//004368 3f2e 0012                MOVE.W   18(A6),-(A7)
//00436c 3f2e 0010                MOVE.W   16(A6),-(A7)
//004370 2f2e 000c                MOVE.L   12(A6),-(A7)
//004374 4eba 42cc                JSR      $+17102 (=0x008642)
//004378 508f                     ADDQ.L   #0,A7
  Mirror(dst,width,height);
//00437a 4e5e                     UNLK     A6
//00437c 4e75                     RTS
}


void TAG00437e(pnt P1,RectPos *P2)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00437e 4e56 0000                LINK     A6,#0
  dReg D0;
//  aReg A3;
//004382 2f0b                     MOVE.L   A3,-(A7)
//004384 266e 000c                MOVE.L   12(A6),A3
  // A3 = P2;
//004388 102b 0004                MOVE.B   4(A3),D0
//00438c 6730                     BEQ      $+50 (=0x0043be)
  ASSERT(verifyRectPos(P2));
  D0W = (UI8)(P2->b.uByte4);
  if (D0B != 0)
  {
//00438e 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//004394 4240                     CLR.W    D0
//004396 102b 0004                MOVE.B   4(A3),D0
//00439a 3f00                     MOVE.W   D0,-(A7)
//00439c 4240                     CLR.W    D0
//00439e 102b 0007                MOVE.B   7(A3),D0
//0043a2 3f00                     MOVE.W   D0,-(A7)
//0043a4 4240                     CLR.W    D0
//0043a6 102b 0006                MOVE.B   6(A3),D0
//0043aa 3f00                     MOVE.W   D0,-(A7)
//0043ac 2f0b                     MOVE.L   A3,-(A7)
//0043ae 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0043b2 2f2e 0008                MOVE.L   8(A6),-(A7)
//0043b6 4eba 44fa                JSR      $+17660 (=0x0088b2)
//0043ba defc 0016                ADD.W    #22,A7
    TAG0088b2(P1,
              d.pViewportBMP,
              P2,
              (UI8)(P2->b.uByte6),
              (UI8)(P2->b.uByte7),
              (UI8)(P2->b.uByte4),
              112,
              10);
  };
//0043be 265f                     MOVE.L   (A7)+,A3
//0043c0 4e5e                     UNLK     A6
//0043c2 4e75                     RTS
}

void TAG0043c4(pnt src,RectPos *P2)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0043c4 4e56 0000                LINK     A6,#0
  //pnt A3;
//0043c8 2f0b                     MOVE.L   A3,-(A7)
//0043ca 266e 000c                MOVE.L   12(A6),A3
  //A3 = P2;
//0043ce 102b 0004                MOVE.B   4(A3),D0
//0043d2 6730                     BEQ      $+50 (=0x004404)
  ASSERT(verifyRectPos(P2));
  if (P2->b.uByte4 != 0)
  {
//0043d4 2f3c 0070 ffff           MOVE.L   #7405567,-(A7)
//0043da 4240                     CLR.W    D0
//0043dc 102b 0004                MOVE.B   4(A3),D0
//0043e0 3f00                     MOVE.W   D0,-(A7)
//0043e2 4240                     CLR.W    D0
//0043e4 102b 0007                MOVE.B   7(A3),D0
//0043e8 3f00                     MOVE.W   D0,-(A7)
//0043ea 4240                     CLR.W    D0
//0043ec 102b 0006                MOVE.B   6(A3),D0
//0043f0 3f00                     MOVE.W   D0,-(A7)
//0043f2 2f0b                     MOVE.L   A3,-(A7)
//0043f4 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0043f8 2f2e 0008                MOVE.L   8(A6),-(A7)
//0043fc 4eba 44b4                JSR      $+17590 (=0x0088b2)
//004400 defc 0016                ADD.W    #22,A7
    TAG0088b2(src,
              d.pViewportBMP,
              P2,
              (UI8)(P2->b.uByte6),
              (UI8)(P2->b.uByte7),
              (UI8)(P2->b.uByte4),
              112,
              -1);
  };
//004404 265f                     MOVE.L   (A7)+,A3
//004406 4e5e                     UNLK     A6
//004408 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
void TAG00440a(pnt P1)
{
  aReg A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00440a 4e56 0000                LINK     A6,#0
//00440e 2f0b                     MOVE.L   A3,-(A7)
//004410 266e 0008                MOVE.L   8(A6),A3
  A3 = P1;
//004414 102b 0004                MOVE.B   4(A3),D0
//004418 6730                     BEQ      $+50 (=0x00444a)
  if (A3[4] == 0) return;
//00441a 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//004420 4240                     CLR.W    D0
//004422 102b 0004                MOVE.B   4(A3),D0
//004426 3f00                     MOVE.W   D0,-(A7)
//004428 4240                     CLR.W    D0
//00442a 102b 0007                MOVE.B   7(A3),D0
//00442e 3f00                     MOVE.W   D0,-(A7)
//004430 4240                     CLR.W    D0
//004432 102b 0006                MOVE.B   6(A3),D0
//004436 3f00                     MOVE.W   D0,-(A7)
//004438 2f0b                     MOVE.L   A3,-(A7)
//00443a 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//00443e 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004442 4eba 446e                JSR      $+17520 (=0x0088b2)
//004446 defc 0016                ADD.W    #22,A7
  TAG0088b2(d.Pointer1848,
            d.pViewportBMP,
            (RectPos *)A3,
            (UI8)(A3[6]),
            (UI8)(A3[7]),
            (UI8)(A3[4]),
            112,
            10);
//00444a 265f                     MOVE.L   (A7)+,A3
//00444c 4e5e                     UNLK     A6
//00444e 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
void TAG004450(pnt P1,pnt P2)
{
  aReg A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004450 4e56 0000                LINK     A6,#0
//004454 2f0b                     MOVE.L   A3,-(A7)
//004456 266e 000c                MOVE.L   12(A6),A3
  A3 = P2;
//00445a 102b 0004                MOVE.B   4(A3),D0
//00445e 674a                     BEQ      $+76 (=0x0044aa)
  if (A3[4] == 0) return;
//004460 4240                     CLR.W    D0
//004462 102b 0005                MOVE.B   5(A3),D0
//004466 3f00                     MOVE.W   D0,-(A7)
//004468 4240                     CLR.W    D0
//00446a 102b 0004                MOVE.B   4(A3),D0
//00446e 3f00                     MOVE.W   D0,-(A7)
//004470 2f2e 0008                MOVE.L   8(A6),-(A7)
//004474 4eba 41cc                JSR      $+16846 (=0x008642=Mirror)
//004478 508f                     ADDQ.L   #0,A7
  Mirror(P1,
         (UI8)(A3[4]),
         (UI8)(A3[5]) );
//00447a 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//004480 4240                     CLR.W    D0
//004482 102b 0004                MOVE.B   4(A3),D0
//004486 3f00                     MOVE.W   D0,-(A7)
//004488 4240                     CLR.W    D0
//00448a 102b 0007                MOVE.B   7(A3),D0
//00448e 3f00                     MOVE.W   D0,-(A7)
//004490 4240                     CLR.W    D0
//004492 102b 0006                MOVE.B   6(A3),D0
//004496 3f00                     MOVE.W   D0,-(A7)
//004498 2f0b                     MOVE.L   A3,-(A7)
//00449a 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//00449e 2f2e 0008                MOVE.L   8(A6),-(A7)
//0044a2 4eba 440e                JSR      $+17424 (=0x0088b2)
  TAG0088b2(P1,
            d.pViewportBMP,
            (RectPos *)A3,
            (UI8)(A3[6]),
            (UI8)(A3[7]),
            (UI8)(A3[4]),
            112,
            10);
//0044a6 defc 0016                ADD.W    #22,A7
//0044aa 265f                     MOVE.L   (A7)+,A3
//0044ac 4e5e                     UNLK     A6
//0044ae 4e75                     RTS
}

void TAG0044b0(i32 P1,RectPos *rect)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0044b0 4e56 0000                LINK     A6,#0
//0044b4 2f0b                     MOVE.L   A3,-(A7)
//0044b6 266e 000a                MOVE.L   10(A6),A3
//0044ba 102b 0004                MOVE.B   4(A3),D0
//0044be 6738                     BEQ      $+58 (=0x0044f8)
  ASSERT(verifyRectPos(rect));
  if (rect->b.uByte4 == 0) return;
//0044c0 2f3c 0070 000a           MOVE.L   #7340042,-(A7)

//0044c6 4240                     CLR.W    D0
//0044c8 102b 0004                MOVE.B   4(A3),D0
//0044cc 3f00                     MOVE.W   D0,-(A7)

//0044ce 4240                     CLR.W    D0
//0044d0 102b 0007                MOVE.B   7(A3),D0
//0044d4 3f00                     MOVE.W   D0,-(A7)

//0044d6 4240                     CLR.W    D0
//0044d8 102b 0006                MOVE.B   6(A3),D0
//0044dc 3f00                     MOVE.W   D0,-(A7)
//0044de 2f0b                     MOVE.L   A3,-(A7)
//0044e0 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0044e4 3f2e 0008                MOVE.W   8(A6),-(A7)
//0044e8 4ead 061e                JSR      1566(A5)    (=0x022a92)
//0044ec 548f                     ADDQ.L   #2,A7
//0044ee 2f00                     MOVE.L   D0,-(A7)
//0044f0 4eba 43c0                JSR      $+17346 (=0x0088b2)
//0044f4 defc 0016                ADD.W    #22,A7
  TAG0088b2( GetGraphicAddress(P1),
             d.pViewportBMP,
             rect,
             rect->b.uByte6,
             rect->b.uByte7,
             rect->b.uByte4,
             112,
             10);
//0044f8 265f                     MOVE.L   (A7)+,A3
//0044fa 4e5e                     UNLK     A6
//0044fc 4e75                     RTS
}

void TAG0044fe(i32 graphic, RectPos *rectpos)
{
  ASSERT(verifyRectPos(rectpos));
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0044fe 4e56 0000                LINK     A6,#0
//004502 2f0b                     MOVE.L   A3,-(A7)
//004504 266e 000a                MOVE.L   10(A6),A3
//004508 102b 0004                MOVE.B   4(A3),D0
//00450c 6758                     BEQ      $+90 (=0x004566)
  if (rectpos->b.uByte4 == 0) return;
//00450e 4240                     CLR.W    D0
//004510 102b 0005                MOVE.B   5(A3),D0
//004514 3f00                     MOVE.W   D0,-(A7)
//004516 4240                     CLR.W    D0
//004518 102b 0004                MOVE.B   4(A3),D0
//00451c 3f00                     MOVE.W   D0,-(A7)
//00451e 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004522 3f2e 0008                MOVE.W   8(A6),-(A7)
//004526 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//00452a 548f                     ADDQ.L   #2,A7
//00452c 2f00                     MOVE.L   D0,-(A7)
//00452e 4eba fe1a                JSR      $-484 (=0x00434a)
//004532 defc 000c                ADD.W    #12,A7
  MakeMirror(GetGraphicAddress(graphic),
             d.Pointer1848,
             rectpos->b.uByte4,
             rectpos->b.uByte5);
//004536 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//00453c 4240                     CLR.W    D0
//00453e 102b 0004                MOVE.B   4(A3),D0
//004542 3f00                     MOVE.W   D0,-(A7)
//004544 4240                     CLR.W    D0
//004546 102b 0007                MOVE.B   7(A3),D0
//00454a 3f00                     MOVE.W   D0,-(A7)
//00454c 4240                     CLR.W    D0
//00454e 102b 0006                MOVE.B   6(A3),D0
//004552 3f00                     MOVE.W   D0,-(A7)
//004554 2f0b                     MOVE.L   A3,-(A7)
//004556 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//00455a 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//00455e 4eba 4352                JSR      $+17236 (=0x0088b2)
//004562 defc 0016                ADD.W    #22,A7
  TAG0088b2(d.Pointer1848,
            d.pViewportBMP,
            rectpos,
            rectpos->b.uByte6,
            rectpos->b.uByte7,
            rectpos->b.uByte4,
            112,
            10);
//004566 265f                     MOVE.L   (A7)+,A3
//004568 4e5e                     UNLK     A6
//00456a 4e75                     RTS
}

void TAG00456c(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00456c 4e56 0000                LINK     A6,#0
//004570 0c6c 0005 f8ce           CMP.W    #5,-1842(A4)
//004576 6432                     BCC      $+52 (=0x0045aa)
  if ((ui16)d.Word1842 < 5)
  {
//004578 302c f8b2                MOVE.W   -1870(A4),D0
//00457c 662c                     BNE      $+46 (=0x0045aa)
    if (d.Word1870 == 0)
    {
//00457e 302c a544                MOVE.W   -23228(A4),D0
//004582 6614                     BNE      $+22 (=0x004598)
      if (d.Word23228 == 0)
      {
//004584 302c d236                MOVE.W   -11722(A4),D0
//004588 c07c 0001                AND.W    #1,D0
//00458c 660a                     BNE      $+12 (=0x004598)
        if ((d.Word11722 & 1) == 0)
        {
//00458e 006c 0001 d236           OR.W     #1,-11722(A4)
//004594 526c f8ce                ADDQ.W   #1,-1842(A4)
          d.Word11722 |= 1;
          d.Word1842++;
        };
      };
//004598 026c fffe d234           AND.W    #65534,-11724(A4)
      d.Word11724 &= 0xfffe;
//00459e 397c 000f f8d4           MOVE.W   #15,-1836(A4)
//0045a4 397c 0001 f8b2           MOVE.W   #1,-1870(A4)
      d.Word1836 = 15;
      d.Word1870 = 1;
    };
  };
//0045aa 4e5e                     UNLK     A6
//0045ac 4e75                     RTS
}

i16 TAG0045ae(i16 P1,i16 P2)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0045ae 4e56 ffa4                LINK     A6,#-92
  dReg D0, D1, D3, D4, D5, D6, D7;
  aReg A0, A2, A3;
  i8  LOCAL_92[70];
  ui8 LOCAL_22[4];
  i16 LOCAL_14;
  i16 LOCAL_12;
  i16 LOCAL_10;
  i16 LOCAL_8;
  i16 LOCAL_6;
  pnt LOCAL_4;
  bool bD0;
//0045b2 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//0045b6 3e2e 0008                MOVE.W   8(A6),D7
//0045ba 3c2e 000a                MOVE.W   10(A6),D6
//0045be 3007                     MOVE.W   D7,D0
  D7W = P1; // Graphic ordinal for this level.
  D6W = P2;
//0045c0 6700 044c                BEQ      $+1102 (=0x004a0e)
  if (D7W == 0) return 0;
//0045c4 5347                     SUBQ.W   #1,D7
  D7W--;
//0045c6 3a07                     MOVE.W   D7,D5
//0045c8 e585                     ASL.L    #2,D5
//0045ca 41ec f6b2                LEA      -2382(A4),A0
//0045ce d0c5                     ADD.W    D5,A0
//0045d0 3a10                     MOVE.W   (A0),D5
  D5W = d.Word2382[D7W][0];
//0045d2 3006                     MOVE.W   D6,D0
//0045d4 3207                     MOVE.W   D7,D1
//0045d6 e581                     ASL.L    #2,D1
//0045d8 41ec f6b4                LEA      -2380(A4),A0
//0045dc d0c1                     ADD.W    D1,A0
//0045de 3210                     MOVE.W   (A0),D1
  D4W = d.Word2382[D7W][1];
//0045e0 3801                     MOVE.W   D1,D4
  D1L = 78 * D4W;
//0045e2 c3fc 004e                MULS     #78,D1
//0045e6 41ec ed1c                LEA      -4836(A4),A0
//0045ea d288                     ADD.L    A0,D1
//0045ec c1fc 0006                MULS     #6,D0
//0045f0 2041                     MOVE.L   D1,A0
//0045f2 d0c0                     ADD.W    D0,A0
//0045f4 41d0                     LEA      (A0),A0
//0045f6 2648                     MOVE.L   A0,A3
  A3 = d.Byte4836 + D1L + 6*D6W;
//0045f8 3f07                     MOVE.W   D7,-(A7)
//0045fa 4ead 0336                JSR      822(A5)    (=0x0095e8)
//0045fe 548f                     ADDQ.L   #2,A7
  LOCAL_12 = TAG0095e8(D7W);//if D7W one of three special??
//004600 3d40 fff4                MOVE.W   D0,-12(A6)
//004604 3007                     MOVE.W   D7,D0
//004606 b06c d75e                CMP.W    -10402(A4),D0
//00460a 57c0                     SEQ      D0
//00460c c07c 0001                AND.W    #1,D0
//004610 3d40 fff6                MOVE.W   D0,-10(A6)
  LOCAL_10 = (D7W == d.NumWallDecoration);
//004614 6710                     BEQ      $+18 (=0x004626)
  if (LOCAL_10 != 0)
  {
//004616 4267                     CLR.W    -(A7)
//004618 3f2c d6d4                MOVE.W   -10540(A4),-(A7)
//00461c 486e ffa4                PEA      -92(A6)
//004620 4ead 02e8                JSR      744(A5)    (=0x00a27c)
//004624 508f                     ADDQ.L   #0,A7
    DecodeText(LOCAL_92, d.rn10540, 0);
  };
//004626 4267                     CLR.W    -(A7)
//004628 4eba ff42                JSR      $-188 (=0x00456c)
//00462c 548f                     ADDQ.L   #2,A7
  TAG00456c();
//00462e 0c46 000a                CMP.W    #10,D6
//004632 6d00 017c                BLT      $+382 (=0x0047b0)
  if (D6W >= 10)
  {
//004636 0c46 000c                CMP.W    #12,D6
//00463a 6600 013c                BNE      $+318 (=0x004778)
    if (D6W == 12)
    {
//00463e 302e fff6                MOVE.W   -10(A6),D0
//004642 6700 00d8                BEQ      $+218 (=0x00471c)
      if (LOCAL_10 != 0)
      {
//004646 2f3c 0070 ffff           MOVE.L   #7405567,-(A7)
//00464c 102c f432                MOVE.B   -3022(A4),D0
//004650 c07c 00ff                AND.W    #255,D0
//004654 3f00                     MOVE.W   D0,-(A7)
//004656 2f3c 005e 001c           MOVE.L   #6160412,-(A7)
//00465c 486c efa0                PEA      -4192(A4)
//004660 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//004664 2f2c f7ca                MOVE.L   -2102(A4),-(A7)
//004668 4eba 4248                JSR      $+16970 (=0x0088b2)
//00466c defc 0016                ADD.W    #22,A7
         TAG0088b2(d.Pointer2102,  // src
                  d.pViewportBMP, // dst
                  (RectPos *)d.Byte4192,     // 0x00,0x4a,0x14,0x5a
                  94,             // src x
                  28,             // src y
                  d.uByte3026[4],// src byte width
                  112,            // dst byte width
                  -1);
//004670 41ee ffa4                LEA      -92(A6),A0
//004674 2648                     MOVE.L   A0,A3
          A3 = LOCAL_92;
//004676 3f3c 0078                MOVE.W   #120,-(A7)
//00467a 4ead 061e                JSR      1566(A5)    (=0x022a92)
//00467e 548f                     ADDQ.L   #2,A7
//004680 2d40 fffc                MOVE.L   D0,-4(A6)
          LOCAL_4 = GetGraphicAddress(120);
//004684 426e fff2                CLR.W    -14(A6)
          LOCAL_14 = 0;
        do
        {
//004688 4247                     CLR.W    D7
//00468a 244b                     MOVE.L   A3,A2
          D7W = 0;
          A2 = A3;
//00468c 6002                     BRA      $+4 (=0x004690)
          while ((UI8)(*(A2++)) < 128) D7W++;
          {
//00468e 5247                     ADDQ.W   #1,D7

//004690 4240                     CLR.W    D0
//004692 101a                     MOVE.B   (A2)+,D0
//004694 b07c 0080                CMP.W    #128,D0
//004698 6df4                     BLT      $-10 (=0x00468e)
          };
//00469a 303c 0070                MOVE.W   #112,D0
//00469e 3207                     MOVE.W   D7,D1
//0046a0 e541                     ASL.W    #2,D1
//0046a2 9041                     SUB.W    D1,D0
//0046a4 1d40 ffea                MOVE.B   D0,-22(A6)
          LOCAL_22[0] = ub(112 - 4*D7W);
//0046a8 c07c 00ff                AND.W    #255,D0
//0046ac 5e40                     ADDQ.W   #7,D0
//0046ae 1d40 ffeb                MOVE.B   D0,-21(A6)
          LOCAL_22[1] = ub(LOCAL_22[0] + 7);
//0046b2 302e fff2                MOVE.W   -14(A6),D0
//0046b6 526e fff2                ADDQ.W   #1,-14(A6)
//0046ba 41ec ef9c                LEA      -4196(A4),A0
//0046be d0c0                     ADD.W    D0,A0
//0046c0 4240                     CLR.W    D0
//0046c2 1010                     MOVE.B   (A0),D0
          LOCAL_22[3] = ub(*(d.Byte4196+(LOCAL_14++)));
//0046c4 1d40 ffed                MOVE.B   D0,-19(A6)
//0046c8 c07c 00ff                AND.W    #255,D0
//0046cc 5f40                     SUBQ.W   #7,D0
//0046ce 1d40 ffec                MOVE.B   D0,-20(A6)
          LOCAL_22[2] = ub(LOCAL_22[3] - 7);
//0046d2 6030                     BRA      $+50 (=0x004704)
          while (D7W-- != 0)
          {
//0046d4 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//0046da 3f3c 0090                MOVE.W   #144,-(A7)
//0046de 4267                     CLR.W    -(A7)
//0046e0 4240                     CLR.W    D0
//0046e2 101b                     MOVE.B   (A3)+,D0
//0046e4 e740                     ASL.W    #3,D0
//0046e6 3f00                     MOVE.W   D0,-(A7)
//0046e8 486e ffea                PEA      -22(A6)
//0046ec 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0046f0 2f2e fffc                MOVE.L   -4(A6),-(A7)
//0046f4 4eba 41bc                JSR      $+16830 (=0x0088b2)
//0046f8 defc 0016                ADD.W    #22,A7
            TAG0088b2(LOCAL_4,
                      d.pViewportBMP,
                      (RectPos*)LOCAL_22,
                      8*(UI8)(*(A3++)),
                      0,
                      144,
                      112,
                      10);
//0046fc 502e ffea                ADDQ.B   #0,-22(A6)
//004700 502e ffeb                ADDQ.B   #0,-21(A6)
            LOCAL_22[0]+=8;
            LOCAL_22[1]+=8;
//004704 3007                     MOVE.W   D7,D0
//004706 5347                     SUBQ.W   #1,D7
//004708 4a40                     TST.W    D0
//00470a 66c8                     BNE      $-54 (=0x0046d4)
          };
//00470c 4240                     CLR.W    D0
//00470e 101b                     MOVE.B   (A3)+,D0
//004710 b07c 0081                CMP.W    #129,D0
//004714 6600 ff72                BNE      $-140 (=0x004688)
        } while ((UI8)(*(A3++)) != 129);
//004718 6000 02ee                BRA      $+752 (=0x004a08)
        return LOCAL_12;
      }; //if (LOCAL_10 != 0)
//00471c 5245                     ADDQ.W   #1,D5
      D5W++;
//00471e 3f3c 0004                MOVE.W   #4,-(A7)
//004722 486c d6d0                PEA      -10544(A4)
//004726 2f0b                     MOVE.L   A3,-(A7)
//004728 4ead 0006                JSR      6(A5)    (=0x0009dc)
//00472c defc 000a                ADD.W    #10,A7
      MemMove(A3, (pnt)d.ViewportObjectButtons[5], 4); // ?????
//004730 396e fff4 d6dc           MOVE.W   -12(A6),-10532(A4)
      d.Word10532 = LOCAL_12;
//004736 302c f8b2                MOVE.W   -1870(A4),D0
//00473a 670a                     BEQ      $+12 (=0x004746)
      d.Word10534 = (d.Word1870!=0) && (D7W==d.Word10404);
//00473c 3007                     MOVE.W   D7,D0
//00473e b06c d75c                CMP.W    -10404(A4),D0
//004742 57c0                     SEQ      D0
//004744 4a00                     TST.B    D0
//004746 56c0                     SNE      D0
//004748 c07c 0001                AND.W    #1,D0
//00474c 3940 d6da                MOVE.W   D0,-10534(A4)
//004750 426c d6d8                CLR.W    -10536(A4)
//004754 4244                     CLR.W    D4
      d.Word10536 = 0;
//004756 601a                     BRA      $+28 (=0x004772)
      for (D4W=0; D4W<1; D4W++)
      {
//004758 3004                     MOVE.W   D4,D0
//00475a e380                     ASL.L    #1,D0
//00475c 41ec d754                LEA      -10412(A4),A0
//004760 d0c0                     ADD.W    D0,A0
//004762 3010                     MOVE.W   (A0),D0
        D0W = d.Word10412[D4W];
//004764 b047                     CMP.W    D7,D0
//004766 6608                     BNE      $+10 (=0x004770)
        if (D7W == D0W)
        {
//004768 397c 0001 d6d8           MOVE.W   #1,-10536(A4)
          d.Word10536 = 1;
//00476e 6008                     BRA      $+10 (=0x004778)
          break;
        };
//004770 5244                     ADDQ.W   #1,D4

//004772 0c44 0001                CMP.W    #1,D4
//004776 6de0                     BLT      $-30 (=0x004758)
      };
    };
//004778 3f05                     MOVE.W   D5,-(A7)
//00477a 4ead 061e                JSR      1566(A5)    (=0x022a92)
//00477e 548f                     ADDQ.L   #2,A7
    A2 = GetGraphicAddress(D5W); // Get graphic bitmap address
//004780 2440                     MOVE.L   D0,A2
//004782 0c46 000b                CMP.W    #11,D6
//004786 6622                     BNE      $+36 (=0x0047aa)
    if (D6W == 11)
    {
//004788 4240                     CLR.W    D0
//00478a 102b 0005                MOVE.B   5(A3),D0
//00478e 3f00                     MOVE.W   D0,-(A7)
//004790 4240                     CLR.W    D0
//004792 102b 0004                MOVE.B   4(A3),D0
//004796 3f00                     MOVE.W   D0,-(A7)
//004798 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//00479c 2f0a                     MOVE.L   A2,-(A7)
//00479e 4eba fbaa                JSR      $-1108 (=0x00434a)
//0047a2 defc 000c                ADD.W    #12,A7
      MakeMirror(A2,
                 d.Pointer1848,
                 (UI8)(*(A3+4)),
                 (UI8)(*(A3+5)));
//0047a6 246c f8c8                MOVE.L   -1848(A4),A2
      A2 = d.Pointer1848;
    };
//0047aa 4244                     CLR.W    D4
    D4W = 0;
//0047ac 6000 0180                BRA      $+386 (=0x00492e)
  }
  else
  {
//0047b0 426e fffa                CLR.W    -6(A6)
    LOCAL_6 = 0;
//0047b4 0c46 0006                CMP.W    #6,D6
//0047b8 57c0                     SEQ      D0
    bD0 = D6W == 6;
//0047ba 4a00                     TST.B    D0
//0047bc 6608                     BNE      $+10 (=0x0047c6)
    if (!bD0)
    {
//0047be 0c46 0001                CMP.W    #1,D6
//0047c2 57c0                     SEQ      D0
//0047c4 4a00                     TST.B    D0
      bD0 = D6W == 1;
    };
//0047c6 c07c 0001                AND.W    #1,D0
//0047ca 3d40 fff8                MOVE.W   D0,-8(A6)
    LOCAL_8 = bD0;
//0047ce 6712                     BEQ      $+20 (=0x0047e2)
    if (LOCAL_8 != 0)
    {
//0047d0 3004                     MOVE.W   D4,D0
//0047d2 c1fc 004e                MULS     #78,D0
//0047d6 41ec ed5e                LEA      -4770(A4),A0
//0047da d0c0                     ADD.W    D0,A0
//0047dc 41d0                     LEA      (A0),A0
//0047de 2448                     MOVE.L   A0,A2
      A2 = (d.Byte4776 + 6) +78*D4W;
//0047e0 6048                     BRA      $+74 (=0x00482a)
    }
    else
    {
//0047e2 0c46 0005                CMP.W    #5,D6
//0047e6 6704                     BEQ      $+6 (=0x0047ec)
//0047e8 4a46                     TST.W    D6
//0047ea 6612                     BNE      $+20 (=0x0047fe)
      if ( (D6W==5) || (D6W==0) )
      {
//0047ec 3004                     MOVE.W   D4,D0
//0047ee c1fc 004e                MULS     #78,D0
//0047f2 41ec ed58                LEA      -4776(A4),A0
//0047f6 d0c0                     ADD.W    D0,A0
//0047f8 41d0                     LEA      (A0),A0
//0047fa 2448                     MOVE.L   A0,A2
        A2 = d.Byte4776 + 78*D4W;
//0047fc 602c                     BRA      $+46 (=0x00482a)
      }
      else
      {
//0047fe 5245                     ADDQ.W   #1,D5
        D5W++;
//004800 3004                     MOVE.W   D4,D0
//004802 c1fc 004e                MULS     #78,D0
//004806 41ec ed64                LEA      -4764(A4),A0
//00480a d0c0                     ADD.W    D0,A0
//00480c 41d0                     LEA      (A0),A0
//00480e 2448                     MOVE.L   A0,A2
        A2 = (d.Byte4776 + 12) + 78*D4W;
//004810 0c46 0007                CMP.W    #7,D6
//004814 6608                     BNE      $+10 (=0x00481e)
        if (D6W == 7)
        {
//004816 3d7c 0006 fffa           MOVE.W   #6,-6(A6)
          LOCAL_6 = 6;
//00481c 600c                     BRA      $+14 (=0x00482a)
        }
        else
        {
//00481e 0c46 0009                CMP.W    #9,D6
//004822 6606                     BNE      $+8 (=0x00482a)
          if (D6W == 9)
          {
//004824 3d7c fffa fffa           MOVE.W   #65530,-6(A6)
            LOCAL_6 = -6;
          };
        };
      };
    };
//00482a 280b                     MOVE.L   A3,D4
//00482c 362e fffa                MOVE.W   -6(A6),D3
//004830 48c3                     EXT.L    D3
//004832 d883                     ADD.L    D3,D4
//004834 2044                     MOVE.L   D4,A0
    A0 = A3 + LOCAL_6;
//004836 1828 0001                MOVE.B   1(A0),D4
    D4W = ((ui8 *)(A0))[1];
//00483a 220b                     MOVE.L   A3,D1
//00483c 362e fffa                MOVE.W   -6(A6),D3
//004840 48c3                     EXT.L    D3
//004842 d283                     ADD.L    D3,D1
//004844 2041                     MOVE.L   D1,A0
    A0 = A3 + LOCAL_6;
    D3W = ((ui8 *)A0)[0];
//004846 c87c 00ff                AND.W    #255,D4
//00484a 4243                     CLR.W    D3
//00484c 1610                     MOVE.B   (A0),D3
//00484e 9843                     SUB.W    D3,D4
    D4W = sw(D4W - D3W);
//004850 303c 0004                MOVE.W   #4,D0
//004854 3207                     MOVE.W   D7,D1
//004856 e541                     ASL.W    #2,D1
//004858 d041                     ADD.W    D1,D0
    D0W = sw(4*D7W + 4);
//00485a 41ec f048                LEA      -4024(A4),A0
//00485e d0c6                     ADD.W    D6,A0
    A0 = d.Byte4024 + D6W;
//004860 4243                     CLR.W    D3
//004862 1610                     MOVE.B   (A0),D3
    D3W = (UI8)(A0[0]);
//004864 d043                     ADD.W    D3,D0
    D0W = sw(D0W + D3W);
//004866 3e00                     MOVE.W   D0,D7
    D7W = D0W;
//004868 3f00                     MOVE.W   D0,-(A7)
//00486a 4ead 05b8                JSR      1464(A5)    (=0x022c90)
//00486e 548f                     ADDQ.L   #2,A7
    D0W = TAG022c90(D7W);
//004870 4a40                     TST.W    D0
//004872 6660                     BNE      $+98 (=0x0048d4)
    if (D0W == 0)
    {
//004874 3f05                     MOVE.W   D5,-(A7)
//004876 4ead 061e                JSR      1566(A5)    (=0x022a92)
//00487a 548f                     ADDQ.L   #2,A7
      LOCAL_4 = GetGraphicAddress(D5W);
//00487c 2d40 fffc                MOVE.L   D0,-4(A6)
//004880 0c46 0004                CMP.W    #4,D6
//004884 6e08                     BGT      $+10 (=0x00488e)
      if (D6W <= 4)
      {
//004886 41ec efd4                LEA      -4140(A4),A0
        A0 = d.Byte4140;
//00488a 2008                     MOVE.L   A0,D0
//00488c 6006                     BRA      $+8 (=0x004894)
      }
      else
      {
//00488e 41ec efc4                LEA      -4156(A4),A0
        A0 = d.Byte4156;
//004892 2008                     MOVE.L   A0,D0
      };
//004894 2f00                     MOVE.L   D0,-(A7)
//
//004896 4240                     CLR.W    D0
//004898 102b 0005                MOVE.B   5(A3),D0
//00489c 3f00                     MOVE.W   D0,-(A7)
//
//00489e 3004                     MOVE.W   D4,D0
//0048a0 5240                     ADDQ.W   #1,D0
//0048a2 3f00                     MOVE.W   D0,-(A7)
//
//0048a4 4240                     CLR.W    D0
//0048a6 102a 0005                MOVE.B   5(A2),D0
//0048aa 3f00                     MOVE.W   D0,-(A7)
//
//0048ac 4240                     CLR.W    D0
//0048ae 102a 0004                MOVE.B   4(A2),D0
//0048b2 e340                     ASL.W    #1,D0
//0048b4 3f00                     MOVE.W   D0,-(A7)
//
//0048b6 3f07                     MOVE.W   D7,-(A7)
//0048b8 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//0048bc 548f                     ADDQ.L   #2,A7
//0048be 2f00                     MOVE.L   D0,-(A7)
//0048c0 2f2e fffc                MOVE.L   -4(A6),-(A7)
//0048c4 4eba 3bd2                JSR      $+15316 (=0x008498)
//0048c8 defc 0014                ADD.W    #20,A7
      ShrinkBLT(LOCAL_4,
                TAG022d38(D7W),
                2*(UI8)(A2[4]),
                (UI8)(A2[5]),
                D4W+1,
                (UI8)(A3[5]),
                A0);
//0048cc 3f07                     MOVE.W   D7,-(A7)
//0048ce 4ead 060c                JSR      1548(A5)    (=0x022d5e)
//0048d2 548f                     ADDQ.L   #2,A7
      TAG022d5e(D7W);
    };
//0048d4 3f07                     MOVE.W   D7,-(A7)
//0048d6 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//0048da 548f                     ADDQ.L   #2,A7
    A2 = TAG022d38(D7W);
//0048dc 2440                     MOVE.L   D0,A2
//0048de 302e fff8                MOVE.W   -8(A6),D0
//0048e2 6732                     BEQ      $+52 (=0x004916)
    if (LOCAL_8 != 0)
    {
//0048e4 4240                     CLR.W    D0
//0048e6 102b 0005                MOVE.B   5(A3),D0
//0048ea 3f00                     MOVE.W   D0,-(A7)
//0048ec 4240                     CLR.W    D0
//0048ee 102b 0004                MOVE.B   4(A3),D0
//0048f2 3f00                     MOVE.W   D0,-(A7)
//0048f4 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//0048f8 2f0a                     MOVE.L   A2,-(A7)
//0048fa 4eba fa4e                JSR      $-1456 (=0x00434a)
//0048fe defc 000c                ADD.W    #12,A7
    MakeMirror(A2,d.Pointer1848,(UI8)(A3[4]),(UI8)(A3[5]));
//004902 246c f8c8                MOVE.L   -1848(A4),A2
    A2 = d.Pointer1848;
//004906 303c 000f                MOVE.W   #15,D0
//00490a 3204                     MOVE.W   D4,D1
//00490c c27c 000f                AND.W    #15,D1
//004910 9041                     SUB.W    D1,D0
//004912 3800                     MOVE.W   D0,D4
    D4W = (I16)(15 - (D4W&15));
//004914 6018                     BRA      $+26 (=0x00492e)
    }
    else
    {
//004916 0c46 0007                CMP.W    #7,D6
//00491a 6610                     BNE      $+18 (=0x00492c)
      if (D6W == 7)
      {
//00491c 4240                     CLR.W    D0
//00491e 102b 0001                MOVE.B   1(A3),D0
        D0W = (UI8)(A3[1]);
//004922 4243                     CLR.W    D3
//004924 1613                     MOVE.B   (A3),D3
        D3W = (UI8)(A3[0]);
//004926 9043                     SUB.W    D3,D0
        D0W = sw(D0W - D3W);
//004928 9840                     SUB.W    D0,D4
        D4W = sw(D4W - D0W);
//00492a 6002                     BRA      $+4 (=0x00492e)
      }
      else
      {
//00492c 4244                     CLR.W    D4
        D4W = 0;
      };
    };
  };
//00492e 302e fff6                MOVE.W   -10(A6),D0
//004932 6762                     BEQ      $+100 (=0x004996)
  if (LOCAL_10 != 0)
  {
//004934 2d4b fffc                MOVE.L   A3,-4(A6)
    LOCAL_4 = A3;
//004938 41ee ffa4                LEA      -92(A6),A0
//00493c 2648                     MOVE.L   A0,A3
    A3 = LOCAL_92;
//00493e 4245                     CLR.W    D5
    D5W = 0;
    do
    {
//004940 6002                     BRA      $+4 (=0x004944)
      while ((UI8)(*A3) < 128) A3++;
//004942 528b                     ADDQ.L   #1,A3

//004944 4240                     CLR.W    D0
//004946 1013                     MOVE.B   (A3),D0
//004948 b07c 0080                CMP.W    #128,D0
//00494c 6df4                     BLT      $-10 (=0x004942)

//00494e 5245                     ADDQ.W   #1,D5
      D5W++;
//004950 4240                     CLR.W    D0
//004952 101b                     MOVE.B   (A3)+,D0
//004954 b07c 0081                CMP.W    #129,D0
//004958 66e6                     BNE      $-24 (=0x004940)
    } while ((UI8)(*(A3++)) != 129);
//00495a 266e fffc                MOVE.L   -4(A6),A3
    A3 = LOCAL_4;
//00495e 0c45 0004                CMP.W    #4,D5
//004962 6c32                     BGE      $+52 (=0x004996)
    if (D5W < 0)
    {
//004964 3f3c 0008                MOVE.W   #8,-(A7)
//004968 486e ffea                PEA      -22(A6)
//00496c 2f0b                     MOVE.L   A3,-(A7)
//00496e 4ead 0006                JSR      6(A5)    (=0x0009dc)
//004972 defc 000a                ADD.W    #10,A7
      MemMove(A3, (pnt)LOCAL_22, 8);
//004976 41ee ffea                LEA      -22(A6),A0
//00497a 2648                     MOVE.L   A0,A3
      A3 = (pnt)LOCAL_22;
//00497c 41ec f048                LEA      -4024(A4),A0
//004980 d0c6                     ADD.W    D6,A0
      A0 = d.Byte4024;
//004982 4240                     CLR.W    D0
//004984 1010                     MOVE.B   (A0),D0
//004986 c1fc 0003                MULS     #3,D0
      D0L = 3*((UI8)(*A0));
//00498a d045                     ADD.W    D5,D0
      D0W = sw(D0W + D5W);
//00498c 41ec ef8b                LEA      -4213(A4),A0
//004990 d0c0                     ADD.W    D0,A0
      A0 = d.Byte4213 + D0W;
//004992 1750 0003                MOVE.B   (A0),3(A3)
      *(A3+3) = *A0;
    };
  };
//004996 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//00499c 4240                     CLR.W    D0
//00499e 102b 0004                MOVE.B   4(A3),D0
//0049a2 3f00                     MOVE.W   D0,-(A7)
//0049a4 4267                     CLR.W    -(A7)
//0049a6 3f04                     MOVE.W   D4,-(A7)
//0049a8 2f0b                     MOVE.L   A3,-(A7)
//0049aa 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0049ae 2f0a                     MOVE.L   A2,-(A7)
//0049b0 4eba 3f00                JSR      $+16130 (=0x0088b2)
//0049b4 defc 0016                ADD.W    #22,A7
  TAG0088b2(A2,  // Pictureframe appears
            d.pViewportBMP,
            (RectPos *)A3, //0x50,0x8f,0x1d,0x47
            D4W,
            0,
            (UI8)(A3[4]),
            112,
            10);
//0049b8 0c46 000c                CMP.W    #12,D6
//0049bc 664a                     BNE      $+76 (=0x004a08)
  if (D6W == 12)
  {
//0049be 302c d6d6                MOVE.W   -10538(A4),D0
    D0W = d.Word10538;
//0049c2 536c d6d6                SUBQ.W   #1,-10538(A4)
    d.Word10538--;
//0049c6 4a40                     TST.W    D0
//0049c8 673e                     BEQ      $+64 (=0x004a08)
    if (D0W != 0)
    {
//0049ca 2f3c 0070 0001           MOVE.L   #7340033,-(A7)
//0049d0 3f3c 0080                MOVE.W   #128,-(A7)
//0049d4 302c d6d6                MOVE.W   -10538(A4),D0
//0049d8 e640                     ASR.W    #3,D0
//0049da c1fc 001d                MULS     #29,D0
//0049de 3f00                     MOVE.W   D0,-(A7)
//0049e0 302c d6d6                MOVE.W   -10538(A4),D0
//0049e4 c07c 0007                AND.W    #7,D0
//0049e8 eb40                     ASL.W    #5,D0
//0049ea 3f00                     MOVE.W   D0,-(A7)
//0049ec 486c f60e                PEA      -2546(A4)
//0049f0 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//0049f4 3f3c 001a                MOVE.W   #26,-(A7)
//0049f8 4ead 061e                JSR      1566(A5)    (=0x022a92)
//0049fc 548f                     ADDQ.L   #2,A7
//0049fe 2f00                     MOVE.L   D0,-(A7)
//004a00 4eba 3eb0                JSR      $+16050 (=0x0088b2)
//004a04 defc 0016                ADD.W    #22,A7
    TAG0088b2(GetGraphicAddress(26),  // Draws portrait
              d.pViewportBMP,
              (RectPos *)d.Byte2546, //0x60,0x7f,0x23,0x3f
              (d.Word10538&7)<<5,
              29 * BITS3_15(d.Word10538),
              128,
              112,
              1);
    };
  }
//004a08 302e fff4                MOVE.W   -12(A6),D0
//004a0c 6004                     BRA      $+6 (=0x004a12)
  return LOCAL_12;
//004a0e 303c 0000                MOVE.W   #0,D0
//004a12 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//004a16 4e5e                     UNLK     A6
//004a18 4e75                     RTS
}

void TAG004a1a(i16 P1,i16 P2)
{
  dReg D0, D3;
  aReg A0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004a1a 4e56 0000                LINK     A6,#0
//004a1e 48e7 0f30                MOVEM.L  D4567A23,-(A7)
  dReg D1, D4, D5, D6, D7;
  aReg A2,A3;
//004a22 3e2e 0008                MOVE.W   8(A6),D7
//004a26 3c2e 000a                MOVE.W   10(A6),D6
  D7W = P1;
  D6W = P2;
//004a2a 3007                     MOVE.W   D7,D0
//004a2c 6700 00f0                BEQ      $+242 (=0x004b1e)
  if (D7W == 0) return;
//004a30 3807                     MOVE.W   D7,D4
//004a32 c87c 8000                AND.W    #32768,D4
//004a36 670e                     BEQ      $+16 (=0x004a46)
  D4W = (UI16)(D7W & 0x8000);
  if (  (D4W == 0)
      ||((D7W = (I16)(D7W&0x7fff)) !=0)) // Seems silly
  {
//004a38 3007                     MOVE.W   D7,D0
//004a3a c07c 7fff                AND.W    #32767,D0
//004a3e 3e00                     MOVE.W   D0,D7
//004a40 6604                     BNE      $+6 (=0x004a46)
//004a42 6000 00ca                BRA      $+204 (=0x004b0e)
//004a46 5347                     SUBQ.W   #1,D7
    D7W -= 1;
    D5W = sw(D7W << 2);
//004a48 3a07                     MOVE.W   D7,D5
//004a4a e585                     ASL.L    #2,D5
//004a4c 41ec f672                LEA      -2446(A4),A0
//004a50 d0c5                     ADD.W    D5,A0
    A0 = &d.Byte2446 + D5W;
//004a52 3a10                     MOVE.W   (A0),D5
    D5W = wordGear(A0);
//004a54 3206                     MOVE.W   D6,D1
    D1W = D6W;
//004a56 41ec f03e                LEA      -4034(A4),A0
//004a5a d0c1                     ADD.W    D1,A0
    A0 = d.Byte4034 + D1W;
//004a5c 4243                     CLR.W    D3
//004a5e 1610                     MOVE.B   (A0),D3
    D3W = (UI8)(*A0);
//004a60 da43                     ADD.W    D3,D5
    D5W = sw(D5W + D3W);
//004a62 3006                     MOVE.W   D6,D0
//004a64 3207                     MOVE.W   D7,D1
    D0W = D6W;
    D1W = sw(D7W << 2);
//004a66 e581                     ASL.L    #2,D1
//004a68 41ec f674                LEA      -2444(A4),A0
//004a6c d0c1                     ADD.W    D1,A0
//004a6e 3210                     MOVE.W   (A0),D1
    A0 = &d.Byte2446 + D1W + 2;
    D1W = wordGear(A0);
//004a70 c3fc 0036                MULS     #54,D1
    D1L = 54*D1W;
//004a74 41ec ec7a                LEA      -4998(A4),A0
//004a78 d288                     ADD.L    A0,D1
    A0 = d.Byte4998;
    D1L += (i32)A0;
//004a7a c1fc 0006                MULS     #6,D0
    D0L = 6*D0W;
//004a7e 2041                     MOVE.L   D1,A0
//004a80 d0c0                     ADD.W    D0,A0
//004a82 41d0                     LEA      (A0),A0
//004a84 2448                     MOVE.L   A0,A2
    A2 = A0 = (pnt)(D1L + D0W);
//004a86 0c46 0008                CMP.W    #8,D6
//004a8a 672a                     BEQ      $+44 (=0x004ab6)
    if (D6W == 8) goto tag004ab6;
//004a8c 0c46 0005                CMP.W    #5,D6
//004a90 6724                     BEQ      $+38 (=0x004ab6)
    if (D6W == 5) goto tag004ab6;
//004a92 0c46 0002                CMP.W    #2,D6
//004a96 671e                     BEQ      $+32 (=0x004ab6)
    if (D6W == 2) goto tag004ab6;
//004a98 0c47 000f                CMP.W    #15,D7
//004a9c 6644                     BNE      $+70 (=0x004ae2)
    if (D7W != 15) goto tag004ae2;
//004a9e 302c f8b6                MOVE.W   -1866(A4),D0
//004aa2 673e                     BEQ      $+64 (=0x004ae2)
    if (d.UseMirrorImages == 0) goto tag004ae2;
//004aa4 0c46 0007                CMP.W    #7,D6
//004aa8 670c                     BEQ      $+14 (=0x004ab6)
    if (D6W == 7) goto tag004ab6;
//004aaa 0c46 0004                CMP.W    #4,D6
//004aae 6706                     BEQ      $+8 (=0x004ab6)
    if (D6W == 4) goto tag004ab6;
//004ab0 0c46 0001                CMP.W    #1,D6
//004ab4 662c                     BNE      $+46 (=0x004ae2)
    if (D6W == 1) goto tag004ae2;
tag004ab6:
//004ab6 4240                     CLR.W    D0
//004ab8 102a 0005                MOVE.B   5(A2),D0
    D0W = 0;
    D0B = *(A2+5);
//004abc 3f00                     MOVE.W   D0,-(A7)
//004abe 4240                     CLR.W    D0
//004ac0 102a 0004                MOVE.B   4(A2),D0
//004ac4 3f00                     MOVE.W   D0,-(A7)
//004ac6 202c f8c8                MOVE.L   -1848(A4),D0
//004aca 2640                     MOVE.L   D0,A3
//004acc 2f00                     MOVE.L   D0,-(A7)
//004ace 3f05                     MOVE.W   D5,-(A7)
//004ad0 4ead 061e                JSR      1566(A5)    (=0x022a92)
//004ad4 548f                     ADDQ.L   #2,A7
//004ad6 2f00                     MOVE.L   D0,-(A7)
//004ad8 4eba f870                JSR      $-1934 (=0x00434a)
//004adc defc 000c                ADD.W    #12,A7
    MakeMirror(GetGraphicAddress(D5W),
               A3 = d.Pointer1848,      // Temp Viewport BMP
               (UI8)(A2[4]),               // Width
               D0W );                  // height
//004ae0 600a                     BRA      $+12 (=0x004aec)
    goto tag004aec;
tag004ae2:
//004ae2 3f05                     MOVE.W   D5,-(A7)
//004ae4 4ead 061e                JSR      1566(A5)    (=0x022a92)
//004ae8 548f                     ADDQ.L   #2,A7
//004aea 2640                     MOVE.L   D0,A3
    A3 = GetGraphicAddress(D5W);
tag004aec:
//004aec 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//004af2 4240                     CLR.W    D0
//004af4 102a 0004                MOVE.B   4(A2),D0
//004af8 3f00                     MOVE.W   D0,-(A7)
//004afa 4267                     CLR.W    -(A7)
//004afc 4267                     CLR.W    -(A7)
//004afe 2f0a                     MOVE.L   A2,-(A7)
//004b00 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//004b04 2f0b                     MOVE.L   A3,-(A7)
//004b06 4eba 3daa                JSR      $+15788 (=0x0088b2)
//004b0a defc 0016                ADD.W    #22,A7
    TAG0088b2(A3,
              d.pViewportBMP,
              (RectPos *)A2,
              0,
              0,
              (UI8)(A2[4]),
              112,
              10);
  };
//004b0e 3004                     MOVE.W   D4,D0
//004b10 670c                     BEQ      $+14 (=0x004b1e)
  if (D4W != 0)
  {
//004b12 3f06                     MOVE.W   D6,-(A7)
//004b14 3f3c 0010                MOVE.W   #16,-(A7)
//004b18 4eba ff00                JSR      $-254 (=0x004a1a)
//004b1c 588f                     ADDQ.L   #4,A7
    TAG004a1a(16, D6W);
  };
//004b1e 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//004b22 4e5e                     UNLK     A6
//004b24 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
void TAG004b26(i32 P1, i32 P2)
{
  dReg D0, D1, D4, D5, D6, D7;
  aReg A0, A2, A3;
  pnt pnt_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004b26 4e56 fffc                LINK     A6,#-4
//004b2a 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//004b2e 3e2e 0008                MOVE.W   8(A6),D7
//004b32 3c2e 000a                MOVE.W   10(A6),D6
  D7W = sw(P1);
  D6W = sw(P2);
//004b36 3007                     MOVE.W   D7,D0
//004b38 6700 011c                BEQ      $+286 (=0x004c56)
  if (D7W == 0) return;
//004b3c 5347                     SUBQ.W   #1,D7
  D7W--;
//004b3e 3a07                     MOVE.W   D7,D5
//004b40 e585                     ASL.L    #2,D5
//004b42 41ec f62e                LEA      -2514(A4),A0
//004b46 d0c5                     ADD.W    D5,A0
//004b48 3a10                     MOVE.W   (A0),D5
  //D5W = wordGear(d.Byte2514 + 4*D7W);
  D5W = d.Word2514[D7W][0];
//004b4a 3006                     MOVE.W   D6,D0
//004b4c 3207                     MOVE.W   D7,D1
//004b4e e581                     ASL.L    #2,D1
//004b50 41ec f630                LEA      -2512(A4),A0
//004b54 d0c1                     ADD.W    D1,A0
//004b56 3210                     MOVE.W   (A0),D1
//004b58 3801                     MOVE.W   D1,D4
  //D4W = wordGear(d.Byte2512 + 4*D7W);
  D4W = d.Word2514[D7W][1];
//004b5a c3fc 0012                MULS     #18,D1
  D1L = 18 * D4W;
//004b5e 41ec ec32                LEA      -5070(A4),A0
//004b62 d288                     ADD.L    A0,D1
//004b64 c1fc 0006                MULS     #6,D0
//004b68 2041                     MOVE.L   D1,A0
//004b6a d0c0                     ADD.W    D0,A0
//004b6c 41d0                     LEA      (A0),A0
//004b6e 2648                     MOVE.L   A0,A3
  A3 = d.Byte5070 + 18*D4W + 6*D6W;
//004b70 0c46 0002                CMP.W    #2,D6
//004b74 6612                     BNE      $+20 (=0x004b88)
  if (D6W == 2)
  {
//004b76 3f05                     MOVE.W   D5,-(A7)
//004b78 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004b7c 548f                     ADDQ.L   #2,A7
//004b7e 2440                     MOVE.L   D0,A2
    A2 = GetGraphicAddress(D5W);
//004b80 3c3c 0030                MOVE.W   #48,D6
    D6W = 48;
//004b84 6000 00ae                BRA      $+176 (=0x004c34)
  }
  else
  {
    D7W = sw(2*D7W + 68 + D6W);
//004b88 303c 0044                MOVE.W   #68,D0
//004b8c 3207                     MOVE.W   D7,D1
//004b8e c3fc 0002                MULS     #2,D1
//004b92 d041                     ADD.W    D1,D0
//004b94 d046                     ADD.W    D6,D0
//004b96 3e00                     MOVE.W   D0,D7
//004b98 3f00                     MOVE.W   D0,-(A7)
//004b9a 4ead 05b8                JSR      1464(A5)    (=0x022c90)
//004b9e 548f                     ADDQ.L   #2,A7
    D0W = TAG022c90(D7W);
//004ba0 4a40                     TST.W    D0
//004ba2 6678                     BNE      $+122 (=0x004c1c)
    if (D0W == 0)
    {
//004ba4 3004                     MOVE.W   D4,D0
//004ba6 c1fc 0012                MULS     #18,D0
//004baa 41ec ec3e                LEA      -5058(A4),A0
//004bae d0c0                     ADD.W    D0,A0
//004bb0 41d0                     LEA      (A0),A0
//004bb2 2448                     MOVE.L   A0,A2
      ASSERT(D4W < 3);
      A2 = &d.Byte5058[18*D4W];
//004bb4 3f05                     MOVE.W   D5,-(A7)
//004bb6 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004bba 548f                     ADDQ.L   #2,A7
//004bbc 2d40 fffc                MOVE.L   D0,-4(A6)
      pnt_4 = GetGraphicAddress(D5W);
//004bc0 4a46                     TST.W    D6
//004bc2 6608                     BNE      $+10 (=0x004bcc)
      if (D6W == 0)
      {
//004bc4 41ec efb4                LEA      -4172(A4),A0
//004bc8 2008                     MOVE.L   A0,D0
        A0 = d.Byte4172;
//004bca 6006                     BRA      $+8 (=0x004bd2)
      }
      else
      {
//004bcc 41ec efa4                LEA      -4188(A4),A0
//004bd0 2008                     MOVE.L   A0,D0
        A0 = d.Byte4188;
      };
//004bd2 2f00                     MOVE.L   D0,-(A7)
//004bd4 4240                     CLR.W    D0
//004bd6 102b 0005                MOVE.B   5(A3),D0
//004bda 3f00                     MOVE.W   D0,-(A7)
//004bdc 4240                     CLR.W    D0
//004bde 102b 0001                MOVE.B   1(A3),D0
//004be2 4243                     CLR.W    D3
//004be4 1613                     MOVE.B   (A3),D3
//004be6 9043                     SUB.W    D3,D0
//004be8 5240                     ADDQ.W   #1,D0
//004bea 3f00                     MOVE.W   D0,-(A7)
//004bec 4240                     CLR.W    D0
//004bee 102a 0005                MOVE.B   5(A2),D0
//004bf2 3f00                     MOVE.W   D0,-(A7)
//004bf4 4240                     CLR.W    D0
//004bf6 102a 0004                MOVE.B   4(A2),D0
//004bfa e340                     ASL.W    #1,D0
//004bfc 3f00                     MOVE.W   D0,-(A7)
//004bfe 3f07                     MOVE.W   D7,-(A7)
//004c00 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//004c04 548f                     ADDQ.L   #2,A7
//004c06 2f00                     MOVE.L   D0,-(A7)
//004c08 2f2e fffc                MOVE.L   -4(A6),-(A7)
//004c0c 4eba 388a                JSR      $+14476 (=0x008498=ShrinkBLT)
//004c10 defc 0014                ADD.W    #20,A7
      ShrinkBLT(pnt_4,
                TAG022d38(D7W),
                2*(UI8)(A2[4]),
                (UI8)(A2[5]),
                (UI8)(A3[1]) - (UI8)(A3[0]) + 1,
                (UI8)(A3[5]),
                A0);
//004c14 3f07                     MOVE.W   D7,-(A7)
//004c16 4ead 060c                JSR      1548(A5)    (=0x022d5e)
//004c1a 548f                     ADDQ.L   #2,A7
      TAG022d5e(D7W);
    };
//004c1c 3f07                     MOVE.W   D7,-(A7)
//004c1e 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//004c22 548f                     ADDQ.L   #2,A7
//004c24 2440                     MOVE.L   D0,A2
    A2 = TAG022d38(D7W);
//004c26 4a46                     TST.W    D6
//004c28 6606                     BNE      $+8 (=0x004c30)
    if (D6W == 0)
    {
//004c2a 3c3c 0018                MOVE.W   #24,D6
//004c2e 6004                     BRA      $+6 (=0x004c34)
      D6W = 24;
    }
    else
    {
//004c30 3c3c 0020                MOVE.W   #32,D6
      D6W = 32;
    };
  };
//004c34 3f3c 0009                MOVE.W   #9,-(A7)
//004c38 3f06                     MOVE.W   D6,-(A7)
//004c3a 4240                     CLR.W    D0
//004c3c 102b 0004                MOVE.B   4(A3),D0
//004c40 3f00                     MOVE.W   D0,-(A7)
//004c42 4267                     CLR.W    -(A7)
//004c44 4267                     CLR.W    -(A7)
//004c46 2f0b                     MOVE.L   A3,-(A7)
//004c48 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004c4c 2f0a                     MOVE.L   A2,-(A7)
//004c4e 4eba 3c62                JSR      $+15460 (=0x0088b2)
  TAG0088b2(A2,
            d.Pointer1848,
            (RectPos *)A3,
            0,
            0,
            (UI8)(A3[4]),
            D6W,
            9);
//004c52 defc 0016                ADD.W    #22,A7
//004c56 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//004c5a 4e5e                     UNLK     A6
//004c5c 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG004c5e
void DrawDoorSwitch(i16 P1, i16 P2)
{
  dReg D0, D1, D4, D5, D6, D7;
  aReg A0, A2, A3;
  pnt pnt_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004c5e 4e56 fffc                LINK     A6,#-4
//004c62 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//004c66 3e2e 0008                MOVE.W   8(A6),D7
  D7W = P1;
  D6W = P2;
//004c6a 3c2e 000a                MOVE.W   10(A6),D6
//004c6e 3007                     MOVE.W   D7,D0
//004c70 6700 0124                BEQ      $+294 (=0x004d96)
  if (D7W == 0) return;
//004c74 5347                     SUBQ.W   #1,D7
  D7W -= 1;
//004c76 3a07                     MOVE.W   D7,D5
//004c78 da7c 013b                ADD.W    #315,D5
  D5W = sw(D7W + 315);
//004c7c 3006                     MOVE.W   D6,D0
//004c7e 41ec efe4                LEA      -4124(A4),A0
//004c82 d0c7                     ADD.W    D7,A0
//004c84 4241                     CLR.W    D1
//004c86 1210                     MOVE.B   (A0),D1
//004c88 3801                     MOVE.W   D1,D4
  D4W = (UI8)(d.Byte4124[D7W]);
//004c8a c3fc 0018                MULS     #24,D1
//004c8e 41ec ec1a                LEA      -5094(A4),A0
//004c92 d288                     ADD.L    A0,D1
//004c94 c1fc 0006                MULS     #6,D0
//004c98 2041                     MOVE.L   D1,A0
//004c9a d0c0                     ADD.W    D0,A0
//004c9c 41d0                     LEA      (A0),A0
//004c9e 2648                     MOVE.L   A0,A3
  A3 = d.Byte5094 + 24*D4W + 6*D6W;
//004ca0 0c46 0003                CMP.W    #3,D6
//004ca4 6620                     BNE      $+34 (=0x004cc6)
  if (D6W == 3)
  {
//004ca6 3f05                     MOVE.W   D5,-(A7)
//004ca8 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004cac 548f                     ADDQ.L   #2,A7
//004cae 2440                     MOVE.L   D0,A2
    A2 = GetGraphicAddress(D5W);
//004cb0 3f3c 0004                MOVE.W   #4,-(A7)
//004cb4 486c d6d0                PEA      -10544(A4)
//004cb8 2f0b                     MOVE.L   A3,-(A7)
//004cba 4ead 0006                JSR      6(A5)    (=0x0009dc=MemMove)
//004cbe defc 000a                ADD.W    #10,A7
    MemMove(A3, (pnt)d.ViewportObjectButtons[5], 4);
//004cc2 6000 00b0                BRA      $+178 (=0x004d74)
  }
  else
  {
//004cc6 303c 0066                MOVE.W   #102,D0
//004cca 3207                     MOVE.W   D7,D1
//004ccc c3fc 0002                MULS     #2,D1
//004cd0 d041                     ADD.W    D1,D0
    D0W = sw(2*D7W + 102);
//004cd2 3206                     MOVE.W   D6,D1
//004cd4 6606                     BNE      $+8 (=0x004cdc)
    if (D6W == 0)
    {
//004cd6 323c 0000                MOVE.W   #0,D1
//004cda 6004                     BRA      $+6 (=0x004ce0)
      D1W = 0;
    }
    else
    {
//004cdc 3206                     MOVE.W   D6,D1
//004cde 5341                     SUBQ.W   #1,D1
      D1W = sw(D6W - 1);
    };
//004ce0 d041                     ADD.W    D1,D0
//004ce2 3e00                     MOVE.W   D0,D7
    D7W = sw(D0W + D1W);
//004ce4 3f00                     MOVE.W   D0,-(A7)
//004ce6 4ead 05b8                JSR      1464(A5)    (=0x022c90)
//004cea 548f                     ADDQ.L   #2,A7
    D0W = TAG022c90(D7W);
//004cec 4a40                     TST.W    D0
//004cee 667a                     BNE      $+124 (=0x004d6a)
    if (D0W == 0)
    {
//004cf0 3004                     MOVE.W   D4,D0
//004cf2 c1fc 0018                MULS     #24,D0
//004cf6 41ec ec2c                LEA      -5076(A4),A0
//004cfa d0c0                     ADD.W    D0,A0
//004cfc 41d0                     LEA      (A0),A0
//004cfe 2448                     MOVE.L   A0,A2
      A2 = d.Byte5076 + 24*D4W;
//004d00 3f05                     MOVE.W   D5,-(A7)
//004d02 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004d06 548f                     ADDQ.L   #2,A7
//004d08 2d40 fffc                MOVE.L   D0,-4(A6)
      pnt_4 = GetGraphicAddress(D5W);
//004d0c 0c46 0002                CMP.W    #2,D6
//004d10 6608                     BNE      $+10 (=0x004d1a)
      if (D6W == 2)
      {
//004d12 41ec efc4                LEA      -4156(A4),A0
//004d16 2008                     MOVE.L   A0,D0
        A0 = d.Byte4156;
//004d18 6006                     BRA      $+8 (=0x004d20)
      }
      else
      {
//004d1a 41ec efd4                LEA      -4140(A4),A0
        A0 = d.Byte4140;
//004d1e 2008                     MOVE.L   A0,D0
      };
//004d20 2f00                     MOVE.L   D0,-(A7)
//004d22 4240                     CLR.W    D0
//004d24 102b 0005                MOVE.B   5(A3),D0
//004d28 3f00                     MOVE.W   D0,-(A7)
//004d2a 4240                     CLR.W    D0
//004d2c 102b 0001                MOVE.B   1(A3),D0
//004d30 4243                     CLR.W    D3
//004d32 1613                     MOVE.B   (A3),D3
//004d34 9043                     SUB.W    D3,D0
//004d36 5240                     ADDQ.W   #1,D0
//004d38 3f00                     MOVE.W   D0,-(A7)
//004d3a 4240                     CLR.W    D0
//004d3c 102a 0005                MOVE.B   5(A2),D0
//004d40 3f00                     MOVE.W   D0,-(A7)
//004d42 4240                     CLR.W    D0
//004d44 102a 0004                MOVE.B   4(A2),D0
//004d48 e340                     ASL.W    #1,D0
//004d4a 3f00                     MOVE.W   D0,-(A7)
//004d4c 3f07                     MOVE.W   D7,-(A7)
//004d4e 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//004d52 548f                     ADDQ.L   #2,A7
//004d54 2f00                     MOVE.L   D0,-(A7)
//004d56 2f2e fffc                MOVE.L   -4(A6),-(A7)
//004d5a 4eba 373c                JSR      $+14142 (=0x008498=ShrinkBLT)
//004d5e defc 0014                ADD.W    #20,A7
      ShrinkBLT(pnt_4,
                TAG022d38(D7W),
                2*(UI8)(A2[4]),
                (UI8)A2[5],
                (UI8)A3[1]-(UI8)A3[0]+1,
                (UI8)A3[5],
                A0);
//004d62 3f07                     MOVE.W   D7,-(A7)
//004d64 4ead 060c                JSR      1548(A5)    (=0x022d5e)
//004d68 548f                     ADDQ.L   #2,A7
      TAG022d5e(D7W);
    };
//004d6a 3f07                     MOVE.W   D7,-(A7)
//004d6c 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//004d70 548f                     ADDQ.L   #2,A7
//004d72 2440                     MOVE.L   D0,A2
    A2 = TAG022d38(D7W);
  };
//004d74 2f3c 0070 000a           MOVE.L   #7340042,-(A7)
//004d7a 4240                     CLR.W    D0
//004d7c 102b 0004                MOVE.B   4(A3),D0
//004d80 3f00                     MOVE.W   D0,-(A7)
//004d82 4267                     CLR.W    -(A7)
//004d84 4267                     CLR.W    -(A7)
//004d86 2f0b                     MOVE.L   A3,-(A7)
//004d88 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//004d8c 2f0a                     MOVE.L   A2,-(A7)
//004d8e 4eba 3b22                JSR      $+15140 (=0x0088b2)
//004d92 defc 0016                ADD.W    #22,A7
  TAG0088b2(A2,
            d.pViewportBMP,
            (RectPos *)A3,
            0,
            0,
            (UI8)(A3[4]),
            112,
            10);
//004d96 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//004d9a 4e5e                     UNLK     A6
//004d9c 4e75                     RTS
}

//*********************************************************
// We got here when we saw a smashed door???
//*********************************************************
//   TAG004d9e
void DrawDoor(i16 index, //8
               i16 P2,    //10
               i16 *P3,   //12
               i16 P4,    //16
               i16 P5,    //18
               pnt P6)    //20
{
  dReg D0, D6, D7;
  aReg A3;
  DB0  *DB0A2;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004d9e 4e56 0000                LINK     A6,#0
//004da2 48e7 0330                MOVEM.L  D67A23,-(A7)
//004da6 3e2e 000a                MOVE.W   10(A6),D7
  ASSERT(index < 0x7f);
  D7W = P2;
//004daa 266e 0014                MOVE.L   20(A6),A3
  A3 = P6;
//004dae 4a47                     TST.W    D7
//004db0 6700 014a                BEQ      $+332 (=0x004efc)
  if (D7W == 0) return;
//004db4 202c d6e0                MOVE.L   -10528(A4),D0
//004db8 322e 0008                MOVE.W   8(A6),D1
//004dbc c2bc 0000 ffff           AND.L    #65535,D1
//004dc2 e581                     ASL.L    #2,D1
//004dc4 d081                     ADD.L    D1,D0
//004dc6 2440                     MOVE.L   D0,A2
  DB0A2 = GetRecordAddressDB0(RN(0,0,index));
  D6W = DB0A2->doorType();
//004dc8 3f2e 0010                MOVE.W   16(A6),-(A7)
//004dcc 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004dd0 302a 0002                MOVE.W   2(A2),D0
//004dd4 c07c 0001                AND.W    #1,D0
//004dd8 3c00                     MOVE.W   D0,D6
//004dda e380                     ASL.L    #1,D0
//004ddc 206e 000c                MOVE.L   12(A6),A0
//004de0 d0c0                     ADD.W    D0,A0
//004de2 3f10                     MOVE.W   (A0),-(A7)
//004de4 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004de8 548f                     ADDQ.L   #2,A7
//
//004dea 2f00                     MOVE.L   D0,-(A7)
//004dec 4ead 0006                JSR      6(A5)    (=0x0009dc=MemMove)
//004df0 defc 000a                ADD.W    #10,A7
  MemMove(GetGraphicAddress(P3[D6W]),
          d.Pointer1848, //temporary bitmap
          P4);
//004df4 3f2e 0012                MOVE.W   18(A6),-(A7)
//004df8 302a 0002                MOVE.W   2(A2),D0
//004dfc c07c 001e                AND.W    #30,D0
//004e00 e248                     LSR.W    #1,D0
//004e02 3f00                     MOVE.W   D0,-(A7)
//004e04 4eba fd20                JSR      $-734 (=0x004b26)
//004e08 588f                     ADDQ.L   #4,A7
  TAG004b26(DB0A2->bits1_4(), P5);
//004e0a 3006                     MOVE.W   D6,D0
//004e0c e380                     ASL.L    #1,D0
//004e0e 41ec d73e                LEA      -10434(A4),A0
//004e12 d0c0                     ADD.W    D0,A0
//004e14 4240                     CLR.W    D0
//004e16 1010                     MOVE.B   (A0),D0
  D0W = d.Word10434[D6W];
//004e18 c07c 0004                AND.W    #4,D0
//004e1c 6744                     BEQ      $+70 (=0x004e62)
  if (D0W & 0x400) // Test most significant byte
  {
//004e1e 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//004e22 4a40                     TST.W    D0
//004e24 671a                     BEQ      $+28 (=0x004e40)
    if (STRandomBool())
    {
//004e26 4240                     CLR.W    D0
//004e28 102b 0005                MOVE.B   5(A3),D0
//004e2c 3f00                     MOVE.W   D0,-(A7)
//004e2e 4240                     CLR.W    D0
//004e30 102b 0004                MOVE.B   4(A3),D0
//004e34 3f00                     MOVE.W   D0,-(A7)
//004e36 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004e3a 4eba 3806                JSR      $+14344 (=0x008642=Mirror)
//004e3e 508f                     ADDQ.L   #0,A7
      Mirror(d.Pointer1848, (UI8)(A3[4]), (UI8)(A3[5]));
    };
//004e40 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//004e44 4a40                     TST.W    D0
//004e46 671a                     BEQ      $+28 (=0x004e62)
    if (STRandomBool())
    {
//004e48 4240                     CLR.W    D0
//004e4a 102b 0005                MOVE.B   5(A3),D0
//004e4e 3f00                     MOVE.W   D0,-(A7)
//004e50 4240                     CLR.W    D0
//004e52 102b 0004                MOVE.B   4(A3),D0
//004e56 3f00                     MOVE.W   D0,-(A7)
//004e58 2f2c f8c8                MOVE.L   -1848(A4),-(A7)
//004e5c 4eba 39e2                JSR      $+14820 (=0x008840)
//004e60 508f                     ADDQ.L   #0,A7
      TAG008840(d.Pointer1848, (UI8)(A3[4]), (UI8)(A3[5]));
    };
  };
//004e62 200b                     MOVE.L   A3,D0
//004e64 41ec f106                LEA      -3834(A4),A0
//004e68 b088                     CMP.L    A0,D0
//004e6a 6612                     BNE      $+20 (=0x004e7e)
  if (A3 == d.Byte3834)
  {
//004e6c 102c cc20                MOVE.B   -13280(A4),D0
//004e70 670c                     BEQ      $+14 (=0x004e7e)
    if (d.SeeThruWalls != 0)
    {
//004e72 2f3c 0011 0002           MOVE.L   #1114114,-(A7)
//004e78 4eba fcac                JSR      $-850 (=0x004b26)
//004e7c 588f                     ADDQ.L   #4,A7
      TAG004b26(17, 2);
    };
  };
//004e7e 0c47 0004                CMP.W    #4,D7
//004e82 660c                     BNE      $+14 (=0x004e90)
  if (D7W == 4)
  {
//004e84 41d3                     LEA      (A3),A0
//004e86 2f08                     MOVE.L   A0,-(A7)
//004e88 4eba f580                JSR      $-2686 (=0x00440a)
//004e8c 588f                     ADDQ.L   #4,A7
    TAG00440a(A3);
//004e8e 606c                     BRA      $+110 (=0x004efc)
  }
  else
  {
//004e90 0c47 0005                CMP.W    #5,D7
//004e94 661a                     BNE      $+28 (=0x004eb0)
    if (D7W == 5)
    {
//004e96 3f2e 0012                MOVE.W   18(A6),-(A7)
//004e9a 3f3c 0010                MOVE.W   #16,-(A7)
//004e9e 4eba fc86                JSR      $-888 (=0x004b26)
//004ea2 588f                     ADDQ.L   #4,A7
      TAG004b26(16, P5);
//004ea4 41d3                     LEA      (A3),A0
//004ea6 2f08                     MOVE.L   A0,-(A7)
//004ea8 4eba f560                JSR      $-2718 (=0x00440a)
//004eac 588f                     ADDQ.L   #4,A7
      TAG00440a(A3);
//004eae 604c                     BRA      $+78 (=0x004efc)
    }
    else
    {
//004eb0 5347                     SUBQ.W   #1,D7
      D7W--;
//004eb2 302a 0002                MOVE.W   2(A2),D0
//004eb6 c07c 0020                AND.W    #32,D0
//004eba ea48                     LSR.W    #5,D0
//004ebc 6716                     BEQ      $+24 (=0x004ed4)
      if (DB0A2->mode())
      {
//004ebe 3007                     MOVE.W   D7,D0
//004ec0 e780                     ASL.L    #3,D0
//004ec2 41eb 0008                LEA      8(A3),A0
//004ec6 d0c0                     ADD.W    D0,A0
//004ec8 41d0                     LEA      (A0),A0
//004eca 2f08                     MOVE.L   A0,-(A7)
//004ecc 4eba f53c                JSR      $-2754 (=0x00440a)
//004ed0 588f                     ADDQ.L   #4,A7
        TAG00440a(A3+8+8*D7W);
//004ed2 6028                     BRA      $+42 (=0x004efc)
      }
      else
      {
//004ed4 3007                     MOVE.W   D7,D0
//004ed6 e780                     ASL.L    #3,D0
//004ed8 41eb 0020                LEA      32(A3),A0
//004edc d0c0                     ADD.W    D0,A0
//004ede 41d0                     LEA      (A0),A0
//004ee0 2f08                     MOVE.L   A0,-(A7)
//004ee2 4eba f526                JSR      $-2776 (=0x00440a)
//004ee6 588f                     ADDQ.L   #4,A7
        TAG00440a(A3+32+8*D7W);
//004ee8 3007                     MOVE.W   D7,D0
//004eea e780                     ASL.L    #3,D0
//004eec 41eb 0038                LEA      56(A3),A0
//004ef0 d0c0                     ADD.W    D0,A0
//004ef2 41d0                     LEA      (A0),A0
//004ef4 2f08                     MOVE.L   A0,-(A7)
//004ef6 4eba f512                JSR      $-2796 (=0x00440a)
//004efa 588f                     ADDQ.L   #4,A7
        TAG00440a(A3+56+8*D7W);
      };
    };
  };
//004efc 4cdf 0cc0                MOVEM.L  (A7)+,D67A23
//004f00 4e5e                     UNLK     A6
//004f02 4e75                     RTS
}
//*********************************************************
//
//*********************************************************
//   TAG004f04
void CheckCeilingPit(i16 P1,RectPos *rect,i32 mapX,i32 mapY,i16 P5)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004f04 4e56 0000                LINK     A6,#0
//004f08 3f07                     MOVE.W   D7,-(A7)
  dReg D0, D7;
//  aReg A0;
  CELLFLAG **ppdA0;
  ASSERT(verifyRectPos(rect));
  //CELLFLAG *pdA0;
  //i32 D7;
//004f0a 486e 0010                PEA      16(A6)
//004f0e 486e 000e                PEA      14(A6)
//004f12 3f3c ffff                MOVE.W   #65535,-(A7)
//004f16 3f2c d746                MOVE.W   -10426(A4),-(A7)
//004f1a 4ead 031e                JSR      798(A5)    (=0x009800)
//004f1e defc 000c                ADD.W    #12,A7
  D7W = IncrementLevel(d.LoadedLevel,-1,&mapX,&mapY);
//004f22 3e00                     MOVE.W   D0,D7
//004f24 4a40                     TST.W    D0
//004f26 6d54                     BLT      $+86 (=0x004f7c)
  if (D7W >= 0)
  {
//004f28 302e 000e                MOVE.W   14(A6),D0
    D0W = sw(mapX);
//004f2c 3207                     MOVE.W   D7,D1
//004f2e e581                     ASL.L    #2,D1
    //D1W = D7W * 4;
//004f30 206c d72e                MOVE.L   -10450(A4),A0
//004f34 d0c1                     ADD.W    D1,A0
    ppdA0 = d.pppdPointer10450[D7W];

//004f36 e580                     ASL.L    #2,D0
    //D0W *= 4;
//004f38 2050                     MOVE.L   (A0),A0
    D0W = ppdA0[mapX][mapY];
//004f3a d0c0                     ADD.W    D0,A0
//004f3c 2050                     MOVE.L   (A0),A0
    //A0 = pntGear(A0) + mapY;
//004f3e d0ee 0010                ADD.W    16(A6),A0
//004f42 4240                     CLR.W    D0
//004f44 1010                     MOVE.B   (A0),D0
    //D0W = (UI8)(*A0);
//004f46 3e00                     MOVE.W   D0,D7
  D7W = D0W;
//004f48 ea40                     ASR.W    #5,D0
  D0W >>= 5;
  D0W &= 0x7ff;
//004f4a 0c40 0002                CMP.W    #2,D0
//004f4e 662c                     BNE      $+46 (=0x004f7c)
    if (D0W == roomPIT)
    {
//004f50 3007                     MOVE.W   D7,D0
//004f52 c07c 0008                AND.W    #8,D0
//004f56 6724                     BEQ      $+38 (=0x004f7c)
      if ((D7W & 8) != 0)
      {
//004f58 302e 0012                MOVE.W   18(A6),D0
//004f5c 6710                     BEQ      $+18 (=0x004f6e)
        if (P5 != 0)
        {
//004f5e 2f2e 000a                MOVE.L   10(A6),-(A7)
//004f62 3f2e 0008                MOVE.W   8(A6),-(A7)
//004f66 4eba f596                JSR      $-2664 (=0x0044fe)
//004f6a 5c8f                     ADDQ.L   #6,A7
          TAG0044fe(P1, rect);
//004f6c 600e                     BRA      $+16 (=0x004f7c)
        }
        else
        {
//004f6e 2f2e 000a                MOVE.L   10(A6),-(A7)
//004f72 3f2e 0008                MOVE.W   8(A6),-(A7)
//004f76 4eba f538                JSR      $-2758 (=0x0044b0)
//004f7a 5c8f                     ADDQ.L   #6,A7
          TAG0044b0(P1, rect);
        };
      };
    };
  };
//004f7c 3e1f                     MOVE.W   (A7)+,D7
//004f7e 4e5e                     UNLK     A6
//004f80 4e75                     RTS
}

void TAG004f82(pnt P1,RectPos *rectpos)
{ // Draw teleporter???
  dReg D0;
  aReg A2;
  ui8 *uA3;
  pnt pnt_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//004f82 4e56 fffc                LINK     A6,#-4
//004f86 48e7 0030                MOVEM.L  A23,-(A7)
//004f8a 266e 0008                MOVE.L   8(A6),A3
  uA3 = (ui8 *)P1;
//004f8e 4240                     CLR.W    D0
//004f90 102b 0003                MOVE.B   3(A3),D0
//004f94 b07c 00ff                CMP.W    #255,D0
//004f98 6608                     BNE      $+10 (=0x004fa2)
  if (uA3[3] == 0xff)
  {
//004f9a 247c 0000 0000           MOVE.L   #0,A2
//004fa0 6060                     BRA      $+98 (=0x005002)
    A2 = NULL;
  }
  else
  {
//004fa2 102b 0004                MOVE.B   4(A3),D0
//004fa6 4241                     CLR.W    D1
//004fa8 122b 0005                MOVE.B   5(A3),D1
//004fac c07c 00ff                AND.W    #255,D0
//004fb0 c1c1                     MULU     D1,D0
    D0W = sw(uA3[4] * uA3[5]);
//004fb2 3f00                     MOVE.W   D0,-(A7)
//004fb4 202c f8c8                MOVE.L   -1848(A4),D0
//004fb8 2440                     MOVE.L   D0,A2
//004fba 2f00                     MOVE.L   D0,-(A7)
//004fbc 303c 0045                MOVE.W   #69,D0
//004fc0 4241                     CLR.W    D1
//004fc2 122b 0003                MOVE.B   3(A3),D1
//004fc6 c27c 007f                AND.W    #127,D1
//004fca d041                     ADD.W    D1,D0
//004fcc 3f00                     MOVE.W   D0,-(A7)
//004fce 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//004fd2 548f                     ADDQ.L   #2,A7
//004fd4 2f00                     MOVE.L   D0,-(A7)
//004fd6 4ead 0006                JSR      6(A5)    (=0x0009dc)
    MemMove(GetGraphicAddress((uA3[3] & 0x7f) + 69),
            A2 = d.Pointer1848,
            D0W);
//004fda defc 000a                ADD.W    #10,A7
//004fde 4240                     CLR.W    D0
//004fe0 102b 0003                MOVE.B   3(A3),D0
//004fe4 c07c 0080                AND.W    #128,D0
//004fe8 6718                     BEQ      $+26 (=0x005002)
    if (uA3[3] & 0x80)
    {
//004fea 4240                     CLR.W    D0
//004fec 102b 0005                MOVE.B   5(A3),D0
//004ff0 3f00                     MOVE.W   D0,-(A7)
//004ff2 4240                     CLR.W    D0
//004ff4 102b 0004                MOVE.B   4(A3),D0
//004ff8 3f00                     MOVE.W   D0,-(A7)
//004ffa 2f0a                     MOVE.L   A2,-(A7)
//004ffc 4eba 3644                JSR      $+13894 (=0x008642)
//005000 508f                     ADDQ.L   #0,A7
      Mirror(A2, uA3[4], uA3[5]);
    };
  };
//005002 4267                     CLR.W    -(A7)
//005004 4ead 05b8                JSR      1464(A5)    (=0x022c90)
  TAG022c90(0);
//005008 548f                     ADDQ.L   #2,A7
//00500a 303c 0049                MOVE.W   #73,D0
//00500e 4243                     CLR.W    D3
//005010 1613                     MOVE.B   (A3),D3
//005012 d043                     ADD.W    D3,D0
//005014 3f00                     MOVE.W   D0,-(A7)
//005016 4ead 061e                JSR      1566(A5)    (=0x022a92=GetGraphicAddress)
//00501a 548f                     ADDQ.L   #2,A7
//00501c 2d40 fffc                MOVE.L   D0,-4(A6)
  pnt_4 = GetGraphicAddress(uA3[0] + 73);
//005020 4267                     CLR.W    -(A7)
//005022 4240                     CLR.W    D0
//005024 102b 0006                MOVE.B   6(A3),D0
//005028 3f00                     MOVE.W   D0,-(A7)
//00502a 4240                     CLR.W    D0
//00502c 102b 0002                MOVE.B   2(A3),D0
//005030 3f00                     MOVE.W   D0,-(A7)
//005032 3f3c 0070                MOVE.W   #112,-(A7)
//005036 4ead 0090                JSR      144(A5)    (=0x001030=Random)
//00503a c07c 001f                AND.W    #31,D0
//00503e 3f00                     MOVE.W   D0,-(A7)
//005040 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//005044 4243                     CLR.W    D3
//005046 162b 0001                MOVE.B   1(A3),D3
//00504a d043                     ADD.W    D3,D0
//00504c 3f00                     MOVE.W   D0,-(A7)
//00504e 2f2e 000c                MOVE.L   12(A6),-(A7)
//005052 4267                     CLR.W    -(A7)
//005054 4ead 05f4                JSR      1524(A5)    (=0x022d38)
//005058 548f                     ADDQ.L   #2,A7
//00505a 2f00                     MOVE.L   D0,-(A7)
//00505c 2f0a                     MOVE.L   A2,-(A7)
//00505e 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//005062 2f2e fffc                MOVE.L   -4(A6),-(A7)
//005066 4eba 3c30                JSR      $+15410 (=0x008c98)
//00506a defc 0020                ADD.W    #32,A7
  TAG008c98(pnt_4,
            d.pViewportBMP,
            A2,
            TAG022d38(0),
            rectpos,
            sw(uA3[1] + STRandomBool()),
            sw(STRandom(32)),
            112,
            uA3[2],
            uA3[6],
            0);
//00506e 4267                     CLR.W    -(A7)
//005070 4ead 060c                JSR      1548(A5)    (=0x022d5e)
//005074 548f                     ADDQ.L   #2,A7
  TAG022d5e(0);
//005076 3f3c 8000                MOVE.W   #32768,-(A7)
//00507a 4ead 05b2                JSR      1458(A5)    (=0x021f5e)
//00507e 548f                     ADDQ.L   #2,A7
  DeleteGraphic(0x8000);
//005080 4cdf 0c00                MOVEM.L  (A7)+,A23
//005084 4e5e                     UNLK     A6
//005086 4e75                     RTS
}




ui8 data008734[256] = { //166 bytes
//008734
    0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
    0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
//008744
    0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
    0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
//008754
    0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
    0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
//008764
    0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
    0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
//008774
    0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
    0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
//008784
    0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
    0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
//008794
    0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
    0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
//0087a4
    0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
    0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
//0087b4
    0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
    0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
//0087c4
    0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
    0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
//0087d4
    0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
    0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
//0087e4
    0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
    0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
//0087f4
    0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
    0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
//008804
    0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
    0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
//008814
    0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
    0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
//008824
    0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
    0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff,
};

void Mirror(pnt bitmap, i32 width, i32 height) //TAG008642
{ // width is in bytes
  dReg D0, D1, D2, D3;
  aReg A1, A2, A3, A4;
  A2 = (pnt)data008734;
  D0L = 0;
  D1L = 0;
  D3W = sw(width);
  width >>= 3;
  A1 = bitmap + D3W - 6;
  A3 = bitmap;
  A4 = A1;
tag008670:
  D2W = sw(width);
tag008672:
  D2W -= 2;
  if (D2W < 0) goto tag0086e6;
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  A1 += 4;
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  A1 += 4;
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  A1 += 4;
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  D0B     = *bitmap;
  D1B     = *(--A1);
  *A1     = *(A2 + D0L);
  *(bitmap++) = *(A2 + D1L);
  if (D2W == 0) goto tag008722;
  A1 -= 12;
  goto tag008672;
tag0086e6:
  D0B  = *bitmap;
  D1B  = *(--A1);
  *A1 = *(A2 + D0L);
  *bitmap = *(A2 + D1L);
  bitmap += 2;
  A1 += 2;;
  D0B  = *bitmap;
  D1B  = *A1;
  *A1 = *(A2 + D0L);
  *bitmap = *(A2 + D1L);
  bitmap += 2;
  A1 += 2;;
  D0B = *bitmap;
  D1B = *A1;
  *A1 = *(A2 + D0L);
  *bitmap = *(A2 + D1L);
  bitmap += 2;
  A1 += 2;;
  D0B = *bitmap;
  D1B = *A1;
  *A1 = *(A2 + D0L);
  *bitmap = *(A2 + D1L);
tag008722:
  height--;
  if (height == 0) goto tag008834;
  A3 += D3W;
  bitmap = A3;
  A4 += D3W;
  A1 = A4;
  goto tag008670;
tag008834:
  return;
}

// TAG008e1c
void Flood(pnt dst,i16 color,i16 num)
{ // number of 16-bit groups
  dReg D0, D1;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//008e1c 4e56 0000                LINK     A6,#0
//008e20 48e7 0310                MOVEM.L  D67A3,-(A7)
//008e24 266e 0008                MOVE.L   8(A6),A3
//008e28 3e2e 000c                MOVE.W   12(A6),D7
//008e2c 3c2e 000e                MOVE.W   14(A6),D6
//008e30 7000                     MOVEQ.L  #0,D0
//008e32 7200                     MOVEQ.L  #0,D1
  D0L = D1L = 0;
//008e34 e24f                     LSR.W    #1,D7
//008e36 6404                     BCC      $+6 (=0x008e3c)
  if (color & 1) D0W = -1;
//008e38 4640                     NOT.W    D0
//008e3a 4840                     SWAP     D0
//008e3c e24f                     LSR.W    #1,D7
//008e3e 6402                     BCC      $+4 (=0x008e42)
  if (color & 2) D0H1 = -1;
//008e40 4640                     NOT.W    D0
//008e42 e24f                     LSR.W    #1,D7
//008e44 6404                     BCC      $+6 (=0x008e4a)
  if (color & 4) D1W = -1;
//008e46 4641                     NOT.W    D1
//008e48 4841                     SWAP     D1
//008e4a e24f                     LSR.W    #1,D7
//008e4c 6402                     BCC      $+4 (=0x008e50)
  if (color & 8) D1H1 = -1;
//008e4e 4641                     NOT.W    D1
  do
  {
//008e50 26c0                     MOVE.L   D0,(A3)+
//008e52 26c1                     MOVE.L   D1,(A3)+
    longGear(dst) = D0L;
    longGear(dst+4) = D1L;
    dst += 8;
//008e54 5346                     SUBQ.W   #1,D6
    num--;
//008e56 66f8                     BNE      $-6 (=0x008e50)
  } while (num != 0);
//008e58 4cdf 08c0                MOVEM.L  (A7)+,D67A3
//008e5c 4e5e                     UNLK     A6
//008e5e 4e75                     RTS
};

/*
void TAG00907e(void (*  func)(i32,i32,i32), //8
               pnt (*  getGraphic)(i32),    //12
               i16    P3  ,                 //16
               pnt    P4  ,                 //18
               i16*   P5  ,                 //22
               i16**  P6  ,                 //26 Patch area pointers
               i16    P7  )                 //30 Number of patch areas
{
  // We appear to modify the front three words of
  // the function func to call a little code
  // patch that gets a different graphic to display.
  // After it executes, the patch puts the function
  // back as it was.  What sense does this make?
  // We are called upon to do this at strange times, IMHO.
//
  dReg D0;
  aReg A0;
  i16 *pwA2;
//
  HopefullyNotNeeded();
//
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00907e 4e56 0000                LINK     A6,#0
//009082 48e7 0030                MOVEM.L  A23,-(A7)
//009086 266e 0008                MOVE.L   8(A6),A3
  do
  { // Look for empty entry at random
//00908a 4ead 0090                JSR      144(A5)    (=0x001030=Random)
//00908e c0bc 0000 ffff           AND.L    #65535,D0
//009094 80ee 001e                DIVU     30(A6),D0
//009098 4840                     SWAP     D0
//00909a e580                     ASL.L    #2,D0
    D0L = Random() % P7;
//00909c 206e 001a                MOVE.L   26(A6),A0
//0090a0 d0c0                     ADD.W    D0,A0
    //A0 = P6 + 4 * D0L;
//0090a2 2450                     MOVE.L   (A0),A2
    pwA2 = P6[D0L];
//0090a4 302a 003c                MOVE.W   60(A2),D0
//0090a8 302a 003c                MOVE.W   60(A2),D0
    D0W = pwA2[30];
//0090ac 66dc                     BNE      $-34 (=0x00908a)
  } while (D0W != 0);
//0090ae 357c 0001 003c           MOVE.W   #1,60(A2)
  // pwA2 will continue to point to the first word of the
  // patch area selected.
  pwA2[30] = 1;
//0090b4 3f3c 002a                MOVE.W   #42,-(A7)
//0090b8 2f0a                     MOVE.L   A2,-(A7)
//0090ba 487a 0054                PEA      $+86 (=0x009110)
//0090be 4ead 0006                JSR      6(A5)    (=0x0009dc=MemMove)
//0090c2 defc 000a                ADD.W    #10,A7
  MemMove(TAG009110,pwA2,42); // 21 words.  Up to the
              // trailing jump without the address
//0090c6 0c53 4ef9                CMP.W    #20217,(A3)
//0090ca 6606                     BNE      $+8 (=0x0090d2)    // is it a JMP ?
//0090cc 266b 0002                MOVE.L   2(A3),A3
//0090d0 60f4                     BRA      $-10 (=0x0090c6)
  // A3 points to first byte of function to be patched.
//0090d2 3f3c 0006                MOVE.W   #6,-(A7)
//0090d6 41ea 003e                LEA      62(A2),A0
//0090da 2f08                     MOVE.L   A0,-(A7)
//0090dc 2f0b                     MOVE.L   A3,-(A7)
//0090de 4ead 0006                JSR      6(A5)    (=0x0009dc=MemMove)
//0090e2 defc 000a                ADD.W    #10,A7
  MemMove(func, pwA2+31, 6); // Save first 6 byes of func
////0090e6 254b 002a                MOVE.L   A3,42(A2)
  pntGear(pwA2+21) = func;  // Patch jumps back to function
////0090ea 256e 000c 002e           MOVE.L   12(A6),46(A2)
  pntGear(pwA2+23) = getGraphic; // The function patch calls
////0090f0 356e 0010 0032           MOVE.W   16(A6),50(A2)
////0090f6 256e 0012 0034           MOVE.L   18(A6),52(A2)
////0090fc 256e 0016 0038           MOVE.L   22(A6),56(A2)
////009102 36fc 4ef9                MOVE.W   #20217,(A3)+
////009106 268a                     MOVE.L   A2,(A3)
//009108 4cdf 0c00                MOVEM.L  (A7)+,A23
//00910c 4e5e                     UNLK     A6
//00910e 4e75                     RTS
//
}
*/
void TAG009110(void)
{
  HopefullyNotNeeded();
  /*
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
009110 00 41fa 003a                LEA      $+60 (=0x00914c),A0
009114 02 4258                     CLR.W    (A0)+
  // Clear the patch in-use flag .. A0 <- pointer to saved words
009116 03 227a 0022                MOVE.L   $+36 (=0x00913a),A1   // put it back as it was
  // Get address of patched function.
00911a 05 22d8                     MOVE.L   (A0)+,(A1)+
00911c 06 3290                     MOVE.W   (A0),(A1)
  // Restore the function to its original state.
00911e 07 207a 001e                MOVE.L   $+32 (=0x00913e),A0
  // A0 <- extra function to call.
009122 09 3f3a 001e                MOVE.W   $+32 (=0x009142),-(A7)
  // Push the graphic number.
009126 11 4e90                     JSR      (A0)   ;getGraphicAddress
  // Get the Graphic address.
009128 12 2040                     MOVE.L   D0,A0
  // A0 <- The graphic address.
00912a 13 2f3a 001c                MOVE.L   $+30 (=0x009148),-(A7)
  // Push the pointer P5.
00912e 15 2f3a 0014                MOVE.L   $+22 (=0x009144),-(A7)
  // Push the pointer P4
009132 17 4e71                     NOP
  // At this point we have on the stack ( P4, P5, graphicNum )
009134 18 4fef 000a                LEA      10(A7),A7
  // Get everything off the stack.
009138 20 4ef9 cdcd cdcd <         JMP      -842150451     DrawCellxxxx(facing,x,y)
  // Go perform the Drawing function as if we had never been here.

//00913e 23 0000 0000 // getGraphic function address goes here
//009142 25 0000      // The graphic number
//009144 26 0000 0000 // P4 (a pointer)
//009148 28 0000 0000 // P5 (a pointer)
//00914c 30 0000      // In-use flag
//00914e 31 0000 0000 0000  // Save first three words of function
*/
}




i16 TAG0095e8(i16 P1)
{
  dReg D0, D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0095e8 4e56 0000                LINK     A6,#0
//0095ec 3f07                     MOVE.W   D7,-(A7)
//0095ee 4a6e 0008                TST.W    8(A6)
//0095f2 6d24                     BLT      $+38 (=0x009618)
  if (P1 < 0) return 0;
//0095f4 4247                     CLR.W    D7
//0095f6 601a                     BRA      $+28 (=0x009612)
  for (D7W=0; D7W<3; D7W++)
  {
//0095f8 3007                     MOVE.W   D7,D0
//0095fa e380                     ASL.L    #1,D0
//0095fc 41ec d756                LEA      -10410(A4),A0
//009600 d0c0                     ADD.W    D0,A0
//009602 3010                     MOVE.W   (A0),D0
    D0W = d.Word10410[D7W];
//009604 b06e 0008                CMP.W    8(A6),D0
//009608 6606                     BNE      $+8 (=0x009610)
    if (P1 == D0W) return 1;
//00960a 303c 0001                MOVE.W   #1,D0
//00960e 600c                     BRA      $+14 (=0x00961c)
//009610 5247                     ADDQ.W   #1,D7
//
//009612 0c47 0003                CMP.W    #3,D7
//009616 6de0                     BLT      $-30 (=0x0095f8)
  };
  return 0;
//009618 303c 0000                MOVE.W   #0,D0
//00961c 3e1f                     MOVE.W   (A7)+,D7
//00961e 4e5e                     UNLK     A6
//009620 4e75                     RTS
}

//   TAG009622
void UpdateXY(i32 facing,
              i32 stepsForward,
              i32 stepsRight,
              i32 *mapX,
              i32 *mapY)
{
  i16 *A0, *A1;
  dReg D1, D2;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009622 4e56 0000                LINK     A6,#0
//009626 48e7 0030                MOVEM.L  A23,-(A7)
//00962a 266e 000e                MOVE.L   14(A6),A3
//00962e 246e 0012                MOVE.L   18(A6),A2
//009632 41ec e3a2                LEA      -7262(A4),A0
//009636 43ec e39a                LEA      -7270(A4),A1
  A0 = d.DeltaX;
  A1 = d.DeltaY;
//00963a 302e 0008                MOVE.W   8(A6),D0
//00963e d040                     ADD.W    D0,D0
//009640 322e 000a                MOVE.W   10(A6),D1
  D1W = sw(stepsForward);
  D2W = D1W;
//009644 3401                     MOVE.W   D1,D2
//009646 c3f0 0000                MULU     0(A0,D0W),D1
  D1L = D1UW * (ui16)A0[facing];
//00964a d353                     ADD.W    D1,(A3)
  *mapX = sw(*mapX + D1W);
//00964c c5f1 0000                MULU     0(A1,D0W),D2
  D2L = D2UW * (ui16)A1[facing];
//009650 d552                     ADD.W    D2,(A2)
  *mapY = sw(*mapY + D2W);
//009652 5440                     ADDQ.W   #2,D0
//009654 0240 0007                AND.W    #7,D0
  facing = (I16)((facing+1)&3);
//009658 322e 000c                MOVE.W   12(A6),D1
//00965c 3401                     MOVE.W   D1,D2
  D2W = D1W = sw(stepsRight);
//00965e c3f0 0000                MULU     0(A0,D0W),D1
  D1L = D1UW * (ui16)A0[facing];
//009662 d353                     ADD.W    D1,(A3)
//009664 c5f1 0000                MULU     0(A1,D0W),D2
  *mapX = sw(*mapX + D1W);
  D2L = D2UW * (ui16)A1[facing];
//009668 d552                     ADD.W    D2,(A2)
  *mapY = sw(*mapY + D2W);
//00966a 4cdf 0c00                MOVEM.L  (A7)+,A23
//00966e 4e5e                     UNLK     A6
//009670 4e75                     RTS
}

//  TAG009672
CELLFLAG GetCellFlags(i32 mapX, i32 mapY)
{
  ROOMTYPE rtD4, rtD5;
  //D7W = P1;
  //D6W = P2;
  bool legalX, legalY;
  legalY = (mapY>=0) && (mapY<d.height);
  legalX = (mapX>=0) && (mapX<d.width);
  if (legalX && legalY)
  {
    return d.LevelCellFlags[mapX][mapY];
  }
  else
  {
    if (legalY)
    { // Then figure out what to do about illegal x.
      if (mapX == -1)
      {
        rtD4 = RoomType(d.LevelCellFlags[0][mapY]);
        if (rtD4 == roomOPEN) return 4; // Stone...4 = east side something??
        if (rtD4 == roomPIT) return 4;
        return 0;
      };
      if (mapX == d.width)
      {
        rtD4 = RoomType(d.LevelCellFlags[d.width-1][mapY]);
        if (rtD4 == roomOPEN) return 1; // Stone...1=west side something??
        if (rtD4 == roomPIT) return 1;
        return 0;
      };
      return 0;
    };
    if (legalX)
    {
      // What to do if Y is illegal
      if (mapY == -1)
      {
        rtD5 = RoomType(d.LevelCellFlags[mapX][0]);
        if (rtD5 == roomOPEN) return 2;//Stone...South side something??
        if (rtD5 == roomPIT) return 2;
        return 0;
      };
      if (mapY == d.height)
      {
        rtD5 = RoomType(d.LevelCellFlags[mapX][mapY-1]);
        if (rtD5 == roomOPEN) return 8;
        if (rtD5 == roomPIT) return 8;
        return 0;
      };
      return 0;
    };
    return 0; // Both x and y illegal.
  };
}

i16 TAG0097a0(i32 facing,
              i32 stepsForward,
              i32 stepsRight,
              i32 x,
              i32 y)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0097a0 4e56 0000                LINK     A6,#0
//0097a4 486e 0010                PEA      16(A6)
//0097a8 486e 000e                PEA      14(A6)
//0097ac 3f2e 000c                MOVE.W   12(A6),-(A7)
//0097b0 3f2e 000a                MOVE.W   10(A6),-(A7)
//0097b4 3f2e 0008                MOVE.W   8(A6),-(A7)
//0097b8 4eba fe68                JSR      $-406 (=0x009622)
//0097bc defc 000e                ADD.W    #14,A7
  UpdateXY(facing, stepsForward, stepsRight, &x, &y);
//0097c0 3f2e 0010                MOVE.W   16(A6),-(A7)
//0097c4 3f2e 000e                MOVE.W   14(A6),-(A7)
//0097c8 4eba fea8                JSR      $-342 (=0x009672)
//0097cc 588f                     ADDQ.L   #4,A7
  D0W = GetCellFlags(x, y);
//0097ce c07c 00ff                AND.W    #255,D0
//0097d2 4e5e                     UNLK     A6
//0097d4 4e75                     RTS
  return (I16)(D0W & 0xff);
}

i16 TAG0097d6(i32 facing,
              i32 stepsForward,
              i32 stepsRight,
              i32 x,
              i32 y)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0097d6 4e56 0000                LINK     A6,#0
//0097da 3f2e 0010                MOVE.W   16(A6),-(A7)
//0097de 3f2e 000e                MOVE.W   14(A6),-(A7)
//0097e2 3f2e 000c                MOVE.W   12(A6),-(A7)
//0097e6 3f2e 000a                MOVE.W   10(A6),-(A7)
//0097ea 3f2e 0008                MOVE.W   8(A6),-(A7)
//0097ee 4eba ffb0                JSR      $-78 (=0x0097a0)
//0097f2 defc 000a                ADD.W    #10,A7
  D0W = TAG0097a0(facing,
                   stepsForward,
                   stepsRight,
                   x,
                   y);
//0097f6 c07c 00ff                AND.W    #255,D0
  D0W &= 0xff;
  D0W >>= 5;
//0097fa ea40                     ASR.W    #5,D0
//0097fc 4e5e                     UNLK     A6
//0097fe 4e75                     RTS
  return D0W;
}

//*********************************************************
//
//*********************************************************
//  TAG009800
i16 IncrementLevel(i32 level,i32 delta,i32 *mapX,i32 *mapY)
//                 8     10      12      16
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009800 4e56 fff4                LINK     A6,#-12
//009804 48e7 0f10                MOVEM.L  D4567A3,-(A7)
  dReg D1, D4;
//  aReg A3;
  LEVELDESC *pldA3;
  i32 absX, absY;
  i32 newLevel;
  i16 LOCAL_2;
  i16 LOCAL_12;
//009808 0c6c 00ff d244           CMP.W    #255,-11708(A4)
//00980e 6608                     BNE      $+10 (=0x009818)
  if (d.partyLevel==255) return -1;
//009810 303c ffff                MOVE.W   #65535,D0
//009814 6000 00e6                BRA      $+232 (=0x0098fc)

//009818 202c d736                MOVE.L   -10442(A4),D0
  pldA3 = &d.pLevelDescriptors[level];
//00981c 322e 0008                MOVE.W   8(A6),D1
//009820 c3fc 0010                MULS     #16,D1
//009824 d081                     ADD.L    D1,D0
//009826 2640                     MOVE.L   D0,A3
//009828 1e2b 0006                MOVE.B   6(A3),D7
//00982c 206e 000c                MOVE.L   12(A6),A0
//009830 ce7c 00ff                AND.W    #255,D7
//009834 de50                     ADD.W    (A0),D7
  absX = pldA3->offsetX + (*mapX);
//009836 1c2b 0007                MOVE.B   7(A3),D6
  absY = pldA3->offsetY + (*mapY);
//00983a 206e 0010                MOVE.L   16(A6),A0
//00983e cc7c 00ff                AND.W    #255,D6
//009842 dc50                     ADD.W    (A0),D6
//009844 3a2b 0008                MOVE.W   8(A3),D5
//009848 ca7c 003f                AND.W    #63,D5
//00984c da6e 000a                ADD.W    10(A6),D5
  newLevel = pldA3->Level() + delta; // Old level + delta
//009850 266c d736                MOVE.L   -10442(A4),A3
  pldA3 = d.pLevelDescriptors;
//009854 426e fffe                CLR.W    -2(A6)

//0098e4 302e fffe                MOVE.W   -2(A6),D0
//0098e8 206c d732                MOVE.L   -10446(A4),A0
//0098ec 4241                     CLR.W    D1
//0098ee 1228 0004                MOVE.B   4(A0),D1
//0098f2 b041                     CMP.W    D1,D0
//0098f4 6d00 ff66                BLT      $-152 (=0x00985c)

  for (LOCAL_2 = 0; LOCAL_2 < ((d.dungeonDatIndex[2]>>8)&0xff); LOCAL_2++)
  { // 0 to #levels-1
//009858 6000 008a                BRA      $+140 (=0x0098e4)
//00985c 302b 0008                MOVE.W   8(A3),D0
//009860 c07c 003f                AND.W    #63,D0
//009864 3205                     MOVE.W   D5,D1
//009866 b041                     CMP.W    D1,D0
//009868 6672                     BNE      $+116 (=0x0098dc)
    if (pldA3->Level() == newLevel)
      // search for new level among level descriptors
    {
//00986a 3007                     MOVE.W   D7,D0
//00986c 4241                     CLR.W    D1
//00986e 122b 0006                MOVE.B   6(A3),D1
//009872 3801                     MOVE.W   D1,D4
//009874 b041                     CMP.W    D1,D0
//009876 6d64                     BLT      $+102 (=0x0098dc)
      D4W = pldA3->offsetX;
      if (absX >= D4W)
      {
//009878 3004                     MOVE.W   D4,D0
//00987a 322b 0008                MOVE.W   8(A3),D1
//00987e c27c 07c0                AND.W    #1984,D1
//009882 ec49                     LSR.W    #6,D1
//009884 d041                     ADD.W    D1,D0
//009886 3d40 fff4                MOVE.W   D0,-12(A6)
//00988a 3007                     MOVE.W   D7,D0
//00988c b06e fff4                CMP.W    -12(A6),D0
//009890 624a                     BHI      $+76 (=0x0098dc)
        D1W = pldA3->LastColumn(); // #columns-1 on new level
        LOCAL_12 = sw(D4W + D1W);
        if ((ui16)absX <= (ui16)LOCAL_12)
        {
//009892 3006                     MOVE.W   D6,D0
//009894 4241                     CLR.W    D1
//009896 122b 0007                MOVE.B   7(A3),D1
//00989a 3801                     MOVE.W   D1,D4
//00989c b041                     CMP.W    D1,D0
//00989e 6d3c                     BLT      $+62 (=0x0098dc)
          D4W = pldA3->offsetY;
          if(absY >= D4W)
          {
//0098a0 3004                     MOVE.W   D4,D0
//0098a2 322b 0008                MOVE.W   8(A3),D1
//0098a6 c27c f800                AND.W    #63488,D1
//0098aa e649                     LSR.W    #3,D1
//0098ac e049                     LSR.W    #8,D1
//0098ae d041                     ADD.W    D1,D0
//0098b0 3d40 fff4                MOVE.W   D0,-12(A6)
//0098b4 3006                     MOVE.W   D6,D0
//0098b6 b06e fff4                CMP.W    -12(A6),D0
//0098ba 6220                     BHI      $+34 (=0x0098dc)
            D1W = pldA3->LastRow();//#rows-1 in new level
            LOCAL_12 =  sw(D4W + D1W);
            if ((ui16)absY <= (ui16)LOCAL_12)
            {
//0098bc 3006                     MOVE.W   D6,D0
//0098be 9044                     SUB.W    D4,D0
//0098c0 206e 0010                MOVE.L   16(A6),A0
//0098c4 3080                     MOVE.W   D0,(A0)
              *mapY = absY - D4W;
//0098c6 3007                     MOVE.W   D7,D0
//0098c8 4243                     CLR.W    D3
//0098ca 162b 0006                MOVE.B   6(A3),D3
//0098ce 9043                     SUB.W    D3,D0
//0098d0 206e 000c                MOVE.L   12(A6),A0
//0098d4 3080                     MOVE.W   D0,(A0)
              *mapX = absX - pldA3->offsetX;
//0098d6 302e fffe                MOVE.W   -2(A6),D0
//0098da 6020                     BRA      $+34 (=0x0098fc)
              return LOCAL_2;
            };
          };
        };
      };
    };
//0098dc d6fc 0010                ADD.W    #16,A3
//0098e0 526e fffe                ADDQ.W   #1,-2(A6)
  pldA3 ++;
  };
//0098f8 303c ffff                MOVE.W   #65535,D0
//
//0098fc 4cdf 08f0                MOVEM.L  (A7)+,D4567A3
//009900 4e5e                     UNLK     A6
//009902 4e75                     RTS
  return -1;
}
//*********************************************************
//
//*********************************************************
DBCOMMON *GetRecordAddress(RN recDesc)
{
  // recDesc(bits 10-13) = database number
  // recDesc(bits 0-9)   = record number within database
//  dReg D0, D2, D3;
//  aReg A0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009984 4e56 0000                LINK     A6,#0
//009988 362e 0008                MOVE.W   8(A6),D3
//00998c 3403                     MOVE.W   D3,D2
//00998e 0243 03ff                AND.W    #1023,D3
//009992 e44a                     LSR.W    #2,D2
//009994 e04a                     LSR.W    #8,D2
//009996 0242 000f                AND.W    #15,D2
  // Some folks call to get the address before checking
  // to see if the object exists at all!
  if ((recDesc == RNnul) || (recDesc == RNeof))
  {
    HopefullyNotNeeded();
    return NULL;
  };
  return db.GetCommonAddress(recDesc);
//00999a 41ec e38a                LEA      -7286(A4),A0
//00999e 7000                     MOVEQ.L  #0,D0
//0099a0 1030 2000                MOVE.B   0(A0,D2W),D0
//0099a4 c0c3                     MULU     D3,D0
//0099a6 41ec d6e0                LEA      -10528(A4),A0
//0099aa e542                     ASL.W    #2,D2
//0099ac d0b0 2000                ADD.L    0(A0,D2W),D0
//0099b0 4e5e                     UNLK     A6
//0099b2 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
DB0 *GetRecordAddressDB0(RN object)
{
  ASSERT(object.db() == 0);
  return (DB0 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB1 *GetRecordAddressDB1(RN object)
{
  ASSERT(object.db() == 1);
  return (DB1 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB2 *GetRecordAddressDB2(RN object)
{
  ASSERT(object.db() == 2);
  return (DB2 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB3 *GetRecordAddressDB3(RN object)
{
  ASSERT(object.db() == 3);
  return (DB3 *) GetRecordAddress(object);
}


//*********************************************************
//
//*********************************************************
DB4 *GetRecordAddressDB4(RN object)
{
  ASSERT(object.db() == 4);
  return (DB4 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB5 *GetRecordAddressDB5(RN object)
{
  ASSERT(object.db() == 5);
  return (DB5 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB6 *GetRecordAddressDB6(RN object)
{
  ASSERT(object.db() == 6);
  return (DB6 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB8 *GetRecordAddressDB8(RN object)
{
  ASSERT(object.db() == 8);
  return (DB8 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB9 *GetRecordAddressDB9(RN object)
{
  ASSERT(object.db() == 9);
  return (DB9 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB10 *GetRecordAddressDB10(RN object)
{
  ASSERT(object.db() == 10);
  return (DB10 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB14 *GetRecordAddressDB14(RN object)
{
  ASSERT(object.db() == 14);
  return (DB14 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
DB15 *GetRecordAddressDB15(RN object)
{
  ASSERT(object.db() == 15);
  return (DB15 *) GetRecordAddress(object);
}

//*********************************************************
//
//*********************************************************
RN FindEmptyDB4Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbMONSTER, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbMONSTER);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB4(result)->Clear();
  return result;
}

//*********************************************************
//
//*********************************************************
RN FindEmptyDB5Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbWEAPON, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbWEAPON);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB5(result)->Clear();
  return result;
}


//*********************************************************
//
//*********************************************************
RN FindEmptyDB6Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbCLOTHING, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbCLOTHING);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB6(result)->Clear();
  return result;
}


//*********************************************************
//
//*********************************************************
RN FindEmptyDB10Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbMISC, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbMISC);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB10(result)->Clear();
  return result;
}

//*********************************************************
//
//*********************************************************
RN FindEmptyDB14Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbMISSILE, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbMISSILE);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB14(result)->Clear();
  return result;
}


//*********************************************************
//
//*********************************************************
RN FindEmptyDB15Entry(bool important)
{//(RN)
  RN result;
  result = db.FindEmptyDBEntry(dbCLOUD, important);
  if (result == RNnul)
  {
    result = MakeMoreRoomInDB(dbCLOUD);
    if (result == RNnul) return RNnul;
  };
  GetRecordAddressDB15(result)->Clear();
  return result;
}

//*********************************************************
//
//*********************************************************
// TAG009a02
RN GetDBRecordLink(RN recName)
{
  // recDesc(bits 10-13) = database number
  // recDesc(bits 0-9)   = record number within database
  DBCOMMON *dbA0;
  ASSERT(recName != RNnul);
  ASSERT(recName != RNeof);
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009a02 4e56 fffc                LINK     A6,#-4
//009a06 3f2e 0008                MOVE.W   8(A6),-(A7)
//009a0a 4eba ff78                JSR      $-134 (=0x009984)
//009a0e 548f                     ADDQ.L   #2,A7
  dbA0 = GetRecordAddress(recName);
//009a10 2d40 fffc                MOVE.L   D0,-4(A6)
//009a14 206e fffc                MOVE.L   -4(A6),A0
//009a18 3010                     MOVE.W   (A0),D0
  return dbA0->link();
//009a1a 4e5e                     UNLK     A6
//009a1c 4e75                     RTS
}

//  TAG009a1e
i16 GetObjectListIndex(i32 mapX,i32 mapY)
{ // Index in ??? of the object-list for this cell.
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009a1e 4e56 0000                LINK     A6,#0
  dReg D0, D4, D5;
  CELLFLAG *pdA3;
//009a22 48e7 0f10                MOVEM.L  D4567A3,-(A7)
//009a26 3e2e 0008                MOVE.W   8(A6),D7
//009a2a 3c2e 000a                MOVE.W   10(A6),D6
//  D7W = x;
//  D6W = y;
//009a2e 3007                     MOVE.W   D7,D0
//009a30 e580                     ASL.L    #2,D0
//009a32 206c d748                MOVE.L   -10424(A4),A0
//009a36 d0c0                     ADD.W    D0,A0
//009a38 2650                     MOVE.L   (A0),A3
  pdA3 = d.LevelCellFlags[mapX];
//009a3a 4a47                     TST.W    D7
//009a3c 6d22                     BLT      $+36 (=0x009a60)
  if (mapX < 0) return -1;
//009a3e 3007                     MOVE.W   D7,D0
//009a40 b06c d744                CMP.W    -10428(A4),D0
//009a44 6c1a                     BGE      $+28 (=0x009a60)
  if (mapX >= d.width) return -1;
//009a46 4a46                     TST.W    D6
//009a48 6d16                     BLT      $+24 (=0x009a60)
  if (mapY < 0) return -1;
//009a4a 3006                     MOVE.W   D6,D0
//009a4c b06c d742                CMP.W    -10430(A4),D0
//009a50 6c0e                     BGE      $+16 (=0x009a60)
  if (mapY >= d.height) return -1;
//009a52 204b                     MOVE.L   A3,A0
//009a54 d0c6                     ADD.W    D6,A0
//009a56 4240                     CLR.W    D0
//009a58 1010                     MOVE.B   (A0),D0
//009a5a c07c 0010                AND.W    #16,D0
  D0W = (I16)(pdA3[mapY] & 16);
//009a5e 6606                     BNE      $+8 (=0x009a66)
  if (D0W == 0) return -1; // Nothing here!
//009a66 4244                     CLR.W    D4
  D4W = 0;
//009a68 3a07                     MOVE.W   D7,D5
//009a6a e385                     ASL.L    #1,D5
//009a6c 206c d74c                MOVE.L   -10420(A4),A0
//009a70 d0c5                     ADD.W    D5,A0
//009a72 3a10                     MOVE.W   (A0),D5
  D5W = d.pwPointer10420[mapX];
//009a74 600c                     BRA      $+14 (=0x009a82)
  for (D4W=0; D4W<mapY; D4W++)
  {
//009a76 4240                     CLR.W    D0
//009a78 101b                     MOVE.B   (A3)+,D0
//009a7a c07c 0010                AND.W    #16,D0
    D0W = (I16)(pdA3[D4W]&16);
//009a7e 6702                     BEQ      $+4 (=0x009a82)
    if (D0W != 0) D5W++;
//009a80 5245                     ADDQ.W   #1,D5

//009a82 3004                     MOVE.W   D4,D0
//009a84 5244                     ADDQ.W   #1,D4
//009a86 b046                     CMP.W    D6,D0
//009a88 66ec                     BNE      $-18 (=0x009a76)
  };
//009a8a 3005                     MOVE.W   D5,D0
  D0W = D5W;
//009a8c 4cdf 08f0                MOVEM.L  (A7)+,D4567A3
//009a90 4e5e                     UNLK     A6
//009a92 4e75                     RTS
  return D0W;
}

//  TAG009a94
RN FindFirstObject(i32 x,i32 y)
{

  i16 LOCAL_2;
  LOCAL_2 = GetObjectListIndex(x, y);
  if ( LOCAL_2 == -1) return RN(RNeof);
  return d.prn10464[LOCAL_2];
}

// TAG009ac8
RN FindFirstDrawableObject(i32 x,i32 y)
{
  RN obj;
  obj = FindFirstObject(x, y);
  while ( (obj != RNeof) && (obj.db() < 4))
  {
    obj = GetDBRecordLink(obj);
  };
  return obj;
}

//*********************************************************
//
//*********************************************************
//   TAG009c4e
void RemoveObjectFromRoom(RN P1,RN P2,i32 mapX,i32 mapY)
{
  dReg D0, D1, D5, D6;
  DBCOMMON *dbA3=NULL;
  RN objD4, objD7;
  i16 *pwA0;
  RN *prnA2;
  ui16 *puwA2;
  CELLFLAG *pdA0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//009c4e 4e56 0000                LINK     A6,#0
//009c52 48e7 0f30                MOVEM.L  D4567A23,-(A7)
  if (TimerTraceActive)
  {
    fprintf(TraceFile,"Removed %04x from %02x(%02x,%02x)\n",
             P1.rid(), d.LoadedLevel, mapX, mapY);
  };
//009c56 3e2e 0008                MOVE.W   8(A6),D7
  objD7 = P1;
//009c5a 0c47 fffe                CMP.W    #65534,D7
//009c5e 6604                     BNE      $+6 (=0x009c64)
//009c60 6000 0134                BRA      $+310 (=0x009d96)
  if (objD7 == RNeof) return;
//009c64 ce7c 3fff                AND.W    #16383,D7
  objD7.pos(0); // clear position leaving only record id
//009c68 4a6e 000c                TST.W    12(A6)
//009c6c 6d00 00ce                BLT      $+208 (=0x009d3c)
  if (mapX >= 0)
  {
//009c70 3f07                     MOVE.W   D7,-(A7)
//009c72 4eba fd10                JSR      $-750 (=0x009984)
//009c76 548f                     ADDQ.L   #2,A7
//009c78 2640                     MOVE.L   D0,A3
    dbA3 = GetRecordAddress(objD7);
    ASSERT(dbA3->link() != RNnul);
//009c7a 3f2e 000e                MOVE.W   14(A6),-(A7)
//009c7e 3f2e 000c                MOVE.W   12(A6),-(A7)
//009c82 4eba fd9a                JSR      $-612 (=0x009a1e)
//009c86 588f                     ADDQ.L   #4,A7
//009c88 3c00                     MOVE.W   D0,D6
    D6W = GetObjectListIndex(mapX, mapY);
//009c8a e380                     ASL.L    #1,D0
//009c8c 206c d720                MOVE.L   -10464(A4),A0
//009c90 d0c0                     ADD.W    D0,A0
//009c92 41d0                     LEA      (A0),A0
//009c94 2448                     MOVE.L   A0,A2
    prnA2 = &d.prn10464[D6W];
    ASSERT(prnA2->ConvertToInteger() != 0xbadd);
//009c96 0c53 fffe                CMP.W    #65534,(A3)
//009c9a 6600 008e                BNE      $+144 (=0x009d2a)
//
//009c9e 3012                     MOVE.W   (A2),D0
//009ca0 c07c 3fff                AND.W    #16383,D0
//009ca4 b047                     CMP.W    D7,D0
//009ca6 6600 0082                BNE      $+132 (=0x009d2a)
    // If specified object has no forward link
    //  and first object in room is the specified object
    //   then remove the entire room from
    if ((dbA3->link() == RNeof) && ((*prnA2).rid()==objD7.rid()) )
    {
//009caa 302e 000c                MOVE.W   12(A6),D0
//009cae e580                     ASL.L    #2,D0
//009cb0 206c d748                MOVE.L   -10424(A4),A0
//009cb4 d0c0                     ADD.W    D0,A0
//009cb6 2050                     MOVE.L   (A0),A0
      pdA0 = d.LevelCellFlags[mapX];
//009cb8 d0ee 000e                ADD.W    14(A6),A0
      ASSERT((pdA0[mapY] & 0x10)!=0);
      pdA0[mapY] &= 0xef; // clear 'room has object'
//009cbc 0210 ffef                AND.B    #-17,(A0)
//009cc0 206c d732                MOVE.L   -10446(A4),A0
      pwA0 = d.dungeonDatIndex;
//009cc4 3028 000a                MOVE.W   10(A0),D0
//009cc8 5340                     SUBQ.W   #1,D0
//009cca 3a00                     MOVE.W   D0,D5
      D5W = sw(pwA0[5] - 1);
//009ccc 9046                     SUB.W    D6,D0
      D0W = sw(2*(D5W - D6W));
//009cce e340                     ASL.W    #1,D0
//009cd0 3f00                     MOVE.W   D0,-(A7)
//009cd2 2f0a                     MOVE.L   A2,-(A7)
//009cd4 200a                     MOVE.L   A2,D0
//009cd6 5480                     ADDQ.L   #2,D0
//009cd8 2f00                     MOVE.L   D0,-(A7)
//009cda 4ead 0006                JSR      6(A5)    (=0x0009dc)
//009cde defc 000a                ADD.W    #10,A7
      MemMove((pnt)(prnA2+1), (pnt)prnA2, D0W);
//009ce2 3005                     MOVE.W   D5,D0
//009ce4 e380                     ASL.L    #1,D0
//009ce6 206c d720                MOVE.L   -10464(A4),A0
//009cea d0c0                     ADD.W    D0,A0
      d.prn10464[D5W] = RN(RNnul); // clear last entry
//009cec 30bc ffff                MOVE.W   #65535,(A0)
//009cf0 202c d74c                MOVE.L   -10420(A4),D0
//009cf4 322e 000c                MOVE.W   12(A6),D1
//009cf8 48c1                     EXT.L    D1
//009cfa e381                     ASL.L    #1,D1
//009cfc d081                     ADD.L    D1,D0
//009cfe 5480                     ADDQ.L   #2,D0
//009d00 2440                     MOVE.L   D0,A2
      //Now we need to fix the column indexes.
      puwA2 = &d.pwPointer10420[mapX+1];
      // Each column after the current column must
      // have its index decremented.  This includes
      // all the columns in higher numbered levels.
//009d02 3a2c d724                MOVE.W   -10460(A4),D5
      D5W = d.numColumnPointers; // for all levels
//009d06 322c d746                MOVE.W   -10426(A4),D1
//009d0a e381                     ASL.L    #1,D1
//009d0c 206c d726                MOVE.L   -10458(A4),A0
//009d10 d0c1                     ADD.W    D1,A0
//009d12 3210                     MOVE.W   (A0),D1
      D1W = d.puwPointer10458[d.LoadedLevel];
      // D1=index of first DUDAD for this level.
//009d14 d26e 000c                ADD.W    12(A6),D1
//009d18 9a41                     SUB.W    D1,D5
      D1W = sw(D1W + mapX);
      //D1=index of DUDAD for this column
//009d1a 5345                     SUBQ.W   #1,D5
      D5W = sw(D5W - D1W);
      D5W--; //#columns following this column in
              // this and following levels.
//009d1c 6002                     BRA      $+4 (=0x009d20)
      while ((D5W--) != 0) (*(puwA2++))--;
//009d1e 535a                     SUBQ.W   #1,(A2)+
//
//009d20 3005                     MOVE.W   D5,D0
//009d22 5345                     SUBQ.W   #1,D5
//009d24 4a40                     TST.W    D0
//009d26 66f6                     BNE      $-8 (=0x009d1e)
      dbA3->link(RNeof);
      return;
//009d28 6068                     BRA      $+106 (=0x009d92)
    };
//009d2a 3012                     MOVE.W   (A2),D0
//009d2c c07c 3fff                AND.W    #16383,D0
//009d30 b047                     CMP.W    D7,D0
//009d32 6604                     BNE      $+6 (=0x009d38)
    if (prnA2->rid() == objD7.rid())
    {
//009d34 3493                     MOVE.W   (A3),(A2)
      *prnA2 = dbA3->link();
//009d36 605a                     BRA      $+92 (=0x009d92)
      dbA3->link(RNeof);
      return;
    }
//009d38 3d52 000a                MOVE.W   (A2),10(A6)
    P2 = *prnA2;
  };
//009d3c 3f2e 000a                MOVE.W   10(A6),-(A7)
//009d40 4eba fcc0                JSR      $-830 (=0x009a02)
//009d44 548f                     ADDQ.L   #2,A7
//009d46 3800                     MOVE.W   D0,D4
  objD4 = GetDBRecordLink(P2);
//009d48 601e                     BRA      $+32 (=0x009d68)
  while (objD4.rid() != objD7.rid())
  {
//009d4a 0c44 fffe                CMP.W    #65534,D4
//009d4e 6706                     BEQ      $+8 (=0x009d56)
//009d50 0c44 ffff                CMP.W    #65535,D4
//009d54 6602                     BNE      $+4 (=0x009d58)
    if ((objD4 == RNeof) || (objD4 == RNnul))
    {
      dbA3->link(RNeof);
      return;
    };
//009d56 603a                     BRA      $+60 (=0x009d92)

//009d58 3004                     MOVE.W   D4,D0
//009d5a 3d40 000a                MOVE.W   D0,10(A6)
    P2 = objD4;
//009d5e 3f00                     MOVE.W   D0,-(A7)
//009d60 4eba fca0                JSR      $-862 (=0x009a02)
//009d64 548f                     ADDQ.L   #2,A7
//009d66 3800                     MOVE.W   D0,D4
    objD4 = GetDBRecordLink(objD4);
//009d68 3004                     MOVE.W   D4,D0
//009d6a c07c 3fff                AND.W    #16383,D0
//009d6e b047                     CMP.W    D7,D0
//009d70 66d8                     BNE      $-38 (=0x009d4a)
  };
//009d72 3f2e 000a                MOVE.W   10(A6),-(A7)
//009d76 4eba fc0c                JSR      $-1010 (=0x009984)
//009d7a 548f                     ADDQ.L   #2,A7
//009d7c 2640                     MOVE.L   D0,A3
  dbA3 = GetRecordAddress(P2);
//009d7e 3f04                     MOVE.W   D4,-(A7)
//009d80 4eba fc80                JSR      $-894 (=0x009a02)
//009d84 548f                     ADDQ.L   #2,A7
//009d86 3680                     MOVE.W   D0,(A3)
  ASSERT(GetDBRecordLink(objD4) != RNnul);
  dbA3->link(GetDBRecordLink(objD4));
//009d88 3f07                     MOVE.W   D7,-(A7)
//009d8a 4eba fbf8                JSR      $-1030 (=0x009984)
//009d8e 548f                     ADDQ.L   #2,A7
//009d90 2640                     MOVE.L   D0,A3
  dbA3 = GetRecordAddress(objD7);
//009d92 36bc fffe                MOVE.W   #65534,(A3)
  dbA3->link(RNeof);
//009d96 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//009d9a 4e5e                     UNLK     A6
//009d9c 4e75                     RTS
}

//   TAG00a27c
void DecodeText(pnt result,RN textID,i16 P3)
{
  dReg D0, D4, D5, D6, D7;
  aReg A2;
  DB2 *DB2A1;
  RN objD4;
  DB2 LOCAL_12;
  pnt LOCAL_8;
  i16 LOCAL_4;
  i8  LOCAL_1;
  D6L = 0xbaddadee;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00a27c 4e56 fff4                LINK     A6,#-12
//00a280 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//00a284 266e 0008                MOVE.L   8(A6),A3
//00a288 3e2e 000e                MOVE.W   14(A6),D7  //00a28c 302e 000c                MOVE.W   12(A6),D0
  //A3 = P1;
  D7W = P3;
  //D0W = textID.idx();
//00a290 c07c 03ff                AND.W    #1023,D0
  //D0W &= 0x3ff;
//00a294 222c d6e8                MOVE.L   -10520(A4),D1
//00a298 e580                     ASL.L    #2,D0
//00a29a 2041                     MOVE.L   D1,A0
//00a29c d0c0                     ADD.W    D0,A0

//00a29e 43d0                     LEA      (A0),A1
  DB2A1 = GetRecordAddressDB2(textID);
//00a2a0 41ee fff4                LEA      -12(A6),A0
  //A0 = (pnt)&LOCAL_12;
//00a2a4 7001                     MOVEQ.L  #1,D0
//00a2a6 4ead 00c6                JSR      198(A5)    (=0x0031c4)
  //MoveWords(&LOCAL_12, DB2A1, 1); // 2 words
  LOCAL_12 = *DB2A1;
//00a2aa 302e fff6                MOVE.W   -10(A6),D0
//00a2ae c07c 0001                AND.W    #1,D0
  //D0W = LOCAL_12[1];
//00a2b2 660a                     BNE      $+12 (=0x00a2be)
//00a2b4 3007                     MOVE.W   D7,D0
//00a2b6 c07c 8000                AND.W    #32768,D0
//00a2ba 6700 014c                BEQ      $+334 (=0x00a408)
  if ((LOCAL_12.b0() != 0) || ((D7W & 0x8000) != 0))
  {
//00a2be ce7c 7fff                AND.W    #32767,D7
//00a2c2 0c47 0001                CMP.W    #1,D7
//00a2c6 660c                     BNE      $+14 (=0x00a2d4)
    if ((D7W & 0x7fff) == 1)
    {
//00a2c8 16fc 000a                MOVE.B   #10,(A3)+
//00a2cc 1d7c 0020 ffff           MOVE.B   #32,-1(A6)
      *(result++) = 10;
      LOCAL_1 = 32;
//00a2d2 6012                     BRA      $+20 (=0x00a2e6)
    }
    else
    {
//00a2d4 4a47                     TST.W    D7
//00a2d6 6608                     BNE      $+10 (=0x00a2e0)
      if (D7W == 0)
      {
//00a2d8 1d7c ff80 ffff           MOVE.B   #-128,-1(A6)
        LOCAL_1 = -128;
//00a2de 6006                     BRA      $+8 (=0x00a2e6)
      }
      else
      {
//00a2e0 1d7c 000a ffff           MOVE.B   #10,-1(A6)
        LOCAL_1 = 10;
      };
    };
//00a2e6 4245                     CLR.W    D5
//00a2e8 426e fffc                CLR.W    -4(A6)
    D5W = 0;
    LOCAL_4 = 0;
//00a2ec 202c d796                MOVE.L   -10346(A4),D0
//00a2f0 322e fff6                MOVE.W   -10(A6),D1
//00a2f4 c27c fff8                AND.W    #65528,D1
//00a2f8 e649                     LSR.W    #3,D1
//00a2fa c2bc 0000 ffff           AND.L    #65535,D1
//00a300 e381                     ASL.L    #1,D1
//00a302 d081                     ADD.L    D1,D0
//00a304 2440                     MOVE.L   D0,A2
    A2 = d.Pointer10346 + 2*LOCAL_12.index();
    for (;;)
    {
//00a306 3005                     MOVE.W   D5,D0
//00a308 660e                     BNE      $+16 (=0x00a318)
      if (D5W == 0)
      {
//00a30a 3c1a                     MOVE.W   (A2)+,D6
        D6W = LE16(wordGear(A2));
        A2 += 2;
//00a30c 3806                     MOVE.W   D6,D4
//00a30e e44c                     LSR.W    #2,D4
//00a310 e04c                     LSR.W    #8,D4
//00a312 c87c 001f                AND.W    #31,D4
        D4W = BITS10_14(D6W);
//00a316 6016                     BRA      $+24 (=0x00a32e)
      }
      else
      {
//00a318 0c45 0001                CMP.W    #1,D5
//00a31c 660a                     BNE      $+12 (=0x00a328)
        if (D5W == 1)
        {
//00a31e 3806                     MOVE.W   D6,D4
//00a320 ea4c                     LSR.W    #5,D4
//00a322 c87c 001f                AND.W    #31,D4
          D4W = BITS5_9(D6W);
//00a326 6006                     BRA      $+8 (=0x00a32e)
        }
        else
        {
//00a328 3806                     MOVE.W   D6,D4
          D4W = BITS0_4(D6W);
//00a32a c87c 001f                AND.W    #31,D4
        };
      };
//00a32e 5245                     ADDQ.W   #1,D5
      D5W++;
//00a330 3005                     MOVE.W   D5,D0
//00a332 c0bc 0000 ffff           AND.L    #65535,D0
//00a338 80fc 0003                DIVU     #3,D0
//00a33c 4840                     SWAP     D0
//00a33e 3a00                     MOVE.W   D0,D5
      D5W = (I16)(D5W % 3);
//00a340 302e fffc                MOVE.W   -4(A6),D0
//00a344 6778                     BEQ      $+122 (=0x00a3be)
      if (LOCAL_4 != 0)
      {
//00a346 4213                     CLR.B    (A3)
      *result = 0;
//00a348 0c6e 001e fffc           CMP.W    #30,-4(A6)
//00a34e 6628                     BNE      $+42 (=0x00a378)
        if (LOCAL_4 == 30)
        {
//00a350 4a47                     TST.W    D7
//00a352 6712                     BEQ      $+20 (=0x00a366)
          if (D7W != 0)
          {
//00a354 3004                     MOVE.W   D4,D0
//00a356 e780                     ASL.L    #3,D0
//00a358 41ec d8e0                LEA      -10016(A4),A0
//00a35c d0c0                     ADD.W    D0,A0
//00a35e 41d0                     LEA      (A0),A0
//00a360 2d48 fff8                MOVE.L   A0,-8(A6)
            LOCAL_8 = d.Byte10016 + 8*D4W;
//00a364 6010                     BRA      $+18 (=0x00a376)
          }
          else
          {
//00a366 3004                     MOVE.W   D4,D0
//00a368 e780                     ASL.L    #3,D0
//00a36a 41ec d7a0                LEA      -10336(A4),A0
//00a36e d0c0                     ADD.W    D0,A0
//00a370 41d0                     LEA      (A0),A0
//00a372 2d48 fff8                MOVE.L   A0,-8(A6)
            LOCAL_8 = d.Byte10336 + 8*D4W;
          };
//00a376 6026                     BRA      $+40 (=0x00a39e)
        }
        else
        {
//00a378 4a47                     TST.W    D7
//00a37a 6712                     BEQ      $+20 (=0x00a38e)
          if (D7W != 0)
          {
//00a37c 3004                     MOVE.W   D4,D0
//00a37e e380                     ASL.L    #1,D0
//00a380 41ec d8a0                LEA      -10080(A4),A0
//00a384 d0c0                     ADD.W    D0,A0
//00a386 41d0                     LEA      (A0),A0
//00a388 2d48 fff8                MOVE.L   A0,-8(A6)
            LOCAL_8 = d.Byte10080 + 2*D4W;
//00a38c 6010                     BRA      $+18 (=0x00a39e)
          }
          else
          {
//00a38e 3004                     MOVE.W   D4,D0
//00a390 e380                     ASL.L    #1,D0
//00a392 41ec d8a0                LEA      -10080(A4),A0
//00a396 d0c0                     ADD.W    D0,A0
//00a398 41d0                     LEA      (A0),A0
//00a39a 2d48 fff8                MOVE.L   A0,-8(A6)
            LOCAL_8 = d.Byte10080 + 2*D4W;
          };
        };
//00a39e 2f2e fff8                MOVE.L   -8(A6),-(A7)
//00a3a2 2f0b                     MOVE.L   A3,-(A7)
//00a3a4 4ead 0078                JSR      120(A5)    (=0x0031e8)
//00a3a8 508f                     ADDQ.L   #0,A7
        StrCat(result, LOCAL_8);
//00a3aa 2f2e fff8                MOVE.L   -8(A6),-(A7)
//00a3ae 4ead 006c                JSR      108(A5)    (=0x0032ba)
//00a3b2 588f                     ADDQ.L   #4,A7
        D0W = StrLen(LOCAL_8);
//00a3b4 48c0                     EXT.L    D0
//00a3b6 d7c0                     ADD.L    D0,A3
        result += D0W;
//00a3b8 426e fffc                CLR.W    -4(A6)
        LOCAL_4 = 0;
//00a3bc 6046                     BRA      $+72 (=0x00a404)
      }
      else
      {
//00a3be 0c44 001c                CMP.W    #28,D4
//00a3c2 6426                     BCC      $+40 (=0x00a3ea)
        if (D4W < 28)
        {
//00a3c4 4a47                     TST.W    D7
//00a3c6 671c                     BEQ      $+30 (=0x00a3e4)
          if (D7W != 0)
          {
//00a3c8 0c44 001a                CMP.W    #26,D4
//00a3cc 6606                     BNE      $+8 (=0x00a3d4)
            if (D4W== 26)
            {
//00a3ce 383c 0020                MOVE.W   #32,D4
              D4W = 32;
//00a3d2 6010                     BRA      $+18 (=0x00a3e4)
            }
            else
            {
//00a3d4 0c44 001b                CMP.W    #27,D4
//00a3d8 6606                     BNE      $+8 (=0x00a3e0)
              if (D4W ==27)
              {
//00a3da 383c 002e                MOVE.W   #46,D4
                D4W = 46;
//00a3de 6004                     BRA      $+6 (=0x00a3e4)
              }
              else
              {
//00a3e0 d87c 0041                ADD.W    #65,D4
                 D4W += 65;
              };
            };
          };
//00a3e4 3004                     MOVE.W   D4,D0
//00a3e6 16c0                     MOVE.B   D0,(A3)+
          *(result++) = D4B;
//00a3e8 601a                     BRA      $+28 (=0x00a404)
        }
        else
        {
//00a3ea 0c44 001c                CMP.W    #28,D4
//00a3ee 6606                     BNE      $+8 (=0x00a3f6)
          if (D4W == 28)
          {
//00a3f0 16ee ffff                MOVE.B   -1(A6),(A3)+
            *(result++) = LOCAL_1;
//00a3f4 600e                     BRA      $+16 (=0x00a404)
          }
          else
          {
//00a3f6 0c44 001e                CMP.W    #30,D4
//00a3fa 6206                     BHI      $+8 (=0x00a402)
            if (D4W <= 30)
            {
//00a3fc 3d44 fffc                MOVE.W   D4,-4(A6)
              LOCAL_4 = D4W;
//00a400 6002                     BRA      $+4 (=0x00a404)
            }
            else
            {
//00a402 6004                     BRA      $+6 (=0x00a408)
              break;
            };
          };
        };
      };
//00a404 6000 ff00                BRA      $-254 (=0x00a306)
    };
  };
//00a408 4a47                     TST.W    D7
//00a40a 6606                     BNE      $+8 (=0x00a412)
  if (D7W == 0)
  {
//00a40c 16bc ff81                MOVE.B   #-127,(A3)
    *result = -127;
//00a410 6002                     BRA      $+4 (=0x00a414)
  }
  else
  {
//00a412 4213                     CLR.B    (A3)
    *result = 0;
  };
//00a414 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//00a418 4e5e                     UNLK     A6
//00a41a 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//  TAG00a41c
i16 Hash(i16 P1,i16 P2,i16 P3)
{ // Called from only one place in program.
  // P1 is a function of mapX, mapY
  // P2 is a function of level, width, and height
  // P3 is a small integer like 30 used as arithmetic modulus.
  // Result is     hash(P1,P2,dungeonDatIndex[0]) % P3
  dReg D0, D1;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00a41c 4e56 0000                LINK     A6,#0
//00a420 302e 0008                MOVE.W   8(A6),D0
  D0L = (ui16)P1 * 31417;
//00a424 c0fc 7ab9                MULU     #31417,D0
//00a428 e248                     LSR.W    #1,D0
  D0W = (I16)((D0W >>= 1) & 0x7fff);
//00a42a 322e 000a                MOVE.W   10(A6),D1
//00a42e c2fc 000b                MULU     #11,D1
  D1L = (ui16)P2 * 11;
//00a432 d041                     ADD.W    D1,D0
  D0W = sw(D0W + D1W);
//00a434 206c d732                MOVE.L   -10446(A4),A0
//00a438 d050                     ADD.W    (A0),D0
  D0W = sw(D0W + d.dungeonDatIndex[0]); // = 13 in my game
//00a43a e448                     LSR.W    #2,D0
//00a43c c0bc 0000 ffff           AND.L    #65535,D0
  D0W = (I16)((D0W >> 2) & 0x3fff);
  D0L &= 0xffff;
//00a442 80ee 000c                DIVU     12(A6),D0
  D0W = sw(D0L % (ui16)(P3));
  return D0W;
//00a446 4840                     SWAP     D0
//00a448 4e5e                     UNLK     A6
//00a44a 4e75                     RTS
}

i16 TAG00a44c(i16 P1,i16 P2,i32 mapX,i32 P4,i16 P5)
{ //P4 = n*mapY  where n=facing
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00a44c 4e56 fffe                LINK     A6,#-2
  i16 LOCAL_2;
//00a450 302e 0008                MOVE.W   8(A6),D0
//00a454 6744                     BEQ      $+70 (=0x00a49a)
  if (P1 != 0)
  {
//00a456 3f2e 0010                MOVE.W   16(A6),-(A7)
//00a45a 303c 0bb8                MOVE.W   #3000,D0
//00a45e 322c d746                MOVE.W   -10426(A4),D1
//00a462 ed41                     ASL.W    #6,D1
//00a464 d041                     ADD.W    D1,D0
//00a466 d06c d744                ADD.W    -10428(A4),D0
//00a46a d06c d742                ADD.W    -10430(A4),D0
//00a46e 3f00                     MOVE.W   D0,-(A7)
//00a470 303c 07d0                MOVE.W   #2000,D0
//00a474 322e 000c                MOVE.W   12(A6),D1
//00a478 eb41                     ASL.W    #5,D1
//00a47a d041                     ADD.W    D1,D0
//00a47c d06e 000e                ADD.W    14(A6),D0
//00a480 3f00                     MOVE.W   D0,-(A7)
//00a482 4eba ff98                JSR      $-102 (=0x00a41c)
//00a486 5c8f                     ADDQ.L   #6,A7
//00a488 3d40 fffe                MOVE.W   D0,-2(A6)
    LOCAL_2 = Hash(
                (I16)(2000 + (mapX<<5) + P4),
                (I16)(3000 + (d.LoadedLevel<<6) + d.width + d.height),
                P5);
//00a48c b06e 000a                CMP.W    10(A6),D0
//00a490 6c08                     BGE      $+10 (=0x00a49a)
    if (LOCAL_2 < P2)
    {
//00a492 302e fffe                MOVE.W   -2(A6),D0
      return sw(LOCAL_2 + 1);
//00a496 5240                     ADDQ.W   #1,D0
//00a498 6004                     BRA      $+6 (=0x00a49e)
    };
  };
//00a49a 303c 0000                MOVE.W   #0,D0
//00a49e 4e5e                     UNLK     A6
//00a4a0 4e75                     RTS
  return 0;
}

void TAG00a4a2(SUMMARIZEROOMDATA *rslt ,
               i16 P2,       //12
               i16 P3,       //14
               i16 P4,       //16
               i32 facing,   //18
               i32 mapX,     //20
               i32 mapY,     //22
               i32 P8)       //24
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00a4a2 4e56 0000                LINK     A6,#0
  dReg D0, D4;
//00a4a6 48e7 0f10                MOVEM.L  D4567A3,-(A7)
//00a4aa 266e 0008                MOVE.L   8(A6),A3
//00a4ae 3e2e 0012                MOVE.W   18(A6),D7
//00a4b2 3c2e 0014                MOVE.W   20(A6),D6
//00a4b6 3a2e 0016                MOVE.W   22(A6),D5
  //D7W = P5;
  //D6W = P6;
  //D5W = P7;
//00a4ba 206c d750                MOVE.L   -10416(A4),A0
//00a4be 3828 000a                MOVE.W   10(A0),D4
//00a4c2 c87c 00f0                AND.W    #240,D4
//00a4c6 e84c                     LSR.W    #4,D4
  D4W = d.pCurLevelDesc->w10b4_7();
  mapY++;
  facing++;
//00a4c8 3f3c 001e                MOVE.W   #30,-(A7)
//00a4cc 5245                     ADDQ.W   #1,D5
//00a4ce 3005                     MOVE.W   D5,D0
//00a4d0 5247                     ADDQ.W   #1,D7
//00a4d2 3207                     MOVE.W   D7,D1
//00a4d4 c27c 0003                AND.W    #3,D1
//00a4d8 5241                     ADDQ.W   #1,D1
//00a4da c1c1                     MULU     D1,D0
//00a4dc 3f00                     MOVE.W   D0,-(A7)
//00a4de 3f06                     MOVE.W   D6,-(A7)
//00a4e0 3f04                     MOVE.W   D4,-(A7)
//00a4e2 3f2e 000c                MOVE.W   12(A6),-(A7)
//00a4e6 4eba ff64                JSR      $-154 (=0x00a44c)
//00a4ea defc 000a                ADD.W    #10,A7
//00a4ee 3740 0004                MOVE.W   D0,4(A3)
  rslt->word4[0] = TAG00a44c(P2,
                             D4W,
                             mapX,
                             mapY*((facing&3)+1),
                             30);
  facing++;
//00a4f2 3f3c 001e                MOVE.W   #30,-(A7)
//00a4f6 3005                     MOVE.W   D5,D0
//00a4f8 5247                     ADDQ.W   #1,D7
//00a4fa 3207                     MOVE.W   D7,D1
//00a4fc c27c 0003                AND.W    #3,D1
//00a500 5241                     ADDQ.W   #1,D1
//00a502 c1c1                     MULU     D1,D0
//00a504 3f00                     MOVE.W   D0,-(A7)
//00a506 3f06                     MOVE.W   D6,-(A7)
//00a508 3f04                     MOVE.W   D4,-(A7)
//00a50a 3f2e 000e                MOVE.W   14(A6),-(A7)
//00a50e 4eba ff3c                JSR      $-194 (=0x00a44c)
//00a512 defc 000a                ADD.W    #10,A7
//00a516 3740 0006                MOVE.W   D0,6(A3)
  rslt->word4[1] = TAG00a44c(P3,D4W,mapX,mapY*((facing&3)+1),30);
  facing++;
//00a51a 3f3c 001e                MOVE.W   #30,-(A7)
//00a51e 3005                     MOVE.W   D5,D0
//00a520 5345                     SUBQ.W   #1,D5
//00a522 5247                     ADDQ.W   #1,D7
//00a524 3207                     MOVE.W   D7,D1
//00a526 c27c 0003                AND.W    #3,D1
//00a52a 5241                     ADDQ.W   #1,D1
//00a52c c1c1                     MULU     D1,D0
//00a52e 3f00                     MOVE.W   D0,-(A7)
//00a530 3f06                     MOVE.W   D6,-(A7)
//00a532 3f04                     MOVE.W   D4,-(A7)
//00a534 3f2e 0010                MOVE.W   16(A6),-(A7)
//00a538 4eba ff12                JSR      $-236 (=0x00a44c)
//00a53c defc 000a                ADD.W    #10,A7
//00a540 3740 0008                MOVE.W   D0,8(A3)
  rslt->word4[2] = TAG00a44c(P4,D4W,mapX,mapY*((facing&3)+1),30);
  mapY--;
//00a544 302e 0018                MOVE.W   24(A6),D0
//00a548 6618                     BNE      $+26 (=0x00a562)
  if (    (P8 != 0)
       || (mapX < 0)
       || (mapX >= d.width)
       || (mapY < 0)
       || (mapY >= d.height) )

//00a54a 4a46                     TST.W    D6
//00a54c 6d14                     BLT      $+22 (=0x00a562)
//00a54e 3006                     MOVE.W   D6,D0
//00a550 b06c d744                CMP.W    -10428(A4),D0
//00a554 6c0c                     BGE      $+14 (=0x00a562)
//00a556 4a45                     TST.W    D5
//00a558 6d08                     BLT      $+10 (=0x00a562)
//00a55a 3005                     MOVE.W   D5,D0
//00a55c b06c d742                CMP.W    -10430(A4),D0
//00a560 6d30                     BLT      $+50 (=0x00a592)
  {
//00a562 383c 0002                MOVE.W   #2,D4
//00a566 6024                     BRA      $+38 (=0x00a58c)
    for (D4W = 2; D4W <= 4; D4W++)
    {
//00a568 3004                     MOVE.W   D4,D0
//00a56a e380                     ASL.L    #1,D0
//00a56c 204b                     MOVE.L   A3,A0
//00a56e d0c0                     ADD.W    D0,A0
//00a570 3010                     MOVE.W   (A0),D0
//00a572 5340                     SUBQ.W   #1,D0
//00a574 3f00                     MOVE.W   D0,-(A7)
//00a576 4eba f070                JSR      $-3982 (=0x0095e8)
//00a57a 548f                     ADDQ.L   #2,A7
      D0W = TAG0095e8(sw(rslt->word4[D4W-2]-1));
//00a57c 4a40                     TST.W    D0
//00a57e 670a                     BEQ      $+12 (=0x00a58a)
      if (D0W != 0)
      {
//00a580 3004                     MOVE.W   D4,D0
//00a582 e380                     ASL.L    #1,D0
//00a584 204b                     MOVE.L   A3,A0
//00a586 d0c0                     ADD.W    D0,A0
//00a588 4250                     CLR.W    (A0)
        rslt->word4[D4W-2] = 0;
      };
//00a58a 5244                     ADDQ.W   #1,D4

//00a58c 0c44 0004                CMP.W    #4,D4
//00a590 6fd6                     BLE      $-40 (=0x00a568)
    };
  };
//00a592 4cdf 08f0                MOVEM.L  (A7)+,D4567A3
//00a596 4e5e                     UNLK     A6
//00a598 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG00a9d4
void LoadLevel(i32 newLevel)
{
  //i32 D7;
  dReg D0;
  //aReg A0, A1;
  //LEVELDESC *pA0;
  //i16 *pwA0;
  //D7W=P1;
  D0W = d.LoadedLevel;
  if (D0W != newLevel)
  {
    d.LoadedLevel = sw(newLevel);
    d.LevelCellFlags = d.pppdPointer10450[newLevel];
    d.pCurLevelDesc = &d.pLevelDescriptors[newLevel];
    d.width = sw(d.pCurLevelDesc->LastColumn()+1);
    d.height = sw(d.pCurLevelDesc->LastRow()+1);
    D0W = BITS8_11(d.pCurLevelDesc->word14);
    //D0W  <<= 1;
    //A0 = (pnt)d.Word9760;
    //A0 += D0W;
    //A1=A0;
    //pwA0 = d.Word10434[0];
    //D0L=0;
    //A0 = (pnt)pwA0;
    //MoveWords(A0, A1, D0W); // 0x0031c4
    d.Word10434[0] = d.Word9760[D0W];
    D0W = BITS12_15(d.pCurLevelDesc->word14);
    //D0L <<= 1;
    //A0 = (pnt)d.Word9760;
    //A0 += D0W;
    //A1 = A0;
    //pwA0 = &d.Word10434[1];
    //D0L=0;
    //A0 = (pnt)pwA0;
    //MoveWords(A0, A1, D0W); // 0031c4;
    d.Word10434[1] = d.Word9760[D0W];
    D0W = d.puwPointer10458[newLevel];
    d.pwPointer10420 = &d.pwPointer10454[D0W];
  };
}

//   TAG00aa9c
void SetPartyLevel(i32 level)
{
  dReg D1;
  d.partyLevel = sw(level);
  LoadLevel(level);
  d.EndOfCELLFLAGS = d.LevelCellFlags[d.width-1] + d.height;//pdD0 = (CELLFLAG *)pD0;
  d.NumWallDecoration = d.pCurLevelDesc->NumWallDecoration();
  D1W=d.pCurLevelDesc->numMonsterType();
  memmove(d.WallDecorationTOC, //For this level
          d.EndOfCELLFLAGS + D1W,
          d.NumWallDecoration);
  d.WallDecorationTOC[d.NumWallDecoration] = 0;
  D1W = sw(D1W + d.NumWallDecoration);
  memmove(d.Byte10378,
          d.EndOfCELLFLAGS + D1W,
          d.pCurLevelDesc->w10b8_11());
  D1W = sw(D1W + d.pCurLevelDesc->w10b8_11());
  memmove(d.Byte10396,
          d.EndOfCELLFLAGS + D1W,
          d.pCurLevelDesc->w12b0_3());
}

//*********************************************************
//
//*********************************************************
//  TAG00ad18
i16 SetMonsterPositionBits(i16 P1,i32 monsterNumber,i32 Pos)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00ad18 4e56 fff6                LINK     A6,#-10
//00ad1c 48e7 0300                MOVEM.L  D67,-(A7)
//00ad20 3e2e 000a                MOVE.W   10(A6),D7
//00ad24 3c2e 000c                MOVE.W   12(A6),D6
  Pos &= 3; // Some folks forget!
//00ad28 cc7c 0003                AND.W    #3,D6
//00ad2c 3207                     MOVE.W   D7,D1
//00ad2e e341                     ASL.W    #1,D1
//00ad30 3e01                     MOVE.W   D1,D7
//00ad32 3006                     MOVE.W   D6,D0
//00ad34 e360                     ASL.W    D1,D0
//00ad36 3c00                     MOVE.W   D0,D6
  Pos <<= 2*monsterNumber; //locate position bits
//00ad38 302e 0008                MOVE.W   8(A6),D0
//00ad3c 323c 0003                MOVE.W   #3,D1
//00ad40 ef61                     ASL.W    D7,D1
//00ad42 4641                     NOT.W    D1
//00ad44 c041                     AND.W    D1,D0
  P1 &= ~(3 << (2*monsterNumber)); // mask out unwanted bits
//00ad46 3d40 fff6                MOVE.W   D0,-10(A6)
//00ad4a 3006                     MOVE.W   D6,D0
//00ad4c 806e fff6                OR.W     -10(A6),D0
  return sw(P1 | Pos);
//00ad50 4cdf 00c0                MOVEM.L  (A7)+,D67
//00ad54 4e5e                     UNLK     A6
//00ad56 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//  TAG00ad58
i32 NextMonsterUpdateTime(ITEM16 *P1,i16 monsterIndex,i16 P3)
{
  dReg D0, D1, D4, D5, D6, D7;
  ITEM16 *pI16A3;
  DB4    *pDB4A2;
  MONSTERTYPE mtD6;
  MONSTERTYPE mt_4;
  bool bool_2;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00ad58 4e56 fffc                LINK     A6,#-4
//00ad5c 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//00ad60 266e 0008                MOVE.L   8(A6),A3
  pI16A3 = P1;
//00ad64 3e2e 000c                MOVE.W   12(A6),D7
  D7W = monsterIndex;
//00ad68 3013                     MOVE.W   (A3),D0
//00ad6a 222c d6f0                MOVE.L   -10512(A4),D1
//00ad6e c1fc 0010                MULS     #16,D0
//00ad72 2041                     MOVE.L   D1,A0
//00ad74 d0c0                     ADD.W    D0,A0
//00ad76 41d0                     LEA      (A0),A0
//00ad78 2448                     MOVE.L   A0,A2
  pDB4A2 = GetRecordAddressDB4(RN(0,4,pI16A3->word0));
  //pDB4A2 = & ((DB4 *)(d.misc1052eight[4]))[pI16A3->word0];
//00ad7a 4245                     CLR.W    D5
//00ad7c 1a2a 0004                MOVE.B   4(A2),D5
  mt_4 = pDB4A2->monsterType();
//00ad80 3d45 fffc                MOVE.W   D5,-4(A6)
//00ad84 cbfc 001a                MULS     #26,D5
//00ad88 41ec da44                LEA      -9660(A4),A0
//00ad8c d0c5                     ADD.W    D5,A0
//00ad8e 3a10                     MOVE.W   (A0),D5
  D5W = d.MonsterDescriptor[mt_4].word4;
//00ad90 4a47                     TST.W    D7
//00ad92 5dc0                     SLT      D0
//00ad94 c07c 0001                AND.W    #1,D0
//00ad98 3d40 fffe                MOVE.W   D0,-2(A6)
  bool_2 = D7W < 0;
//00ad9c 670a                     BEQ      $+12 (=0x00ada8)
  if (bool_2)
  {
//00ad9e 3e2a 000e                MOVE.W   14(A2),D7
//00ada2 ce7c 0060                AND.W    #96,D7
//00ada6 ea4f                     LSR.W    #5,D7
    D7W = BITS5_6(pDB4A2->word14);
  };
  do  // while D7W >= 0
  {
//00ada8 41eb 000c                LEA      12(A3),A0
//00adac d0c7                     ADD.W    D7,A0
//00adae 4246                     CLR.W    D6
//00adb0 1c10                     MOVE.B   (A0),D6
    D6W = pI16A3->uByte12[D7W];
//00adb2 cc7c 00c0                AND.W    #192,D6
    D6W &= 0xc0;
//00adb6 3805                     MOVE.W   D5,D4
//00adb8 e84c                     LSR.W    #4,D4
//00adba e04c                     LSR.W    #8,D4
//00adbc c87c 0003                AND.W    #3,D4
    D4W = BITS12_13(D5W);
//00adc0 6724                     BEQ      $+38 (=0x00ade6)
    if (D4W != 0)
    {
//00adc2 4ead 0090                JSR      144(A5)    (=0x001030=Random)
//00adc6 c0bc 0000 ffff           AND.L    #65535,D0
//00adcc 80c4                     DIVU     D4,D0
//00adce 4840                     SWAP     D0
//00add0 3800                     MOVE.W   D0,D4
      D4W = (i16)STRandom(D4W);
//00add2 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00add6 4a40                     TST.W    D0
//00add8 670a                     BEQ      $+12 (=0x00ade4)
      if (STRandomBool())
      {
//00adda 3004                     MOVE.W   D4,D0
//00addc 4440                     NEG.W    D0
//00adde c07c 0007                AND.W    #7,D0
//00ade2 3800                     MOVE.W   D0,D4
        D4W = (I16)((-D4W) & 7);
      };
//00ade4 8c44                     OR.W     D4,D6
      D6W |= D4W;
    };
//00ade6 3805                     MOVE.W   D5,D4
//00ade8 ec4c                     LSR.W    #6,D4
//00adea e04c                     LSR.W    #8,D4
//00adec c87c 0003                AND.W    #3,D4
    D4W = BITS14_15(D5W);
//00adf0 6728                     BEQ      $+42 (=0x00ae1a)
    if (D4W != 0)
    {
//00adf2 4ead 0090                JSR      144(A5)    (=0x001030=Random)
//00adf6 c0bc 0000 ffff           AND.L    #65535,D0
//00adfc 80c4                     DIVU     D4,D0
//00adfe 4840                     SWAP     D0
//00ae00 3800                     MOVE.W   D0,D4
      D4W = (i16)STRandom(D4W);
//00ae02 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00ae06 4a40                     TST.W    D0
//00ae08 670a                     BEQ      $+12 (=0x00ae14)
      if (STRandomBool())
      {
//00ae0a 3004                     MOVE.W   D4,D0
//00ae0c 4440                     NEG.W    D0
//00ae0e c07c 0007                AND.W    #7,D0
//00ae12 3800                     MOVE.W   D0,D4
        D4W = (I16)((-D4W) & 7);
      };
//00ae14 3004                     MOVE.W   D4,D0
//00ae16 e740                     ASL.W    #3,D0
//00ae18 8c40                     OR.W     D0,D6
      D6W |= D4W << 3;
    };
//00ae1a 302e 000e                MOVE.W   14(A6),D0
//00ae1e 676c                     BEQ      $+110 (=0x00ae8c)
    if (P3 != 0)
    {
//00ae20 3005                     MOVE.W   D5,D0
//00ae22 c07c 0200                AND.W    #512,D0
//00ae26 675a                     BEQ      $+92 (=0x00ae82)
      if (D5W & 0x200)
      {
//00ae28 3006                     MOVE.W   D6,D0
//00ae2a c07c 0080                AND.W    #128,D0
//00ae2e 672e                     BEQ      $+48 (=0x00ae5e)
//00ae30 0c6e 0012 fffc           CMP.W    #18,-4(A6)
//00ae36 6626                     BNE      $+40 (=0x00ae5e)
        if ( (D6W & 0x80) && (mt_4 == mon_DethKnight) )
        {
//00ae38 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00ae3c 4a40                     TST.W    D0
//00ae3e 671c                     BEQ      $+30 (=0x00ae5c)
          if (STRandomBool())
          {
//00ae40 303c 0040                MOVE.W   #64,D0
//00ae44 b146                     EOR.W    D0,D6
            D6W ^= 0x40;
//00ae46 3f3c 0001                MOVE.W   #1,-(A7)
//00ae4a 3f2c cd26                MOVE.W   -13018(A4),-(A7)
//00ae4e 3f2c cd28                MOVE.W   -13016(A4),-(A7)
//00ae52 3f3c 0010                MOVE.W   #16,-(A7)
//00ae56 4ead 0120                JSR      288(A5)    (=0x00219a)
//00ae5a 508f                     ADDQ.L   #0,A7
            QueueSound(16, d.Word13016, d.Word13018, 1);
          };
//00ae5c 6022                     BRA      $+36 (=0x00ae80)
        }
        else
        {
//00ae5e 3006                     MOVE.W   D6,D0
//00ae60 c07c 0080                AND.W    #128,D0
//00ae64 6708                     BEQ      $+10 (=0x00ae6e)
//00ae66 3005                     MOVE.W   D5,D0
//00ae68 c07c 0400                AND.W    #1024,D0
//00ae6c 6612                     BNE      $+20 (=0x00ae80)
          if (   ((D6W & 0x80) == 0)
              || ((D5W & 0x400) == 0) )
          {
//00ae6e 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00ae72 4a40                     TST.W    D0
//00ae74 6706                     BEQ      $+8 (=0x00ae7c)
            if (STRandomBool())
            {
//00ae76 8c7c 0040                OR.W     #64,D6
              D6W |= 0x40;
//00ae7a 6004                     BRA      $+6 (=0x00ae80)
            }
            else
            {
              D6W &= 0xffbf;
//00ae7c cc7c ffbf                AND.W    #65471,D6
            };
          };
        };
//00ae80 6004                     BRA      $+6 (=0x00ae86)
      }
      else
      {
//00ae82 cc7c ffbf                AND.W    #65471,D6
        D6W &= 0xffbf;
      };
//00ae86 8c7c 0080                OR.W     #128,D6
      D6W |= 0x80;
//00ae8a 6024                     BRA      $+38 (=0x00aeb0)
    }
    else
    {
//00ae8c 3005                     MOVE.W   D5,D0
//00ae8e c07c 0004                AND.W    #4,D0
//00ae92 6714                     BEQ      $+22 (=0x00aea8)
      if (D5W & 4)
      {
//00ae94 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00ae98 4a40                     TST.W    D0
//00ae9a 6706                     BEQ      $+8 (=0x00aea2)
        if (STRandomBool())
        {
//00ae9c 8c7c 0040                OR.W     #64,D6
          D6W |= 0x40;
//00aea0 6004                     BRA      $+6 (=0x00aea6)
        }
        else
        {
//00aea2 cc7c ffbf                AND.W    #65471,D6
          D6W &= 0xffbf;
        };
//00aea6 6004                     BRA      $+6 (=0x00aeac)
      }
      else
      {
//00aea8 cc7c ffbf                AND.W    #65471,D6
        D6W &= 0xffbf;
      };
//00aeac cc7c ff7f                AND.W    #65407,D6
      D6W &= 0xff7f;
    };
//00aeb0 3006                     MOVE.W   D6,D0
//00aeb2 41eb 000c                LEA      12(A3),A0
//00aeb6 d0c7                     ADD.W    D7,A0
//00aeb8 1080                     MOVE.B   D0,(A0)
    pI16A3->uByte12[D7W] = D6B;
//00aeba 302e fffe                MOVE.W   -2(A6),D0
//00aebe 670a                     BEQ      $+12 (=0x00aeca)
    if (!bool_2) break;
//00aec0 3007                     MOVE.W   D7,D0
//00aec2 5347                     SUBQ.W   #1,D7
    D7W--;
//00aec4 4a40                     TST.W    D0
//00aec6 6600 fee0                BNE      $-286 (=0x00ada8)
  } while  (D7W >= 0);
//00aeca 4246                     CLR.W    D6
//00aecc 1c2a 0004                MOVE.B   4(A2),D6
  mtD6 = pDB4A2->monsterType();
//00aed0 cdfc 001a                MULS     #26,D6
//00aed4 41ec da54                LEA      -9644(A4),A0
//00aed8 d0c6                     ADD.W    D6,A0
//00aeda 3c10                     MOVE.W   (A0),D6
  D6W = d.MonsterDescriptor[mtD6].word20;
//00aedc 202c d23a                MOVE.L   -11718(A4),D0
  //D0L = d.Time;
//00aee0 322e 000e                MOVE.W   14(A6),D1
//00aee4 670a                     BEQ      $+12 (=0x00aef0)
  if (P3 != 0)
  {
//00aee6 3206                     MOVE.W   D6,D1
//00aee8 e049                     LSR.W    #8,D1
//00aeea c27c 000f                AND.W    #15,D1
    D1W = (I16)((D6W >> 8) & 15);
//00aeee 6008                     BRA      $+10 (=0x00aef8)
  }
  else
  {
//00aef0 3206                     MOVE.W   D6,D1
//00aef2 e849                     LSR.W    #4,D1
//00aef4 c27c 000f                AND.W    #15,D1
    D1W = (I16)((D6W >> 4) & 15);
  };
//00aef8 c2bc 0000 ffff           AND.L    #65535,D1
//00aefe d081                     ADD.L    D1,D0
  D0L = d.Time + (D1L & 0xffff);
//00af00 2f00                     MOVE.L   D0,-(A7)
//00af02 4ead 0084                JSR      132(A5)    (=0x001050=RandomBool)
//00af06 2200                     MOVE.L   D0,D1
  D0L += STRandomBool();
//00af08 201f                     MOVE.L   (A7)+,D0
//00af0a c2bc 0000 ffff           AND.L    #65535,D1
//00af10 d081                     ADD.L    D1,D0
  return D0L;
//00af12 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//00af16 4e5e                     UNLK     A6
//00af18 4e75                     RTS
}

void TAG00af1a(i32 mapX,i32 mapY)
{
  DB4 *DB4A3;
  TIMER timer;
  RN obj;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00af1a 4e56 fff6                LINK     A6,#-10
//00af1e 2f0b                     MOVE.L   A3,-(A7)
//00af20 3f2e 000a                MOVE.W   10(A6),-(A7)
//00af24 3f2e 0008                MOVE.W   8(A6),-(A7)
//00af28 4eba fc44                JSR      $-954 (=0x00ab6e)
//00af2c 588f                     ADDQ.L   #4,A7
//00af2e 3f00                     MOVE.W   D0,-(A7)
//00af30 4eba ea52                JSR      $-5548 (=0x009984)
//00af34 548f                     ADDQ.L   #2,A7
//00af36 2640                     MOVE.L   D0,A3
  obj = FindFirstMonster(mapX, mapY);
  if ((obj != RNnul) && (obj != RNeof))
  {
    DB4A3 = GetRecordAddressDB4(obj);
//00af38 302b 000e                MOVE.W   14(A3),D0
//00af3c c07c 000f                AND.W    #15,D0
//00af40 0c40 0004                CMP.W    #4,D0
//00af44 6514                     BCS      $+22 (=0x00af5a)
    if ((DB4A3->word14 & 15) >= 4)
    {
//00af46 303c 0000                MOVE.W   #0,D0
//00af4a c07c 000f                AND.W    #15,D0
//00af4e 3600                     MOVE.W   D0,D3
//00af50 026b fff0 000e           AND.W    #65520,14(A3)
//00af56 876b 000e                OR.W     D3,14(A3)
      DB4A3->word14 = (I16)(DB4A3->word14 & 0xfff0);
    };
  }
  else
  {
    HopefullyNotNeeded();
    DB4A3 = NULL;
    timer.uByte5 = 255;
    // This happened in the prison when the only
    // character died from lack of food.
  };
//00af5a 202c d23a                MOVE.L   -11718(A4),D0
//00af5e 5280                     ADDQ.L   #1,D0
//00af60 322c d746                MOVE.W   -10426(A4),D1
//00af64 48c1                     EXT.L    D1
//00af66 263c 0000 0018           MOVE.L   #24,D3
//00af6c e7a1                     ASL.L    D3,D1
//00af6e 8081                     OR.L     D1,D0
//00af70 2d40 fff6                MOVE.L   D0,-10(A6)
  timer.time = (d.Time+1) | (d.LoadedLevel << 24);
//00af74 1d7c 0025 fffa           MOVE.B   #37,-6(A6)
  timer.function = 37;
//00af7a 303c 00ff                MOVE.W   #255,D0
//00af7e 4241                     CLR.W    D1
//00af80 122b 0004                MOVE.B   4(A3),D1
//00af84 c3fc 001a                MULS     #26,D1
//00af88 41ec da46                LEA      -9658(A4),A0
//00af8c d0c1                     ADD.W    D1,A0
//00af8e 4243                     CLR.W    D3
//00af90 1610                     MOVE.B   (A0),D3
//00af92 9043                     SUB.W    D3,D0
//00af94 1d40 fffb                MOVE.B   D0,-5(A6)
  timer.uByte5 =
        ub(255 - d.MonsterDescriptor[DB4A3->monsterType()].uByte6);
//00af98 422e fffe                CLR.B    -2(A6)
  timer.uByte8=0;
//00af9c 302e 0008                MOVE.W   8(A6),D0
//00afa0 1d40 fffc                MOVE.B   D0,-4(A6)
  timer.uByte6 = (i8)mapX;
//00afa4 302e 000a                MOVE.W   10(A6),D0
//00afa8 1d40 fffd                MOVE.B   D0,-3(A6)
  timer.uByte7 = (i8)mapY;
//00afac 486e fff6                PEA      -10(A6)
//00afb0 4ead 0366                JSR      870(A5)    (=0x00ffbe)
//00afb4 588f                     ADDQ.L   #4,A7
  SetTimer(&timer);
//00afb6 265f                     MOVE.L   (A7)+,A3
//00afb8 4e5e                     UNLK     A6
//00afba 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG00afbc
void DeleteMonsterMovementTimers(i32 mapX,i32 mapY)
{ //Timer types 29 to 41
  dReg D0, D4, D5;
  TIMER *psA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00afbc 4e56 0000                LINK     A6,#0
//00afc0 48e7 0f10                MOVEM.L  D4567A3,-(A7)
//00afc4 3e2e 0008                MOVE.W   8(A6),D7
//00afc8 3c2e 000a                MOVE.W   10(A6),D6
  //D7W = P1;
  //D6W = P2;
//00afcc 266c cd3e                MOVE.L   -12994(A4),A3
  psA3 = d.Timers;
//00afd0 4245                     CLR.W    D5
//00afd2 6048                     BRA      $+74 (=0x00b01c)
  for (D5W=0; D5W<d.MaxTimers; psA3++,D5W++)
  {
//00afd4 2013                     MOVE.L   (A3),D0
//00afd6 263c 0000 0018           MOVE.L   #24,D3
//00afdc e6a0                     LSR.L    D3,D0
    D0L = (psA3->time >> 24) & 0xff;
//00afde 322c d746                MOVE.W   -10426(A4),D1
//00afe2 b041                     CMP.W    D1,D0
//00afe4 6630                     BNE      $+50 (=0x00b016)
    if (D0L != d.LoadedLevel) continue;
//00afe6 4240                     CLR.W    D0
//00afe8 102b 0004                MOVE.B   4(A3),D0
    D4W = D0W = psA3->function;
//00afec 3800                     MOVE.W   D0,D4
//00afee 0c40 001c                CMP.W    #28,D0
//00aff2 6322                     BLS      $+36 (=0x00b016)
    if (D0W < 29) continue;
//00aff4 0c44 002a                CMP.W    #42,D4
//00aff8 641c                     BCC      $+30 (=0x00b016)
    if (D4W > 41) continue;
//00affa 4240                     CLR.W    D0
//00affc 102b 0006                MOVE.B   6(A3),D0
//00b000 b047                     CMP.W    D7,D0
//00b002 6612                     BNE      $+20 (=0x00b016)
    if (psA3->uByte6 != mapX) continue;
//00b004 4240                     CLR.W    D0
//00b006 102b 0007                MOVE.B   7(A3),D0
//00b00a b046                     CMP.W    D6,D0
//00b00c 6608                     BNE      $+10 (=0x00b016)
    if (psA3->uByte7 != mapY) continue;
    TraceTimer(psA3, D5W, "Delete");
//00b00e 3f05                     MOVE.W   D5,-(A7)
//00b010 4ead 0360                JSR      864(A5)    (=0x00ff4c)
//00b014 548f                     ADDQ.L   #2,A7
    DeleteTimer(D5W);
//00b016 d6fc 000a                ADD.W    #10,A3
//00b01a 5245                     ADDQ.W   #1,D5

//00b01c 3005                     MOVE.W   D5,D0
//00b01e b06c cd42                CMP.W    -12990(A4),D0
//00b022 65b0                     BCS      $-78 (=0x00afd4)
  };
//00b024 4cdf 08f0                MOVEM.L  (A7)+,D4567A3
//00b028 4e5e                     UNLK     A6
//00b02a 4e75                     RTS
}

void TAG00b062(RN P1,i32 mapX,i32 mapY)
{
  dReg D0, D1, D6;
  aReg A0;
  ITEM26 *pI26A0;
  ITEM16 *pi16A2;
  DB4    *pDB4A3;
  i16 item16Index;
  i16 LOCAL_6;
  static i32 message = 0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00b062 4e56 fff8                LINK     A6,#-8
//00b066 48e7 0330                MOVEM.L  D67A23,-(A7)
//00b06a 246c cd2e                MOVE.L   -13010(A4),A2
//00b06e 426e fff8                CLR.W    -8(A6)
  A0 = NULL;
  pi16A2 = d.Item16;
  item16Index = 0;
//00b072 6016                     BRA      $+24 (=0x00b08a)
  while (pi16A2->word0 >= 0)
  {
//00b074 526e fff8                ADDQ.W   #1,-8(A6)
    item16Index++;
//00b078 302e fff8                MOVE.W   -8(A6),D0
//00b07c b06c cd2c                CMP.W    -13012(A4),D0
//00b080 6504                     BCS      $+6 (=0x00b086)
    if (item16Index >= d.MaxITEM16)
    {
      UI_MessageBox("The Atari code did bad things at this point.\n"
                    "I am going to try to increase the maximum.\n"
                    "If you don't like this, close the program\n"
                    "before saving the game.",
                    "Too many monsters",
                    MESSAGE_OK);
      if (d.MaxITEM16 < 300)
      {
        i32 i, newMax;
        newMax = d.MaxITEM16 + 50;
        d.Item16 = (ITEM16 *)realloc(d.Item16, 16 * newMax);
        for (i=d.MaxITEM16; i< newMax; i++)
        {
          d.Item16[i].word0 = -1;
        };
        pi16A2 = &d.Item16[d.MaxITEM16];
        d.MaxITEM16 = sw(newMax);
        break;
      }
      else
      {
        UI_MessageBox("I give up",
                      "Cannot reallocate monster data",
                      MESSAGE_OK);
        die (0);
      };
    };
//00b082 6000 00c6                BRA      $+200 (=0x00b14a)
//00b086 d4fc 0010                ADD.W    #16,A2
    pi16A2++;
//00b08a 4a52                     TST.W    (A2)
//00b08c 6ce6                     BGE      $-24 (=0x00b074)
  };
//00b08e 526c cd2a                ADDQ.W   #1,-13014(A4)
  d.ITEM16QueLen++;
//00b092 322e 0008                MOVE.W   8(A6),D1
//00b096 c27c 03ff                AND.W    #1023,D1
//00b09a 3481                     MOVE.W   D1,(A2)
  pi16A2->word0 = D1W = P1.idx();
//00b09c c3fc 0010                MULS     #16,D1
//00b0a0 202c d6f0                MOVE.L   -10512(A4),D0
//00b0a4 d081                     ADD.L    D1,D0
//00b0a6 2640                     MOVE.L   D0,A3
  pDB4A3 = GetRecordAddressDB4(RN(0,dbMONSTER,D1W));
  //pDB4A3 = &((DB4 *)(d.misc1052eight[4]))[D1W];
//00b0a8 156b 0005 0003           MOVE.B   5(A3),3(A2)
  pi16A2->positions() = pDB4A3->groupIndex();
//00b0ae 302e fff8                MOVE.W   -8(A6),D0
//00b0b2 1740 0005                MOVE.B   D0,5(A3)
  pDB4A3->groupIndex(item16Index);
//00b0b6 302e 000a                MOVE.W   10(A6),D0
//00b0ba 1540 000a                MOVE.B   D0,10(A2)
  pi16A2->uByte10 = (i8)mapX;
//00b0be 1540 0008                MOVE.B   D0,8(A2)
  pi16A2->uByte8 = (i8)mapX;
//00b0c2 302e 000c                MOVE.W   12(A6),D0
//00b0c6 1540 000b                MOVE.B   D0,11(A2)
  pi16A2->uByte11 = (i8)mapY;
//00b0ca 1540 0009                MOVE.B   D0,9(A2)
  pi16A2->uByte9 = (i8)mapY;
//00b0ce 202c d23a                MOVE.L   -11718(A4),D0
//00b0d2 90bc 0000 007f           SUB.L    #127,D0
//00b0d8 1540 0004                MOVE.B   D0,4(A2)
  pi16A2->uByte4 = (UI8)(d.Time - 127); // about 20 seconds
//00b0dc 4240                     CLR.W    D0
//00b0de 102b 0004                MOVE.B   4(A3),D0
//00b0e2 c1fc 001a                MULS     #26,D0
//00b0e6 41ec da42                LEA      -9662(A4),A0
//00b0ea d0c0                     ADD.W    D0,A0
  pI26A0 = &d.MonsterDescriptor[pDB4A3->monsterType()];
//00b0ec 3010                     MOVE.W   (A0),D0
//00b0ee c07c 0003                AND.W    #3,D0
//00b0f2 0c40 0001                CMP.W    #1,D0
//00b0f6 57c0                     SEQ      D0
//00b0f8 c07c 0001                AND.W    #1,D0
  SET(D0B, (pI26A0->word2 & 3) == 1);
//00b0fc 3d40 fffa                MOVE.W   D0,-6(A6)
  LOCAL_6 = (I16)(D0W & 1);
//00b100 3c2b 000e                MOVE.W   14(A3),D6
//00b104 cc7c 0060                AND.W    #96,D6
//00b108 ea4e                     LSR.W    #5,D6
  D6W = BITS5_6(pDB4A3->word14);
  do
  {
    // Set all members of group facing same direction.
//00b10a 302b 000e                MOVE.W   14(A3),D0
    D0W = pDB4A3->word14;
//00b10e c07c 0300                AND.W    #768,D0
//00b112 e048                     LSR.W    #8,D0
//00b114 3f00                     MOVE.W   D0,-(A7)
//00b116 3f06                     MOVE.W   D6,-(A7)
//00b118 4240                     CLR.W    D0
//00b11a 102a 0002                MOVE.B   2(A2),D0
//00b11e 3f00                     MOVE.W   D0,-(A7)
//00b120 4eba fbf6                JSR      $-1032 (=0x00ad18)
//00b124 5c8f                     ADDQ.L   #6,A7
    D0W = SetMonsterPositionBits(
             pi16A2->facings(),  //Old facing byte
             D6W,               //MonsterNumber
             BITS8_9(D0W));     //Monster's facing
//00b126 1540 0002                MOVE.B   D0,2(A2)
    pi16A2->facings() = D0B;
//00b12a 3006                     MOVE.W   D6,D0
//00b12c 41ea 000c                LEA      12(A2),A0
//00b130 d0c0                     ADD.W    D0,A0
//00b132 4210                     CLR.B    (A0)
    pi16A2->uByte12[D6W] = 0;
//00b134 3006                     MOVE.W   D6,D0
//00b136 5346                     SUBQ.W   #1,D6
//00b138 4a40                     TST.W    D0
//00b13a 66ce                     BNE      $-48 (=0x00b10a)
    D0W = D6W--;
  } while (D0W != 0);
//00b13c 4267                     CLR.W    -(A7)
//00b13e 3f3c ffff                MOVE.W   #65535,-(A7)
//00b142 2f0a                     MOVE.L   A2,-(A7)
//00b144 4eba fc12                JSR      $-1004 (=0x00ad58)
//00b148 508f                     ADDQ.L   #0,A7
  NextMonsterUpdateTime(pi16A2, -1, 0);
//00b14a 4cdf 0cc0                MOVEM.L  (A7)+,D67A23
//00b14e 4e5e                     UNLK     A6
//00b150 4e75                     RTS
}

void TAG00b152(i32 P1)
{
  ITEM16 *pi16A3;
  DB4 *pDB4A2;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00b152 4e56 0000                LINK     A6,#0
//00b156 48e7 0330                MOVEM.L  D67A23,-(A7)
//
//00b15a 302e 0008                MOVE.W   8(A6),D0
//00b15e b06c cd2c                CMP.W    -13012(A4),D0
//00b162 6212                     BHI      $+20 (=0x00b176)
//
//00b164 302e 0008                MOVE.W   8(A6),D0
//00b168 c1fc 0010                MULS     #16,D0
//00b16c 206c cd2e                MOVE.L   -13010(A4),A0
//00b170 d0c0                     ADD.W    D0,A0
//00b172 4a50                     TST.W    (A0)
//00b174 6c02                     BGE      $+4 (=0x00b178)
  if ( (P1 > d.MaxITEM16) || (d.Item16[P1].word0 < 0) ) return;
//00b176 6070                     BRA      $+114 (=0x00b1e8)
//
//00b178 302e 0008                MOVE.W   8(A6),D0
//00b17c c1fc 0010                MULS     #16,D0
//00b180 206c cd2e                MOVE.L   -13010(A4),A0
//00b184 d0c0                     ADD.W    D0,A0
//00b186 41d0                     LEA      (A0),A0
//00b188 2648                     MOVE.L   A0,A3
  ASSERT(P1 < d.MaxITEM16);
  pi16A3 = &d.Item16[P1];
//00b18a 3013                     MOVE.W   (A3),D0
//00b18c 222c d6f0                MOVE.L   -10512(A4),D1
//00b190 c1fc 0010                MULS     #16,D0
//00b194 2041                     MOVE.L   D1,A0
//00b196 d0c0                     ADD.W    D0,A0
//00b198 41d0                     LEA      (A0),A0
//00b19a 2448                     MOVE.L   A0,A2
  pDB4A2 = GetRecordAddressDB4(RN(0,dbMONSTER,pi16A3->word0));
  //pDB4A2 = &((DB4 *)d.misc1052eight[4])[pi16A3->word0];
//00b19c 536c cd2a                SUBQ.W   #1,-13014(A4)
  d.ITEM16QueLen--;
//00b1a0 156b 0003 0005           MOVE.B   3(A3),5(A2)
  pDB4A2->groupIndex(pi16A3->positions());
//00b1a6 4240                     CLR.W    D0
//00b1a8 102b 0002                MOVE.B   2(A3),D0
//00b1ac c07c 0003                AND.W    #3,D0
//00b1b0 c07c 0003                AND.W    #3,D0
//00b1b4 3600                     MOVE.W   D0,D3
//00b1b6 e143                     ASL.W    #8,D3
//00b1b8 026a fcff 000e           AND.W    #64767,14(A2)
//00b1be 876a 000e                OR.W     D3,14(A2)
  SETWBITS8_9(pDB4A2->word14, pi16A3->facings());
//00b1c2 302a 000e                MOVE.W   14(A2),D0
//00b1c6 c07c 000f                AND.W    #15,D0
//00b1ca 0c40 0004                CMP.W    #4,D0
//00b1ce 6514                     BCS      $+22 (=0x00b1e4)
  if (BITS0_3(pDB4A2->word14) >=4)
  {
//00b1d0 303c 0000                MOVE.W   #0,D0
//00b1d4 c07c 000f                AND.W    #15,D0
//00b1d8 3600                     MOVE.W   D0,D3
//00b1da 026a fff0 000e           AND.W    #65520,14(A2)
//00b1e0 876a 000e                OR.W     D3,14(A2)
    SETWBITS0_3(pDB4A2->word14, 0);
  };
//00b1e4 36bc ffff                MOVE.W   #65535,(A3)
  pi16A3->word0 = -1;
//00b1e8 4cdf 0cc0                MOVEM.L  (A7)+,D67A23
//00b1ec 4e5e                     UNLK     A6
//00b1ee 4e75                     RTS
}

void TAG00bd40(void)
{
  i32 i=0;
  while (d.ITEM16QueLen != 0)
  {
    if ( d.Item16[i].word0 >= 0)
    {
      TAG00b152(i);
    };
    i++;
  };
}

//   TAG00bd70
void ProcessMonstersOnLevel(void)
{ // I don't know what 'process' means.  :-(
  dReg D0;
  RN *prnA2;
  //DUDAD16 *pdA3;
  CELLFLAG *ubA3;
  RN objD5;
  i32 mapX, mapY;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//00bd70 4e56 0000                LINK     A6,#0
//00bd74 48e7 0730                MOVEM.L  D567A23,-(A7)
//00bd78 206c d748                MOVE.L   -10424(A4),A0
//00bd7c 2650                     MOVE.L   (A0),A3
  ubA3 = d.LevelCellFlags[0];
//00bd7e 206c d74c                MOVE.L   -10420(A4),A0
//00bd82 3010                     MOVE.W   (A0),D0
  D0W = *d.pwPointer10420;
//00bd84 e380                     ASL.L    #1,D0
//00bd86 206c d720                MOVE.L   -10464(A4),A0
//00bd8a d0c0                     ADD.W    D0,A0
//00bd8c 41d0                     LEA      (A0),A0
//00bd8e 2448                     MOVE.L   A0,A2
  prnA2 = &d.prn10464[D0W];
  for (mapX=0; mapX<d.width; mapX++)
  {
//00bd90 4247                     CLR.W    D7
//00bd92 6060                     BRA      $+98 (=0x00bdf4)
//00bd94 4246                     CLR.W    D6
//00bd96 6050                     BRA      $+82 (=0x00bde8)
    for (mapY=0; mapY<d.height; mapY++)
    {
//00bd98 4240                     CLR.W    D0
//00bd9a 101b                     MOVE.B   (A3)+,D0
//00bd9c c07c 0010                AND.W    #16,D0
//00bda0 6744                     BEQ      $+70 (=0x00bde6)
      if (((*ubA3++)&16) != 0)
      {
//00bda2 3a1a                     MOVE.W   (A2)+,D5
        objD5 = *(prnA2++);
        while (objD5 != RNeof)
        {
//00bda4 3005                     MOVE.W   D5,D0
//00bda6 c07c 3c00                AND.W    #15360,D0
//00bdaa e448                     LSR.W    #2,D0
//00bdac e048                     LSR.W    #8,D0
          ASSERT(objD5.ConvertToInteger() != 0xfeff);
          D0W = objD5.db();
//00bdae 0c40 0004                CMP.W    #4,D0
//00bdb2 6622                     BNE      $+36 (=0x00bdd6)
          if (D0W == 4) // Is it a monster?
          {
//00bdb4 3f06                     MOVE.W   D6,-(A7)
//00bdb6 3f07                     MOVE.W   D7,-(A7)
//00bdb8 4eba f202                JSR      $-3580 (=0x00afbc)
//00bdbc 588f                     ADDQ.L   #4,A7
            DeleteMonsterMovementTimers(mapX, mapY); // type 29-41
//00bdbe 3f06                     MOVE.W   D6,-(A7)
//00bdc0 3f07                     MOVE.W   D7,-(A7)
//00bdc2 3f05                     MOVE.W   D5,-(A7)
//00bdc4 4eba f29c                JSR      $-3426 (=0x00b062)
//00bdc8 5c8f                     ADDQ.L   #6,A7
            TAG00b062(objD5, mapX, mapY);
//00bdca 3f06                     MOVE.W   D6,-(A7)
//00bdcc 3f07                     MOVE.W   D7,-(A7)
//00bdce 4eba f14a                JSR      $-3764 (=0x00af1a)
//00bdd2 588f                     ADDQ.L   #4,A7
            TAG00af1a(mapX, mapY);
//00bdd4 6010                     BRA      $+18 (=0x00bde6)
            break;
          }
//00bdd6 3f05                     MOVE.W   D5,-(A7)
//00bdd8 4eba dc28                JSR      $-9174 (=0x009a02)
//00bddc 548f                     ADDQ.L   #2,A7
          objD5 = GetDBRecordLink(objD5);
//00bdde 3a00                     MOVE.W   D0,D5
//00bde0 0c40 fffe                CMP.W    #65534,D0
//00bde4 66be                     BNE      $-64 (=0x00bda4)
        }; //while
      };
//00bde6 5246                     ADDQ.W   #1,D6

//00bde8 3006                     MOVE.W   D6,D0
//00bdea 322c d742                MOVE.W   -10430(A4),D1
//00bdee b041                     CMP.W    D1,D0
//00bdf0 65a6                     BCS      $-88 (=0x00bd98)
    };
//00bdf2 5247                     ADDQ.W   #1,D7
//
//00bdf4 3007                     MOVE.W   D7,D0
//00bdf6 322c d744                MOVE.W   -10428(A4),D1
//00bdfa b041                     CMP.W    D1,D0
//00bdfc 6596                     BCS      $-104 (=0x00bd94)
  };
//00bdfe 4cdf 0ce0                MOVEM.L  (A7)+,D567A23
//00be02 4e5e                     UNLK     A6
//00be04 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG00be06
void InitializeItem16(void)
{
  i32 i;
  if (d.Word11686 != 0) d.MaxITEM16 = 60;
  d.Item16 = (ITEM16 *)malloc(16*d.MaxITEM16);
  for (i=0; i < d.MaxITEM16; i++) d.Item16[i].word0 = -1;
}

//*********************************************************
//
//*********************************************************
i16 TAG00dcf2(pnt /*P1*/)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
/* All of this does nothing!
////00dcf2 4e56 0000                LINK     A6,#0
////00dcf6 2f0b                     MOVE.L   A3,-(A7)
////00dcf8 266e 0008                MOVE.L   8(A6),A3
////00dcfc 4240                     CLR.W    D0
////00dcfe 101b                     MOVE.B   (A3)+,D0
////00dd00 b07c 0007                CMP.W    #7,D0
////00dd04 665e                     BNE      $+96 (=0x00dd64)
////00dd06 4240                     CLR.W    D0
////00dd08 101b                     MOVE.B   (A3)+,D0
////00dd0a b07c 0050                CMP.W    #80,D0
////00dd0e 6654                     BNE      $+86 (=0x00dd64)
////00dd10 4240                     CLR.W    D0
////00dd12 101b                     MOVE.B   (A3)+,D0
////00dd14 b07c 0041                CMP.W    #65,D0
////00dd18 664a                     BNE      $+76 (=0x00dd64)
////00dd1a 4240                     CLR.W    D0
////00dd1c 101b                     MOVE.B   (A3)+,D0
////00dd1e b07c 0043                CMP.W    #67,D0
////00dd22 6640                     BNE      $+66 (=0x00dd64)
////00dd24 4240                     CLR.W    D0
////00dd26 101b                     MOVE.B   (A3)+,D0
////00dd28 b07c 0045                CMP.W    #69,D0
////00dd2c 6636                     BNE      $+56 (=0x00dd64)
////00dd2e 4240                     CLR.W    D0
////00dd30 101b                     MOVE.B   (A3)+,D0
////00dd32 b07c 002f                CMP.W    #47,D0
////00dd36 662c                     BNE      $+46 (=0x00dd64)
////00dd38 4240                     CLR.W    D0
////00dd3a 101b                     MOVE.B   (A3)+,D0
////00dd3c b07c 0046                CMP.W    #70,D0
////00dd40 6622                     BNE      $+36 (=0x00dd64)
////00dd42 4240                     CLR.W    D0
////00dd44 1013                     MOVE.B   (A3),D0
////00dd46 b07c 0042                CMP.W    #66,D0
////00dd4a 6618                     BNE      $+26 (=0x00dd64)
////00dd4c 4240                     CLR.W    D0
////00dd4e 102b 01f7                MOVE.B   503(A3),D0
////00dd52 b07c 0046                CMP.W    #70,D0
////00dd56 660c                     BNE      $+14 (=0x00dd64)
////00dd58 4240                     CLR.W    D0
////00dd5a 102b 01f8                MOVE.B   504(A3),D0
////00dd5e b07c 0042                CMP.W    #66,D0
////00dd62 6706                     BEQ      $+8 (=0x00dd6a)
////00dd64 4e71                     NOP
////00dd66 4e71                     NOP
////00dd68 4e71                     NOP
*/
//00dd6a 586c dd44                ADDQ.W   #4,-8892(A4)
  d.Word8892+=4;
//00dd6e 303c 0004                MOVE.W   #4,D0
//00dd72 265f                     MOVE.L   (A7)+,A3
//00dd74 4e5e                     UNLK     A6
//00dd76 4e75                     RTS
  return 4;
}

//*********************************************************
//
//*********************************************************
i16 TAG011594(i16 chIdx,
              CHARDESC *pChar,
              i16 srcPossession,
              i16 dstPossession)
{//(i16)
  dReg D5, D6, D7;
  RN   objD0;
  CHARDESC *pcA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//011594 4e56 0000                LINK     A6,#0
//011598 48e7 0710                MOVEM.L  D567A3,-(A7)
//01159c 3e2e 0008                MOVE.W   8(A6),D7
  D7W = chIdx;
//0115a0 266e 000a                MOVE.L   10(A6),A3
  pcA3 = pChar;
//0115a4 3c2e 000e                MOVE.W   14(A6),D6
  D6W = srcPossession;
//0115a8 3a2e 0010                MOVE.W   16(A6),D5
  D5W = dstPossession;
//0115ac 3006                     MOVE.W   D6,D0
//0115ae e380                     ASL.L    #1,D0
//0115b0 41eb 00d4                LEA      212(A3),A0
//0115b4 d0c0                     ADD.W    D0,A0
//0115b6 3010                     MOVE.W   (A0),D0
//0115b8 c07c 3c00                AND.W    #15360,D0
//0115bc e448                     LSR.W    #2,D0
//0115be e048                     LSR.W    #8,D0
//0115c0 0c40 0005                CMP.W    #5,D0
//0115c4 661c                     BNE      $+30 (=0x0115e2)
  if (pChar->possessions[D6W].db() != 5) return 0;
//0115c6 3f05                     MOVE.W   D5,-(A7)
//0115c8 3f06                     MOVE.W   D6,-(A7)
//0115ca 3f07                     MOVE.W   D7,-(A7)
//0115cc 4ead 04f2                JSR      1266(A5)    (=0x015a66)
//0115d0 588f                     ADDQ.L   #4,A7
  objD0 = RemovePossession(D7W, D6W);
//0115d2 3f00                     MOVE.W   D0,-(A7)
//0115d4 3f07                     MOVE.W   D7,-(A7)
//0115d6 4ead 04e6                JSR      1254(A5)    (=0x015c4c)
//0115da 5c8f                     ADDQ.L   #6,A7
  PlaceObjectOnCharacter(D7W, objD0, D5W);
//0115dc 303c 0001                MOVE.W   #1,D0
//0115e0 6004                     BRA      $+6 (=0x0115e6)
  return (1);
//0115e2 303c 0000                MOVE.W   #0,D0
//0115e6 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//0115ea 4e5e                     UNLK     A6
//0115ec 4e75                     RTS
}

void TAG0115ee(i32 chIdx, i32 byte6)
{ //(void)
  //Called by timer 11 when byte 6 not zero
  dReg D0, D5, D6, D7;
  CHARDESC *pcA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0115ee 4e56 0000                LINK     A6,#0
//0115f2 48e7 0710                MOVEM.L  D567A3,-(A7)
//0115f6 3e2e 0008                MOVE.W   8(A6),D7
//0115fa 3c2e 000a                MOVE.W   10(A6),D6
  D6W = sw(byte6);
  D7W = sw(chIdx);
//0115fe 3007                     MOVE.W   D7,D0
//011600 c1fc 0320                MULS     #800,D0
//011604 41ec bf9e                LEA      -16482(A4),A0
//011608 d0c0                     ADD.W    D0,A0
//01160a 41d0                     LEA      (A0),A0
//01160c 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[chIdx];
//01160e 3006                     MOVE.W   D6,D0
//011610 e380                     ASL.L    #1,D0
//011612 41eb 00d4                LEA      212(A3),A0
//011616 d0c0                     ADD.W    D0,A0
//011618 0c50 ffff                CMP.W    #65535,(A0)
//01161c 6702                     BEQ      $+4 (=0x011620)
//01161e 603c                     BRA      $+62 (=0x01165c)
  if (pcA3->possessions[D6W] != RNnul) return;
//011620 3f06                     MOVE.W   D6,-(A7)
//011622 3f3c 000c                MOVE.W   #12,-(A7)
//011626 2f0b                     MOVE.L   A3,-(A7)
//011628 3f07                     MOVE.W   D7,-(A7)
//01162a 4eba ff68                JSR      $-150 (=0x011594)
//01162e defc 000a                ADD.W    #10,A7
  D0W = TAG011594(D7W, pcA3, 12, D6W);
//011632 4a40                     TST.W    D0
//011634 6702                     BEQ      $+4 (=0x011638)
//011636 6024                     BRA      $+38 (=0x01165c)
  if (D0W != 0) return;
//011638 3a3c 0007                MOVE.W   #7,D5
//01163c 6018                     BRA      $+26 (=0x011656)
  for (D5W = 7; D5W <= 9; D5W++)
  {
//01163e 3f06                     MOVE.W   D6,-(A7)
//011640 3f05                     MOVE.W   D5,-(A7)
//011642 2f0b                     MOVE.L   A3,-(A7)
//011644 3f07                     MOVE.W   D7,-(A7)
//011646 4eba ff4c                JSR      $-178 (=0x011594)
//01164a defc 000a                ADD.W    #10,A7
    D0W = TAG011594(D7W, pcA3, D5W, D6W);
//01164e 4a40                     TST.W    D0
//011650 6702                     BEQ      $+4 (=0x011654)
//011652 6008                     BRA      $+10 (=0x01165c)
    if (D0W != 0) return;
//
//011654 5245                     ADDQ.W   #1,D5
//
//011656 0c45 0009                CMP.W    #9,D5
//01165a 63e2                     BLS      $-28 (=0x01163e)
  };
//01165c 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//011660 4e5e                     UNLK     A6
//011662 4e75                     RTS
}

//**********************************************************
//
//**********************************************************
//   TAG011664
void MarkAllChar1000(void)
{
  i32 i;
  for (i=0; i<d.NumCharacter; i++)
  {
    d.CH16482[i].word48 |= 0x1000;
  };
  DrawAllCharacterState();
}

//*********************************************************
//
//*********************************************************
void TAG0138ec(void)
{//(void)
  dReg D7;
  RN objD5;
  CHARDESC *pcA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0138ec 4e56 fff8                LINK     A6,#-8
//0138f0 48e7 0730                MOVEM.L  D567A23,-(A7)
//0138f4 302c d25a                MOVE.W   -11686(A4),D0
//0138f8 6600 008e                BNE      $+144 (=0x013988)
  if (d.Word11686 == 0)
  {
//0138fc 302c bf40                MOVE.W   -16576(A4),D0
//013900 3a00                     MOVE.W   D0,D5
//013902 0c40 ffff                CMP.W    #65535,D0
//013906 6612                     BNE      $+20 (=0x01391a)
    objD5 = d.objectInHand;
    if (objD5 == RNnul)
    {
//013908 397c 0001 bf3e           MOVE.W   #1,-16578(A4)
      d.EmptyHanded = 1;
//01390e 397c ffff bf42           MOVE.W   #65535,-16574(A4)
      d.ObjectType16574 = -1;
//013914 4ead 0192                JSR      402(A5)    (=0x002572)
      TAG002572();
//013918 600c                     BRA      $+14 (=0x013926)
    }
    else
    {
//01391a 3f3c 0001                MOVE.W   #1,-(A7)
//01391e 3f05                     MOVE.W   D5,-(A7)
//013920 4eba 1da2                JSR      $+7588 (=0x0156c4)
//013924 588f                     ADDQ.L   #4,A7
      TAG0156c4(objD5, 1);
    };
//013926 41ec bf9e                LEA      -16482(A4),A0
//01392a 2648                     MOVE.L   A0,A3
    pcA3 = d.CH16482;
//01392c 4247                     CLR.W    D7
    D7W = 0;
    while (D7W < d.NumCharacter)
    {
//01392e 6016                     BRA      $+24 (=0x013946)
//013930 026b 007f 0030           AND.W    #127,48(A3)
//013936 006b 9400 0030           OR.W     #37888,48(A3)
      pcA3->word48 &= 0x7f;
      pcA3->word48 |= 0x9400;
//01393c 3007                     MOVE.W   D7,D0
//01393e 5247                     ADDQ.W   #1,D7
      D7W++;
      pcA3++; // +800
//013940 200b                     MOVE.L   A3,D0
//013942 d6fc 0320                ADD.W    #800,A3

//013946 3007                     MOVE.W   D7,D0
//013948 b06c d24c                CMP.W    -11700(A4),D0
//01394c 65e2                     BCS      $-28 (=0x013930)
    };
//01394e 4eba 1a46                JSR      $+6728 (=0x015396)
    DrawAllCharacterState();
//013952 302c bf48                MOVE.W   -16568(A4),D0
//013956 3e00                     MOVE.W   D0,D7
//013958 0c40 ffff                CMP.W    #65535,D0
    D7W = d.HandChar; // owner of cursor
//01395c 670e                     BEQ      $+16 (=0x01396c)
    if (D7W != -1)
    {
//01395e 397c ffff bf48           MOVE.W   #65535,-16568(A4)
      // I have commented the following line.
      // What happened was that the Atari code
      // added the weight of the object in hand already.
      // So when we set d.HandChar to -1 and call
      // SetHandOwner the weight gets added again.
      // By leaving d.HandChar alone, SetHandOwner will
      // subtract the weight before adding it again.
      //d.HandChar = -1;
      // The following patch has been put into the
      // Atari code
      //  01395e  4e71  NOP   patch used to be 397c
      //  013960  4e71  NOP   patch used to be ffff
      //  013962  4e71  NOP   patch used to be bf48

//013964 3f07                     MOVE.W   D7,-(A7)
//013966 4ead 0564                JSR      1380(A5)    (=0x019cb2)
//01396a 548f                     ADDQ.L   #2,A7
      SetHandOwner(D7W);
    };
//01396c 302c b0d6                MOVE.W   -20266(A4),D0
//013970 3e00                     MOVE.W   D0,D7
//013972 0c40 ffff                CMP.W    #65535,D0
//013976 670e                     BEQ      $+16 (=0x013986)
    D7W = d.MagicCaster;
    if (D7W != -1)
    {
//013978 397c ffff b0d6           MOVE.W   #65535,-20266(A4)
      d.MagicCaster = -1;
//01397e 3f07                     MOVE.W   D7,-(A7)
//013980 4ead 0534                JSR      1332(A5)    (=0x01b726)
//013984 548f                     ADDQ.L   #2,A7
      SelectMagicCaster(D7W);
    };
//013986 6012                     BRA      $+20 (=0x01399a)
    return;
  };
//013988 397c ffff bf40           MOVE.W   #65535,-16576(A4)
//01398e 397c ffff bf42           MOVE.W   #65535,-16574(A4)
//013994 397c 0001 bf3e           MOVE.W   #1,-16578(A4)
  d.objectInHand = RN(RNnul);
  d.ObjectType16574 = -1;
  d.EmptyHanded = 1;

//01399a 4cdf 0ce0                MOVEM.L  (A7)+,D567A23
//01399e 4e5e                     UNLK     A6
//0139a0 4e75                     RTS
}

//   TAG011692
RESTARTABLE _ProcessTimers(void) // Parameter ignored, I think.
{//(void)
  static dReg D0;
  static i32 function;
  static i32 param;
  static i32 chIdx;
  static i16 *pwA3;
  static CHARDESC *pcA0;
  static TIMER  timeEnt;
  static i16 cP4, cP3, cP2, cP1;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
  //i32 saveD7 = D7;
  //pnt saveA3 = A3;
  function=0;
  while (CheckForTimerEvent() != 0)
  {
    GetNextTimerEntry(&timeEnt);//TAG01022c
    D0L = timeEnt.time >> 24; // Level
    LoadLevel(D0W);
    if ((ui32)d.Long11732 <= (ui32)d.Time)
    {
      function = timeEnt.function;
    };
    if ( (function > 28) && (function < 42) )
    {
      ProcessTimers29to41(
                timeEnt.uByte6, //TAG00cbc6
                timeEnt.uByte7,
                timeEnt.function,
                timeEnt.uByte8);
    }
    else
    {
      switch (function)
      {
      case 48:
      case 49: MissileTimer(&timeEnt); break;
      case 1:  ProcessTimer1(&timeEnt); break;
      case 25: ProcessTimer25(&timeEnt); break;
      case 7:  ProcessTimer7(&timeEnt); break;
      case 2:  ProcessTimer2(&timeEnt); break;
      case 10: ProcessTimer10(&timeEnt); break;
      case 9:  ProcessTimer9(&timeEnt); break;
      case 8:  ProcessTimer8(&timeEnt); break;
      case 6:  ProcessTimer6(_1_,&timeEnt); break;
      case 11:
               chIdx=timeEnt.uByte5;
               EnableCharacterAction(chIdx);
               if (timeEnt.uByte6 != 0)
               {
                 TAG0115ee(timeEnt.uByte5,
                           timeEnt.uByte6-1);
               };
               D0B = timeEnt.uByte5;
               DrawCharacterState(D0W);//top box,pos,dir,weapon
               break;
      case 12: TAG011366(timeEnt.uByte5); break;
      case 13: ProcessTimer13(&timeEnt); break;
      case 5:  ProcessTimer5(&timeEnt); break;
      case 60:
      case 61: ProcessTimer60and61(&timeEnt); break;//TAG011192
      case 65: ProcessTimer65(&timeEnt); break;
      case 20:
               cP4 = 1;
               D0B = timeEnt.uByte7;
               cP3 = D0W;
               D0B = timeEnt.uByte6;
               cP2 = D0W;
               cP1 = timeEnt.word8();
               QueueSound(cP1, cP2, cP3, cP4);
               break;
      case 24:
               D0W = d.Word11694;
               if (D0W != 0) break;
               D0W = 0;
               D0B = timeEnt.uByte7;
               cP4 = D0W;
               D0B = timeEnt.uByte6;
               cP3 = D0W;
               //cP2 = 0;
               //cP1 = timeEnt.WB8.word;
               RemoveObjectFromRoom(timeEnt.obj8(), // Get word 8 as RN
                                    RN(RNempty),
                                    cP3,
                                    cP4);
               pwA3=(i16 *)GetRecordAddress(timeEnt.obj8());
               //A0 = A3;
               pwA3[0] = -1;
               break;
      case 53: SetWatchdogTimer(); break;
      case 70:
               LoadLevel(d.partyLevel);
               ProcessLightLevelTimer(&timeEnt);
               SelectPaletteForLightLevel();
               break;
      case 71: d.Invisible--; break;
      case 72:
               //D0 = (ui8)LOCAL_1[5] *800;
               pcA0 = &d.CH16482[timeEnt.uByte5]; //d.Byte16208;
               //A0 += D0;
               //D0W = wordGear(A0)-wordGear(&LOCAL_1[6]);
               {
                 i32 oldShieldLevel, newShieldLevel, chIdx;
                 char ascii[10];
                 oldShieldLevel = pcA0->shieldStrength/50;
                 pcA0->shieldStrength =
                       uw(pcA0->shieldStrength - timeEnt.uWord6());
                 newShieldLevel = pcA0->shieldStrength/50;
                 if (oldShieldLevel != newShieldLevel)
                 {
                   chIdx = timeEnt.uByte5;
                   PrintLinefeed();
                   QuePrintLines(d.Byte1386[chIdx],pcA0->name);
                   QuePrintLines(d.Byte1386[chIdx]," SHIELD DECREASED TO ");
                   sprintf(ascii, "%d", pcA0->shieldStrength);
                   QuePrintLines(d.Byte1386[chIdx], ascii);
                 };
               };
               //wordGear(A0) = D0W;
               //D0 = (ui8)LOCAL_1[5] * 800;
               //A0 = d.Byte16434;
               //A0 += D0;
               pcA0->word48 |= 0x1000;
               //wordGear(A0) |= 0x1000;
               DrawCharacterState(timeEnt.uByte5); //top box etc
               break;
      case 73: d.SeeThruWalls--; break;
      case 74:
               d.Word13278 = sw(d.Word13278-wordGear(&timeEnt.uByte6));
               MarkAllChar1000();
               break;
      case 75:
               //D0W = (ui8)LOCAL_1[5];
               //D7W = D0W;
               param = timeEnt.uByte5;
               //D0 = D0W * 800;
               //A0 = d.Byte16440 + D0;
               //(*A0)--;
               pcA0 = &d.CH16482[param];
               pcA0->uByte42--;
               TAG017274(param,timeEnt.word6());
               break;
      case 77:
               d.Word13274 = sw(d.Word13274-wordGear(&timeEnt.uByte6));
               MarkAllChar1000();
               break;
      case 78: d.Word13276 = sw(d.Word13276 - wordGear(&timeEnt.uByte6));
               break;
      case 79: d.Byte13279--; break;
      case 22: // It appears that a timer of type 22 is created
               // when a game is restarted from a saved game.
               TAG00dcf2(&d.Byte11678);  // This function NOP'ed out! Always returns 4.
                // Looks like something deleted here
                // Looks like  if (result==???) d.Word000554=136;
               break;
      default: die(0, "Illegal Timer Entry");
      }; // switch (function)
    };
    LoadLevel(d.partyLevel); //00a9d4
  }; // while()
  //D7=saveD7;
  //A1=saveA3;
  RETURN;
}

RESTARTABLE _TAG014368(const i32 button)
{//(void)
  static dReg D0, D1, D5, D6, D7;
  static aReg A0;
  static DB3      *DB3A2;
  static RN objD4;
  static i32 mapX, mapY;
  static CHARDESC *pcA3;
  static RectPos LOCAL_12;
  RESTARTMAP
    RESTART(2)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//014368 4e56 fff4                LINK     A6,#-12
//01436c 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//014370 302c d24c                MOVE.W   -11700(A4),D0
//014374 5340                     SUBQ.W   #1,D0
//014376 3e00                     MOVE.W   D0,D7
  D7W = sw(d.NumCharacter-1);
//014378 c1fc 0320                MULS     #800,D0
//01437c 41ec bf9e                LEA      -16482(A4),A0
//014380 d0c0                     ADD.W    D0,A0
//014382 41d0                     LEA      (A0),A0
//014384 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[D7W];
//014386 0c6e 00a2 0008           CMP.W    #162,8(A6)
//01438c 6600 009c                BNE      $+158 (=0x01442a)
  if (button==162)
  {
//014390 3f3c 0004                MOVE.W   #4,-(A7)
//014394 4eba 4f5e                JSR      $+20320 (=0x0192f4)
//014398 548f                     ADDQ.L   #2,A7
    TAG0192f4(4);
//01439a 426c d258                CLR.W    -11688(A4)
    d.PotentialCharacterOrdinal = 0;
//01439e 0c6c 0001 d24c           CMP.W    #1,-11700(A4)
//0143a4 660a                     BNE      $+12 (=0x0143b0)
    if (d.NumCharacter == 1)
    {
//0143a6 3f3c ffff                MOVE.W   #65535,-(A7)
//0143aa 4ead 0564                JSR      1380(A5)    (=0x019cb2)
//0143ae 548f                     ADDQ.L   #2,A7
      SetHandOwner(-1);
    };
//0143b0 536c d24c                SUBQ.W   #1,-11700(A4)
    d.NumCharacter--;
//0143b4 426e fff8                CLR.W    -8(A6)
    LOCAL_12.w.y1 = 0;
//0143b8 3d7c 001c fffa           MOVE.W   #28,-6(A6)
    LOCAL_12.w.y2 = 28;
//0143be 3007                     MOVE.W   D7,D0
//0143c0 c0fc 0045                MULU     #69,D0
//0143c4 3d40 fff4                MOVE.W   D0,-12(A6)
    LOCAL_12.w.x1 = sw(69 * D7W);
//0143c8 d07c 0042                ADD.W    #66,D0
//0143cc 3d40 fff6                MOVE.W   D0,-10(A6)
    LOCAL_12.w.x2 = sw(LOCAL_12.w.x1 + 66);
//0143d0 426c a608                CLR.W    -23032(A4)
//0143d4 4ead 0024                JSR      36(A5)    (=0x002fd2)
    STHideCursor();
//0143d8 3f3c 00a0                MOVE.W   #160,-(A7)
//0143dc 4267                     CLR.W    -(A7)
//0143de 486e fff4                PEA      -12(A6)
//0143e2 2f2c d132                MOVE.L   -11982(A4),-(A7)
//0143e6 4ead 01ce                JSR      462(A5)    (=0x008e60)
//0143ea defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,
                  &LOCAL_12,
                  0,
                  160);
//0143ee 3f3c 00a0                MOVE.W   #160,-(A7)
//0143f2 4267                     CLR.W    -(A7)
//0143f4 4240                     CLR.W    D0
//0143f6 102b 001d                MOVE.B   29(A3),D0
//0143fa 5840                     ADDQ.W   #4,D0
//0143fc 906c d246                SUB.W    -11706(A4),D0
    D0W = (UI8)(pcA3->position+4-d.partyFacing);
//014400 c07c 0003                AND.W    #3,D0
//014404 e540                     ASL.W    #2,D0
//014406 e380                     ASL.L    #1,D0
    //D0L = (D0W & 3) << 3;
//014408 41ec fa52                LEA      -1454(A4),A0
//01440c d0c0                     ADD.W    D0,A0
//01440e 41d0                     LEA      (A0),A0
    //A0 = &d.wRectPos1454[D0W&3];
//014410 2f08                     MOVE.L   A0,-(A7)
//014412 2f2c d132                MOVE.L   -11982(A4),-(A7)
//014416 4ead 01ce                JSR      462(A5)    (=0x008e60)
//01441a defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,
                  &d.wRectPos1454[D0W&3],
                  0,
                  160);
//01441e 4ead 057c                JSR      1404(A5)    (=0x0207cc)
    TAG0207cc();
//014422 4ead 00a8                JSR      168(A5)    (=0x003026)
    STShowCursor();
//014426 6000 0246                BRA      $+584 (=0x01466e)
  }
  else
  {
//01442a 426c d258                CLR.W    -11688(A4)
    d.PotentialCharacterOrdinal = 0;
//01442e 3d6c d24a fffe           MOVE.W   -11702(A4),-2(A6)
    mapX = d.partyX;
//014434 3d6c d248 fffc           MOVE.W   -11704(A4),-4(A6)
    mapY = d.partyY;
//01443a 322c d246                MOVE.W   -11706(A4),D1
    D1W = d.partyFacing;
//01443e e381                     ASL.L    #1,D1
//014440 41ec e3a2                LEA      -7262(A4),A0
//014444 d0c1                     ADD.W    D1,A0
//014446 3210                     MOVE.W   (A0),D1
    D1W = d.DeltaX[D1W];
//014448 302e fffe                MOVE.W   -2(A6),D0
//01444c d041                     ADD.W    D1,D0
//01444e 3d40 fffe                MOVE.W   D0,-2(A6)
    mapX += D1W;
//014452 322c d246                MOVE.W   -11706(A4),D1
    D1W = d.partyFacing;
//014456 e381                     ASL.L    #1,D1
//014458 41ec e39a                LEA      -7270(A4),A0
//01445c d0c1                     ADD.W    D1,A0
//01445e 3210                     MOVE.W   (A0),D1
    D1W = d.DeltaY[D1W];
//014460 302e fffc                MOVE.W   -4(A6),D0
//014464 d041                     ADD.W    D1,D0
//014466 3d40 fffc                MOVE.W   D0,-4(A6)
    mapY += D1W;

//01446a 4246                     CLR.W    D6
//01446c 6028                     BRA      $+42 (=0x014496)
    for (D6W=0; D6W<30; D6W++)
    {
//01446e 3006                     MOVE.W   D6,D0
//014470 e380                     ASL.L    #1,D0
//014472 41eb 00d4                LEA      212(A3),A0
//014476 d0c0                     ADD.W    D0,A0
//014478 3010                     MOVE.W   (A0),D0
      objD4 = pcA3->possessions[D6W];
//01447a 3800                     MOVE.W   D0,D4
//01447c 0c40 ffff                CMP.W    #65535,D0
//014480 6712                     BEQ      $+20 (=0x014494)
      if (objD4 != RNnul)
      {
//014482 3f2e fffc                MOVE.W   -4(A6),-(A7)
//014486 3f2e fffe                MOVE.W   -2(A6),-(A7)
//01448a 4267                     CLR.W    -(A7)
//01448c 3f04                     MOVE.W   D4,-(A7)
//01448e 4ead 028e                JSR      654(A5)    (=0x009c4e)
        RemoveObjectFromRoom(objD4, RN(RNempty), mapX, mapY);
//014492 508f                     ADDQ.L   #0,A7
      };
//014494 5246                     ADDQ.W   #1,D6

//014496 0c46 001e                CMP.W    #30,D6
//01449a 65d2                     BCS      $-44 (=0x01446e)
    };
//01449c 3f2e fffc                MOVE.W   -4(A6),-(A7)
//0144a0 3f2e fffe                MOVE.W   -2(A6),-(A7)
//0144a4 4ead 0246                JSR      582(A5)    (=0x009a94)
//0144a8 588f                     ADDQ.L   #4,A7
//0144aa 3800                     MOVE.W   D0,D4
    objD4 = FindFirstObject(mapX, mapY);
    for (;;)
    {
//0144ac 3004                     MOVE.W   D4,D0
//0144ae c07c 3c00                AND.W    #15360,D0
//0144b2 e448                     LSR.W    #2,D0
//0144b4 e048                     LSR.W    #8,D0
      D0W = objD4.db();
//0144b6 0c40 0003                CMP.W    #3,D0
//0144ba 6612                     BNE      $+20 (=0x0144ce)
      if (D0W == dbACTUATOR)
      {
//0144bc 3f04                     MOVE.W   D4,-(A7)
//0144be 4ead 0312                JSR      786(A5)    (=0x009984)
//0144c2 548f                     ADDQ.L   #2,A7
//0144c4 2440                     MOVE.L   D0,A2
        DB3A2 = GetRecordAddressDB3(objD4);
//0144c6 026a ff80 0002           AND.W    #65408,2(A2)
        DB3A2->actuatorType(0);
//0144cc 600c                     BRA      $+14 (=0x0144da)
        break;
      };
//0144ce 3f04                     MOVE.W   D4,-(A7)
//0144d0 4ead 0222                JSR      546(A5)    (=0x009a02)
//0144d4 548f                     ADDQ.L   #2,A7
//0144d6 3800                     MOVE.W   D0,D4
      objD4 = GetDBRecordLink(objD4);
//0144d8 60d2                     BRA      $-44 (=0x0144ac)
    };
//0144da 0c6e 00a1 0008           CMP.W    #161,8(A6)
//0144e0 6600 00fc                BNE      $+254 (=0x0145de)
    if (button == 161)
    {
//0144e4 2f0b                     MOVE.L   A3,-(A7)
//0144e6 4eba f9bc                JSR      $-1602 (=0x013ea4)
//0144ea 588f                     ADDQ.L   #4,A7
      ReIncarnate(_2_, pcA3);
//0144ec 3f3c 0078                MOVE.W   #120,-(A7)
//0144f0 41eb 005c                LEA      92(A3),A0
//0144f4 2f08                     MOVE.L   A0,-(A7)
//0144f6 4ead 0054                JSR      84(A5)    (=0x000a84)
//0144fa 5c8f                     ADDQ.L   #6,A7
      ClearMemory((pnt)pcA3->skills92,120);
//0144fc 3c3c 0001                MOVE.W   #1,D6
//014500 6060                     BRA      $+98 (=0x014562)
      for (D6W=1; D6W<=6; D6W++)
      {
//014502 3a06                     MOVE.W   D6,D5
//014504 cbfc 0003                MULS     #3,D5
//014508 41eb 0047                LEA      71(A3),A0
//01450c d0c5                     ADD.W    D5,A0
//01450e 4245                     CLR.W    D5
//014510 1a10                     MOVE.B   (A0),D5
        D5W = pcA3->Attributes[D6W].Current();
//014512 e645                     ASR.W    #3,D5
        D5W >>= 3;
//014514 3006                     MOVE.W   D6,D0
//014516 c1fc 0003                MULS     #3,D0
//01451a 41eb 0047                LEA      71(A3),A0
//01451e d0c0                     ADD.W    D0,A0
//014520 4240                     CLR.W    D0
//014522 1010                     MOVE.B   (A0),D0
        D0W = pcA3->Attributes[D6W].Current();
        D0W = sw(D0W - D5W);
//014524 9045                     SUB.W    D5,D0
//014526 3f00                     MOVE.W   D0,-(A7)
//014528 3006                     MOVE.W   D6,D0
//01452a c1fc 0003                MULS     #3,D0
//01452e 41eb 0048                LEA      72(A3),A0
//014532 d0c0                     ADD.W    D0,A0
//014534 4240                     CLR.W    D0
//014536 1010                     MOVE.B   (A0),D0
        D1W = pcA3->Attributes[D6W].Minimum();
//014538 3f00                     MOVE.W   D0,-(A7)
//01453a 4ead 00d2                JSR      210(A5)    (=0x000ff8)
//01453e 588f                     ADDQ.L   #4,A7
        D0W = sw(Larger(D1W, D0W));
//014540 3206                     MOVE.W   D6,D1
//014542 c3fc 0003                MULS     #3,D1
//014546 41eb 0046                LEA      70(A3),A0
//01454a d0c1                     ADD.W    D1,A0
//01454c 1080                     MOVE.B   D0,(A0)
        pcA3->Attributes[D6W].Maximum(D0W);
//01454e 3206                     MOVE.W   D6,D1
//014550 c3fc 0003                MULS     #3,D1
//014554 41eb 0047                LEA      71(A3),A0
//014558 d0c1                     ADD.W    D1,A0
//01455a c07c 00ff                AND.W    #255,D0
//01455e 1080                     MOVE.B   D0,(A0)
        pcA3->Attributes[D6W].Current(D0W);
//014560 5246                     ADDQ.W   #1,D6
//
//014562 0c46 0006                CMP.W    #6,D6
//014566 639a                     BLS      $-100 (=0x014502)
      };
//014568 302b 0034                MOVE.W   52(A3),D0
//01456c e240                     ASR.W    #1,D0
//01456e 3740 0034                MOVE.W   D0,52(A3)
      pcA3->HP >>= 1;
//014572 302b 0036                MOVE.W   54(A3),D0
//014576 e240                     ASR.W    #1,D0
//014578 3740 0036                MOVE.W   D0,54(A3)
      pcA3->maxHP >>= 1;
//01457c 302b 0038                MOVE.W   56(A3),D0
//014580 e240                     ASR.W    #1,D0
//014582 3740 0038                MOVE.W   D0,56(A3)
      pcA3->stamina >>= 1;
//014586 302b 003a                MOVE.W   58(A3),D0
//01458a e240                     ASR.W    #1,D0
//01458c 3740 003a                MOVE.W   D0,58(A3)
      pcA3->maxStamina >>= 1;
//014590 302b 003c                MOVE.W   60(A3),D0
//014594 e240                     ASR.W    #1,D0
//014596 3740 003c                MOVE.W   D0,60(A3)
      pcA3->mana >>= 1;
//01459a 302b 003e                MOVE.W   62(A3),D0
//01459e e240                     ASR.W    #1,D0
//0145a0 3740 003e                MOVE.W   D0,62(A3)
      pcA3->maxMana >>= 1;
//0145a4 4246                     CLR.W    D6
      //D6W = 0;
//0145a6 6030                     BRA      $+50 (=0x0145d8)
      for (D6W=0; D6W<12; D6W++)
      {
//0145a8 4ead 0090                JSR      144(A5)    (=0x001030)
        D5W = sw((STRandom() & 0xffff) % 7);
//0145ac c0bc 0000 ffff           AND.L    #65535,D0
//0145b2 80fc 0007                DIVU     #7,D0
//0145b6 4840                     SWAP     D0
//0145b8 3a00                     MOVE.W   D0,D5
//0145ba 3005                     MOVE.W   D5,D0
//0145bc c1fc 0003                MULS     #3,D0
//0145c0 41eb 0047                LEA      71(A3),A0
//0145c4 d0c0                     ADD.W    D0,A0
//0145c6 5210                     ADDQ.B   #1,(A0)
        pcA3->Attributes[D5W].IncrCurrent(1);
//0145c8 3005                     MOVE.W   D5,D0
//0145ca c1fc 0003                MULS     #3,D0
//0145ce 41eb 0046                LEA      70(A3),A0
//0145d2 d0c0                     ADD.W    D0,A0
//0145d4 5210                     ADDQ.B   #1,(A0)
        pcA3->Attributes[D5W].IncrMaximum(1);

//0145d6 5246                     ADDQ.W   #1,D6
//
//0145d8 0c46 000c                CMP.W    #12,D6
//0145dc 65ca                     BCS      $-52 (=0x0145a8)
      };
    };
//0145de 0c6c 0001 d24c           CMP.W    #1,-11700(A4)
//0145e4 6618                     BNE      $+26 (=0x0145fe)
    if (d.NumCharacter == 1)
    {
//0145e6 296c d23a cd66           MOVE.L   -11718(A4),-12954(A4)
      d.Long12954 = d.Time;
//0145ec 4267                     CLR.W    -(A7)
//0145ee 4ead 0564                JSR      1380(A5)    (=0x019cb2)
//0145f2 548f                     ADDQ.L   #2,A7
      SetHandOwner(0);
//0145f4 4267                     CLR.W    -(A7)
//0145f6 4ead 0534                JSR      1332(A5)    (=0x01b726)
//0145fa 548f                     ADDQ.L   #2,A7
      SelectMagicCaster(0);
//0145fc 600a                     BRA      $+12 (=0x014608)
    }
    else
    {
//0145fe 3f2c b0d6                MOVE.W   -20266(A4),-(A7)
//014602 4ead 054c                JSR      1356(A5)    (=0x01b50e)
//014606 548f                     ADDQ.L   #2,A7
      TAG01b50e(d.MagicCaster);
    };
//014608 4ead 00ba                JSR      186(A5)    (=0x001c02)
    PrintLinefeed();
//01460c 41d3                     LEA      (A3),A0
//01460e 2f08                     MOVE.L   A0,-(A7)
//014610 3007                     MOVE.W   D7,D0
//014612 41ec fa96                LEA      -1386(A4),A0
//014616 d0c0                     ADD.W    D0,A0
//014618 4240                     CLR.W    D0
//01461a 1010                     MOVE.B   (A0),D0
//01461c 3c00                     MOVE.W   D0,D6
//01461e 3f00                     MOVE.W   D0,-(A7)
//014620 4ead 00c0                JSR      192(A5)    (=0x001aa8)
//014624 5c8f                     ADDQ.L   #6,A7
    QuePrintLines(D6W = (UI8)(d.Byte1386[D7W]), pcA3->name);
//014626 0c6e 00a0 0008           CMP.W    #160,8(A6)
//01462c 6608                     BNE      $+10 (=0x014636)
    if (button == 160)
    {
//01462e 41ec 004c                LEA      76(A4),A0
//014632 2008                     MOVE.L   A0,D0
      A0 = " RESURRECTED";
//014634 6006                     BRA      $+8 (=0x01463c)
    }
    else
    {
//014636 41ec 005a                LEA      90(A4),A0
//01463a 2008                     MOVE.L   A0,D0
      A0 = " REINCARNATED";
    };
//01463c 2f00                     MOVE.L   D0,-(A7)
//01463e 3f06                     MOVE.W   D6,-(A7)
//014640 4ead 00c0                JSR      192(A5)    (=0x001aa8)
//014644 5c8f                     ADDQ.L   #6,A7
      QuePrintLines(D6W, A0);
//014646 3f3c 0004                MOVE.W   #4,-(A7)
//01464a 4eba 4ca8                JSR      $+19626 (=0x0192f4)
//01464e 548f                     ADDQ.L   #2,A7
      TAG0192f4(4);
//014650 4ead 057c                JSR      1404(A5)    (=0x0207cc)
      TAG0207cc();
//014654 0c6c ffff bf48           CMP.W    #65535,-16568(A4)
//01465a 6606                     BNE      $+8 (=0x014662)
    if (d.HandChar == -1) // owner of cursor
    {
//01465c 303c 0000                MOVE.W   #0,D0
      D0W = 0;
//014660 6004                     BRA      $+6 (=0x014666)
    }
    else
    {
//014662 303c 0001                MOVE.W   #1,D0
      D0W = 1;
    };
//014666 3f00                     MOVE.W   D0,-(A7)
//014668 4ead 0144                JSR      324(A5)    (=0x002456)
//01466c 548f                     ADDQ.L   #2,A7
    SetCursorShape(D0W);
  };
//01466e 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//014672 4e5e                     UNLK     A6
//014674 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
//   TAG015396
void DrawAllCharacterState(void)
{//(void)
  static i32 i;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//015396 4e56 0000                LINK     A6,#0
//01539a 3f07                     MOVE.W   D7,-(A7)
//01539c 3f3c 0025                MOVE.W   #37,-(A7)
//0153a0 4ead 0138                JSR      312(A5)    (=0x001ce8)
//0153a4 548f                     ADDQ.L   #2,A7
  //What if we simply don't do this????NotImplemented(0x99977a);//wvbl(_1_);
//0153a6 4247                     CLR.W    D7
//0153a8 600a                     BRA      $+12 (=0x0153b4)
  for (i=0; i<d.NumCharacter; i++)
  {
//0153aa 3f07                     MOVE.W   D7,-(A7)
//0153ac 4eba fa32                JSR      $-1484 (=0x014de0)
//0153b0 548f                     ADDQ.L   #2,A7
    DrawCharacterState(i);//box at top, pos, dir, weapon
//0153b2 5247                     ADDQ.W   #1,D7
//0153b4 3007                     MOVE.W   D7,D0
//0153b6 b06c d24c                CMP.W    -11700(A4),D0
//0153ba 65ee                     BCS      $-16 (=0x0153aa)
  };
//0153bc 3e1f                     MOVE.W   (A7)+,D7
//0153be 4e5e                     UNLK     A6
//0153c0 4e75                     RTS
  return;
}

void TAG0156c4(RN object,i16 P2)
{//(void)
  dReg D1;
  CHARDESC *pcA0;
  OBJECTTYPE ot;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0156c4 4e56 0000                LINK     A6,#0
//0156c8 0c6e ffff 0008           CMP.W    #65535,8(A6)
//0156ce 6604                     BNE      $+6 (=0x0156d4)
//0156d0 6000 0094                BRA      $+150 (=0x015766)
  if (object == RNnul) return;
//0156d4 426c bf3e                CLR.W    -16578(A4)
  d.EmptyHanded = 0;
//0156d8 2f2c bf44                MOVE.L   -16572(A4),-(A7)
//0156dc 302e 0008                MOVE.W   8(A6),D0
//0156e0 3940 bf40                MOVE.W   D0,-16576(A4)
//0156e4 3f00                     MOVE.W   D0,-(A7)
//0156e6 4ead 014a                JSR      330(A5)    (=0x001188)
//0156ea 548f                     ADDQ.L   #2,A7
//0156ec 3940 bf42                MOVE.W   D0,-16574(A4)
//0156f0 3f00                     MOVE.W   D0,-(A7)
//0156f2 4ead 004e                JSR      78(A5)    (=0x00135c)
//0156f6 5c8f                     ADDQ.L   #6,A7
  ot = GetModifiedObjectType(d.objectInHand = object);
  d.ObjectType16574 = (I16)ot;
  GetIconGraphic(ot, d.Pointer16572);
//0156f8 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();
//0156fc 3f2e 0008                MOVE.W   8(A6),-(A7)
//015700 4ead 0162                JSR      354(A5)    (=0x0012a0)
//015704 548f                     ADDQ.L   #2,A7
  DrawNameOfHeldObject(object);
//015706 302e 000a                MOVE.W   10(A6),D0
//01570a 6708                     BEQ      $+10 (=0x015714)
  if (P2)
  {
//01570c 397c 0001 d220           MOVE.W   #1,-11744(A4)
//015712 600a                     BRA      $+12 (=0x01571e)
    d.Word11744 = 1;
  }
  else
  {
//015714 2f2c bf44                MOVE.L   -16572(A4),-(A7)
//015718 4ead 00f6                JSR      246(A5)    (=0x002480)
//01571c 588f                     ADDQ.L   #4,A7
    CreateObjectCursor(d.Pointer16572);
  };
//01571e 4ead 00a8                JSR      168(A5)    (=0x003026)
  STShowCursor();
//015722 0c6c ffff bf48           CMP.W    #65535,-16568(A4)
//015728 673c                     BEQ      $+62 (=0x015766)
  if (d.HandChar == -1) return; //owner of cursor
//01572a 3f2e 0008                MOVE.W   8(A6),-(A7)
//01572e 4ead 0234                JSR      564(A5)    (=0x0091e6)
//015732 548f                     ADDQ.L   #2,A7
//015734 2200                     MOVE.L   D0,D1
  D1L = GetObjectWeight(object);
//015736 302c bf48                MOVE.W   -16568(A4),D0
//01573a c1fc 0320                MULS     #800,D0
//01573e 41ec c0ae                LEA      -16210(A4),A0
//015742 d0c0                     ADD.W    D0,A0
  pcA0 = &d.CH16482[d.HandChar]; //owner of cursor
  pcA0->load = sw(pcA0->load + D1W);
  ASSERT(pcA0->load < MAXLOAD);
//015744 3010                     MOVE.W   (A0),D0
//015746 d041                     ADD.W    D1,D0
//015748 3080                     MOVE.W   D0,(A0)
//01574a 302c bf48                MOVE.W   -16568(A4),D0
//01574e c1fc 0320                MULS     #800,D0
//015752 41ec bfce                LEA      -16434(A4),A0
//015756 d0c0                     ADD.W    D0,A0
//015758 0050 0200                OR.W     #512,(A0)
  pcA0->word48 |= 0x200;
//01575c 3f2c bf48                MOVE.W   -16568(A4),-(A7)
//015760 4eba f67e                JSR      $-2432 (=0x014de0)
//015764 548f                     ADDQ.L   #2,A7
  DrawCharacterState(d.HandChar); //owner of cursor
//015766 4e5e                     UNLK     A6
//015768 4e75                     RTS
}

//  TAG01479c
i16 GetCharacterToDamage(i32 newX, i32 newY , i32 direction)
{ // We got here when we ran into a wall.
  // Direction appears to be 180 degrees from
  // direction party just moved.  Perhaps it is the
  // side of the cell we encountered.
  // We also got here when a Deth Knight attacked the
  // party.  In that case, direction was STRandom0_3().
  // Probably the position within the monster party
  // from which tha attack came???.
  dReg D6, D7;
  i32 xDist, yDist;
  ui8 attackOrder[4];
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01479c 4e56 fffc                LINK     A6,#-4
//0147a0 48e7 0300                MOVEM.L  D67,-(A7)
//0147a4 302c d24c                MOVE.W   -11700(A4),D0
//0147a8 677c                     BEQ      $+126 (=0x014826)
  if (d.NumCharacter == 0) return -1;
//0147aa 302e 0008                MOVE.W   8(A6),D0
//0147ae 322c d24a                MOVE.W   -11702(A4),D1
//0147b2 9041                     SUB.W    D1,D0
  xDist = abs(newX - d.partyX); // How far we moved?
//0147b4 3f00                     MOVE.W   D0,-(A7)
//0147b6 4ead 00ea                JSR      234(A5)    (=0x000fd2)
//0147ba 548f                     ADDQ.L   #2,A7
//0147bc 2f00                     MOVE.L   D0,-(A7)
//0147be 302e 000a                MOVE.W   10(A6),D0
//0147c2 322c d248                MOVE.W   -11704(A4),D1
//0147c6 9041                     SUB.W    D1,D0
//0147c8 3f00                     MOVE.W   D0,-(A7)
//0147ca 4ead 00ea                JSR      234(A5)    (=0x000fd2)
//0147ce 548f                     ADDQ.L   #2,A7
  yDist = abs(newY - d.partyY);
//0147d0 2200                     MOVE.L   D0,D1
//0147d2 201f                     MOVE.L   (A7)+,D0
//0147d4 d041                     ADD.W    D1,D0
//0147d6 0c40 0001                CMP.W    #1,D0
//0147da 6e4a                     BGT      $+76 (=0x014826)
  if (xDist + yDist > 1) return -1;
//0147dc 3f2e 000c                MOVE.W   12(A6),-(A7)
//0147e0 3f2e 000a                MOVE.W   10(A6),-(A7)
//0147e4 3f2e 0008                MOVE.W   8(A6),-(A7)
//0147e8 3f2c d248                MOVE.W   -11704(A4),-(A7)
//0147ec 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//0147f0 486e fffc                PEA      -4(A6)
//0147f4 4ead 0396                JSR      918(A5)    (=0x00f694)
//0147f8 defc 000e                ADD.W    #14,A7
  //Pretend we are being attacked by the wall!  The
  //two people closest to the wall are affected.
  DetermineAttackOrder(attackOrder,
                       d.partyX, d.partyY,
                       newX,newY,direction);
//0147fc 4247                     CLR.W    D7
//0147fe 6020                     BRA      $+34 (=0x014820)
  for (D7W=0; D7W<4; D7W++)
  {
//014800 3007                     MOVE.W   D7,D0
//014802 41ee fffc                LEA      -4(A6),A0
//014806 d0c0                     ADD.W    D0,A0
//014808 4240                     CLR.W    D0
//01480a 1010                     MOVE.B   (A0),D0
//01480c 3f00                     MOVE.W   D0,-(A7)
//01480e 4eba ff3e                JSR      $-192 (=0x01474e)
//014812 548f                     ADDQ.L   #2,A7
//014814 3c00                     MOVE.W   D0,D6
    D6W = CharacterAtPosition(attackOrder[D7W]);
//014816 4a40                     TST.W    D0
//014818 6d04                     BLT      $+6 (=0x01481e)
    if (D6W >= 0)
    {
//01481a 3006                     MOVE.W   D6,D0
//01481c 600c                     BRA      $+14 (=0x01482a)
      return D6W;
    };
//01481e 5247                     ADDQ.W   #1,D7
//
//014820 0c47 0004                CMP.W    #4,D7
//014824 65da                     BCS      $-36 (=0x014800)
  };
//014826 303c ffff                MOVE.W   #65535,D0
  return -1;
//01482a 4cdf 00c0                MOVEM.L  (A7)+,D67
//01482e 4e5e                     UNLK     A6
//014830 4e75                     RTS
}

//*********************************************************
// 20 has already been subtracted from button number.
// It should be 0 to 7 for the 8 hands shown in
// in the 4 portrait areas.  It should be 8 to 20
// for the clothes on the 'paper doll'.  It should be
// 21 to 37 for the backpack items.  It will be 30 to
// 37 for items in the chest.
//
//*********************************************************
//   TAG015e0c
void HandleClothingClick(i32 button)
{//(void)
  dReg D0, D6, D7;
  RN objD4, objD5;
  //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//015e0c 4e56 0000                LINK     A6,#0
//015e10 48e7 0f00                MOVEM.L  D4567,-(A7)
//015e14 0c6e 0008 0008           CMP.W    #8,8(A6)
//015e1a 6442                     BCC      $+68 (=0x015e5e)
  if (button < 8)
  {
//015e1c 302c d258                MOVE.W   -11688(A4),D0
//015e20 6704                     BEQ      $+6 (=0x015e26)
//015e22 6000 00f2                BRA      $+244 (=0x015f16)
    if (d.PotentialCharacterOrdinal != 0) return;
//015e26 3e2e 0008                MOVE.W   8(A6),D7
    D7W = sw(button);
//015e2a e24f                     LSR.W    #1,D7
    D7W /= 2;
//015e2c 3007                     MOVE.W   D7,D0
//015e2e b06c d24c                CMP.W    -11700(A4),D0
//015e32 641c                     BCC      $+30 (=0x015e50)
    if (D7W >= d.NumCharacter) return;
//015e34 3007                     MOVE.W   D7,D0
//015e36 5240                     ADDQ.W   #1,D0
//015e38 322c bf1a                MOVE.W   -16614(A4),D1
//015e3c b041                     CMP.W    D1,D0
//015e3e 6710                     BEQ      $+18 (=0x015e50)
    if (D7W+1 == d.SelectedCharacterOrdinal) return;
//015e40 3007                     MOVE.W   D7,D0
//015e42 c1fc 0320                MULS     #800,D0
//015e46 41ec bfd2                LEA      -16430(A4),A0
//015e4a d0c0                     ADD.W    D0,A0
//015e4c 3010                     MOVE.W   (A0),D0
//015e4e 6604                     BNE      $+6 (=0x015e54)
//015e50 6000 00c4                BRA      $+198 (=0x015f16)
    if (d.CH16482[D7W].HP == 0) return;
//015e54 3c2e 0008                MOVE.W   8(A6),D6
//015e58 cc7c 0001                AND.W    #1,D6
    D6W = (I16)(button & 1); // Left or right hand.
//015e5c 600c                     BRA      $+14 (=0x015e6a)
  }
  else
  {
//015e5e 3e2c bf1a                MOVE.W   -16614(A4),D7
//015e62 5347                     SUBQ.W   #1,D7
    D7W = sw(d.SelectedCharacterOrdinal-1);
//015e64 3c2e 0008                MOVE.W   8(A6),D6
//015e68 5146                     SUBQ.W   #0,D6
    D6W = sw(button-8);
//015e6a 3a2c bf40                MOVE.W   -16576(A4),D5
  };
  objD5 = d.objectInHand;
//015e6e 0c46 001e                CMP.W    #30,D6
//015e72 650e                     BCS      $+16 (=0x015e82)
  if (D6W >= 30)
  {
//015e74 3806                     MOVE.W   D6,D4
//015e76 e384                     ASL.L    #1,D4
//015e78 41ec becc                LEA      -16692(A4),A0
//015e7c d0c4                     ADD.W    D4,A0
//015e7e 3810                     MOVE.W   (A0),D4
    objD4 = d.rnChestContents[D6W-30];
//015e80 6016                     BRA      $+24 (=0x015e98)
  }
  else
  {
//015e82 3806                     MOVE.W   D6,D4
//015e84 3207                     MOVE.W   D7,D1
//015e86 c3fc 0320                MULS     #800,D1
//015e8a 41ec c072                LEA      -16270(A4),A0
//015e8e d0c1                     ADD.W    D1,A0
//015e90 e384                     ASL.L    #1,D4
//015e92 41d0                     LEA      (A0),A0
//015e94 d0c4                     ADD.W    D4,A0
//015e96 3810                     MOVE.W   (A0),D4
    objD4 = d.CH16482[D7W].possessions[D6W];
  };
//015e98 0c44 ffff                CMP.W    #65535,D4
//015e9c 6608                     BNE      $+10 (=0x015ea6)
//015e9e 0c45 ffff                CMP.W    #65535,D5
//015ea2 6602                     BNE      $+4 (=0x015ea6)
//015ea4 6070                     BRA      $+114 (=0x015f16)
  if ( (objD4==RNnul) && (objD5==RNnul) ) return;
//015ea6 0c45 ffff                CMP.W    #65535,D5
//015eaa 6724                     BEQ      $+38 (=0x015ed0)
  if (objD5 != RNnul)
  {
//015eac 3f05                     MOVE.W   D5,-(A7)
//015eae 4ead 0306                JSR      774(A5)    (=0x009308=GetWeaponIndex)
//015eb2 548f                     ADDQ.L   #2,A7
    D0W = GetObjectIndex(objD5);
//015eb4 c1fc 0006                MULS     #6,D0
//015eb8 41ec df46                LEA      -8378(A4),A0
//015ebc d0c0                     ADD.W    D0,A0
//015ebe 3010                     MOVE.W   (A0),D0
    D0W = d.ObjDesc[D0W].word4;// Possible carry locations
//015ec0 3206                     MOVE.W   D6,D1
//015ec2 e381                     ASL.L    #1,D1
//015ec4 41ec fbee                LEA      -1042(A4),A0
//015ec8 d0c1                     ADD.W    D1,A0
//015eca c050                     AND.W    (A0),D0
//015ecc 6602                     BNE      $+4 (=0x015ed0)
//015ece 6046                     BRA      $+72 (=0x015f16)
    if ((D0W & d.CarryLocation[D6W]) == 0) return;
  };
//015ed0 4ead 0024                JSR      36(A5)    (=0x002fd2=HideCursor)
  STHideCursor();
//015ed4 0c45 ffff                CMP.W    #65535,D5
//015ed8 6704                     BEQ      $+6 (=0x015ede)
  if (objD5 != RNnul)
  {
//015eda 4eba f88e                JSR      $-1904 (=0x01576a)
    RemoveObjectFromHand();
  };
//015ede 0c44 ffff                CMP.W    #65535,D4
//015ee2 6714                     BEQ      $+22 (=0x015ef8)
  if (objD4 != RNnul)
  {
//015ee4 3f06                     MOVE.W   D6,-(A7)
//015ee6 3f07                     MOVE.W   D7,-(A7)
//015ee8 4eba fb7c                JSR      $-1154 (=0x015a66)
//015eec 588f                     ADDQ.L   #4,A7
    RemovePossession(D7W, D6W);
//015eee 4267                     CLR.W    -(A7)
//015ef0 3f04                     MOVE.W   D4,-(A7)
//015ef2 4eba f7d0                JSR      $-2094 (=0x0156c4)
//015ef6 588f                     ADDQ.L   #4,A7
    TAG0156c4(objD4, 0);
  };
//015ef8 0c45 ffff                CMP.W    #65535,D5
//015efc 670c                     BEQ      $+14 (=0x015f0a)
  if (objD5 != RNnul)
  {
//015efe 3f06                     MOVE.W   D6,-(A7)
//015f00 3f05                     MOVE.W   D5,-(A7)
//015f02 3f07                     MOVE.W   D7,-(A7)
//015f04 4eba fd46                JSR      $-696 (=0x015c4c)
//015f08 5c8f                     ADDQ.L   #6,A7
    PlaceObjectOnCharacter(D7W, objD5, D6W);
  };
//015f0a 3f07                     MOVE.W   D7,-(A7)
//015f0c 4eba eed2                JSR      $-4396 (=0x014de0=DrawCharacterState)
  DrawCharacterState(D7W);
//015f10 548f                     ADDQ.L   #2,A7
//015f12 4ead 00a8                JSR      168(A5)    (=0x003026=ShowCursor)
  STShowCursor();
//015f16 4cdf 00f0                MOVEM.L  (A7)+,D4567
//015f1a 4e5e                     UNLK     A6
//015f1c 4e75                     RTS
}

//  TAG01657e
i16 TimeToMove(CHARDESC *pChar)
{ // Speed of character???
  dReg D0, D1, D6, D7;
  i16 time;
  OBJECTTYPE otD0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01657e 4e56 0000                LINK     A6,#0
//016582 48e7 0710                MOVEM.L  D567A3,-(A7)
//016586 266e 0008                MOVE.L   8(A6),A3
  //pcA3 = chIdx
//01658a 2f0b                     MOVE.L   A3,-(A7)
//01658c 4eba ff7a                JSR      $-132 (=0x016508)
//016590 588f                     ADDQ.L   #4,A7
//016592 3c00                     MOVE.W   D0,D6
  D6L = MaxLoad(pChar);
//016594 322b 0110                MOVE.W   272(A3),D1
//016598 3e01                     MOVE.W   D1,D7
  D7W = pChar->load;
//01659a b041                     CMP.W    D1,D0
//01659c 6332                     BLS      $+52 (=0x0165d0)
  if (D6L > D7W)
  {
//01659e 3a3c 0002                MOVE.W   #2,D5
    time = 2;
//0165a2 3007                     MOVE.W   D7,D0
//0165a4 c0bc 0000 ffff           AND.L    #65535,D0
//0165aa e780                     ASL.L    #3,D0
    D0L = 8 * (D7W & 0xffff);
//0165ac 3206                     MOVE.W   D6,D1
//0165ae c2bc 0000 ffff           AND.L    #65535,D1
//0165b4 2f01                     MOVE.L   D1,-(A7)
    D1L = 5 * (D6L & 0xffff);
//0165b6 223c 0000 0005           MOVE.L   #5,D1
//0165bc 2f01                     MOVE.L   D1,-(A7)
//0165be 4ead 0168                JSR      360(A5)    (=0x003158)
//0165c2 221f                     MOVE.L   (A7)+,D1
    //D1L = 5 * D1L;
//0165c4 b081                     CMP.L    D1,D0
//0165c6 6f02                     BLE      $+4 (=0x0165ca)
    if (D0L > D1L)
    {
//0165c8 5245                     ADDQ.W   #1,D5
      time++;
    };
//0165ca 3e3c 0001                MOVE.W   #1,D7
    D7W = 1;
//0165ce 6018                     BRA      $+26 (=0x0165e8)
  }
  else
  {
//0165d0 3a3c 0004                MOVE.W   #4,D5
    time = 4;
//0165d4 3207                     MOVE.W   D7,D1
//0165d6 9246                     SUB.W    D6,D1
//0165d8 e541                     ASL.W    #2,D1
    D1L = (4 * (D7W - D6L)) & 0xffff;
//0165da c2bc 0000 ffff           AND.L    #65535,D1
//0165e0 82c6                     DIVU     D6,D1
//0165e2 da41                     ADD.W    D1,D5
    time = sw(time + D1L / D6L);
//0165e4 3e3c 0002                MOVE.W   #2,D7
    D7W = 2;
  };
//0165e8 302b 0032                MOVE.W   50(A3),D0
//0165ec c07c 0020                AND.W    #32,D0
//0165f0 6702                     BEQ      $+4 (=0x0165f4)
  if (pChar->ouches & 32)
  {
//0165f2 da47                     ADD.W    D7,D5
    time = sw(time + D7W);
  };
//0165f4 3f2b 00de                MOVE.W   222(A3),-(A7)
//0165f8 4ead 014a                JSR      330(A5)    (=0x001188)
//0165fc 548f                     ADDQ.L   #2,A7
  otD0 = GetModifiedObjectType(pChar->possessions[5]); // feet
//0165fe 0c40 00c2                CMP.W    #194,D0
//016602 6602                     BNE      $+4 (=0x016606)
  if (otD0 == obj_BootsOfSpeed)
  {
//016604 5345                     SUBQ.W   #1,D5
    time--;
  };
//016606 3005                     MOVE.W   D5,D0
  return time;
//016608 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01660c 4e5e                     UNLK     A6
//01660e 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG0169a8
void WakeUp(void)
{//(void)
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0169a8 4e56 fffe                LINK     A6,#-2
//0169ac 397c 0001 d228           MOVE.W   #1,-11736(A4)
  d.clockTick = 1;
//0169b2 426c d256                CLR.W    -11690(A4)
  d.PartySleeping = 0;
//0169b6 397c 000a d230           MOVE.W   #10,-11728(A4)
  d.Speed = sw(speedTable[gameSpeed].vblPerTick);
//0169bc 4ead 01c2                JSR      450(A5)    (=0x004308)
  FloorAndCeilingOnly();
//0169c0 41ec bcd4                LEA      -17196(A4),A0
//0169c4 2948 be18                MOVE.L   A0,-16872(A4)
  d.PrimaryButtonList = &d.Buttons17196[0];
//0169c8 41ec bc68                LEA      -17304(A4),A0
//0169cc 2948 be14                MOVE.L   A0,-16876(A4)
  d.SecondaryButtonList = &d.Buttons17304[0];
//0169d0 41ec b820                LEA      -18400(A4),A0
//0169d4 2948 be10                MOVE.L   A0,-16880(A4)
  d.pKeyXlate2 = (KeyXlate *)&d.Byte18400;
//0169d8 41ec b804                LEA      -18428(A4),A0
//0169dc 2948 be0c                MOVE.L   A0,-16884(A4)
  d.pKeyXlate1 = (KeyXlate *)&d.Byte18428;
//0169e0 4ead 052e                JSR      1326(A5)    (=0x01953a)
  TAG01953a();
//0169e4 4ead 057c                JSR      1404(A5)    (=0x0207cc)
  TAG0207cc();
//0169e8 4e5e                     UNLK     A6
//0169ea 4e75                     RTS
}

i16 TAG0169ec(i32 mapX, i32 mapY)
{
  dReg D6, D7;
  i16  *pwA3;
//0169f4 1c2c cc28                MOVE.B   -13272(A4),D6
//0169f8 cc7c 00ff                AND.W    #255,D6
  D6W = d.NumHistEnt; //entry count
//0169fc 676c                     BEQ      $+110 (=0x016a6a)
  if (D6W != 0)
  {
    D7W = sw(  ((d.LoadedLevel & 63) << 10)
              | (mapY << 5)
              | (mapX & 31)  );
//016a44 3006                     MOVE.W   D6,D0
//016a46 5346                     SUBQ.W   #1,D6
//016a48 e380                     ASL.L    #1,D0
//016a4a 41ec cc2c                LEA      -13268(A4),A0
//016a4e d0c0                     ADD.W    D0,A0
//016a50 41d0                     LEA      (A0),A0
    pwA3 = &d.Word13268[D6W];
    D6W--;
//016a52 2008                     MOVE.L   A0,D0
//016a54 2640                     MOVE.L   D0,A3
    do
    {
//016a56 3023                     MOVE.W   -(A3),D0
      pwA3--;
//016a58 b047                     CMP.W    D7,D0
//016a5a 6606                     BNE      $+8 (=0x016a62)
      if (*pwA3 == D7W)
      {
//016a5c 3006                     MOVE.W   D6,D0
//016a5e 5240                     ADDQ.W   #1,D0
        return sw(D6W + 1);
//016a60 600c                     BRA      $+14 (=0x016a6e)
      };
//016a62 3006                     MOVE.W   D6,D0
//016a64 5346                     SUBQ.W   #1,D6
//016a66 4a40                     TST.W    D0
//016a68 66ec                     BNE      $-18 (=0x016a56)
    } while (D6W-- != 0);
  };
  return 0;
}

void TAG016e54(void)
{//(void)
  dReg D0, D1, D3, D4, D5, D6, D7;
  aReg A0;
  TIMER *ptA2;
  CHARDESC *pcA3;
  RectPos LOCAL_20;
  TIMER LOCAL_12;
  i16 LOCAL_2;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//016e54 4e56 ffec                LINK     A6,#-20
//016e58 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//016e5c 41ec bf9e                LEA      -16482(A4),A0
//016e60 2648                     MOVE.L   A0,A3
  Instrumentation(icntTAG016e54);
  pcA3 = d.CH16482;
//016e62 4247                     CLR.W    D7
//016e64 6000 01f0                BRA      $+498 (=0x017056)
  for (D7W=0; D7W < d.NumCharacter; D7W++, pcA3++)
  {
//016e68 3007                     MOVE.W   D7,D0
//016e6a e380                     ASL.L    #1,D0
//016e6c 41ec bf4a                LEA      -16566(A4),A0
//016e70 d0c0                     ADD.W    D0,A0
//016e72 3010                     MOVE.W   (A0),D0
    D4W = d.PendingOuches[D7W];
//016e74 3800                     MOVE.W   D0,D4
//016e76 816b 0032                OR.W     D0,50(A3)
    pcA3->ouches |= D4W;
//016e7a 3007                     MOVE.W   D7,D0
//016e7c e380                     ASL.L    #1,D0
//016e7e 41ec bf4a                LEA      -16566(A4),A0
//016e82 d0c0                     ADD.W    D0,A0
//016e84 4250                     CLR.W    (A0)
    d.PendingOuches[D7W] = 0;
//016e86 3007                     MOVE.W   D7,D0
//016e88 e380                     ASL.L    #1,D0
//016e8a 41ec bf52                LEA      -16558(A4),A0
//016e8e d0c0                     ADD.W    D0,A0
//016e90 3010                     MOVE.W   (A0),D0
//016e92 3c00                     MOVE.W   D0,D6
    D6W = d.PendingDamage[D7W];
//016e94 6604                     BNE      $+6 (=0x016e9a)
//016e96 6000 01b4                BRA      $+438 (=0x01704c)
    if (D6W == 0) continue;
//016e9a 3007                     MOVE.W   D7,D0
//016e9c e380                     ASL.L    #1,D0
//016e9e 41ec bf52                LEA      -16558(A4),A0
//016ea2 d0c0                     ADD.W    D0,A0
//016ea4 4250                     CLR.W    (A0)
    d.PendingDamage[D7W] = 0;
//016ea6 302b 0034                MOVE.W   52(A3),D0
//016eaa 3a00                     MOVE.W   D0,D5
    D5W = pcA3->HP;
//016eac 6604                     BNE      $+6 (=0x016eb2)
//016eae 6000 019c                BRA      $+414 (=0x01704c)
    if (D5W== 0) continue;
//016eb2 3005                     MOVE.W   D5,D0
//016eb4 9046                     SUB.W    D6,D0
//016eb6 3a00                     MOVE.W   D0,D5
    D5W = sw(D5W - D6W);
//016eb8 4a40                     TST.W    D0
//016eba 6e0c                     BGT      $+14 (=0x016ec8)
    if (D5W <= 0)
    {
//016ebc 3f07                     MOVE.W   D7,-(A7)
//016ebe 4eba fd9a                JSR      $-612 (=0x016c5a)
//016ec2 548f                     ADDQ.L   #2,A7
      KillCharacter(D7W);
//016ec4 6000 0186                BRA      $+392 (=0x01704c)
      continue;
    };
//016ec8 3745 0034                MOVE.W   D5,52(A3)
    pcA3->HP = D5W;
//016ecc 006b 0100 0030           OR.W     #256,48(A3)
    pcA3->word48 |= 0x100;
//016ed2 3004                     MOVE.W   D4,D0
//016ed4 6706                     BEQ      $+8 (=0x016edc)
    if (D4W)
    {
//016ed6 006b 2000 0030           OR.W     #8192,48(A3)
      pcA3->word48 |= 0x2000;
    };
//016edc 3a07                     MOVE.W   D7,D5
//016ede cafc 0045                MULU     #69,D5
    D5L = 69 * D7W;
//016ee2 426e fff0                CLR.W    -16(A6)
    LOCAL_20.w.y1 = 0;
//016ee6 4ead 0024                JSR      36(A5)    (=0x002fd2)
    STHideCursor();
//016eea 3007                     MOVE.W   D7,D0
//016eec 5240                     ADDQ.W   #1,D0
//016eee 322c bf1a                MOVE.W   -16614(A4),D1
//016ef2 b041                     CMP.W    D1,D0
//016ef4 6658                     BNE      $+90 (=0x016f4e)
    if (D7W + 1 == d.SelectedCharacterOrdinal)
    {
//016ef6 3d7c 001c fff2           MOVE.W   #28,-14(A6)
      LOCAL_20.w.y2 = 28;
//016efc 3005                     MOVE.W   D5,D0
//016efe 5e40                     ADDQ.W   #7,D0
//016f00 3d40 ffec                MOVE.W   D0,-20(A6)
      LOCAL_20.w.x1 = sw(D5W + 7);
//016f04 d07c 001f                ADD.W    #31,D0
//016f08 3d40 ffee                MOVE.W   D0,-18(A6)
      LOCAL_20.w.x2 = sw(LOCAL_20.w.x1 + 31);
//016f0c 2f3c 0010 000a           MOVE.L   #1048586,-(A7)
//016f12 486e ffec                PEA      -20(A6)
//016f16 3f3c 0010                MOVE.W   #16,-(A7)
//016f1a 4ead 061e                JSR      1566(A5)    (=0x022a92)
//016f1e 548f                     ADDQ.L   #2,A7
//016f20 2f00                     MOVE.L   D0,-(A7)
//016f22 4ead 00a2                JSR      162(A5)    (=0x000f7e)
//016f26 defc 000c                ADD.W    #12,A7
      BLT2Screen(GetGraphicAddress(16), &LOCAL_20, 16, 10);
//016f2a 0c46 000a                CMP.W    #10,D6
//016f2e 6406                     BCC      $+8 (=0x016f36)
      if (D6W < 10)
      {
//016f30 da7c 0015                ADD.W    #21,D5
        D5W += 21;
//016f34 6010                     BRA      $+18 (=0x016f46)
      }
      else
      {
//016f36 0c46 0064                CMP.W    #100,D6
//016f3a 6406                     BCC      $+8 (=0x016f42)
        if (D6W < 100)
        {
//016f3c da7c 0012                ADD.W    #18,D5
          D5W += 18;
//016f40 6004                     BRA      $+6 (=0x016f46)
        }
        else
        {
//016f42 da7c 000f                ADD.W    #15,D5
          D5W += 15;
        };
      };
//016f46 3d7c 0010 fffe           MOVE.W   #16,-2(A6)
      LOCAL_2 = 16;
//016f4c 6054                     BRA      $+86 (=0x016fa2)
    }
    else
    {
//016f4e 3d7c 0006 fff2           MOVE.W   #6,-14(A6)
      LOCAL_20.w.y2 = 6;
//016f54 3005                     MOVE.W   D5,D0
//016f56 3d40 ffec                MOVE.W   D0,-20(A6)
      LOCAL_20.w.x1 = D5W;
//016f5a d07c 002f                ADD.W    #47,D0
//016f5e 3d40 ffee                MOVE.W   D0,-18(A6)
      LOCAL_20.w.x2 = sw(LOCAL_20.w.x1 + 47);
//016f62 2f3c 0018 000a           MOVE.L   #1572874,-(A7)
//016f68 486e ffec                PEA      -20(A6)
//016f6c 3f3c 000f                MOVE.W   #15,-(A7)
//016f70 4ead 061e                JSR      1566(A5)    (=0x022a92)
//016f74 548f                     ADDQ.L   #2,A7
//016f76 2f00                     MOVE.L   D0,-(A7)
//016f78 4ead 00a2                JSR      162(A5)    (=0x000f7e)
//016f7c defc 000c                ADD.W    #12,A7
      BLT2Screen(GetGraphicAddress(15),&LOCAL_20,24,10);
//016f80 0c46 000a                CMP.W    #10,D6
//016f84 6406                     BCC      $+8 (=0x016f8c)
      if (D6W < 10)
      {
//016f86 da7c 0013                ADD.W    #19,D5
        D5W += 19;
//016f8a 6010                     BRA      $+18 (=0x016f9c)
      }
      else
      {
//016f8c 0c46 0064                CMP.W    #100,D6
//016f90 6406                     BCC      $+8 (=0x016f98)
        if (D6W < 100)
        {
//016f92 da7c 0010                ADD.W    #16,D5
          D5W += 16;
//016f96 6004                     BRA      $+6 (=0x016f9c)
        }
        else
        {
//016f98 da7c 000d                ADD.W    #13,D5
          D5W += 13;
        };
      };
//016f9c 3d7c 0005 fffe           MOVE.W   #5,-2(A6)
      LOCAL_2 = 5;
    };
//016fa2 3f3c 0003                MOVE.W   #3,-(A7)
//016fa6 4267                     CLR.W    -(A7)
//016fa8 3f06                     MOVE.W   D6,-(A7)
//016faa 4eba db4a                JSR      $-9396 (=0x014af6)
//016fae 5c8f                     ADDQ.L   #6,A7
    A0 = TAG014af6(D6W, 0, 3);
//016fb0 2f00                     MOVE.L   D0,-(A7)
//016fb2 2f3c 000f 0008           MOVE.L   #983048,-(A7)
//016fb8 3f2e fffe                MOVE.W   -2(A6),-(A7)
//016fbc 3f05                     MOVE.W   D5,-(A7)
//016fbe 4ead 0108                JSR      264(A5)    (=0x001c42)
//016fc2 defc 000c                ADD.W    #12,A7
    TextOutToScreen(D5W, LOCAL_2, 15, 8, A0);
//016fc6 302b 002e                MOVE.W   46(A3),D0
//016fca 3a00                     MOVE.W   D0,D5
    D5W = pcA3->timerIndex;
//016fcc 0c40 ffff                CMP.W    #65535,D0
//016fd0 6636                     BNE      $+56 (=0x017008)
    if (D5W == -1)
    {
//016fd2 1d7c 000c fff8           MOVE.B   #12,-8(A6)
      LOCAL_12.function = 12;
//016fd8 202c d23a                MOVE.L   -11718(A4),D0
//016fdc 5a80                     ADDQ.L   #5,D0
      D0L = d.Time + 5;
//016fde 322c d244                MOVE.W   -11708(A4),D1
//016fe2 48c1                     EXT.L    D1
      D1L = d.partyLevel;
//016fe4 263c 0000 0018           MOVE.L   #24,D3
      D3L = 24;
//016fea e7a1                     ASL.L    D3,D1
      D1L <<= D3W;
//016fec 8081                     OR.L     D1,D0
      D0L |= D1L;
//016fee 2d40 fff4                MOVE.L   D0,-12(A6)
      LOCAL_12.time = D0L;
//016ff2 3007                     MOVE.W   D7,D0
//016ff4 1d40 fff9                MOVE.B   D0,-7(A6)
      LOCAL_12.uByte5 = (i8)D7W;
//016ff8 486e fff4                PEA      -12(A6)
//016ffc 4ead 0366                JSR      870(A5)    (=0x00ffbe)
//017000 588f                     ADDQ.L   #4,A7
//017002 3740 002e                MOVE.W   D0,46(A3)
      pcA3->timerIndex = SetTimer(&LOCAL_12);
//017006 6038                     BRA      $+58 (=0x017040)
    }
    else
    {
//017008 3005                     MOVE.W   D5,D0
//01700a c1fc 000a                MULS     #10,D0
//01700e 206c cd3e                MOVE.L   -12994(A4),A0
//017012 d0c0                     ADD.W    D0,A0
//017014 41d0                     LEA      (A0),A0
//017016 2448                     MOVE.L   A0,A2
      ptA2 = &d.Timers[D5W];
//017018 202c d23a                MOVE.L   -11718(A4),D0
//01701c 5a80                     ADDQ.L   #5,D0
      D0L = d.Time+5;
//01701e 322c d244                MOVE.W   -11708(A4),D1
      D1L = d.partyLevel << 24;
//017022 48c1                     EXT.L    D1
//017024 263c 0000 0018           MOVE.L   #24,D3
//01702a e7a1                     ASL.L    D3,D1
//01702c 8081                     OR.L     D1,D0
//01702e 2480                     MOVE.L   D0,(A2)
      ptA2->time = D0L | D1L;
//017030 3f05                     MOVE.W   D5,-(A7)
//017032 4ead 03cc                JSR      972(A5)    (=0x00fdce)
//017036 548f                     ADDQ.L   #2,A7
//017038 3f00                     MOVE.W   D0,-(A7)
//01703a 4ead 0372                JSR      882(A5)    (=0x00fdf0)
//01703e 548f                     ADDQ.L   #2,A7
      TAG00fdf0(FindTimerPosition(D5W));
    };
//017040 3f07                     MOVE.W   D7,-(A7)
//017042 4eba dd9c                JSR      $-8802 (=0x014de0)
//017046 548f                     ADDQ.L   #2,A7
    DrawCharacterState(D7W);
//017048 4ead 00a8                JSR      168(A5)    (=0x003026)
    STShowCursor();
   //continue here
//01704c 3007                     MOVE.W   D7,D0
//01704e 5247                     ADDQ.W   #1,D7
//017050 200b                     MOVE.L   A3,D0
//017052 d6fc 0320                ADD.W    #800,A3

//017056 3007                     MOVE.W   D7,D0
//017058 b06c d24c                CMP.W    -11700(A4),D0
//01705c 6500 fe0a                BCS      $-500 (=0x016e68)
  };
//017060 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//017064 4e5e                     UNLK     A6
//017066 4e75                     RTS
}


void TAG017274(i32 chIdx,i32 P2)
{//(void)
  CHARDESC  *pch_14;
  TIMER    timer;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//017274 4e56 fff2                LINK     A6,#-14
//017278 0c6e ffff 0008           CMP.W    #65535,8(A6)
//01727e 670c                     BEQ      $+14 (=0x01728c)
//017280 302e 0008                MOVE.W   8(A6),D0
//017284 5240                     ADDQ.W   #1,D0
//017286 b06c d258                CMP.W    -11688(A4),D0
//01728a 6604                     BNE      $+6 (=0x017290)
//01728c 6000 00b2                BRA      $+180 (=0x017340)
  if (chIdx == -1) return;
  if (chIdx+1 == d.PotentialCharacterOrdinal) return;
//017290 302e 0008                MOVE.W   8(A6),D0
//017294 c1fc 0320                MULS     #800,D0
//017298 41ec bf9e                LEA      -16482(A4),A0
//01729c d0c0                     ADD.W    D0,A0
//01729e 41d0                     LEA      (A0),A0
//0172a0 2d48 fff2                MOVE.L   A0,-14(A6)
  pch_14 = &d.CH16482[chIdx];
//0172a4 4267                     CLR.W    -(A7)
//0172a6 4267                     CLR.W    -(A7)
//0172a8 302e 000a                MOVE.W   10(A6),D0
//0172ac ec48                     LSR.W    #6,D0
//0172ae 3f00                     MOVE.W   D0,-(A7)
//0172b0 3f3c 0001                MOVE.W   #1,-(A7)
//0172b4 4ead 00d2                JSR      210(A5)    (=0x000ff8=Max)
//0172b8 588f                     ADDQ.L   #4,A7
//0172ba 3f00                     MOVE.W   D0,-(A7)
//0172bc 3f2e 0008                MOVE.W   8(A6),-(A7)
//0172c0 4eba fda6                JSR      $-600 (=0x017068=DamageCharacter)
//0172c4 508f                     ADDQ.L   #0,A7
  DamageCharacter(chIdx, Larger(1, P2/64), 0, 0);
//0172c6 206e fff2                MOVE.L   -14(A6),A0
//0172ca 0068 0100 0030           OR.W     #256,48(A0)
  pch_14->word48 |= 0x100;
//0172d0 302e 0008                MOVE.W   8(A6),D0
//0172d4 5240                     ADDQ.W   #1,D0
//0172d6 b06c bf1a                CMP.W    -16614(A4),D0
//0172da 6610                     BNE      $+18 (=0x0172ec)
//0172dc 4a6c bf18                TST.W    -16616(A4)
//0172e0 660a                     BNE      $+12 (=0x0172ec)
  if (    (chIdx+1 == d.SelectedCharacterOrdinal)
       && (d.WpnChestOrScroll == 0)   )
  {
//0172e2 206e fff2                MOVE.L   -14(A6),A0
//0172e6 0068 0800 0030           OR.W     #2048,48(A0)
    pch_14->word48 |= 0x800;
  };
//0172ec 536e 000a                SUBQ.W   #1,10(A6)
  P2--;
//0172f0 6744                     BEQ      $+70 (=0x017336)
  if (P2 != 0)
  {
//0172f2 206e fff2                MOVE.L   -14(A6),A0
//0172f6 5228 002a                ADDQ.B   #1,42(A0)
    pch_14->uByte42++;
//0172fa 1d7c 004b fffa           MOVE.B   #75,-6(A6)
    timer.function = 75;
//017300 302e 0008                MOVE.W   8(A6),D0
//017304 1d40 fffb                MOVE.B   D0,-5(A6)
    timer.uByte5 = (i8)chIdx;
//017308 202c d23a                MOVE.L   -11718(A4),D0
//01730c d0bc 0000 0024           ADD.L    #36,D0
//017312 322c d244                MOVE.W   -11708(A4),D1
//017316 48c1                     EXT.L    D1
//017318 263c 0000 0018           MOVE.L   #24,D3
//01731e e7a1                     ASL.L    D3,D1
//017320 8081                     OR.L     D1,D0
//017322 2d40 fff6                MOVE.L   D0,-10(A6)
    timer.time = (d.Time+36)|(d.partyLevel<<24);
//017326 3d6e 000a fffc           MOVE.W   10(A6),-4(A6)
    timer.word6() = sw(P2);
//01732c 486e fff6                PEA      -10(A6)
//017330 4ead 0366                JSR      870(A5)    (=0x00ffbe=SetTimer)
//017334 588f                     ADDQ.L   #4,A7
    SetTimer(&timer);
  };
//017336 3f2e 0008                MOVE.W   8(A6),-(A7)
//01733a 4eba daa4                JSR      $-9562 (=0x014de0=DrawCharacterState)
//01733e 548f                     ADDQ.L   #2,A7
  DrawCharacterState(chIdx);
//017340 4e5e                     UNLK     A6
//017342 4e75                     RTS
}

//   TAG017fa8
void SelectPaletteForLightLevel(void)
{
  dReg D0, D4, D5, D6;
  RN objD0;
  OBJECTTYPE otD0;
  i16  LOCAL_26[8];
  RN   LOCAL_10;
  i16 temp;
  CHARDESC *pCharacter;
  i16 *pwA0, *pwA2, *pwA3;
  i32 totalLight;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//017fa8 4e56 ffe6                LINK     A6,#-26
//017fac 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//017fb0 206c d750                MOVE.L   -10416(A4),A0
//017fb4 3028 000c                MOVE.W   12(A0),D0
//017fb8 c07c f000                AND.W    #61440,D0
//017fbc e848                     LSR.W    #4,D0
//017fbe e048                     LSR.W    #8,D0
  D0W = d.pCurLevelDesc->experienceMultiplier();
//017fc0 4a40                     TST.W    D0
//017fc2 6608                     BNE      $+10 (=0x017fcc)
  if (D0W == 0)
  {
//017fc4 426c d24e                CLR.W    -11698(A4)
    d.CurrentPalette = 0;
//017fc8 6000 014c                BRA      $+334 (=0x018116)
  }
  else
  {
//017fcc 3a3c 0004                MOVE.W   #4,D5
//017fd0 41ec bf9e                LEA      -16482(A4),A0
    pCharacter = &d.CH16482[0];
//017fd4 2d48 fff8                MOVE.L   A0,-8(A6)
//017fd8 41ee ffe6                LEA      -26(A6),A0
//017fdc 2648                     MOVE.L   A0,A3
    pwA3 = LOCAL_26;
//017fde 6000 0082                BRA      $+132 (=0x018062)
    for (D5W=4; (D5W--)!=0; pCharacter++)
    {
//017fe2 383c 0002                MOVE.W   #2,D4
//017fe6 606a                     BRA      $+108 (=0x018052)
      for (D4W=2; (D4W--)!=0; pwA3++)
      {
//017fe8 3004                     MOVE.W   D4,D0
//017fea 206e fff8                MOVE.L   -8(A6),A0
//017fee e380                     ASL.L    #1,D0
//017ff0 41e8 00d4                LEA      212(A0),A0
//017ff4 d0c0                     ADD.W    D0,A0
//017ff6 3010                     MOVE.W   (A0),D0
        objD0 = pCharacter->possessions[D4W];
//017ff8 3d40 fff6                MOVE.W   D0,-10(A6)
        LOCAL_10 = objD0;
//017ffc 3f00                     MOVE.W   D0,-(A7)
//017ffe 4ead 0186                JSR      390(A5)    (=0x00114c)
//018002 548f                     ADDQ.L   #2,A7
        otD0 = GetBasicObjectType(objD0);
//018004 0c40 0004                CMP.W    #4,D0
//018008 6d44                     BLT      $+70 (=0x01804e)
        if (   (otD0 >= obj_Torch_a)
            && (otD0 <= obj_Torch_d) )
        {
//01800a 3004                     MOVE.W   D4,D0
//01800c 206e fff8                MOVE.L   -8(A6),A0
//018010 e380                     ASL.L    #1,D0
//018012 41e8 00d4                LEA      212(A0),A0
//018016 d0c0                     ADD.W    D0,A0
//018018 3010                     MOVE.W   (A0),D0
//01801a 3d40 fff6                MOVE.W   D0,-10(A6)
//01801e 3f00                     MOVE.W   D0,-(A7)
//018020 4ead 0186                JSR      390(A5)    (=0x00114c)
//018024 548f                     ADDQ.L   #2,A7
//018026 0c40 0007                CMP.W    #7,D0
//01802a 6e22                     BGT      $+36 (=0x01804e)
//01802c 3f2e fff6                MOVE.W   -10(A6),-(A7)
//018030 4ead 0312                JSR      786(A5)    (=0x009984)
//018034 548f                     ADDQ.L   #2,A7
          pwA0 = (i16 *)GetRecordAddress(LOCAL_10);
//018036 2d40 fffc                MOVE.L   D0,-4(A6)
//01803a 206e fffc                MOVE.L   -4(A6),A0
//01803e 3028 0002                MOVE.W   2(A0),D0
//018042 c07c 3c00                AND.W    #15360,D0
//018046 e448                     LSR.W    #2,D0
//018048 e048                     LSR.W    #8,D0
//01804a 3680                     MOVE.W   D0,(A3)
          *pwA3 = BITS12_15(pwA0[1]);
//01804c 6002                     BRA      $+4 (=0x018050)
          continue;
        };
//01804e 4253                     CLR.W    (A3)
        *pwA3 = 0;
//018050 548b                     ADDQ.L   #2,A3
//018052 3004                     MOVE.W   D4,D0
//018054 5344                     SUBQ.W   #1,D4
//018056 4a40                     TST.W    D0
//018058 668e                     BNE      $-112 (=0x017fe8)
      };
//01805a 06ae 0000 0320 fff8      ADD.L    #800,-8(A6)
//018062 3005                     MOVE.W   D5,D0
//018064 5345                     SUBQ.W   #1,D5
//018066 4a40                     TST.W    D0
//018068 6600 ff78                BNE      $-134 (=0x017fe2)
    };
//01806c 41ee ffe6                LEA      -26(A6),A0
//018070 2648                     MOVE.L   A0,A3
    pwA3 = LOCAL_26;
//018072 4244                     CLR.W    D4
//018074 6034                     BRA      $+54 (=0x0180aa)
    for (D4W=0; D4W!=4; D4W++, pwA3++)
    { // Find four brightest torches.
//018076 3a3c 0007                MOVE.W   #7,D5
//01807a 9a44                     SUB.W    D4,D5
      D5W = sw(7 - D4W);
//01807c 3004                     MOVE.W   D4,D0
//01807e e380                     ASL.L    #1,D0
//018080 41ee ffe8                LEA      -24(A6),A0
//018084 d0c0                     ADD.W    D0,A0
//018086 41d0                     LEA      (A0),A0
//018088 2448                     MOVE.L   A0,A2
      for (pwA2=&LOCAL_26[D4W+1]; (D5W--)!=0; pwA2++)
      {
//01808a 6012                     BRA      $+20 (=0x01809e)
//01808c 3012                     MOVE.W   (A2),D0
//01808e b053                     CMP.W    (A3),D0
//018090 6f0a                     BLE      $+12 (=0x01809c)
        if (*pwA2 > *pwA3)
        {
//018092 3d52 fff6                MOVE.W   (A2),-10(A6)
          temp = *pwA2;
//018096 3493                     MOVE.W   (A3),(A2)
          *pwA2 = *pwA3;
//018098 36ae fff6                MOVE.W   -10(A6),(A3)
          *pwA3 = temp;
        };
//01809c 548a                     ADDQ.L   #2,A2

//01809e 3005                     MOVE.W   D5,D0
//0180a0 5345                     SUBQ.W   #1,D5
//0180a2 4a40                     TST.W    D0
//0180a4 66e6                     BNE      $-24 (=0x01808c)
      };
//0180a6 548b                     ADDQ.L   #2,A3
//0180a8 5244                     ADDQ.W   #1,D4

//0180aa 0c44 0004                CMP.W    #4,D4
//0180ae 66c6                     BNE      $-56 (=0x018076)
    };
//0180b0 3c3c 0006                MOVE.W   #6,D6
//0180b4 383c 0005                MOVE.W   #5,D4
//0180b8 4247                     CLR.W    D7
    D6W = 6;
    D4W = 5;
    totalLight = 0;
//0180ba 41ee ffe6                LEA      -26(A6),A0
//0180be 2648                     MOVE.L   A0,A3
//0180c0 6028                     BRA      $+42 (=0x0180ea)
    for (pwA3=LOCAL_26; (D4W--)!=0; pwA3++)
    {
//0180c2 3013                     MOVE.W   (A3),D0
//0180c4 6722                     BEQ      $+36 (=0x0180e8)
      if (*pwA3 == 0) continue;
//0180c6 3013                     MOVE.W   (A3),D0
//0180c8 e380                     ASL.L    #1,D0
//0180ca 41ec fbce                LEA      -1074(A4),A0
//0180ce d0c0                     ADD.W    D0,A0
//0180d0 3010                     MOVE.W   (A0),D0
      D0W = d.Word1074[*pwA3];
//0180d2 ed60                     ASL.W    D6,D0
      D0W <<= D6W;
//0180d4 ec48                     LSR.W    #6,D0
      D0W = (I16)((D0W >> 6) & 0x3ff); // logical shift
//0180d6 de40                     ADD.W    D0,D7
      totalLight += D0W; // Sum of torches
//0180d8 3006                     MOVE.W   D6,D0
//0180da 5340                     SUBQ.W   #1,D0
//0180dc 3f00                     MOVE.W   D0,-(A7)
//0180de 4267                     CLR.W    -(A7)
//0180e0 4ead 00d2                JSR      210(A5)    (=0x000ff8)
//0180e4 588f                     ADDQ.L   #4,A7
      D6W = sw(Larger(0, D6W-1));
//0180e6 3c00                     MOVE.W   D0,D6
//0180e8 548b                     ADDQ.L   #2,A3

//0180ea 3004                     MOVE.W   D4,D0
//0180ec 5344                     SUBQ.W   #1,D4
//0180ee 4a40                     TST.W    D0
//0180f0 66d0                     BNE      $-46 (=0x0180c2)
    };
//0180f2 de6c cc1e                ADD.W    -13282(A4),D7
//0180f6 41ec fbc2                LEA      -1086(A4),A0
    totalLight += d.Brightness;
//0180fa 2648                     MOVE.L   A0,A3
    pwA3 = d.PaletteBrightness;
//0180fc 4a47                     TST.W    D7
//0180fe 6f0e                     BLE      $+16 (=0x01810e)
    if (totalLight > 0)
    {
//018100 4244                     CLR.W    D4
      D4W = 0;
      while (*(pwA3++) > totalLight)
      {
//018102 6002                     BRA      $+4 (=0x018106)
//018104 5244                     ADDQ.W   #1,D4
        D4W++;
//018106 301b                     MOVE.W   (A3)+,D0
//018108 b047                     CMP.W    D7,D0
//01810a 6ef8                     BGT      $-6 (=0x018104)
      };
//01810c 6004                     BRA      $+6 (=0x018112)
    }
    else
    {
//01810e 383c 0005                MOVE.W   #5,D4
      D4W = 5;
    };
//018112 3944 d24e                MOVE.W   D4,-11698(A4)
    d.CurrentPalette = D4W;
  };
//018116 397c 0001 d1fe           MOVE.W   #1,-11778(A4)
  d.Word11778 = 1;
//01811c 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//018120 4e5e                     UNLK     A6
//018122 4e75                     RTS
}

//   TAG018124
void NinetySecondUpdate(void)
{ //(void)
  // Decrement bits 10-13 of weapon types 4, 5, 6, and 7
  dReg D0, D3, D5, D6, D7;
  OBJECTTYPE otD4;
  DB5      *DB5A2;
  CHARDESC *pcA3;
#ifdef _DEBUG
  verifyppq();
#endif
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//018124 4e56 0000                LINK     A6,#0
//018128 48e7 0f30                MOVEM.L  D4567A23,-(A7)
//01812c 4245                     CLR.W    D5
  D5W = 0;
//01812e 3e2c d24c                MOVE.W   -11700(A4),D7
  D7W = d.NumCharacter;
//018132 302c d258                MOVE.W   -11688(A4),D0
//018136 6702                     BEQ      $+4 (=0x01813a)
  if (d.PotentialCharacterOrdinal != 0) D7W--;
//018138 5347                     SUBQ.W   #1,D7
//01813a 41ec bf9e                LEA      -16482(A4),A0
//01813e 2648                     MOVE.L   A0,A3
  pcA3 = d.CH16482;
//018140 6000 009c                BRA      $+158 (=0x0181de)
  while ((D7W--) != 0)
  {
//018144 3c3c 0002                MOVE.W   #2,D6
//018148 6000 0086                BRA      $+136 (=0x0181d0)
    for (D6W=2; (D6W--)!= 0;) // One and zero.  Hands.
    {
//01814c 3006                     MOVE.W   D6,D0
//01814e e380                     ASL.L    #1,D0
//018150 41eb 00d4                LEA      212(A3),A0
//018154 d0c0                     ADD.W    D0,A0
//018156 3f10                     MOVE.W   (A0),-(A7)
//018158 4ead 014a                JSR      330(A5)    (=0x001188)
//01815c 548f                     ADDQ.L   #2,A7
//01815e 3800                     MOVE.W   D0,D4
      otD4 = GetModifiedObjectType(pcA3->possessions[D6W]);
//018160 0c44 0004                CMP.W    #4,D4
//018164 6d6a                     BLT      $+108 (=0x0181d0)
      if (otD4 < obj_Torch_a) continue;
//018166 0c44 0007                CMP.W    #7,D4
//01816a 6e64                     BGT      $+102 (=0x0181d0)
      if (otD4 > obj_Torch_d) continue;
//01816c 3006                     MOVE.W   D6,D0
//01816e e380                     ASL.L    #1,D0
//018170 41eb 00d4                LEA      212(A3),A0
//018174 d0c0                     ADD.W    D0,A0
//018176 3f10                     MOVE.W   (A0),-(A7)
//018178 4ead 0312                JSR      786(A5)    (=0x009984)
//01817c 548f                     ADDQ.L   #2,A7
//01817e 2440                     MOVE.L   D0,A2
      DB5A2 = GetRecordAddressDB5(pcA3->possessions[D6W]);
//018180 302a 0002                MOVE.W   2(A2),D0
//018184 c07c 3c00                AND.W    #15360,D0
//018188 e448                     LSR.W    #2,D0
//01818a e048                     LSR.W    #8,D0
      D0W = DB5A2->charges();
//01818c 6742                     BEQ      $+68 (=0x0181d0)
      if (D0W == 0) continue;
//01818e 302a 0002                MOVE.W   2(A2),D0
//018192 c07c 3c00                AND.W    #15360,D0
//018196 e448                     LSR.W    #2,D0
//018198 e048                     LSR.W    #8,D0
//01819a 5340                     SUBQ.W   #1,D0
//01819c c07c 000f                AND.W    #15,D0
      D3W = (I16)((D0W-1) & 15);
//0181a0 3600                     MOVE.W   D0,D3
//0181a2 e543                     ASL.W    #2,D3
//0181a4 e143                     ASL.W    #8,D3
      //D3W <<= 10;
//0181a6 026a c3ff 0002           AND.W    #50175,2(A2)
      //dbA2->word(2) &= 0xc3ff;
      //dbA2->word(2) |= D3W;
      DB5A2->charges(D3W);
//0181ac 876a 0002                OR.W     D3,2(A2)
//0181b0 3000                     MOVE.W   D0,D0
//0181b2 4a40                     TST.W    D0
//0181b4 6616                     BNE      $+24 (=0x0181cc)
      if (D3W == 0)
      {
//0181b6 303c 0000                MOVE.W   #0,D0
//0181ba c07c 0001                AND.W    #1,D0
//0181be 3600                     MOVE.W   D0,D3
//0181c0 ef43                     ASL.W    #7,D3
//0181c2 026a ff7f 0002           AND.W    #65407,2(A2)
        DB5A2->important(false);
//0181c8 876a 0002                OR.W     D3,2(A2)
//0181cc 3a3c 0001                MOVE.W   #1,D5
        D5W = 1;
      };
//
//0181d0 3006                     MOVE.W   D6,D0
//0181d2 5346                     SUBQ.W   #1,D6
//0181d4 4a40                     TST.W    D0
//0181d6 6600 ff74                BNE      $-138 (=0x01814c)
    };
//0181da d6fc 0320                ADD.W    #800,A3
    pcA3++;

//0181de 3007                     MOVE.W   D7,D0
//0181e0 5347                     SUBQ.W   #1,D7
//0181e2 4a40                     TST.W    D0
//0181e4 6600 ff5e                BNE      $-160 (=0x018144)
  };
//0181e8 0c6c 0088 fc6a           CMP.W    #136,-918(A4)
//0181ee 6034                     BRA      $+54 (=0x018224)
  //if (0)
  //{
//0181f0 3f3c 0006                MOVE.W   #6,-(A7)
//0181f4 486c cd44                PEA      -12988(A4)
//0181f8 486c be76                PEA      -16778(A4)
//0181fc 41ec d204                LEA      -11772(A4),A0
//018200 2008                     MOVE.L   A0,D0
//018202 d0bc 0000 005e           ADD.L    #94,D0
//018208 2f00                     MOVE.L   D0,-(A7)
//01820a 3f3c 0015                MOVE.W   #21,-(A7)
//01820e 486d 061e                PEA      1566(A5)
//018212 487a f604                PEA      $-2554 (=0x017818)
//018216 4ead 01f2                JSR      498(A5)    (=0x00907e)
//01821a defc 0018                ADD.W    #24,A7
//01821e 397c 0088 fc6a           MOVE.W   #136,-918(A4)
  //};
//018224 3005                     MOVE.W   D5,D0
//018226 6708                     BEQ      $+10 (=0x018230)
  if (D5W != 0)
  {
//018228 4eba fd7e                JSR      $-640 (=0x017fa8)
    SelectPaletteForLightLevel();
//01822c 4eba d2da                JSR      $-11556 (=0x015508)
    DrawEightHands();
  };
//018230 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//018234 4e5e                     UNLK     A6
//018236 4e75                     RTS
}


void TAG019018(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//019018 4e56 0000                LINK     A6,#0
//01901c 4eba f9b6                JSR      $-1608 (=0x0189d4)
  TAG0189d4();
//019020 4267                     CLR.W    -(A7)
//019022 4ead 01ec                JSR      492(A5)    (=0x0042da=MarkViewportUpdated)
//019026 548f                     ADDQ.L   #2,A7
  MarkViewportUpdated(0);
//019028 397c 0001 a5b2           MOVE.W   #1,-23118(A4)
//01902e 4ead 00a8                JSR      168(A5)    (=0x003026=ShowCursor)
  STShowCursor();
//019032 4e5e                     UNLK     A6
//019034 4e75                     RTS
}


void TAG019264(void)
{//(void)
  RN objD7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//019264 4e56 0000                LINK     A6,#0
//019268 3f07                     MOVE.W   D7,-(A7)
//01926a 2f3c 000c 000d           MOVE.L   #786445,-(A7)
//019270 3f3c 00ca                MOVE.W   #202,-(A7)
//019274 4eba ea04                JSR      $-5626 (=0x017c7a)
//019278 5c8f                     ADDQ.L   #6,A7
  DrawSmallIcon(obj_Special_d, 12, 13);//Eye back to normal.
//01927a 4eba f758                JSR      $-2214 (=0x0189d4)
  TAG0189d4();
//01927e 4267                     CLR.W    -(A7)
//019280 4ead 01ec                JSR      492(A5)    (=0x0042da)
//019284 548f                     ADDQ.L   #2,A7
  MarkViewportUpdated(0);
//019286 302c bf40                MOVE.W   -16576(A4),D0
//01928a 3e00                     MOVE.W   D0,D7
  objD7 = d.objectInHand;
//01928c 0c40 ffff                CMP.W    #65535,D0
//019290 6708                     BEQ      $+10 (=0x01929a)
  if (objD7 != RNnul)
  {
//019292 3f07                     MOVE.W   D7,-(A7)
//019294 4ead 0162                JSR      354(A5)    (=0x0012a0)
//019298 548f                     ADDQ.L   #2,A7
    DrawNameOfHeldObject(objD7);
  };
//01929a 4ead 00a8                JSR      168(A5)    (=0x003026)
  STShowCursor();
//01929e 3e1f                     MOVE.W   (A7)+,D7
//0192a0 4e5e                     UNLK     A6
//0192a2 4e75                     RTS
}
//*********************************************************
//
//*********************************************************
void TAG0192f4(i32 chIdx)
{//(void)
  dReg D0, D6;
  CHARDESC *pcA3;
  i32 itemNum;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0192f4 4e56 0000                LINK     A6,#0
//0192f8 48e7 0310                MOVEM.L  D67A3,-(A7)
//0192fc 3e2e 0008                MOVE.W   8(A6),D7
//019300 0c47 0004                CMP.W    #4,D7
//019304 6714                     BEQ      $+22 (=0x01931a)
  if (chIdx != 4)
  {
//019306 3007                     MOVE.W   D7,D0
//019308 c1fc 0320                MULS     #800,D0
//01930c 41ec bfd2                LEA      -16430(A4),A0
//019310 d0c0                     ADD.W    D0,A0
//019312 3010                     MOVE.W   (A0),D0
    D0W = d.CH16482[chIdx].HP;
//019314 6604                     BNE      $+6 (=0x01931a)
//019316 6000 01a0                BRA      $+418 (=0x0194b8)
    if (D0W == 0) return;
  };
//01931a 302c d210                MOVE.W   -11760(A4),D0
//01931e 6606                     BNE      $+8 (=0x019326)
  if (d.Word11760 != 0) return;
//019320 302c d214                MOVE.W   -11756(A4),D0
//019324 6704                     BEQ      $+6 (=0x01932a)
//019326 6000 0190                BRA      $+402 (=0x0194b8)
  if (d.Word11756 != 0) return;
//01932a 397c 0001 d228           MOVE.W   #1,-11736(A4)
  d.clockTick = 1;
//019330 0c6c 022b f8d6           CMP.W    #555,-1834(A4)
//019336 6014                     BRA      $+22 (=0x01934c)
//
//019338 202c d23a                MOVE.L   -11718(A4),D0
//01933c 90ac bf28                SUB.L    -16600(A4),D0
//019340 0c80 0000 03e8           CMP.L    #1000,D0
//019346 6f04                     BLE      $+6 (=0x01934c)
//019348 526c d216                ADDQ.W   #1,-11754(A4)
//
//01934c 3c2c bf1a                MOVE.W   -16614(A4),D6
  D6W = d.SelectedCharacterOrdinal;
//019350 3007                     MOVE.W   D7,D0
//019352 5240                     ADDQ.W   #1,D0
//019354 b046                     CMP.W    D6,D0
//019356 6604                     BNE      $+6 (=0x01935c)
  if ((chIdx+1) == D6W)
  {
//019358 3e3c 0004                MOVE.W   #4,D7
    chIdx = 4;
  };
//01935c 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();
//019360 3006                     MOVE.W   D6,D0
//019362 6774                     BEQ      $+118 (=0x0193d8)
  if (D6W)
  {
//019364 426c bf1a                CLR.W    -16614(A4)
    d.SelectedCharacterOrdinal = 0;
//019368 4eba ea3a                JSR      $-5572 (=0x017da4)
    TAG017da4();
//01936c 3006                     MOVE.W   D6,D0
//01936e c1fc 0320                MULS     #800,D0
//019372 41ec bc7e                LEA      -17282(A4),A0
//019376 d0c0                     ADD.W    D0,A0
//019378 41d0                     LEA      (A0),A0
//01937a 2648                     MOVE.L   A0,A3
    pcA3 = &d.CH16482[D6W-1];
//01937c 302b 0034                MOVE.W   52(A3),D0
//019380 6718                     BEQ      $+26 (=0x01939a)
    if (pcA3->HP)
    {
//019382 302c d258                MOVE.W   -11688(A4),D0
//019386 6612                     BNE      $+20 (=0x01939a)
      if (d.PotentialCharacterOrdinal == 0)
      {
//019388 006b 1000 0030           OR.W     #4096,48(A3)
        pcA3->word48 |= 0x1000;
//01938e 3006                     MOVE.W   D6,D0
//019390 5340                     SUBQ.W   #1,D0
//019392 3f00                     MOVE.W   D0,-(A7)
//019394 4eba ba4a                JSR      $-17844 (=0x014de0)
//019398 548f                     ADDQ.L   #2,A7
        DrawCharacterState(D6W-1);//box at top, pos, dir,weapon
      };
    };
//01939a 302c d256                MOVE.W   -11690(A4),D0
//01939e 6708                     BEQ      $+10 (=0x0193a8)
    if (d.PartySleeping)
    {
//0193a0 4ead 00a8                JSR      168(A5)    (=0x003026)
      STShowCursor();
//0193a4 6000 0112                BRA      $+276 (=0x0194b8)
      return;
    };
//0193a8 0c47 0004                CMP.W    #4,D7
//0193ac 662a                     BNE      $+44 (=0x0193d8)
    if (chIdx == 4)
    {
//0193ae 397c 0001 d21e           MOVE.W   #1,-11746(A4)
      d.Word11746 = 1;
//0193b4 4ead 0528                JSR      1320(A5)    (=0x01b880)
      DrawMovementButtons();
//0193b8 4ead 00a8                JSR      168(A5)    (=0x003026)
      STShowCursor();
//0193bc 41ec bc68                LEA      -17304(A4),A0
//0193c0 2948 be14                MOVE.L   A0,-16876(A4)
      d.SecondaryButtonList = d.Buttons17304;
//0193c4 41ec b804                LEA      -18428(A4),A0
//0193c8 2948 be0c                MOVE.L   A0,-16884(A4)
      d.pKeyXlate1 = d.Byte18428;
//0193cc 4ead 052e                JSR      1326(A5)    (=0x01953a)
      TAG01953a();
//0193d0 4ead 01c2                JSR      450(A5)    (=0x004308)
      FloorAndCeilingOnly(); //Clears viewport to floor and ceiling
//0193d4 6000 00e2                BRA      $+228 (=0x0194b8)
    return;
    };
  };
//0193d8 426c a608                CLR.W    -23032(A4)
  d.UseByteCoordinates = 0;
//0193dc 3007                     MOVE.W   D7,D0
//0193de 5240                     ADDQ.W   #1,D0
//0193e0 3940 bf1a                MOVE.W   D0,-16614(A4)
  d.SelectedCharacterOrdinal = sw(chIdx + 1);
//0193e4 3006                     MOVE.W   D6,D0
//0193e6 660c                     BNE      $+14 (=0x0193f4)
  if (D6W == 0)
  {
//0193e8 4267                     CLR.W    -(A7)
//0193ea 486c ffe4                PEA      -28(A4)
//0193ee 4ead 0204                JSR      516(A5)    (=0x009058)
//0193f2 5c8f                     ADDQ.L   #6,A7
    ShadeRectangleInScreen((RectPos *)d.Word28, 0);
  };
//0193f4 3007                     MOVE.W   D7,D0
//0193f6 c1fc 0320                MULS     #800,D0
//0193fa 41ec bf9e                LEA      -16482(A4),A0
//0193fe d0c0                     ADD.W    D0,A0
//019400 41d0                     LEA      (A0),A0
//019402 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[chIdx];
//019404 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//019408 3f3c 0011                MOVE.W   #17,-(A7)
//01940c 4ead 0618                JSR      1560(A5)    (=0x022a60)
//019410 5c8f                     ADDQ.L   #6,A7
  TAG022a60(17, d.pViewportBMP);
//019412 302c d258                MOVE.W   -11688(A4),D0
//019416 6716                     BEQ      $+24 (=0x01942e)
  if (d.PotentialCharacterOrdinal)
  {
//019418 2f3c 000c 0070           MOVE.L   #786544,-(A7)
//01941e 486c fbba                PEA      -1094(A4)
//019422 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//019426 4ead 01ce                JSR      462(A5)    (=0x008e60)
//01942a defc 000c                ADD.W    #12,A7
    FillRectangle(d.pViewportBMP, (RectPos *)d.Byte1094, 12, 112);
  };
//01942e 486c 020e                PEA      526(A4)
//019432 2f3c 0074 000d           MOVE.L   #7602189,-(A7)
//019438 3f3c 0005                MOVE.W   #5,-(A7)
//01943c 4ead 00f0                JSR      240(A5)    (=0x001c16)
//019440 defc 000a                ADD.W    #10,A7
  TextToViewport(5, 116, 13, "HEALTH");
//019444 486c 0216                PEA      534(A4)
//019448 2f3c 007c 000d           MOVE.L   #8126477,-(A7)
//01944e 3f3c 0005                MOVE.W   #5,-(A7)
//019452 4ead 00f0                JSR      240(A5)    (=0x001c16)
//019456 defc 000a                ADD.W    #10,A7
  TextToViewport(5, 124, 13, "STAMINA");
//01945a 486c 021e                PEA      542(A4)
//01945e 2f3c 0084 000d           MOVE.L   #8650765,-(A7)
//019464 3f3c 0005                MOVE.W   #5,-(A7)
//019468 4ead 00f0                JSR      240(A5)    (=0x001c16)
//01946c defc 000a                ADD.W    #10,A7
  TextToViewport(5, 132, 13, "MANA");

//019470 4246                     CLR.W    D6
//019472 600c                     BRA      $+14 (=0x019480)
  for (itemNum=0; itemNum<30; itemNum++)
  {
    DisplayBackpackItem(chIdx, itemNum);
  };
//019486 006b 5b80 0030           OR.W     #23424,48(A3)
  pcA3->word48 |= 0x5b80;
//01948c 3f3c 0025                MOVE.W   #37,-(A7)
//019490 4ead 0138                JSR      312(A5)    (=0x001ce8)
//019494 548f                     ADDQ.L   #2,A7
  //NotImplemented(0x99977b);//wvbl(_1_);
//019496 3f07                     MOVE.W   D7,-(A7)
//019498 4eba b946                JSR      $-18104 (=0x014de0)
//01949c 548f                     ADDQ.L   #2,A7
  DrawCharacterState(chIdx); // Box at top,pos,dir,weapon

//01949e 397c 0001 a59a           MOVE.W   #1,-23142(A4)
  d.NewCursorShape = 1;
//0194a4 4ead 00a8                JSR      168(A5)    (=0x003026)
  STShowCursor();
//0194a8 41ec baa0                LEA      -17760(A4),A0
//0194ac 2948 be14                MOVE.L   A0,-16876(A4)
  d.SecondaryButtonList = d.Buttons17760;
//0194b0 42ac be0c                CLR.L    -16884(A4)
  d.pKeyXlate1 = NULL;
//0194b4 4ead 052e                JSR      1326(A5)    (=0x01953a)
  TAG01953a();
//0194b8 4cdf 08c0                MOVEM.L  (A7)+,D67A3
//0194bc 4e5e                     UNLK     A6
//0194be 4e75                     RTS
  return;
}

i16 TAG0194c0(pnt)
{
  d.Word4012 = 3217;
  d.Word19218 = 12777;
  return 1;
}

void TAG01953a(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01953a 4e56 0000                LINK     A6,#0
//01953e 600a                     BRA      $+12 (=0x01954a)
  while (UI_CONSTAT()!=0) UI_DIRECT_CONIN();
  {
//019540 3f3c 0007                MOVE.W   #7,-(A7)
//019544 4ead 0096                JSR      150(A5)    (=0x001cf8)
//019548 548f                     ADDQ.L   #2,A7

//01954a 3f3c 000b                MOVE.W   #11,-(A7)
//01954e 4ead 0096                JSR      150(A5)    (=0x001cf8)
//019552 548f                     ADDQ.L   #2,A7
//019554 4a40                     TST.W    D0
//019556 66e8                     BNE      $-22 (=0x019540)
  };
//019558 397c 0001 be26           MOVE.W   #1,-16858(A4)
//01955e 426c be2a                CLR.W    -16854(A4)
//019562 397c 0004 be28           MOVE.W   #4,-16856(A4)
//019568 426c be26                CLR.W    -16858(A4)
  d.MouseInterlock = 1;
  d.MouseQStart = 0;
  d.MouseQEnd = 4;
  d.MouseInterlock = 0;
//01956c 4e5e                     UNLK     A6
//01956e 4e75                     RTS
}

i16 SearchButtonList(btn *ent,i32 x,i32 y,i32 buttons) //TAG019570
{
  dReg D0, D4, D5, D6, D7;
// ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//019570 4e56 0000                LINK     A6,#0
//019574 48e7 0f10                MOVEM.L  D4567A3,-(A7)
  //SaveRegs(0x0f10);
//019578 266e 0008                MOVE.L   8(A6),A3
//01957c 3e2e 000c                MOVE.W   12(A6),D7
//019580 3c2e 000e                MOVE.W   14(A6),D6
//019584 3a2e 0010                MOVE.W   16(A6),D5
  //A3 = firstEnt;
  D7W = sw(x);
  if ( (D7W >= 512) && (D7W <= 515)) D7W = 0xee;
  D6W = sw(y);
  D5W = sw(buttons);
//019588 b7fc 0000 0000           CMP.L    #0,A3
//01958e 6706                     BEQ      $+8 (=0x019596)
//019590 302c be1c                MOVE.W   -16868(A4),D0
//019594 6706                     BEQ      $+8 (=0x01959c)
  if ((ent == NULL) || (d.Word16868 != 0))
  {
    D0L=0;
  }
//019596 303c 0000                MOVE.W   #0,D0
//01959a 6036                     BRA      $+56 (=0x0195d2)
  else
  {
    while ((D4W = ent->word0) != 0)
    {
//01959c 602e                     BRA      $+48 (=0x0195cc)
//01959e 3007                     MOVE.W   D7,D0
//0195a0 b06b 0004                CMP.W    4(A3),D0
//0195a4 6e22                     BGT      $+36 (=0x0195c8)
      if (   (D7W <  ent->xMax)
          && (D7W >= ent->xMin)
          && (D6W >= ent->yMin)
          && (D6W <  ent->yMax) )
      {
//0195a6 3007                     MOVE.W   D7,D0
//0195a8 b06b 0002                CMP.W    2(A3),D0
//0195ac 6d1a                     BLT      $+28 (=0x0195c8)

//0195ae 3006                     MOVE.W   D6,D0
//0195b0 b06b 0006                CMP.W    6(A3),D0
//0195b4 6d12                     BLT      $+20 (=0x0195c8)

//0195b6 3006                     MOVE.W   D6,D0
//0195b8 b06b 0008                CMP.W    8(A3),D0
//0195bc 6e0a                     BGT      $+12 (=0x0195c8)
//0195be 3005                     MOVE.W   D5,D0
//0195c0 c06b 000a                AND.W    10(A3),D0
        if (D5W & ent->button) break;
//0195c4 6702                     BEQ      $+4 (=0x0195c8)
//0195c6 6008                     BRA      $+10 (=0x0195d0)
      }
//0195c8 d6fc 000c                ADD.W    #12,A3
      ent++;
    };
//0195cc 3813                     MOVE.W   (A3),D4
//0195ce 66ce                     BNE      $-48 (=0x01959e)
//0195d0 3004                     MOVE.W   D4,D0
    D0W = D4W;
  };
//0195d2 4cdf 08f0                MOVEM.L  (A7)+,D4567A3
  //RestoreRegs(0x08f0);
//0195d6 4e5e                     UNLK     A6
//0195d8 4e75                     RTS
  return D0W;
}

//   TAG0195da
void OnMouseClick(i32 x,i32 y,i32 buttons)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  dReg D0, D4, D5, D6, D7;
//0195da 4e56 0000                LINK     A6,#0
  //SaveRegs(0x0f00);
//0195de 48e7 0f00                MOVEM.L  D4567,-(A7)
//0195e2 3e2e 0008                MOVE.W   8(A6),D7
//0195e6 3c2e 000a                MOVE.W   10(A6),D6
  D7W = sw(x);
  D6W = sw(y);
//0195ea 4aec be26                TAS      -16858(A4)
//0195ee 6732                     BEQ      $+52 (=0x019622)
  if (d.MouseInterlock != 0)
  {
//0195f0 397c 0001 be24           MOVE.W   #1,-16860(A4)
    d.FakeMouseClick = 1;
//0195f6 3947 be22                MOVE.W   D7,-16862(A4)
//0195fa 3946 be20                MOVE.W   D6,-16864(A4)
    d.FakeMouseX = D7W;
    d.FakeMouseY = D6W;
//0195fe 396e 000c be1e           MOVE.W   12(A6),-16866(A4)
    d.FakeMouseButton = sw(buttons);
//019604 302c be1c                MOVE.W   -16868(A4),D0
    D0W = d.Word16868;
//019608 6000 00a4                BRA      $+166 (=0x0196ae)
  }
  else
  {
    d.MouseInterlock = 1;
//019622 302c be28                MOVE.W   -16856(A4),D0
//019626 5440                     ADDQ.W   #2,D0
//019628 3a00                     MOVE.W   D0,D5
    D0W = D5W = sw(d.MouseQEnd + 2);
//01962a 0c40 0004                CMP.W    #4,D0
//01962e 6f02                     BLE      $+4 (=0x019632)
    if (D0W > 4) D5W -= 5;
//019630 5b45                     SUBQ.W   #5,D5
//019632 3005                     MOVE.W   D5,D0
  D0W = D5W;
//019634 b06c be2a                CMP.W    -16854(A4),D0
//019638 6602                     BNE      $+4 (=0x01963c)
//01963a 6072                     BRA      $+116 (=0x0196ae)
    if (D0W != d.MouseQStart)
    { //We cannot make End = Start-1 because that means empty!
//01963c 3f2e 000c                MOVE.W   12(A6),-(A7)
//019640 3f06                     MOVE.W   D6,-(A7)
//019642 3f07                     MOVE.W   D7,-(A7)
//019644 2f2c be18                MOVE.L   -16872(A4),-(A7)
//019648 4eba ff26                JSR      $-216 (=0x019570)
//01964c defc 000a                ADD.W    #10,A7
      D4W = SearchButtonList(d.PrimaryButtonList,
                              D7W,
                              D6W,
                              buttons);
//019650 3800                     MOVE.W   D0,D4
//019652 4a44                     TST.W    D4
//019654 6616                     BNE      $+24 (=0x01966c)
      if (D4W == 0)
      {
//019656 3f2e 000c                MOVE.W   12(A6),-(A7)
//01965a 3f06                     MOVE.W   D6,-(A7)
//01965c 3f07                     MOVE.W   D7,-(A7)
//01965e 2f2c be14                MOVE.L   -16876(A4),-(A7)
//019662 4eba ff0c                JSR      $-242 (=0x019570)
//019666 defc 000a                ADD.W    #10,A7
      D4W = SearchButtonList(d.SecondaryButtonList,
                              D7W,
                              D6W,
                              buttons);
//01966a 3800                     MOVE.W   D0,D4
      };
//01966c 4a44                     TST.W    D4
//01966e 673a                     BEQ      $+60 (=0x0196aa)
      if (D4W != 0)
      {
//019670 3005                     MOVE.W   D5,D0
//019672 5345                     SUBQ.W   #1,D5
//019674 4a40                     TST.W    D0
//019676 6604                     BNE      $+6 (=0x01967c)
        D5W--;
        if (D5W < 0) D5W = 4;
//019678 3a3c 0004                MOVE.W   #4,D5
//01967c 3005                     MOVE.W   D5,D0
//01967e 3940 be28                MOVE.W   D0,-16856(A4)
        d.MouseQEnd = D5W;
//019682 c1fc 0006                MULS     #6,D0
      //A0 = d.Pointer16848 + D5W * 6;
//019686 41ec be30                LEA      -16848(A4),A0
//01968a d0c0                     ADD.W    D0,A0
//01968c 3084                     MOVE.W   D4,(A0)
        d.MouseQueue[D5W].num = D4W;
//01968e 3005                     MOVE.W   D5,D0
//019690 c1fc 0006                MULS     #6,D0
//019694 41ec be2c                LEA      -16852(A4),A0
//019698 d0c0                     ADD.W    D0,A0
      //A0 = d.Pointer16852 + D5W * 6;
//01969a 3087                     MOVE.W   D7,(A0)
        d.MouseQueue[D5W].x = D7W;
//01969c 3005                     MOVE.W   D5,D0
//01969e c1fc 0006                MULS     #6,D0
//0196a2 41ec be2e                LEA      -16850(A4),A0
//0196a6 d0c0                     ADD.W    D0,A0
      //A0 = d.Pointer16850 + 6 * D5W;
//0196a8 3086                     MOVE.W   D6,(A0)
        d.MouseQueue[D5W].y =  D6W;
      };
//0196aa 426c be26                CLR.W    -16858(A4)
      d.MouseInterlock = 0;
    };
  };
  //RestoreRegs(0x00f0);
//0196ae 4cdf 00f0                MOVEM.L  (A7)+,D4567
//0196b2 4e5e                     UNLK     A6
//0196b4 4e75                     RTS
}


//   TAG0196b6
void CauseFakeMouseClick(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0196b6 4e56 0000                LINK     A6,#0
//0196ba 302c be24                MOVE.W   -16860(A4),D0
//0196be 6716                     BEQ      $+24 (=0x0196d6)
  if (d.FakeMouseClick != 0)
  {
//0196c0 426c be24                CLR.W    -16860(A4)
    d.FakeMouseClick = 0;
//0196c4 3f2c be1e                MOVE.W   -16866(A4),-(A7)
//0196c8 3f2c be20                MOVE.W   -16864(A4),-(A7)
//0196cc 3f2c be22                MOVE.W   -16862(A4),-(A7)
//0196d0 4eba ff08                JSR      $-246 (=0x0195da)
//0196d4 5c8f                     ADDQ.L   #6,A7
    OnMouseClick(d.FakeMouseX,
                 d.FakeMouseY,
                 d.FakeMouseButton);
  };
//0196d6 4e5e                     UNLK     A6
//0196d8 4e75                     RTS
}

//   TAG01979c
RESTARTABLE _FlashButton(const i32 x1,
                         const i32 x2,
                         const i32 y1,
                         const i32 y2)
{//(void)
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
  while (d.FlashButnActive != 0) {};
  d.FlashButnX1 = sw(x1);
  d.FlashButnX2 = sw(x2);
  d.FlashButnY1 = sw(y1);
  d.FlashButnY2 = sw(y2);
  d.FlashButnActive = 1;
  wvbl(_1_);
  display();
  RETURN;
}

//   TAG0197f6
void TraverseStairway(i32 UpOrDown)
{ //(void)
  // non-zero =decrement level = up
  i32 x, y;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0197f6 4e56 fffe                LINK     A6,#-2
//0197fa 4267                     CLR.W    -(A7)
//0197fc 3f3c ffff                MOVE.W   #65535,-(A7)
//019800 3f2c d248                MOVE.W   -11704(A4),-(A7)
//019804 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019808 3f3c ffff                MOVE.W   #65535,-(A7)
//01980c 4ead 03a2                JSR      930(A5)    (=0x011f52=MoveObject)
//019810 defc 000a                ADD.W    #10,A7
  MoveObject(RN(RNnul), d.partyX, d.partyY, -1, 0);
//019814 486c d248                PEA      -11704(A4)
//019818 486c d24a                PEA      -11702(A4)
//01981c 302e 0008                MOVE.W   8(A6),D0
//019820 6706                     BEQ      $+8 (=0x019828)
//019822 303c ffff                MOVE.W   #65535,D0
//019826 6004                     BRA      $+6 (=0x01982c)
//019828 303c 0001                MOVE.W   #1,D0
//01982c 3f00                     MOVE.W   D0,-(A7)
//01982e 3f2c d244                MOVE.W   -11708(A4),-(A7)
//019832 4ead 031e                JSR      798(A5)    (=0x009800)
//019836 defc 000c                ADD.W    #12,A7
//01983a 3940 d21c                MOVE.W   D0,-11748(A4)
  // Compute relative x and y on the new level.
  x = d.partyX;
  y = d.partyY;
  d.Word11748 = IncrementLevel(d.partyLevel,
                               UpOrDown == 0 ? 1 : -1,
                               &x,
                               &y);
  d.partyX = sw(x);
  d.partyY = sw(y);
//01983e 3f2c d21c                MOVE.W   -11748(A4),-(A7)
//019842 4ead 027c                JSR      636(A5)    (=0x00a9d4)
//019846 548f                     ADDQ.L   #2,A7
  LoadLevel(d.Word11748);
//019848 3f2c d248                MOVE.W   -11704(A4),-(A7)
//01984c 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019850 4ead 030c                JSR      780(A5)    (=0x009904)
//019854 588f                     ADDQ.L   #4,A7
//019856 3f00                     MOVE.W   D0,-(A7)
//019858 4ead 0462                JSR      1122(A5)    (=0x01470c)
//01985c 548f                     ADDQ.L   #2,A7
  SetPartyFacing(ExitStairsDirection(d.partyX, d.partyY));
//01985e 3f2c d244                MOVE.W   -11708(A4),-(A7)
//019862 4ead 027c                JSR      636(A5)    (=0x00a9d4)
  LoadLevel(d.partyLevel);
//019866 548f                     ADDQ.L   #2,A7
//019868 4e5e                     UNLK     A6
//01986a 4e75                     RTS
}

//   TAG01986c
RESTARTABLE _TurnParty(const i32 direction)
{ //(void)
  // 1 = left
  // 2 = right
  static dReg D0, D7;
  static i32 deltaFacing;
  RESTARTMAP
    RESTART(2)
    RESTART(3)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01986c 4e56 0000                LINK     A6,#0
//019870 3f07                     MOVE.W   D7,-(A7)
//019872 397c 0001 d228           MOVE.W   #1,-11736(A4)
  if (extraTicks)
  {
    d.clockTick = 1;
  };
//019878 0c6e 0001 0008           CMP.W    #1,8(A6)
//01987e 6614                     BNE      $+22 (=0x019894)
  if (direction ==1) // Turn left
  {
//019880 2f3c 007d 0091           MOVE.L   #8192145,-(A7)
//019886 2f3c 00ea 0105           MOVE.L   #15335685,-(A7)
//01988c 4eba ff0e                JSR      $-240 (=0x01979c)
//019890 508f                     ADDQ.L   #0,A7
    FlashButton(_2_,0xea, 0x0105, 0x7d, 0x91);
//019892 6012                     BRA      $+20 (=0x0198a6)
  }
  else // Turn right
  {
//019894 2f3c 007d 0091           MOVE.L   #8192145,-(A7)
//01989a 2f3c 0123 013e           MOVE.L   #19071294,-(A7)
//0198a0 4eba fefa                JSR      $-260 (=0x01979c)
    FlashButton(_3_,0x123, 0x13e, 0x7d, 0x91);
//0198a4 508f                     ADDQ.L   #0,A7
  };
//0198a6 3f2c d248                MOVE.W   -11704(A4),-(A7)
//0198aa 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//0198ae 4ead 0252                JSR      594(A5)    (=0x009672)
//0198b2 588f                     ADDQ.L   #4,A7
  D0W = GetCellFlags(d.partyX, d.partyY);
//0198b4 c07c 00ff                AND.W    #255,D0
//0198b8 3e00                     MOVE.W   D0,D7
  D7W = (I16)(D0W & 0xff);
//0198ba ea48                     LSR.W    #5,D0
//0198bc 0c40 0003                CMP.W    #3,D0
//0198c0 6610                     BNE      $+18 (=0x0198d2)
  if (BITS5_15(D0W) == 3)
  {
//0198c2 3007                     MOVE.W   D7,D0
//0198c4 c07c 0004                AND.W    #4,D0
//0198c8 3f00                     MOVE.W   D0,-(A7)
//0198ca 4eba ff2a                JSR      $-212 (=0x0197f6)
//0198ce 548f                     ADDQ.L   #2,A7
    TraverseStairway(D7W & 4); //non-zero = down
//0198d0 6056                     BRA      $+88 (=0x019928)
  }
  else
  {
//0198d2 4267                     CLR.W    -(A7)
//0198d4 2f3c ffff 0001           MOVE.L   #-65535,-(A7)
//0198da 3f2c d248                MOVE.W   -11704(A4),-(A7)
//0198de 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//0198e2 4ead 0354                JSR      852(A5)    (=0x013380)
//0198e6 defc 000a                ADD.W    #10,A7
    PlaceObjectInRoom(d.partyX,
                      d.partyY,
                      RN(RNnul),
                      1,  // In current room
                      0); // Not a loose object
//0198ea 302c d246                MOVE.W   -11706(A4),D0
//0198ee 0c6e 0002 0008           CMP.W    #2,8(A6)
//0198f4 6606                     BNE      $+8 (=0x0198fc)
    if (direction == 2)
    {
//0198f6 323c 0001                MOVE.W   #1,D1
      deltaFacing = 1; // Increment to facing
//0198fa 6004                     BRA      $+6 (=0x019900)
    }
    else
    {
//0198fc 323c 0003                MOVE.W   #3,D1
      deltaFacing = 3; // Increment to facing
    };
//019900 d041                     ADD.W    D1,D0
//019902 c07c 0003                AND.W    #3,D0
//019906 3f00                     MOVE.W   D0,-(A7)
//019908 4ead 0462                JSR      1122(A5)    (=0x01470c)
//01990c 548f                     ADDQ.L   #2,A7
    SetPartyFacing((d.partyFacing + deltaFacing) & 3);
//01990e 2f3c 0001 0001           MOVE.L   #65537,-(A7)
//019914 3f3c ffff                MOVE.W   #65535,-(A7)
//019918 3f2c d248                MOVE.W   -11704(A4),-(A7)
//01991c 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019920 4ead 0354                JSR      852(A5)    (=0x013380)
    PlaceObjectInRoom(d.partyX,
                      d.partyY,
                      RN(RNnul),
                      1,         // In current room
                      1);        // Loose object
//019924 defc 000a                ADD.W    #10,A7
  };
//019928 3e1f                     MOVE.W   (A7)+,D7
//01992a 4e5e                     UNLK     A6
//01992c 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
//   TAG01992e
RESTARTABLE _MoveParty(const i32 button)
{//(void)
  // 03 Move forward
  // 04 Slide right
  // 05 Move backward
  // 06 Slide Left
  static dReg D0, D5, D7;
  static ROOMTYPE rtD6;
  static RN objD0;
  static i32 charIdx;
  static i32 i;
  static bool bD0, bD5;
  static ui16 LOCAL_30;
  static i32 maxLoad;
  static i16 LOCAL_10;
  static i16 LOCAL_8;
  static bool startingInStairwell; //LOCAL_6
  static i32 newY;
  static i32 newX;
  static CHARDESC *pCharacter;
  static MOVEBUTN *pmpA2;
  static i32 direction;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
  direction = button;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01992e 4e56 ffe2                LINK     A6,#-30
//019932 48e7 0f30                MOVEM.L  D4567A23,-(A7)
  if (TimerTraceActive)
  {
    char *pdir;
    fprintf(TraceFile,"MoveParty ");
    switch(direction)
    {
    case 3: pdir="forward"; break;
    case 4: pdir="right";   break;
    case 5: pdir="backward";break;
    case 6: pdir="left";    break;
    default:pdir="????";
    };
    fprintf(TraceFile,"%s from ", pdir);
    printAbsLocation(TraceFile,d.partyLevel,d.partyX,d.partyY);
    fprintf(TraceFile,"\n");
  };
  direction -= 3;
//019936 397c 0001 d228           MOVE.W   #1,-11736(A4)
  if (extraTicks)
  {
    d.clockTick = 1;
  };
//01993c 41ec bf9e                LEA      -16482(A4),A0
//019940 2648                     MOVE.L   A0,A3
//019942 4244                     CLR.W    D4
//019944 6030                     BRA      $+50 (=0x019976)
  for (charIdx=0, pCharacter = &d.CH16482[0];
       charIdx < d.NumCharacter;
       charIdx++, pCharacter++)
  {
//019946 2f0b                     MOVE.L   A3,-(A7)
//019948 4ead 048c                JSR      1164(A5)    (=0x016508)
//01994c 588f                     ADDQ.L   #4,A7
    maxLoad = MaxLoad(pCharacter); //A multiple of 10. why??
//01994e 3d40 ffec                MOVE.W   D0,-20(A6)
//019952 302b 0110                MOVE.W   272(A3),D0
//019956 c0fc 0003                MULU     #3,D0
//01995a c0bc 0000 ffff           AND.L    #65535,D0
//019960 80ee ffec                DIVU     -20(A6),D0
//019964 5240                     ADDQ.W   #1,D0
    D0W = sw((3*pCharacter->load)/maxLoad + 1);
//019966 3f00                     MOVE.W   D0,-(A7)
//019968 3f04                     MOVE.W   D4,-(A7)
//01996a 4ead 0480                JSR      1152(A5)    (=0x01742a)
//01996e 588f                     ADDQ.L   #4,A7
    AdjustStamina(charIdx, D0W);
//019970 d6fc 0320                ADD.W    #800,A3
//019974 5244                     ADDQ.W   #1,D4

//019976 3004                     MOVE.W   D4,D0
//019978 b06c d24c                CMP.W    -11700(A4),D0
//01997c 65c8                     BCS      $-54 (=0x019946)
  };
//01997e 382e 0008                MOVE.W   8(A6),D4
//019982 5744                     SUBQ.W   #3,D4
//019984 3004                     MOVE.W   D4,D0
//019986 e780                     ASL.L    #3,D0
//019988 41ec b7c0                LEA      -18496(A4),A0
//01998c d0c0                     ADD.W    D0,A0
//01998e 41d0                     LEA      (A0),A0
  pmpA2 = &d.MoveButn18496[direction];
////019990 2448                     MOVE.L   A0,A2
//019992 3f2a 0006                MOVE.W   6(A2),-(A7)
//019996 3f2a 0004                MOVE.W   4(A2),-(A7)
//01999a 3f2a 0002                MOVE.W   2(A2),-(A7)
//01999e 3f12                     MOVE.W   (A2),-(A7)
//0199a0 4eba fdfa                JSR      $-516 (=0x01979c)
//0199a4 508f                     ADDQ.L   #0,A7
  FlashButton(_1_,
              pmpA2->x1,   // TAG1979c
              pmpA2->x2,
              pmpA2->y1,
              pmpA2->y2);
//0199a6 302c d248                MOVE.W   -11704(A4),D0
//0199aa 3d40 fffc                MOVE.W   D0,-4(A6)
//0199ae 3f00                     MOVE.W   D0,-(A7)
//0199b0 302c d24a                MOVE.W   -11702(A4),D0
//0199b4 3d40 fffe                MOVE.W   D0,-2(A6)
//0199b8 3f00                     MOVE.W   D0,-(A7)
//0199ba 4ead 0252                JSR      594(A5)    (=0x009672)
//0199be 588f                     ADDQ.L   #4,A7
  D7W = GetCellFlags(newX=d.partyX, newY=d.partyY);//TAG009672
//0199c0 c07c 00ff                AND.W    #255,D0
//0199c4 3e00                     MOVE.W   D0,D7
//0199c6 ea48                     LSR.W    #5,D0
//0199c8 0c40 0003                CMP.W    #3,D0
//0199cc 57c0                     SEQ      D0
  startingInStairwell = RoomType((ui8)D7W)==roomSTAIRS;//Cell's type
//0199ce c07c 0001                AND.W    #1,D0
//0199d2 3d40 fffa                MOVE.W   D0,-6(A6)
//0199d6 302e fffa                MOVE.W   -6(A6),D0
//0199da 6718                     BEQ      $+26 (=0x0199f4)
//0199dc 0c44 0002                CMP.W    #2,D4
//0199e0 6612                     BNE      $+20 (=0x0199f4)
  if (startingInStairwell && (direction==2)) // backward
  {
//0199e2 3007                     MOVE.W   D7,D0
//0199e4 c07c 0004                AND.W    #4,D0
//0199e8 3f00                     MOVE.W   D0,-(A7)
//0199ea 4eba fe0a                JSR      $-500 (=0x0197f6)
//0199ee 548f                     ADDQ.L   #2,A7
    if (TimerTraceActive)
    {
      fprintf(TraceFile,"Before TraverseStairway party is at ");
      printAbsLocation(TraceFile,d.partyLevel,d.partyX,d.partyY);
      fprintf(TraceFile,"\n");
    };
    TraverseStairway(D7W & 4);//non-zero = decrement level = UP
    if (TimerTraceActive)
    {
      fprintf(TraceFile,"After TraverseStairway party is at ");
      printAbsLocation(TraceFile,d.partyLevel,d.partyX,d.partyY);
      fprintf(TraceFile,"\n");
    };
//0199f0 6000 023a                BRA      $+572 (=0x019c2c)
  }
  else
  {
//0199f4 486e fffc                PEA      -4(A6)
//0199f8 486e fffe                PEA      -2(A6)
//
//0199fc 3004                     MOVE.W   D4,D0
//0199fe e380                     ASL.L    #1,D0
//019a00 41ec b778                LEA      -18568(A4),A0
//019a04 d0c0                     ADD.W    D0,A0
//019a06 3f10                     MOVE.W   (A0),-(A7)
//
//019a08 3004                     MOVE.W   D4,D0
//019a0a e380                     ASL.L    #1,D0
//019a0c 41ec b780                LEA      -18560(A4),A0
//019a10 d0c0                     ADD.W    D0,A0
//019a12 3f10                     MOVE.W   (A0),-(A7)
//
//019a14 3f2c d246                MOVE.W   -11706(A4),-(A7)
//019a18 4ead 023a                JSR      570(A5)    (=0x009622)
//019a1c defc 000e                ADD.W    #14,A7
    UpdateXY( //TAG009622
          d.partyFacing,
          d.Word18560[direction], // #steps forward (-1, 0, or 1)
          d.Word18568[direction], // #steps right (-1, 0, or 1)
          &newX,   // original x to be updated
          &newY);  // original y to be updated


//019a20 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019a24 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019a28 4ead 0252                JSR      594(A5)    (=0x009672)
//019a2c 588f                     ADDQ.L   #4,A7
    D7W = GetCellFlags(newX, newY);
//019a2e c07c 00ff                AND.W    #255,D0
//019a32 3e00                     MOVE.W   D0,D7
//019a34 ea48                     LSR.W    #5,D0
//019a36 3c00                     MOVE.W   D0,D6
    rtD6 = RoomType(D7UB);
//019a38 0c46 0003                CMP.W    #3,D6
//019a3c 6638                     BNE      $+58 (=0x019a76)
    if (rtD6 == roomSTAIRS)
    {
//019a3e 4267                     CLR.W    -(A7)
//019a40 3f3c ffff                MOVE.W   #65535,-(A7)
//019a44 3f2c d248                MOVE.W   -11704(A4),-(A7)
//019a48 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019a4c 3f3c ffff                MOVE.W   #65535,-(A7)
//019a50 4ead 03a2                JSR      930(A5)    (=0x011f52)
//019a54 defc 000a                ADD.W    #10,A7
      MoveObject(RN(RNnul),d.partyX,d.partyY,-1,0);
//019a58 396e fffe d24a           MOVE.W   -2(A6),-11702(A4)
//019a5e 396e fffc d248           MOVE.W   -4(A6),-11704(A4)
      d.partyX = sw(newX);
      d.partyY = sw(newY);
//019a64 3007                     MOVE.W   D7,D0
//019a66 c07c 0004                AND.W    #4,D0
//019a6a 3f00                     MOVE.W   D0,-(A7)
//019a6c 4eba fd88                JSR      $-630 (=0x0197f6)
//019a70 548f                     ADDQ.L   #2,A7
      TraverseStairway(D7W&4);//non-zero = decrement level = UP
//019a72 6000 01b8                BRA      $+442 (=0x019c2c)
    }
    else
    {
//019a76 4245                     CLR.W    D5
      D5W = 0;
//019a78 4a46                     TST.W    D6
//019a7a 6606                     BNE      $+8 (=0x019a82)
      if (rtD6 == roomSTONE) // Stone wall
      {
//019a7c 3a3c 0001                MOVE.W   #1,D5
        D5W = 1; // Inhibit movement.
//019a80 6050                     BRA      $+82 (=0x019ad2)
      }
      else
      {
//019a82 0c46 0004                CMP.W    #4,D6
//019a86 662a                     BNE      $+44 (=0x019ab2)
        if (rtD6 == roomDOOR)
        {
          i32 doorCondition = D7W & 7;
          D5W = (   (doorCondition!=0)
                  && (doorCondition!=1)
                  && (doorCondition!=5));
//019a88 3a07                     MOVE.W   D7,D5
//019a8a ca7c 0007                AND.W    #7,D5
          //D5W = D7W & 7;//Door type
//019a8e 4a45                     TST.W    D5
//019a90 56c0                     SNE      D0
          //bD0 = D5W!=0; //???
//019a92 4a00                     TST.B    D0
//019a94 6708                     BEQ      $+10 (=0x019a9e)
          //if (bD0)
          //{
//019a96 0c45 0001                CMP.W    #1,D5
//019a9a 56c0                     SNE      D0
            //bD0 = D5W!=1;
//019a9c 4a00                     TST.B    D0
          //};
//019a9e 4a00                     TST.B    D0
//019aa0 6708                     BEQ      $+10 (=0x019aaa)
          //if (bD0)
          //{
//019aa2 0c45 0005                CMP.W    #5,D5
//019aa6 56c0                     SNE      D0
            //bD0 = D5W!=5;
//019aa8 4a00                     TST.B    D0
          //};
//019aaa c07c 0001                AND.W    #1,D0
//019aae 3a00                     MOVE.W   D0,D5
          //D5W = bD0;
//019ab0 6020                     BRA      $+34 (=0x019ad2)
        }
        else
        {
//019ab2 0c46 0006                CMP.W    #6,D6
//019ab6 661a                     BNE      $+28 (=0x019ad2)
          if (rtD6 == roomFALSEWALL)
          {
//019ab8 3a07                     MOVE.W   D7,D5
//019aba ca7c 0004                AND.W    #4,D5
//019abe 57c5                     SEQ      D5
            bD5 = (D7W&4)==0;
//019ac0 4a05                     TST.B    D5
//019ac2 670a                     BEQ      $+12 (=0x019ace)
            if (bD5)
            {
//019ac4 3a07                     MOVE.W   D7,D5
//019ac6 ca7c 0001                AND.W    #1,D5
//019aca 57c5                     SEQ      D5
              bD5 = (D7W&1)==0;
//019acc 4a05                     TST.B    D5
            };
//019ace ca7c 0001                AND.W    #1,D5
            D5W = bD5; // passable if bit 2 set or bit 0 set
          };
        };
      };
//019ad2 4a6c d24c                TST.W    -11700(A4)
//019ad6 6604                     BNE      $+6 (=0x019adc)
//019ad8 6000 00ca                BRA      $+204 (=0x019ba4)
      if (d.NumCharacter != 0)
      {
//019adc 3005                     MOVE.W   D5,D0
//019ade 6700 0094                BEQ      $+150 (=0x019b74)
        if (D5W != 0) // Inhibit movement
        {
//019ae2 2f3c 0018 0002           MOVE.L   #1572866,-(A7)

//019ae8 3f3c 0001                MOVE.W   #1,-(A7)

//019aec 322c d246                MOVE.W   -11706(A4),D1
//019af0 5441                     ADDQ.W   #2,D1
//019af2 3004                     MOVE.W   D4,D0
//019af4 d041                     ADD.W    D1,D0
//019af6 3800                     MOVE.W   D0,D4
//019af8 c07c 0003                AND.W    #3,D0
          // Find which side of new cell we are crossing
          direction = direction + (d.partyFacing + 2);
//019afc 3f00                     MOVE.W   D0,-(A7)
//
//019afe 3f2e fffc                MOVE.W   -4(A6),-(A7)
//
//019b02 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019b06 4ead 0438                JSR      1080(A5)    (=0x01479c)
//019b0a 5c8f                     ADDQ.L   #6,A7
          LOCAL_8 = GetCharacterToDamage(
                       newX,
                       newY,
                       direction & 3);
//019b0c 3d40 fff8                MOVE.W   D0,-8(A6)
//019b10 3f00                     MOVE.W   D0,-(A7)
//019b12 4ead 03ea                JSR      1002(A5)    (=0x017068)
//019b16 508f                     ADDQ.L   #0,A7
          D5W = sw(DamageCharacter(LOCAL_8, 1, 24, 2));//TAG017068
//019b18 3a00                     MOVE.W   D0,D5
//019b1a 3004                     MOVE.W   D4,D0
//019b1c 5240                     ADDQ.W   #1,D0
//019b1e c07c 0003                AND.W    #3,D0
//019b22 3f00                     MOVE.W   D0,-(A7)
//019b24 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019b28 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019b2c 4ead 0438                JSR      1080(A5)    (=0x01479c)
//019b30 5c8f                     ADDQ.L   #6,A7
          LOCAL_10 = GetCharacterToDamage(
                newX,
                newY,
                (direction+1) & 3);
////019b32 3d40 fff6                MOVE.W   D0,-10(A6)
//019b36 3d40 ffe2                MOVE.W   D0,-30(A6)
          LOCAL_30 = LOCAL_10;
//019b3a 302e fff8                MOVE.W   -8(A6),D0
//019b3e b06e ffe2                CMP.W    -30(A6),D0
//019b42 6716                     BEQ      $+24 (=0x019b5a)
          if (LOCAL_8 != LOCAL_30)
          {
//019b44 2f3c 0018 0002           MOVE.L   #1572866,-(A7)
//019b4a 3f3c 0001                MOVE.W   #1,-(A7)
//019b4e 3f2e fff6                MOVE.W   -10(A6),-(A7)
//019b52 4ead 03ea                JSR      1002(A5)    (=0x017068)
//019b56 508f                     ADDQ.L   #0,A7
            D5W |= DamageCharacter(LOCAL_10,1,24,2);
//019b58 8a40                     OR.W     D0,D5
          };
//019b5a 3005                     MOVE.W   D5,D0
//019b5c 6714                     BEQ      $+22 (=0x019b72)
          if (D5W != 0)
          {
//019b5e 4267                     CLR.W    -(A7)
//019b60 3f2c d248                MOVE.W   -11704(A4),-(A7)
//019b64 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019b68 3f3c 0012                MOVE.W   #18,-(A7)
//019b6c 4ead 0120                JSR      288(A5)    (=0x00219a)
//019b70 508f                     ADDQ.L   #0,A7
            QueueSound(18, d.partyX, d.partyY, 0);
          };
//019b72 6030                     BRA      $+50 (=0x019ba4)
        }
        else
        {
//019b74 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019b78 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019b7c 4ead 0270                JSR      624(A5)    (=0x00ab6e)
//019b80 588f                     ADDQ.L   #4,A7
          objD0 = FindFirstMonster(newX, newY);
//019b82 0c40 fffe                CMP.W    #65534,D0
//019b86 56c0                     SNE      D0
          bD0 = objD0 != RNeof;
//019b88 c07c 0001                AND.W    #1,D0
//019b8c 3a00                     MOVE.W   D0,D5
          D5W = bD0;
//019b8e 6714                     BEQ      $+22 (=0x019ba4)
          if (D5W != 0)
          {
//019b90 4267                     CLR.W    -(A7)
//019b92 3f3c ffff                MOVE.W   #65535,-(A7)
//019b96 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019b9a 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019b9e 4ead 0216                JSR      534(A5)    (=0x00cbc6)
//019ba2 508f                     ADDQ.L   #0,A7
            ProcessTimers29to41(newX, newY, -1, 0);
          };
        };
      };
//019ba4 3005                     MOVE.W   D5,D0
//019ba6 670a                     BEQ      $+12 (=0x019bb2)
      if (D5W != 0)
      {
//019ba8 4eba f990                JSR      $-1646 (=0x01953a)
        TAG01953a();
//019bac 426c d228                CLR.W    -11736(A4)
        d.clockTick = 0;
//019bb0 607a                     BRA      $+124 (=0x019c2c)
      }
      else
      {
//019bb2 302e fffa                MOVE.W   -6(A6),D0
//019bb6 671a                     BEQ      $+28 (=0x019bd2)
        if (startingInStairwell)
        {
//019bb8 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019bbc 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019bc0 4267                     CLR.W    -(A7)
//019bc2 2f3c ffff ffff           MOVE.L   #-1,-(A7)
//019bc8 4ead 03a2                JSR      930(A5)    (=0x011f52)
//019bcc defc 000a                ADD.W    #10,A7
          MoveObject(RN(RNnul), -1, 0, newX, newY);
//019bd0 601c                     BRA      $+30 (=0x019bee)
        }
        else
        {
//019bd2 3f2e fffc                MOVE.W   -4(A6),-(A7)
//019bd6 3f2e fffe                MOVE.W   -2(A6),-(A7)
//019bda 3f2c d248                MOVE.W   -11704(A4),-(A7)
//019bde 3f2c d24a                MOVE.W   -11702(A4),-(A7)
//019be2 3f3c ffff                MOVE.W   #65535,-(A7)
//019be6 4ead 03a2                JSR      930(A5)    (=0x011f52)
//019bea defc 000a                ADD.W    #10,A7
          MoveObject(RN(RNnul),d.partyX,d.partyY,newX,newY);
        };
//019bee 3e3c 0001                MOVE.W   #1,D7
//019bf2 41ec bf9e                LEA      -16482(A4),A0
//019bf6 2648                     MOVE.L   A0,A3
        D7W = 1;
//019bf8 4244                     CLR.W    D4
//019bfa 6020                     BRA      $+34 (=0x019c1c)
        for (i=0, pCharacter=d.CH16482;
             i < d.NumCharacter;
             i++, pCharacter++)
             {  // Find slowest character
//019bfc 302b 0034                MOVE.W   52(A3),D0
//019c00 6714                     BEQ      $+22 (=0x019c16)
          if (pCharacter->HP != 0)
          {
//019c02 2f0b                     MOVE.L   A3,-(A7)
//019c04 4ead 0408                JSR      1032(A5)    (=0x01657e)
//019c08 588f                     ADDQ.L   #4,A7
            D0W = TimeToMove(pCharacter); // Speed of character
//019c0a 3f00                     MOVE.W   D0,-(A7)
//019c0c 3f07                     MOVE.W   D7,-(A7)
//019c0e 4ead 00d2                JSR      210(A5)    (=0x000ff8)
//019c12 588f                     ADDQ.L   #4,A7
            D7W = sw(Larger(D7W, D0W));
//019c14 3e00                     MOVE.W   D0,D7
          };
//019c16 d6fc 0320                ADD.W    #800,A3
//019c1a 5244                     ADDQ.W   #1,D4
//
//019c1c 3004                     MOVE.W   D4,D0
//019c1e b06c d24c                CMP.W    -11700(A4),D0
//019c22 65d8                     BCS      $-38 (=0x019bfc)
        };
//019c24 3947 d242                MOVE.W   D7,-11710(A4)
        d.Word11710 = D7W;
        d.Word11712 = 0;
//019c28 426c d240                CLR.W    -11712(A4)
      };
    };
  };
//019c2c 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
//019c30 4e5e                     UNLK     A6
//019c32 4e75                     RTS
  RETURN;
}
//*********************************************************
//
//*********************************************************
void TAG019c34(i32 button,i32 clickX, i32 clickY)
{//(void)
  dReg D5, D6, D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//019c34 4e56 0000                LINK     A6,#0
//019c38 48e7 0700                MOVEM.L  D567,-(A7)
//019c3c 3e2e 000a                MOVE.W   10(A6),D7
  D7W = sw(clickX);
  D6W = sw(clickY);
//019c40 3c2e 000c                MOVE.W   12(A6),D6
//019c44 302e 0008                MOVE.W   8(A6),D0
//019c48 5240                     ADDQ.W   #1,D0
//019c4a 322c bf1a                MOVE.W   -16614(A4),D1
//019c4e b041                     CMP.W    D1,D0
//019c50 660c                     BNE      $+14 (=0x019c5e)
  if (button+1 == d.SelectedCharacterOrdinal)
  {
//019c52 3f2e 0008                MOVE.W   8(A6),-(A7)
//019c56 4eba 005a                JSR      $+92 (=0x019cb2)
//019c5a 548f                     ADDQ.L   #2,A7
    SetHandOwner(button);
//019c5c 604c                     BRA      $+78 (=0x019caa)
  }
  else
  {
//019c5e 3f3c 0002                MOVE.W   #2,-(A7)
//019c62 3f06                     MOVE.W   D6,-(A7)
//019c64 3f07                     MOVE.W   D7,-(A7)
//019c66 486c b8d8                PEA      -18216(A4)
//019c6a 4eba f904                JSR      $-1786 (=0x019570)
//019c6e defc 000a                ADD.W    #10,A7
//019c72 3a00                     MOVE.W   D0,D5
    D5W = SearchButtonList(d.Buttons18216,D7W, D6W,2);
//019c74 0c45 0010                CMP.W    #16,D5
//019c78 6516                     BCS      $+24 (=0x019c90)
//
//019c7a 0c45 0013                CMP.W    #19,D5
//019c7e 6210                     BHI      $+18 (=0x019c90)
    if ( (D5W>=16) && (D5W<=19) )
    {
//019c80 3005                     MOVE.W   D5,D0
//019c82 907c 0010                SUB.W    #16,D0
//019c86 3f00                     MOVE.W   D0,-(A7)
//019c88 4eba 0028                JSR      $+42 (=0x019cb2)
//019c8c 548f                     ADDQ.L   #2,A7
      SetHandOwner(D5W-16);
//019c8e 601a                     BRA      $+28 (=0x019caa)
    }
    else
    {
//019c90 0c45 0014                CMP.W    #20,D5
//019c94 6514                     BCS      $+22 (=0x019caa)
//019c96 0c45 001b                CMP.W    #27,D5
//019c9a 620e                     BHI      $+16 (=0x019caa)
      if ( (D5W>=20) && (D5W<=27) )
      {
//019c9c 3005                     MOVE.W   D5,D0
//019c9e 907c 0014                SUB.W    #20,D0
//019ca2 3f00                     MOVE.W   D0,-(A7)
//019ca4 4ead 045c                JSR      1116(A5)    (=0x015e0c)
        HandleClothingClick(D5W-20);
//019ca8 548f                     ADDQ.L   #2,A7
      };
    };
  };
//019caa 4cdf 00e0                MOVEM.L  (A7)+,D567
//019cae 4e5e                     UNLK     A6
//019cb0 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG019cb2
void SetHandOwner(i32 chIdx)
{//(void)
  dReg D1, D6;
  CHARDESC *pcA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//019cb2 4e56 0000                LINK     A6,#0
//019cb6 48e7 0710                MOVEM.L  D567A3,-(A7)
//019cba 3e2e 0008                MOVE.W   8(A6),D7
  //D7W = P1;
//019cbe 3007                     MOVE.W   D7,D0
//019cc0 b06c bf48                CMP.W    -16568(A4),D0
//019cc4 6716                     BEQ      $+24 (=0x019cdc)
  if (chIdx == d.HandChar) return; //owner of cursor
//019cc6 0c47 ffff                CMP.W    #65535,D7
//019cca 6714                     BEQ      $+22 (=0x019ce0)
//019ccc 3007                     MOVE.W   D7,D0
//019cce c1fc 0320                MULS     #800,D0
//019cd2 41ec bfd2                LEA      -16430(A4),A0
//019cd6 d0c0                     ADD.W    D0,A0
//019cd8 3010                     MOVE.W   (A0),D0
//019cda 6604                     BNE      $+6 (=0x019ce0)
//019cdc 6000 00aa                BRA      $+172 (=0x019d88)
  if ( (chIdx != -1) && (d.CH16482[chIdx].HP==0) ) return;
//019ce0 0c6c ffff bf48           CMP.W    #65535,-16568(A4)
//019ce6 6740                     BEQ      $+66 (=0x019d28)
  if (d.HandChar != -1) //owner of cursor
  {
//019ce8 302c bf48                MOVE.W   -16568(A4),D0
//019cec 3c00                     MOVE.W   D0,D6
    D6W = d.HandChar; //owner of cursor
//019cee c1fc 0320                MULS     #800,D0
//019cf2 41ec bfce                LEA      -16434(A4),A0
//019cf6 d0c0                     ADD.W    D0,A0
//019cf8 0050 0280                OR.W     #640,(A0)
    d.CH16482[D6W].word48 |= 0x280;
//019cfc 3f2c bf40                MOVE.W   -16576(A4),-(A7)
//019d00 4ead 0234                JSR      564(A5)    (=0x0091e6)
//019d04 548f                     ADDQ.L   #2,A7
//019d06 2200                     MOVE.L   D0,D1
    D1L = GetObjectWeight(d.objectInHand);
//019d08 3006                     MOVE.W   D6,D0
//019d0a c1fc 0320                MULS     #800,D0
//019d0e 41ec c0ae                LEA      -16210(A4),A0
//019d12 d0c0                     ADD.W    D0,A0
//019d14 3010                     MOVE.W   (A0),D0
    d.CH16482[D6W].load = sw(d.CH16482[D6W].load - D1W);
//019d16 9041                     SUB.W    D1,D0
//019d18 3080                     MOVE.W   D0,(A0)
//019d1a 397c ffff bf48           MOVE.W   #65535,-16568(A4)
    d.HandChar = -1; //owner of cursor
//019d20 3f06                     MOVE.W   D6,-(A7)
//019d22 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//019d26 548f                     ADDQ.L   #2,A7
    DrawCharacterState(D6W);//box at top,pos,dir,weapon
  };
//019d28 0c47 ffff                CMP.W    #65535,D7
//019d2c 6608                     BNE      $+10 (=0x019d36)
  if (chIdx == -1)
  {
//019d2e 397c ffff bf48           MOVE.W   #65535,-16568(A4)
    d.HandChar = -1;//owner of cursor
//019d34 6052                     BRA      $+84 (=0x019d88)
    return;
  };
//019d36 3007                     MOVE.W   D7,D0
//019d38 3940 bf48                MOVE.W   D0,-16568(A4)
  d.HandChar = sw(chIdx);//owner of cursor
//019d3c c1fc 0320                MULS     #800,D0
//019d40 41ec bf9e                LEA      -16482(A4),A0
//019d44 d0c0                     ADD.W    D0,A0
//019d46 41d0                     LEA      (A0),A0
//019d48 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[chIdx];
//019d4a 302c d246                MOVE.W   -11706(A4),D0
//019d4e 1740 001c                MOVE.B   D0,28(A3)
  pcA3->facing = (ui8)d.partyFacing;
//019d52 3f2c bf40                MOVE.W   -16576(A4),-(A7)
//019d56 4ead 0234                JSR      564(A5)    (=0x0091e6)
//019d5a 548f                     ADDQ.L   #2,A7
//019d5c 2200                     MOVE.L   D0,D1
  D1L = GetObjectWeight(d.objectInHand);
//019d5e 3007                     MOVE.W   D7,D0
//019d60 c1fc 0320                MULS     #800,D0
//019d64 41ec c0ae                LEA      -16210(A4),A0
//019d68 d0c0                     ADD.W    D0,A0
//019d6a 3010                     MOVE.W   (A0),D0
//019d6c d041                     ADD.W    D1,D0
//019d6e 3080                     MOVE.W   D0,(A0)
  d.CH16482[chIdx].load = sw(d.CH16482[chIdx].load + D1W);
//019d70 3007                     MOVE.W   D7,D0
//019d72 5240                     ADDQ.W   #1,D0
//019d74 b06c d258                CMP.W    -11688(A4),D0
//019d78 670e                     BEQ      $+16 (=0x019d88)
  if (chIdx+1 == d.PotentialCharacterOrdinal) return;
//019d7a 006b 0680 0030           OR.W     #1664,48(A3)
  pcA3->word48 |= 0x680;
//019d80 3f07                     MOVE.W   D7,-(A7)
//019d82 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//019d86 548f                     ADDQ.L   #2,A7
  DrawCharacterState(chIdx);//box at top,pos,dir,weapon
//019d88 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//019d8c 4e5e                     UNLK     A6
//019d8e 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
void TAG0196da(i32 key)
{
  dReg D0, D5, D6;
  KeyXlate *pxA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0196da 4e56 0000                LINK     A6,#0
//0196de 48e7 0710                MOVEM.L  D567A3,-(A7)
//0196e2 2e2e 0008                MOVE.L   8(A6),D7
//0196e6 202c be10                MOVE.L   -16880(A4),D0
//0196ea 2640                     MOVE.L   D0,A3
//0196ec 4a80                     TST.L    D0
//0196ee 6604                     BNE      $+6 (=0x0196f4)
  pxA3 = d.pKeyXlate2;
//0196f0 6000 00a2                BRA      $+164 (=0x019794)
  if (pxA3 != NULL)
  {
//0196f4 397c 0001 be26           MOVE.W   #1,-16858(A4)
    d.MouseInterlock = 1;
//0196fa 2007                     MOVE.L   D7,D0
//0196fc 4a40                     TST.W    D0
//0196fe 6606                     BNE      $+8 (=0x019706)
    if ((key&0xffff) == 0) key >>= 8; // Move scan code to lower word
    {
//019700 2007                     MOVE.L   D7,D0
//019702 e080                     ASR.L    #8,D0
//019704 2e00                     MOVE.L   D0,D7
    };
//019706 302c be28                MOVE.W   -16856(A4),D0
//01970a 5440                     ADDQ.W   #2,D0
    D0W = sw(d.MouseQEnd+2);
//01970c 3c00                     MOVE.W   D0,D6
//01970e 0c40 0004                CMP.W    #4,D0
    D6W = D0W;
    if (D0W > 4) D6W -= 5;
//019712 6f02                     BLE      $+4 (=0x019716)
//019714 5b46                     SUBQ.W   #5,D6
//019716 3006                     MOVE.W   D6,D0
//019718 b06c be2a                CMP.W    -16854(A4),D0
    if (D6W != d.MouseQStart)
//01971c 6602                     BNE      $+4 (=0x019720)
//01971e 606c                     BRA      $+110 (=0x01978c)
    {
//019720 602a                     BRA      $+44 (=0x01974c)
      while ((D5W = pxA3->mouse) != 0)
      {
//019722 2007                     MOVE.L   D7,D0
//019724 b06b 0002                CMP.W    2(A3),D0
        if (pxA3->keyscan == (key&0xffff))
        {
//019728 6620                     BNE      $+34 (=0x01974a)
//01972a 3006                     MOVE.W   D6,D0
          D0W = D6W;
          D6W--;
//01972c 5346                     SUBQ.W   #1,D6
//01972e 4a40                     TST.W    D0
//019730 6604                     BNE      $+6 (=0x019736)
          if (D0W == 0) D6W = 4;
//019732 3c3c 0004                MOVE.W   #4,D6
//019736 3006                     MOVE.W   D6,D0
//019738 3940 be28                MOVE.W   D0,-16856(A4)
          d.MouseQEnd = D6W;
//01973c c1fc 0006                MULS     #6,D0
//019740 41ec be30                LEA      -16848(A4),A0
//019744 d0c0                     ADD.W    D0,A0
//019746 3085                     MOVE.W   D5,(A0)
          d.MouseQueue[D6W].num = D5W;
//019748 6042                     BRA      $+68 (=0x01978c)
          break;
        };
        pxA3++;
//01974a 588b                     ADDQ.L   #4,A3

//01974c 3a13                     MOVE.W   (A3),D5
//01974e 66d2                     BNE      $-44 (=0x019722)
      };
//019750 202c be0c                MOVE.L   -16884(A4),D0
//019754 2640                     MOVE.L   D0,A3
      pxA3 = d.pKeyXlate1;
//019756 4a80                     TST.L    D0
//019758 6602                     BNE      $+4 (=0x01975c)
//01975a 6030                     BRA      $+50 (=0x01978c)
      if (pxA3 != NULL)
      {
//01975c 602a                     BRA      $+44 (=0x019788)
        while ((D5W = pxA3->mouse) != 0)
        {
//01975e 2007                     MOVE.L   D7,D0
//019760 b06b 0002                CMP.W    2(A3),D0
//019764 6620                     BNE      $+34 (=0x019786)
          if ((key&0xffff) == pxA3->keyscan)
          {
//019766 3006                     MOVE.W   D6,D0
//019768 5346                     SUBQ.W   #1,D6
//01976a 4a40                     TST.W    D0
//01976c 6604                     BNE      $+6 (=0x019772)
//01976e 3c3c 0004                MOVE.W   #4,D6
            D0W = D6W;
            D6W--;
            if (D0W == 0) D6W = 4;
//019772 3006                     MOVE.W   D6,D0
//019774 3940 be28                MOVE.W   D0,-16856(A4)
            d.MouseQEnd = D6W;
//019778 c1fc 0006                MULS     #6,D0
//01977c 41ec be30                LEA      -16848(A4),A0
//019780 d0c0                     ADD.W    D0,A0
//019782 3085                     MOVE.W   D5,(A0)
            d.MouseQueue[D6W].num = D5W;
//019784 6006                     BRA      $+8 (=0x01978c)
          };
//019786 588b                     ADDQ.L   #4,A3
          pxA3++;
//019788 3a13                     MOVE.W   (A3),D5
//01978a 66d2                     BNE      $-44 (=0x01975e)
        };
      };
    };
//01978c 426c be26                CLR.W    -16858(A4)
    d.MouseInterlock = 0;
//019790 4eba ff24                JSR      $-218 (=0x0196b6)
    CauseFakeMouseClick();
  };
//019794 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//019798 4e5e                     UNLK     A6
//01979a 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01a6ea(const i32 mouseX, const i32 mouseY)
{//(void)
  static dReg D0, D4, D5, D6, D7;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01a6ea 4e56 0000                LINK     A6,#0
//01a6ee 48e7 0f00                MOVEM.L  D4567,-(A7)
//01a6f2 3e2e 0008                MOVE.W   8(A6),D7
//01a6f6 3c2e 000a                MOVE.W   10(A6),D6
  D7W = sw(mouseX);
  D6W = sw(mouseY);
//01a6fa 0c6c ffff bf48           CMP.W    #65535,-16568(A4)
//01a700 57c4                     SEQ      D4
//01a702 c87c 0001                AND.W    #1,D4
  D4W = sw((d.HandChar == -1) ? 1: 0); //owner of cursor
//01a706 302c bf18                MOVE.W   -16616(A4),D0
  D0W = d.WpnChestOrScroll;
//01a70a b07c 0004                CMP.W    #4,D0
//01a70e 6702                     BEQ      $+4 (=0x01a712)
//01a710 6030                     BRA      $+50 (=0x01a742)
  if (D0W == 4) //If displaying chest in weapon hand
  {
//01a712 3004                     MOVE.W   D4,D0
//01a714 6702                     BEQ      $+4 (=0x01a718)
//01a716 605c                     BRA      $+94 (=0x01a774)
    if (D4W != 0) RETURN;
//01a718 3f3c 0002                MOVE.W   #2,-(A7)
//01a71c 3f06                     MOVE.W   D6,-(A7)
//01a71e 3f07                     MOVE.W   D7,-(A7)
//01a720 486c b86c                PEA      -18324(A4)
//01a724 4eba ee4a                JSR      $-4532 (=0x019570)
//01a728 defc 000a                ADD.W    #10,A7
//01a72c 3a00                     MOVE.W   D0,D5
    D5W = SearchButtonList(d.Buttons18324,D7W, D6W,2);
//01a72e 4a45                     TST.W    D5
//01a730 670e                     BEQ      $+16 (=0x01a740)
    if (D5W)
    {
//01a732 3005                     MOVE.W   D5,D0
//01a734 907c 0014                SUB.W    #20,D0
      //D0W = D5W - 20;
//01a738 3f00                     MOVE.W   D0,-(A7)
//01a73a 4ead 045c                JSR      1116(A5)    (=0x015e0c)
//01a73e 548f                     ADDQ.L   #2,A7
      HandleClothingClick(D5W-20);
    };
//01a740 6032                     BRA      $+52 (=0x01a774)
  }
  else
  {
//01a742 b07c 0005                CMP.W    #5,D0
//01a746 6702                     BEQ      $+4 (=0x01a74a)
//01a748 602a                     BRA      $+44 (=0x01a774)
    if (D0W == 5)
    {
//01a74a 302c bf3e                MOVE.W   -16578(A4),D0
//01a74e 6602                     BNE      $+4 (=0x01a752)
//01a750 6022                     BRA      $+36 (=0x01a774)
      if (d.EmptyHanded)
      {
//01a752 3f3c 0002                MOVE.W   #2,-(A7)
//01a756 3f06                     MOVE.W   D6,-(A7)
//01a758 3f07                     MOVE.W   D7,-(A7)
//01a75a 486c b83c                PEA      -18372(A4)
//01a75e 4eba ee10                JSR      $-4590 (=0x019570)
//01a762 defc 000a                ADD.W    #10,A7
//01a766 3a00                     MOVE.W   D0,D5
        D5W = SearchButtonList(d.Buttons18372, D7W, D6W, 2);
//01a768 4a45                     TST.W    D5
//01a76a 6708                     BEQ      $+10 (=0x01a774)
        if (D5W)
        {
//01a76c 3f05                     MOVE.W   D5,-(A7)
//01a76e 4ead 03f0                JSR      1008(A5)    (=0x014368)
//01a772 548f                     ADDQ.L   #2,A7
          TAG014368(_1_, D5W);
        };
      };
    };
  };
//01a774 4cdf 00f0                MOVEM.L  (A7)+,D4567
//01a778 4e5e                     UNLK     A6
//01a77a 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
//   TAG01a77c
void DisplaySleepScreen(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01a77c 4e56 0000                LINK     A6,#0
//01a780 3f3c 0770                MOVE.W   #1904,-(A7)
//01a784 4267                     CLR.W    -(A7)
//01a786 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//01a78a 4ead 01c8                JSR      456(A5)    (=0x008e1c)
//01a78e 508f                     ADDQ.L   #0,A7
  Flood(d.pViewportBMP, 0, 1904);
//01a790 486c 0224                PEA      548(A4)
//01a794 4267                     CLR.W    -(A7)
//01a796 2f3c 0045 0004           MOVE.L   #4521988,-(A7)
//01a79c 2f3c 0070 005d           MOVE.L   #7340125,-(A7)
//01a7a2 2f2c d25e                MOVE.L   -11682(A4),-(A7)
//01a7a6 4ead 017a                JSR      378(A5)    (=0x00154c)
//01a7aa defc 0012                ADD.W    #18,A7
  TextOut(d.pViewportBMP,
          112,
          93,
          65,
          4,
          0,
          "WAKE UP");
//01a7ae 4e5e                     UNLK     A6
//01a7b0 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG0197d2
void WaitForButtonFlash(void)
{//(void)
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0197d2 4e56 0000                LINK     A6,#0
//0197d6 302c d202                MOVE.W   -11774(A4),D0
//0197da 66fa                     BNE      $-4 (=0x0197d6)
  Instrumentation(icntTAG0197d2);
  while (d.FlashButnActive != 0)
  {
    NotImplemented(0x197dc); //pumper();
  };
//0197dc 302c d200                MOVE.W   -11776(A4),D0
//0197e0 6710                     BEQ      $+18 (=0x0197f2)



  /* I am going to attempt to write this differently
     so that WaitForButtonFlash does not have to
     be restartable.  God help me.
  if (d.Word11776 != 0)
  {
//0197e2 397c 0001 d202           MOVE.W   #1,-11774(A4)
    d.FlashButnActive = 1;
//0197e8 3f3c 0025                MOVE.W   #37,-(A7)
//0197ec 4ead 0138                JSR      312(A5)    (=0x001ce8)
//0197f0 548f                     ADDQ.L   #2,A7
    wvbl(_1_);
  };
*/
      //Here is the attempted rewrite.
      //We will call CheckVBL and sleep until
      //the vbl count is increased.
      if (d.Word11776 != 0)
      {
        ui32 oldCount;
        d.FlashButnActive = 1;
        oldCount = VBLCount+1;
        //checkVBL(); //Give it a chance to happen right now!
        //Make sure we wait at least 20 milliseconds.
        while (oldCount > VBLCount)
        {
          if (VBLMultiplier == 1) UI_Sleep(20);
          checkVBL();
        };
      };
      //OK...Let us see if that works.




//0197f2 4e5e                     UNLK     A6
//0197f4 4e75                     RTS
  return;
}

//*********************************************************
//
//*********************************************************
//   TAG01ad6c
pnt GetAttackTypeName(i32 P1)
{
  dReg D7;
  aReg A3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01ad6c 4e56 0000                LINK     A6,#0
//01ad70 48e7 0110                MOVEM.L  D7A3,-(A7)
//01ad74 1e2e 0009                MOVE.B   9(A6),D7
  D7W = (I16)(P1 & 0xff);
//01ad78 4240                     CLR.W    D0
//01ad7a 1007                     MOVE.B   D7,D0
//01ad7c b07c 00ff                CMP.W    #255,D0
//01ad80 6608                     BNE      $+10 (=0x01ad8a)
  if (D7W == 255)
  {
//01ad82 41ec 0238                LEA      568(A4),A0
//01ad86 2008                     MOVE.L   A0,D0
    return "";
//01ad88 6018                     BRA      $+26 (=0x01ada2)
  }
  else
  {
//01ad8a 41ec b262                LEA      -19870(A4),A0
//01ad8e 2648                     MOVE.L   A0,A3
    for (A3 = d.AttackNames; D7W-- != 0; )
    {
//01ad90 6004                     BRA      $+6 (=0x01ad96)
//01ad92 101b                     MOVE.B   (A3)+,D0
//01ad94 66fc                     BNE      $-2 (=0x01ad92)
      while ( *(A3++) != 0) {}; // Skip one string
//01ad96 4240                     CLR.W    D0
//01ad98 1007                     MOVE.B   D7,D0
//01ad9a 5307                     SUBQ.B   #1,D7
//01ad9c 4a00                     TST.B    D0
//01ad9e 66f2                     BNE      $-12 (=0x01ad92)
    };
//01ada0 200b                     MOVE.L   A3,D0
    return A3;
  };
//01ada2 4cdf 0880                MOVEM.L  (A7)+,D7A3
//01ada6 4e5e                     UNLK     A6
//01ada8 4e75                     RTS
}


void TAG01b098(void)
{
  dReg D7;
  RectPos *prA3;
  pnt pName;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b098 4e56 0000                LINK     A6,#0
//01b09c 48e7 0110                MOVEM.L  D7A3,-(A7)
//01b0a0 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();
//01b0a4 426c a608                CLR.W    -23032(A4)
  d.UseByteCoordinates = 0;
//01b0a8 3f3c 00a0                MOVE.W   #160,-(A7)
//01b0ac 4267                     CLR.W    -(A7)
//01b0ae 486c ffec                PEA      -20(A4)
//01b0b2 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01b0b6 4ead 01ce                JSR      462(A5)    (=0x008e60)
//01b0ba defc 000c                ADD.W    #12,A7
  FillRectangle(d.LogicalScreenBase,
                (RectPos *)d.Word20,
                0,
                160);
//01b0be 302c b0e4                MOVE.W   -20252(A4),D0
//01b0c2 671c                     BEQ      $+30 (=0x01b0e0)
  if (d.Word20252)
  {
//01b0c4 4247                     CLR.W    D7
    for (D7W=0; D7W<d.NumCharacter; D7W++)
    {
//01b0c6 600c                     BRA      $+14 (=0x01b0d4)
//01b0c8 3007                     MOVE.W   D7,D0
//01b0ca 5247                     ADDQ.W   #1,D7
//01b0cc 3f00                     MOVE.W   D0,-(A7)
//01b0ce 4eba fe62                JSR      $-412 (=0x01af32)
//01b0d2 548f                     ADDQ.L   #2,A7
      DrawAttackButton(D7W);

//01b0d4 3007                     MOVE.W   D7,D0
//01b0d6 b06c d24c                CMP.W    -11700(A4),D0
//01b0da 65ec                     BCS      $-18 (=0x01b0c8)
    };
//01b0dc 6000 00d2                BRA      $+212 (=0x01b1b0)
  }
  else
  {
//01b0e0 302c b0ea                MOVE.W   -20246(A4),D0
//01b0e4 6700 00ca                BEQ      $+204 (=0x01b1b0)
    if (d.AttackingCharacterOrdinal)
    {
//01b0e8 41ec b116                LEA      -20202(A4),A0
//01b0ec 2648                     MOVE.L   A0,A3
      prA3 = &d.wRectPos20202;
//01b0ee 102c b0de                MOVE.B   -20258(A4),D0
//01b0f2 c07c 00ff                AND.W    #255,D0
//01b0f6 b07c 00ff                CMP.W    #255,D0
//01b0fa 6606                     BNE      $+8 (=0x01b102)
      if (d.PossibleAttackTypes[2] == -1)
      {
//01b0fc 41ec b10e                LEA      -20210(A4),A0
//01b100 2648                     MOVE.L   A0,A3
        prA3 = &d.wRectPos20210;
      };
//01b102 102c b0dd                MOVE.B   -20259(A4),D0
//01b106 c07c 00ff                AND.W    #255,D0
//01b10a b07c 00ff                CMP.W    #255,D0
//01b10e 6606                     BNE      $+8 (=0x01b116)
      if (d.PossibleAttackTypes[1] == -1)
      {
//01b110 41ec b106                LEA      -20218(A4),A0
//01b114 2648                     MOVE.L   A0,A3
        prA3 = &d.wRectPos20218;
      };
//01b116 2f3c 0030 ffff           MOVE.L   #3211263,-(A7)
//01b11c 2f0b                     MOVE.L   A3,-(A7)
//01b11e 3f3c 000a                MOVE.W   #10,-(A7)
//01b122 4ead 061e                JSR      1566(A5)    (=0x022a92)
//01b126 548f                     ADDQ.L   #2,A7
//01b128 2f00                     MOVE.L   D0,-(A7)
//01b12a 4ead 00a2                JSR      162(A5)    (=0x000f7e)
//01b12e defc 000c                ADD.W    #12,A7
      BLT2Screen(GetGraphicAddress(10),
                prA3,
                48,
                -1);
//01b132 3f3c 0007                MOVE.W   #7,-(A7)
//01b136 302c b0ea                MOVE.W   -20246(A4),D0
//01b13a c1fc 0320                MULS     #800,D0
//01b13e 41ec bc7e                LEA      -17282(A4),A0
//01b142 d0c0                     ADD.W    D0,A0
//01b144 41d0                     LEA      (A0),A0
//01b146 2f08                     MOVE.L   A0,-(A7)
//01b148 3f3c 0004                MOVE.W   #4,-(A7)
//01b14c 4267                     CLR.W    -(A7)
//01b14e 2f3c 00eb 0053           MOVE.L   #15401043,-(A7)
//01b154 3f3c 00a0                MOVE.W   #160,-(A7)
//01b158 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01b15c 4ead 016e                JSR      366(A5)    (=0x0017d0)
//01b160 defc 0014                ADD.W    #20,A7
      TAG0017d0(d.LogicalScreenBase,
                160,
                235,
                83,
                0,
                4,
                d.CH16482[d.AttackingCharacterOrdinal-1].name,
                7);

//01b164 4247                     CLR.W    D7

//01b166 6042                     BRA      $+68 (=0x01b1aa)
      for (D7W=0; D7W<3; D7W++)
      {
//01b168 3f3c 000c                MOVE.W   #12,-(A7)
//01b16c 3007                     MOVE.W   D7,D0
//01b16e 41ec b0dc                LEA      -20260(A4),A0
//01b172 d0c0                     ADD.W    D0,A0
//01b174 4240                     CLR.W    D0
//01b176 1010                     MOVE.B   (A0),D0
//01b178 3f00                     MOVE.W   D0,-(A7)
//01b17a 4eba fbf0                JSR      $-1038 (=0x01ad6c)
//01b17e 548f                     ADDQ.L   #2,A7
        pName = GetAttackTypeName(d.PossibleAttackTypes[D7W]);
//01b180 2f00                     MOVE.L   D0,-(A7)
//01b182 4267                     CLR.W    -(A7)
//01b184 3f3c 0004                MOVE.W   #4,-(A7)
//01b188 303c 005d                MOVE.W   #93,D0
//01b18c 3207                     MOVE.W   D7,D1
//01b18e c2fc 000c                MULU     #12,D1
//01b192 d041                     ADD.W    D1,D0
//01b194 3f00                     MOVE.W   D0,-(A7)
//01b196 2f3c 00a0 00f1           MOVE.L   #10486001,-(A7)
//01b19c 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01b1a0 4ead 016e                JSR      366(A5)    (=0x0017d0)
//01b1a4 defc 0014                ADD.W    #20,A7
        TAG0017d0(d.LogicalScreenBase,
                  160,
                  241,
                  12 * D7W + 93,
                  4,
                  0,
                  pName,
                  12);
//01b1a8 5247                     ADDQ.W   #1,D7

//01b1aa 0c47 0003                CMP.W    #3,D7
//01b1ae 65b8                     BCS      $-70 (=0x01b168)
      };
    };
  };
//01b1b0 4ead 00a8                JSR      168(A5)    (=0x003026)
//01b1b4 426c b0e6                CLR.W    -20250(A4)
  STShowCursor();
  d.Word20250 = 0;
//01b1b8 4cdf 0880                MOVEM.L  (A7)+,D7A3
//01b1bc 4e5e                     UNLK     A6
//01b1be 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01b29a(void)
{//(void)
  static dReg D0, D1, D7;
  static CHARDESC *pcA3=NULL;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b29a 4e56 0000                LINK     A6,#0
//01b29e 48e7 0710                MOVEM.L  D567A3,-(A7)
//01b2a2 302c d24c                MOVE.W   -11700(A4),D0
//01b2a6 6604                     BNE      $+6 (=0x01b2ac)
//01b2a8 6000 00ca                BRA      $+204 (=0x01b374)
  if (d.NumCharacter == 0) RETURN;
//01b2ac 302c d256                MOVE.W   -11690(A4),D0
//01b2b0 6606                     BNE      $+8 (=0x01b2b8)
//01b2b2 302c d258                MOVE.W   -11688(A4),D0
//01b2b6 671a                     BEQ      $+28 (=0x01b2d2)
  if ( (d.PartySleeping) || (d.PotentialCharacterOrdinal) )
  {
//01b2b8 302c b0ea                MOVE.W   -20246(A4),D0
//01b2bc 6708                     BEQ      $+10 (=0x01b2c6)
    if (d.AttackingCharacterOrdinal)
    {
//01b2be 4eba ff00                JSR      $-254 (=0x01b1c0)
      TAG01b1c0();
//01b2c2 6000 00b0                BRA      $+178 (=0x01b374)
      RETURN;
    };
//01b2c6 302c d258                MOVE.W   -11688(A4),D0
//01b2ca 6604                     BNE      $+6 (=0x01b2d0)
//01b2cc 6000 00a6                BRA      $+168 (=0x01b374)
    if (d.PotentialCharacterOrdinal == 0) RETURN;
//01b2d0 6058                     BRA      $+90 (=0x01b32a)
  }
  else
  {
//01b2d2 41ec bf9e                LEA      -16482(A4),A0
//01b2d6 2648                     MOVE.L   A0,A3
    pcA3 = d.CH16482;
//01b2d8 4247                     CLR.W    D7
    for (D7W=0; D7W<d.NumCharacter; D7W++, pcA3++)
    {
//01b2da 3007                     MOVE.W   D7,D0
//01b2dc b06c bf48                CMP.W    -16568(A4),D0
//01b2e0 6736                     BEQ      $+56 (=0x01b318)
      if (D7W != d.HandChar) //owner of cursro
      {
//01b2e2 3007                     MOVE.W   D7,D0
//01b2e4 5240                     ADDQ.W   #1,D0
//01b2e6 b06c b0ea                CMP.W    -20246(A4),D0
//01b2ea 672c                     BEQ      $+46 (=0x01b318)
        if (D7W+1 != d.AttackingCharacterOrdinal)
        {
//01b2ec 102b 0029                MOVE.B   41(A3),D0
//01b2f0 6726                     BEQ      $+40 (=0x01b318)
          if (pcA3->uByte41)
          {
//01b2f2 102b 001c                MOVE.B   28(A3),D0
            D0W = pcA3->facing;
//01b2f6 4241                     CLR.W    D1
//01b2f8 122b 0028                MOVE.B   40(A3),D1
            D1W = pcA3->facing3;
//01b2fc c07c 00ff                AND.W    #255,D0
//01b300 b041                     CMP.W    D1,D0
//01b302 6714                     BEQ      $+22 (=0x01b318)
            if (D0W != D1W)
            {
//01b304 176b 0028 001c           MOVE.B   40(A3),28(A3)
              pcA3->facing = pcA3->facing3;
//01b30a 006b 0400 0030           OR.W     #1024,48(A3)
              pcA3->word48 |= 0x400;
//01b310 3f07                     MOVE.W   D7,-(A7)
//01b312 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//01b316 548f                     ADDQ.L   #2,A7
              DrawCharacterState(D7W);//box at top,pos,dir.weapon
            };
          };
        };
      };
//01b318 422b 0029                CLR.B    41(A3)
      pcA3->uByte41 = 0;
//01b31c d6fc 0320                ADD.W    #800,A3
//01b320 5247                     ADDQ.W   #1,D7
//01b322 3007                     MOVE.W   D7,D0
//01b324 b06c d24c                CMP.W    -11700(A4),D0
//01b328 65b0                     BCS      $-78 (=0x01b2da)
    };
  };
//01b32a 302c b0e6                MOVE.W   -20250(A4),D0
//01b32e 6744                     BEQ      $+70 (=0x01b374)
  if (d.Word20250 == 0) RETURN;
//01b330 302c b0ea                MOVE.W   -20246(A4),D0
//01b334 6622                     BNE      $+36 (=0x01b358)
  if (d.AttackingCharacterOrdinal == 0)
  {
//01b336 302c b0d8                MOVE.W   -20264(A4),D0
//01b33a 6710                     BEQ      $+18 (=0x01b34c)
    if (d.Word20264)
    {
//01b33c 3f2c b0d8                MOVE.W   -20264(A4),-(A7)
//01b340 4eba fa68                JSR      $-1430 (=0x01adaa)
//01b344 548f                     ADDQ.L   #2,A7
      FlashAttackDamage(_1_,d.Word20264);
//01b346 426c b0d8                CLR.W    -20264(A4)
      d.Word20264 = 0;
//01b34a 600a                     BRA      $+12 (=0x01b356)
    }
    else
    {
//01b34c 397c 0001 b0e4           MOVE.W   #1,-20252(A4)
      d.Word20252 = 1;
//01b352 4eba fd44                JSR      $-698 (=0x01b098)
      TAG01b098();
    };
//01b356 601c                     BRA      $+30 (=0x01b374)
    RETURN;
  };
//01b358 426c b0e4                CLR.W    -20252(A4)
  d.Word20252 = 0;
//01b35c 006b 8000 0030           OR.W     #32768,48(A3)
  pcA3->word48 |= 0x8000;
//01b362 302c b0ea                MOVE.W   -20246(A4),D0
//01b366 5340                     SUBQ.W   #1,D0
//01b368 3f00                     MOVE.W   D0,-(A7)
//01b36a 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//01b36e 548f                     ADDQ.L   #2,A7
  DrawCharacterState(d.AttackingCharacterOrdinal-1);//box at top, pos, dir,weapon
//01b370 4eba fd26                JSR      $-728 (=0x01b098)
  TAG01b098();
//01b374 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01b378 4e5e                     UNLK     A6
//01b37a 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
void TAG01b408(i16 P1)
{
  dReg D0, D1, D5, D6, D7;
  CHARDESC *pcA3;
  i16 LOCAL_12;
  i8  LOCAL_2[2];
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b408 4e56 fff4                LINK     A6,#-12
//01b40c 48e7 0710                MOVEM.L  D567A3,-(A7)
//01b410 302c b0d6                MOVE.W   -20266(A4),D0
//01b414 c1fc 0320                MULS     #800,D0
//01b418 41ec bf9e                LEA      -16482(A4),A0
//01b41c d0c0                     ADD.W    D0,A0
//01b41e 41d0                     LEA      (A0),A0
//01b420 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[d.MagicCaster];
//01b422 0c6e 0002 0008           CMP.W    #2,8(A6)
//01b428 6670                     BNE      $+114 (=0x01b49a)
  if (P1 ==2)
  {
//01b42a 3f3c 0240                MOVE.W   #576,-(A7)
//01b42e 486c ae96                PEA      -20842(A4)
//01b432 486c aa16                PEA      -21994(A4)
//01b436 4ead 0006                JSR      6(A5)    (=0x0009dc)
//01b43a defc 000a                ADD.W    #10,A7
    MemMove(d.Byte21994, d.Byte20842, 576);
//01b43e 3c3c 0001                MOVE.W   #1,D6
    D6W = 1;
//01b442 3a3c 0006                MOVE.W   #6,D5
    D5W = 6;
//01b446 4241                     CLR.W    D1
//01b448 122b 0021                MOVE.B   33(A3),D1
    D1W = (UI8)(pcA3->byte33);
//01b44c cbc1                     MULU     D1,D5
//01b44e 3d45 fff4                MOVE.W   D5,-12(A6)
    LOCAL_12 = sw(D1W * D5W);
//01b452 3a3c 0060                MOVE.W   #96,D5
    D5W = 96;
//01b456 da6e fff4                ADD.W    -12(A6),D5
    D5W = sw(D5W + LOCAL_12);
//01b45a 422e ffff                CLR.B    -1(A6)
    LOCAL_2[1] = 0;
    for (D7W=0; D7W<6; D7W++)
    {
//01b45e 4247                     CLR.W    D7
//01b460 6030                     BRA      $+50 (=0x01b492)

//01b462 1005                     MOVE.B   D5,D0
//01b464 5205                     ADDQ.B   #1,D5
//01b466 1d40 fffe                MOVE.B   D0,-2(A6)
      LOCAL_2[0] = D5B;
      D5B++;
      D6W += 14;
//01b46a 486e fffe                PEA      -2(A6)
//01b46e 4267                     CLR.W    -(A7)
//01b470 2f3c 0008 0004           MOVE.L   #524292,-(A7)
//01b476 3006                     MOVE.W   D6,D0
//01b478 d07c 000e                ADD.W    #14,D0
//01b47c 3c00                     MOVE.W   D0,D6
//01b47e 3f00                     MOVE.W   D0,-(A7)
//01b480 3f3c 0030                MOVE.W   #48,-(A7)
//01b484 486c ae96                PEA      -20842(A4)
//01b488 4ead 017a                JSR      378(A5)    (=0x00154c)
//01b48c defc 0012                ADD.W    #18,A7
      TextOut(d.Byte20842, 48, D6W, 8, 4, 0, LOCAL_2);

//01b490 5247                     ADDQ.W   #1,D7
//
//01b492 0c47 0006                CMP.W    #6,D7
//01b496 65ca                     BCS      $-52 (=0x01b462)
    };
//01b498 606c                     BRA      $+110 (=0x01b506)
  }
  else
  {
//01b49a 0c6e 0003 0008           CMP.W    #3,8(A6)
//01b4a0 6664                     BNE      $+102 (=0x01b506)
    if (P1 == 3)
    {
//01b4a2 3f3c 0240                MOVE.W   #576,-(A7)
//01b4a6 486c ae96                PEA      -20842(A4)
//01b4aa 486c ac56                PEA      -21418(A4)
//01b4ae 4ead 0006                JSR      6(A5)    (=0x0009dc)
//01b4b2 defc 000a                ADD.W    #10,A7
      MemMove(d.Byte21418, d.Byte20842, 576);
//01b4b6 422e ffff                CLR.B    -1(A6)
      LOCAL_2[1] = 0;
//01b4ba 3c3c 0008                MOVE.W   #8,D6
      D6W = 8;

//01b4be 4247                     CLR.W    D7
//01b4c0 603e                     BRA      $+64 (=0x01b500)
      for (D7W=0; D7W<4; D7W++)
      {
//01b4c2 3007                     MOVE.W   D7,D0
//01b4c4 41eb 0022                LEA      34(A3),A0
//01b4c8 d0c0                     ADD.W    D0,A0
//01b4ca 1010                     MOVE.B   (A0),D0
        D0B = pcA3->incantation[D7W];
//01b4cc 1d40 fffe                MOVE.B   D0,-2(A6)
        LOCAL_2[0] = D0B;
//01b4d0 4880                     EXT.W    D0
//01b4d2 4a40                     TST.W    D0
//01b4d4 6602                     BNE      $+4 (=0x01b4d8)
//01b4d6 602e                     BRA      $+48 (=0x01b506)
        if (D0B == 0) break;
        D6W += 9;
//01b4d8 486e fffe                PEA      -2(A6)
//01b4dc 4267                     CLR.W    -(A7)
//01b4de 2f3c 0008 0004           MOVE.L   #524292,-(A7)
//01b4e4 3006                     MOVE.W   D6,D0
//01b4e6 d07c 0009                ADD.W    #9,D0
//01b4ea 3c00                     MOVE.W   D0,D6
//01b4ec 3f00                     MOVE.W   D0,-(A7)
//01b4ee 3f3c 0030                MOVE.W   #48,-(A7)
//01b4f2 486c ae96                PEA      -20842(A4)
//01b4f6 4ead 017a                JSR      378(A5)    (=0x00154c)
//01b4fa defc 0012                ADD.W    #18,A7
        TextOut(d.Byte20842, 48, D6W, 8, 4, 0, LOCAL_2);

//01b4fe 5247                     ADDQ.W   #1,D7
//
//01b500 0c47 0004                CMP.W    #4,D7
//01b504 65bc                     BCS      $-66 (=0x01b4c2)
      };
    };
  };
//01b506 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01b50a 4e5e                     UNLK     A6
//01b50c 4e75                     RTS
}


void TAG01b8ae(void)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b8ae 4e56 0000                LINK     A6,#0
//01b8b2 4267                     CLR.W    -(A7)
//01b8b4 4267                     CLR.W    -(A7)
//01b8b6 486c a7d6                PEA      -22570(A4)
//01b8ba 3f3c 000b                MOVE.W   #11,-(A7)
//01b8be 4ead 0606                JSR      1542(A5)    (=0x022b86)
//01b8c2 defc 000a                ADD.W    #10,A7
  ReadAndExpandGraphic(
          11,          // graphic number
          d.Byte22570, // destination
          0,
          0);
//01b8c6 4e5e                     UNLK     A6
//01b8c8 4e75                     RTS
}

//   TAG01b8ca
void DrawSpellMenuCharacters(i16 P1)
{
  dReg D5, D6, D7;
  i8 b_2[2];
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b8ca 4e56 fffe                LINK     A6,#-2
//01b8ce 48e7 0700                MOVEM.L  D567,-(A7)
//01b8d2 422e ffff                CLR.B    -1(A6)
  b_2[1] = 0;
//01b8d6 3a3c 0060                MOVE.W   #96,D5
//01b8da 323c 0006                MOVE.W   #6,D1
//01b8de c2ee 0008                MULU     8(A6),D1
//01b8e2 da41                     ADD.W    D1,D5
  D5W = sw(6 * P1 + 96);
//01b8e4 3c3c 00e1                MOVE.W   #225,D6
  D6W = 225;
//01b8e8 4247                     CLR.W    D7
//01b8ea 6028                     BRA      $+42 (=0x01b914)
  for (D7W = 0; D7W < 6; D7W++)
  {
//01b8ec 1005                     MOVE.B   D5,D0
//01b8ee 5205                     ADDQ.B   #1,D5
//01b8f0 1d40 fffe                MOVE.B   D0,-2(A6)
    b_2[0] = (i8)D5W++;
//01b8f4 486e fffe                PEA      -2(A6)
//01b8f8 4267                     CLR.W    -(A7)
//01b8fa 2f3c 003a 0004           MOVE.L   #3801092,-(A7)
//01b900 3006                     MOVE.W   D6,D0
//01b902 d07c 000e                ADD.W    #14,D0
//01b906 3c00                     MOVE.W   D0,D6
//01b908 3f00                     MOVE.W   D0,-(A7)
//01b90a 4ead 0108                JSR      264(A5)    (=0x001c42)
//01b90e defc 000c                ADD.W    #12,A7
    TextOutToScreen(D6W += 14, 58, 4, 0, b_2);

//01b912 5247                     ADDQ.W   #1,D7
//
//01b914 0c47 0006                CMP.W    #6,D7
//01b918 65d2                     BCS      $-44 (=0x01b8ec)
  };
//01b91a 4cdf 00e0                MOVEM.L  (A7)+,D567
//01b91e 4e5e                     UNLK     A6
//01b920 4e75                     RTS
}

void TAG01b990(i16 button)
{//(void)
  dReg D0, D1, D5, D6, D7;
  aReg A0;
  CHARDESC *pcA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01b990 4e56 0000                LINK     A6,#0
//01b994 48e7 0710                MOVEM.L  D567A3,-(A7)
//01b998 302c b0d6                MOVE.W   -20266(A4),D0
//01b99c c1fc 0320                MULS     #800,D0
//01b9a0 41ec bf9e                LEA      -16482(A4),A0
//01b9a4 d0c0                     ADD.W    D0,A0
//01b9a6 41d0                     LEA      (A0),A0
//01b9a8 2648                     MOVE.L   A0,A3
  pcA3 = &d.CH16482[d.MagicCaster];
////01b9aa 4247                     CLR.W    D7
  D7W = (UI8)(pcA3->byte33);
//01b9ac 1e2b 0021                MOVE.B   33(A3),D7
//01b9b0 3c07                     MOVE.W   D7,D6
//01b9b2 cdfc 0006                MULS     #6,D6
//01b9b6 41ec b5be                LEA      -19010(A4),A0
//01b9ba dc88                     ADD.L    A0,D6
//01b9bc 2046                     MOVE.L   D6,A0
//01b9be d0ee 0008                ADD.W    8(A6),A0
  A0 = d.Byte19010 + 6*D7W;
//01b9c2 4246                     CLR.W    D6
//01b9c4 1c10                     MOVE.B   (A0),D6
  D6W = (UI8)(A0[button]);
//01b9c6 3007                     MOVE.W   D7,D0
//01b9c8 671e                     BEQ      $+32 (=0x01b9e8)
  if (D7W != 0)
  {
//01b9ca 3006                     MOVE.W   D6,D0
//01b9cc 122b 0022                MOVE.B   34(A3),D1
//01b9d0 4881                     EXT.W    D1
    D5W = sw(pcA3->incantation[0] - 96);
//01b9d2 927c 0060                SUB.W    #96,D1
//01b9d6 3a01                     MOVE.W   D1,D5
//01b9d8 41ec b5b8                LEA      -19016(A4),A0
//01b9dc d0c1                     ADD.W    D1,A0
    //A0 = d.Byte19016 + D5W;
//01b9de 4241                     CLR.W    D1
//01b9e0 1210                     MOVE.B   (A0),D1
    D1W = (UI8)(d.Byte19016[D5W]);
//01b9e2 c0c1                     MULU     D1,D0
//01b9e4 e648                     LSR.W    #3,D0
//01b9e6 3c00                     MOVE.W   D0,D6
    D6W = sw((D6W * D1W) >> 3);
  };
//01b9e8 3006                     MOVE.W   D6,D0
//01b9ea 322b 003c                MOVE.W   60(A3),D1
//01b9ee b041                     CMP.W    D1,D0
//01b9f0 6260                     BHI      $+98 (=0x01ba52)
  if (D6W > pcA3->mana) return;
//01b9f2 3006                     MOVE.W   D6,D0
//01b9f4 916b 003c                SUB.W    D0,60(A3)
  pcA3->mana = sw(pcA3->mana - D6W);
//01b9f8 006b 0100 0030           OR.W     #256,48(A3)
  pcA3->word48 |=0x100;
//01b9fe 303c 0060                MOVE.W   #96,D0
//01ba02 3207                     MOVE.W   D7,D1
///01ba04 c2fc 0006                MULU     #6,D1
//01ba08 d041                     ADD.W    D1,D0
//01ba0a d06e 0008                ADD.W    8(A6),D0
  D0W = sw(6 * D7W + 96 + button);
//01ba0e 3207                     MOVE.W   D7,D1
//01ba10 41eb 0022                LEA      34(A3),A0
//01ba14 d0c1                     ADD.W    D1,A0
  //A0 = pcA3->byte34 + D7W;
//01ba16 1080                     MOVE.B   D0,(A0)
  pcA3->incantation[D7W] = D0B;
//01ba18 3007                     MOVE.W   D7,D0
//01ba1a 41eb 0023                LEA      35(A3),A0
//01ba1e d0c0                     ADD.W    D0,A0
//01ba20 4210                     CLR.B    (A0)
  pcA3->incantation[D7W + 1] = 0;
//01ba22 3007                     MOVE.W   D7,D0
//01ba24 5240                     ADDQ.W   #1,D0
//01ba26 c07c 0003                AND.W    #3,D0
//01ba2a 3e00                     MOVE.W   D0,D7
  D7W = (I16)((D7W + 1) & 3);
//01ba2c 1740 0021                MOVE.B   D0,33(A3)
  pcA3->byte33 = (i8)D7W;
//01ba30 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();
//01ba34 3f07                     MOVE.W   D7,-(A7)
//01ba36 4eba fe92                JSR      $-364 (=0x01b8ca)
//01ba3a 548f                     ADDQ.L   #2,A7
  DrawSpellMenuCharacters(D7W);//Zo, Rath, etc
//01ba3c 2f0b                     MOVE.L   A3,-(A7)
//01ba3e 4eba fee2                JSR      $-284 (=0x01b922)
//01ba42 588f                     ADDQ.L   #4,A7
  DisplayMagicSelection(pcA3);
//01ba44 3f2c b0d6                MOVE.W   -20266(A4),-(A7)
//01ba48 4ead 04a4                JSR      1188(A5)    (=0x014de0)
//01ba4c 548f                     ADDQ.L   #2,A7
  DrawCharacterState(d.MagicCaster);
//01ba4e 4ead 00a8                JSR      168(A5)    (=0x003026)
  STShowCursor();
//01ba52 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01ba56 4e5e                     UNLK     A6
//01ba58 4e75                     RTS
}




//   Tag01cefa
void SubstDiskLetter(pnt dest,pnt src,i16 flag)
{
  aReg A0;
  i16 LOCAL_6;
  pnt LOCAL_4;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01cefa 4e56 fffa                LINK     A6,#-6
//01cefe 206e 0008                MOVE.L   8(A6),A0
  A0 = dest;
//01cf02 4210                     CLR.B    (A0)
  *A0 = 0;
//01cf04 3f3c 007e                MOVE.W   #126,-(A7)
//01cf08 2f2e 000c                MOVE.L   12(A6),-(A7)
//01cf0c 4ead 0174                JSR      372(A5)    (=0x0032e4)
//01cf10 5c8f                     ADDQ.L   #6,A7
  LOCAL_4 = StrChr(src, '~');
//01cf12 2d40 fffc                MOVE.L   D0,-4(A6)
//01cf16 4a80                     TST.L    D0
//01cf18 6768                     BEQ      $+106 (=0x01cf82)
  if (LOCAL_4 != NULL)
  {
//01cf1a 202e fffc                MOVE.L   -4(A6),D0
//01cf1e 90ae 000c                SUB.L    12(A6),D0
    LOCAL_6 = sw(LOCAL_4 - src); // index of tilde in P2
//01cf22 3d40 fffa                MOVE.W   D0,-6(A6)
//01cf26 3f00                     MOVE.W   D0,-(A7)
//01cf28 2f2e 000c                MOVE.L   12(A6),-(A7)
//01cf2c 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cf30 4ead 018c                JSR      396(A5)    (=0x00327c)
//01cf34 defc 000a                ADD.W    #10,A7
    StrCpy(dest,src,LOCAL_6); // Copy up to tilde
//01cf38 302e fffa                MOVE.W   -6(A6),D0
//01cf3c 206e 0008                MOVE.L   8(A6),A0
//01cf40 d0c0                     ADD.W    D0,A0
    A0 = dest + LOCAL_6 ;
//01cf42 4210                     CLR.B    (A0)
    *A0 = 0; // mark the end
//01cf44 302c a646                MOVE.W   -22970(A4),D0
//01cf48 6608                     BNE      $+10 (=0x01cf52)
    if ((d.Word22970 != 0) || (flag == 1))
    {
//01cf4a 0c6e 0001 0010           CMP.W    #1,16(A6)
//01cf50 6610                     BNE      $+18 (=0x01cf62)
//01cf52 2f2c a638                MOVE.L   -22984(A4),-(A7)
//01cf56 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cf5a 4ead 0078                JSR      120(A5)    (=0x0031e8)
//01cf5e 508f                     ADDQ.L   #0,A7
      StrCat(dest, d.Pointer22984);
//01cf60 600e                     BRA      $+16 (=0x01cf70)
    }
    else
    {
//01cf62 2f2c a634                MOVE.L   -22988(A4),-(A7)
//01cf66 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cf6a 4ead 0078                JSR      120(A5)    (=0x0031e8)
//01cf6e 508f                     ADDQ.L   #0,A7
      StrCat(dest, d.Pointer22988);
    };
//01cf70 202e fffc                MOVE.L   -4(A6),D0
//01cf74 5280                     ADDQ.L   #1,D0
//01cf76 2f00                     MOVE.L   D0,-(A7)
//01cf78 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cf7c 4ead 0078                JSR      120(A5)    (=0x0031e8)
//01cf80 508f                     ADDQ.L   #0,A7
    StrCat(dest, LOCAL_4+1);
  };
//01cf82 4e5e                     UNLK     A6
//01cf84 4e75                     RTS
}

i16 StreamInput(pnt dest,i32 num) //TAG01cf86
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01cf86 4e56 0000                LINK     A6,#0
  dReg D0, D7;
//01cf8a 2f07                     MOVE.L   D7,-(A7)
//01cf8c 302e 000c                MOVE.W   12(A6),D0
  D0W = sw(num);
  D0H1 = 0;
//01cf90 c0bc 0000 ffff           AND.L    #65535,D0
//01cf96 2e00                     MOVE.L   D0,D7
  D7 = D0;
//01cf98 4a80                     TST.L    D0
//01cf9a 6606                     BNE      $+8 (=0x01cfa2)
  if (D0L == 0) return 1;
//01cf9c 303c 0001                MOVE.W   #1,D0
//01cfa0 605c                     BRA      $+94 (=0x01cffe)
//01cfa2 302c a7b8                MOVE.W   -22600(A4),D0
//01cfa6 6734                     BEQ      $+54 (=0x01cfdc)
  if (d.inStreamBuffered != 0)
  {
//01cfa8 2007                     MOVE.L   D7,D0
//01cfaa b0ac a7b0                CMP.L    -22608(A4),D0
//01cfae 6f06                     BLE      $+8 (=0x01cfb6)
    if (D7L > d.inStreamLeft) return 0;
//01cfb0 303c 0000                MOVE.W   #0,D0
//01cfb4 6048                     BRA      $+74 (=0x01cffe)
//01cfb6 2007                     MOVE.L   D7,D0
//01cfb8 3f00                     MOVE.W   D0,-(A7)
//01cfba 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cfbe 2f2c a7b4                MOVE.L   -22604(A4),-(A7)
//01cfc2 4ead 0006                JSR      6(A5)    (=0x0009dc)
//01cfc6 defc 000a                ADD.W    #10,A7
    MemMove(d.inStreamNext, dest, D7W);
//01cfca 2007                     MOVE.L   D7,D0
//01cfcc d1ac a7b4                ADD.L    D0,-22604(A4)
    d.inStreamNext += D7L;
//01cfd0 2007                     MOVE.L   D7,D0
//01cfd2 91ac a7b0                SUB.L    D0,-22608(A4)
    d.inStreamLeft -= D7L;
//01cfd6 303c 0001                MOVE.W   #1,D0
//01cfda 6022                     BRA      $+36 (=0x01cffe)
    return 1;
  };
//01cfdc 2f2e 0008                MOVE.L   8(A6),-(A7)
//01cfe0 2007                     MOVE.L   D7,D0
//01cfe2 2f00                     MOVE.L   D0,-(A7)
//01cfe4 3f2c a7ce                MOVE.W   -22578(A4),-(A7)
//01cfe8 3f3c 003f                MOVE.W   #63,-(A7)
//01cfec 4ead 0096                JSR      150(A5)    (=0x001cf8)
//01cff0 defc 000c                ADD.W    #12,A7
  D0L = READ(d.datafileHandle, D7L, dest);
//01cff4 2207                     MOVE.L   D7,D1
//01cff6 b081                     CMP.L    D1,D0
//01cff8 57c0                     SEQ      D0
//01cffa c07c 0001                AND.W    #1,D0
  D0W = sw(D0L==D7L ? 1 : 0);
//01cffe 2e1f                     MOVE.L   (A7)+,D7
//01d000 4e5e                     UNLK     A6
//01d002 4e75                     RTS
  return D0W;
}

// TAG01d048

i16 Unscramble(pnt buf, i32 initialHash, i32 numword)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01d048 4e56 0000                LINK     A6,#0
  dReg D5, D6, D7;
  aReg A3;
//01d04c 48e7 0710                MOVEM.L  D567A3,-(A7)
//01d050 266e 0008                MOVE.L   8(A6),A3
  A3 = buf;
//01d054 3e2e 000c                MOVE.W   12(A6),D7
//01d058 3c2e 000e                MOVE.W   14(A6),D6
  D7W = (UI16)initialHash;
  D6W = sw(numword);
//01d05c 3a07                     MOVE.W   D7,D5
  D5W = (UI16)initialHash;
  do
  {
//01d05e da53                     ADD.W    (A3),D5
    D5W = (UI16)(D5W + LE16(wordGear(A3)));
//01d060 3007                     MOVE.W   D7,D0
    //D0W = D7W;
//01d062 b153                     EOR.W    D0,(A3)
    wordGear(A3) ^= LE16(D7W);
//01d064 da5b                     ADD.W    (A3)+,D5
    D5W = (UI16)(D5W + LE16(wordGear(A3)));
    A3 += 2;
//01d066 de46                     ADD.W    D6,D7
    D7W = (UI16)(D7W + D6W);
//01d068 5346                     SUBQ.W   #1,D6
    D6W--;
//01d06a 66f2                     BNE      $-12 (=0x01d05e)
  } while (D6W != 0);
//01d06c 3005                     MOVE.W   D5,D0
  return D5W;
//01d06e 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01d072 4e5e                     UNLK     A6
//01d074 4e75                     RTS
}

// TAG01d0a4
i16 UnscrambleStream(pnt dest,i32 size, i16 initialHash, i16 Checksum)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01d0a4 4e56 fff6                LINK     A6,#-10
//01d0a8 3f2e 000c                MOVE.W   12(A6),-(A7)
//01d0ac 2f2e 0008                MOVE.L   8(A6),-(A7)
//01d0b0 4eba fed4                JSR      $-298 (=0x01cf86)
//01d0b4 5c8f                     ADDQ.L   #6,A7
  D0W = StreamInput(dest,size);
//01d0b6 4a40                     TST.W    D0
//01d0b8 6726                     BEQ      $+40 (=0x01d0e0)
  if (D0W == 0) return 0;
//01d0ba 302e 000c                MOVE.W   12(A6),D0
//01d0be e248                     LSR.W    #1,D0
//01d0c0 3f00                     MOVE.W   D0,-(A7)
//01d0c2 3f2e 000e                MOVE.W   14(A6),-(A7)
//01d0c6 2f2e 0008                MOVE.L   8(A6),-(A7)
//01d0ca 4eba ff7c                JSR      $-130 (=0x01d048)
//01d0ce 508f                     ADDQ.L   #0,A7
  D0W = Unscramble(dest, initialHash, size/2);
//01d0d0 3d40 fff6                MOVE.W   D0,-10(A6)
//01d0d4 302e 0010                MOVE.W   16(A6),D0
//01d0d8 b06e fff6                CMP.W    -10(A6),D0
  if (D0W == Checksum) return 1;
  return 0;
//01d0dc 57c0                     SEQ      D0
//01d0de 4a00                     TST.B    D0
//01d0e0 56c0                     SNE      D0
//01d0e2 c07c 0001                AND.W    #1,D0
//01d0e6 4e5e                     UNLK     A6
//01d0e8 4e75                     RTS
}

i16 FetchDataBytes(i8 *P1,i16 *P2,i32 P3) //TAG01d138
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01d138 4e56 0000                LINK     A6,#0
//01d13c 48e7 0330                MOVEM.L  D67A23,-(A7)
  dReg D0, D6, D7;
  i16 *uwA2;
  aReg A3;
//01d140 266e 0008                MOVE.L   8(A6),A3
//01d144 246e 000c                MOVE.L   12(A6),A2
//01d148 3e2e 0010                MOVE.W   16(A6),D7
  A3 = P1;
  uwA2 = P2;
  D7W = sw(P3);
//01d14c 4246                     CLR.W    D6
  D6W = 0;
//01d14e 3f07                     MOVE.W   D7,-(A7)
//01d150 2f0b                     MOVE.L   A3,-(A7)
//01d152 4eba fe32                JSR      $-460 (=0x01cf86)
//01d156 5c8f                     ADDQ.L   #6,A7
  D0W = StreamInput(A3, D7W);
//01d158 4a40                     TST.W    D0
//01d15a 6606                     BNE      $+8 (=0x01d162)
  if (D0W == 0)
  {
//01d15c 303c 0000                MOVE.W   #0,D0
    D0W = 0;
//01d160 6018                     BRA      $+26 (=0x01d17a)
  }
  else
  {
//01d162 6006                     BRA      $+8 (=0x01d16a)
    while (D7W != 0)
    {
      D7W--;
//01d164 4240                     CLR.W    D0
//01d166 101b                     MOVE.B   (A3)+,D0
//01d168 dc40                     ADD.W    D0,D6
      D6W = (UI16)(D6W + *((ui8 *)A3));
      A3++;
//01d16a 3007                     MOVE.W   D7,D0
//01d16c 5347                     SUBQ.W   #1,D7
//01d16e 4a40                     TST.W    D0
//01d170 66f2                     BNE      $-12 (=0x01d164)
    };
//01d172 3006                     MOVE.W   D6,D0
//01d174 d152                     ADD.W    D0,(A2)
    *uwA2 = (UI16)(*uwA2 + D6W);
//01d176 303c 0001                MOVE.W   #1,D0
    D0W = 1;
  };
//01d17a 4cdf 0cc0                MOVEM.L  (A7)+,D67A23
//01d17e 4e5e                     UNLK     A6
//01d180 4e75                     RTS
  return D0W;
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01daa8(const i32 P1,const i32 P2)
{//(i16)
  static dReg D0, D5, D6, D7;
  static aReg A3;
  static i8  LOCAL_80[80];
  RESTARTMAP
    RESTART(1)
    RESTART(2)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01daa8 4e56 ffb0                LINK     A6,#-80
  //SaveRegs(0x0710);
//01daac 48e7 0710                MOVEM.L  D567A3,-(A7)
//01dab0 3e2e 0008                MOVE.W   8(A6),D7
//01dab4 3c2e 000a                MOVE.W   10(A6),D6
  D7W = sw(P1);
  D6W = sw(P2);
//01dab8 3a3c 0001                MOVE.W   #1,D5
  D5W = 1;
//01dabc 3f3c 0001                MOVE.W   #1,-(A7)
//01dac0 4e71                     NOP
//01dac2 4240                     CLR.W    D0
  D0W = 0;
//01dac4 548f                     ADDQ.L   #2,A7
//01dac6 4a40                     TST.W    D0
//01dac8 660a                     BNE      $+12 (=0x01dad4)
  if ((D0W == 0) && (D7W !=2))
  {
//01daca 0c47 0002                CMP.W    #2,D7
//01dace 6704                     BEQ      $+6 (=0x01dad4)
//01dad0 3005                     MOVE.W   D5,D0
    D0W = D5W;
//01dad2 606a                     BRA      $+108 (=0x01db3e)
  }
  else
  {
//01dad4 267c 0000 0000           MOVE.L   #0,A3
    A3 = NULL;
//01dada 3f3c 0001                MOVE.W   #1,-(A7)
//01dade 2f2c a6b0                MOVE.L   -22864(A4),-(A7)
//01dae2 486e ffb0                PEA      -80(A6)
//01dae6 4eba f412                JSR      $-3052 (=0x01cefa)
//01daea defc 000a                ADD.W    #10,A7
    SubstDiskLetter(LOCAL_80, d.Pointer22864, 1);
//01daee 603a                     BRA      $+60 (=0x01db2a)
    do
    {
      while (D7W != 0)
      {
//01daf0 3a3c 0000                MOVE.W   #0,D5
//01daf4 3e05                     MOVE.W   D5,D7
        D7W = D5W = 0;
//01daf6 3f06                     MOVE.W   D6,-(A7)
//01daf8 3f06                     MOVE.W   D6,-(A7)
//01dafa 3f06                     MOVE.W   D6,-(A7)
//01dafc 42a7                     CLR.L    -(A7)
//01dafe 42a7                     CLR.L    -(A7)
//01db00 42a7                     CLR.L    -(A7)
//01db02 2f2c a684                MOVE.L   -22908(A4),-(A7)
//01db06 486e ffb0                PEA      -80(A6)
//01db0a 2f0b                     MOVE.L   A3,-(A7)
//01db0c 4eba fb6a                JSR      $-1172 (=0x01d678)
//01db10 defc 001e                ADD.W    #30,A7
        DoMenu(_2_,
               A3, // TAG01d678
               LOCAL_80,
               d.Pointer22908,
               NULL,
               NULL,
               NULL,
               D6W,
               D6W,
               D6W );
//01db14 2f3c 0001 0001           MOVE.L   #65537,-(A7)
//01db1a 3f06                     MOVE.W   D6,-(A7)
//01db1c 3f3c 0001                MOVE.W   #1,-(A7)
//01db20 4eba f6aa                JSR      $-2388 (=0x01d1cc)
//01db24 508f                     ADDQ.L   #0,A7
        WaitForMenuSelect(_1_, 1, D6W, 1, 1);
//01db26 266c a6dc                MOVE.L   -22820(A4),A3
        A3 = d.Pointer22820;
//01db2a 3007                     MOVE.W   D7,D0
//01db2c 66c2                     BNE      $-60 (=0x01daf0)
      };
//01db2e 3f3c 0001                MOVE.W   #1,-(A7)
//01db32 4e71                     NOP
//01db34 4240                     CLR.W    D0
      D0W = 0;
//01db36 548f                     ADDQ.L   #2,A7
//01db38 4a40                     TST.W    D0
//01db3a 66b4                     BNE      $-74 (=0x01daf0)
    } while (D0W != 0);
//01db3c 3005                     MOVE.W   D5,D0
    D0W = D5W;
  };
//01db3e 4cdf 08e0                MOVEM.L  (A7)+,D567A3
  //RestoreRegs(0x08e0);
//01db42 4e5e                     UNLK     A6
//01db44 4e75                     RTS
  RETURN_i16(D0W);
}

//*********************************************************
//
//*********************************************************
//  TAG01db46
i16 ReadUnscrambleBlock(pnt P1)
{
  dReg D0, D5, D6, D7;
  aReg A3;
  bool bD0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01db46 4e56 0000                LINK     A6,#0
//01db4a 48e7 0710                MOVEM.L  D567A3,-(A7)
  A3 = P1;
//01db4e 3f3c 0200                MOVE.W   #512,-(A7)
//01db52 202e 0008                MOVE.L   8(A6),D0
//01db56 2640                     MOVE.L   D0,A3
//01db58 2f00                     MOVE.L   D0,-(A7)
//01db5a 4eba f42a                JSR      $-3028 (=0x01cf86)
//01db5e 5c8f                     ADDQ.L   #6,A7
  D0W = StreamInput(A3, 512); //TAG01cf86
//01db60 4a40                     TST.W    D0
//01db62 6606                     BNE      $+8 (=0x01db6a)
//01db64 303c 0000                MOVE.W   #0,D0
//01db68 6040                     BRA      $+66 (=0x01dbaa)
  if (D0W == 0) return 0;
//01db6a 3e3c 0020                MOVE.W   #32,D7
//01db6e 4246                     CLR.W    D6
  for (D7W=32, D6W=0; D7W!=0; D7W--)
  {
//01db70 dc5b                     ADD.W    (A3)+,D6
    D6W = (UI16)(D6W + LE16(wordGear(A3)));
    A3 += 2;
//01db72 301b                     MOVE.W   (A3)+,D0
//01db74 b146                     EOR.W    D0,D6
    D6W ^= LE16(wordGear(A3));
//01db76 9c5b                     SUB.W    (A3)+,D6
    A3 += 2;
    D6W = (UI16)(D6W - LE16(wordGear(A3)));
    A3 += 2;
//01db78 301b                     MOVE.W   (A3)+,D0
//01db7a b146                     EOR.W    D0,D6
    D6W ^= LE16(wordGear(A3));
    A3 += 2;
//01db7c 5347                     SUBQ.W   #1,D7
//01db7e 66f0                     BNE      $-14 (=0x01db70)
  };
//01db80 3f3c 0080                MOVE.W   #128,-(A7)
//01db84 206e 0008                MOVE.L   8(A6),A0
//01db88 3f28 003a                MOVE.W   58(A0),-(A7)
//01db8c 2f0b                     MOVE.L   A3,-(A7)
//01db8e 4eba f4b8                JSR      $-2886 (=0x01d048)
//01db92 508f                     ADDQ.L   #0,A7
  Unscramble(A3, LE16(wordGear(P1+58)), 128);
//01db94 4245                     CLR.W    D5
  for (D7W=128, D5W = 0; D7W!=0; D7W--)
  {
//01db96 3e3c 0080                MOVE.W   #128,D7
//01db9a da5b                     ADD.W    (A3)+,D5
    D5W = (UI16)(D5W + LE16(wordGear(A3)));
    A3 += 2;
//01db9c 5347                     SUBQ.W   #1,D7
//01db9e 66fa                     BNE      $-4 (=0x01db9a)
  };
//01dba0 3006                     MOVE.W   D6,D0
//01dba2 b045                     CMP.W    D5,D0
//01dba4 57c0                     SEQ      D0
  bD0 = D6W == D5W;
//01dba6 c07c 0001                AND.W    #1,D0
  D0W = bD0;
//01dbaa 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//01dbae 4e5e                     UNLK     A6
//01dbb0 4e75                     RTS
  return D0W;
}




//*********************************************************
//
//*********************************************************
//   TAG01f140
RESTARTABLE _DisplayChaosStrikesBack(void)
{//(void)
  static dReg D0, D4, D5, D6, D7;
  static aReg A2, A3;
  static PALETTE  LOCAL_340;
  static str10 LOCAL_308[18]; // 10-byte structures
  static pnt LOCAL_96;
  static pnt LOCAL_92;
  static pnt LOCAL_88[18];
  static pnt screenBuffers[2];
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
    RESTART(4)
    RESTART(5)
    RESTART(6)
    RESTART(7)
    RESTART(8)
  END_RESTARTMAP
  d.UseByteCoordinates=0;
  if (d.Long23306 < 138060) RETURN;
  D4L = (i32)allocateMemory(133056,0); // Temporary allocation
  D4L = (D4L + 256) & 0xffffff00; // Round to 256-byte boundary
  A3 = (pnt)D4L;
#ifdef GraphicsDebug
  GrphDbg = fopen("GraphicsDebug.txt","w");
#endif
  ReadAndExpandGraphic(1, A3, 0, 0);
#ifdef GraphicsDebug
  fclose(GrphDbg);
  GrphDbg = NULL;
#endif
  D4L += 32000;
  A2 = d.LogicalScreenBase;
  setscreen(NULL, A2, 0);
  screenBuffers[1] = A2;
  screenBuffers[0] = A3;
  LOCAL_92 = A3;
  fillMemory((i16 *)&LOCAL_340, 16, 2, 2); // 16 words  of 0x0002
  FadeToPalette(_1_,&LOCAL_340);  // darken screen
  ClearMemory(A2, 32000);
  TAG0088b2(A3,           // Move "Presents" to video memory
            d.LogicalScreenBase,
            (RectPos *)d.Word52,
            0,      // src x
            137,    // src y offset
            160,    // src bytes/line
            160,    // dst bytes/line
            -1);
  LOCAL_340.color[15] = 0x777;
  FadeToPalette(_3_,&LOCAL_340); // Display "Presents"
  LOCAL_96 = (pnt)D4L;
  TAG0088b2(A3,       //src = graphic 1
	         (pnt)D4L,  //dst
			     (RectPos *)d.Word44, //dest offsets and sizes
			     0,        //src x offset
			     80,       //src y offset
			     160,      //src bytes/line
			     160,      //dst bytes/line
			     -1);           // "Strikes Back"
  D4L += 9120; // =57 * 160
  D6W = 12;
  D5W = 48;
  for (D7W=0; D7W<18; D7W++)
  {
    LOCAL_88[D7W] = (pnt)D4L;
    ShrinkBLT(A3, LOCAL_88[D7W], 320, 80, D5W, D6W, d.Byte156);
    D0W = sw(320 - D5W);
    D0L = D0W;
    D0L /= 2;
    LOCAL_308[D7W].w1 = D0W;
    D0W = sw(D0W + D5L - 1);
    LOCAL_308[D7W].w2 = D0W;
    D0W = sw((160 - D6W)/2);
    LOCAL_308[D7W].w3 = D0W;
    D0W = sw(D0W + D6W - 1);
    LOCAL_308[D7W].w4 = D0W;
    D0W = sw(((D5W+15)/16)*8);
    LOCAL_308[D7W].w5 = D0W;
    D0L = D0W*D6W;
    D0L &= 0xffff;
    D4L += D0L;
    D6L +=4;
    D5L +=16;
  };
  LOCAL_340.color[15] = 2;
  FadeToPalette(_6_,&LOCAL_340); // Remove "Presents"
  ClearMemory(LOCAL_92, 32000);
  setscreen((pnt)-1, LOCAL_92, -1);
  ClearMemory(A2, 32000);
  setscreen((pnt)-1, A2, -1);
  LOCAL_340.color[3] = 0x0541;
  LOCAL_340.color[4] = 0x0421;
  LOCAL_340.color[5] = 0x0651;
  LOCAL_340.color[6] = 0x0531;
  LOCAL_340.color[8] = 0x0772;
  LOCAL_340.color[15] =0x0700;
  LOCAL_340.color[10] = 0x0002;
  LOCAL_340.color[12] = 0x0002;
  FadeToPalette(_7_, &LOCAL_340); // Just delay.
  for (D7W=0; D7W<18; D7W++) // expanding "CHAOS"
  {
    VBLDelay(_2_,2);
    display();
    TAG0088b2(LOCAL_88[D7W],
              screenBuffers[D7W&1],
              (RectPos *)&LOCAL_308[D7W],
              0,
              0,
              LOCAL_308[D7W].w5,
              160,
              -1);
    setscreen((pnt)-1, screenBuffers[D7W&1], -1); // Swap video buffers
  };
  VBLDelay(_4_,10);
  // Put "Strikes Back" on screen.
  TAG0088b2(LOCAL_96,
            d.LogicalScreenBase,
            (RectPos *)d.Word36,
            0,
            0,
            160,
            160,
            0);
  LOCAL_340.color[10] = 0;  // Black print
  LOCAL_340.color[12] = 0x0700;  // Red background
  FadeToPalette(_8_,&LOCAL_340);
  VBLDelay(_5_,20);
  FreeTempMemory(133056); // Release temporary memory:
  //D4=saveD4;D5=saveD5;D6=saveD6;D7=saveD7;A2=saveA2;A3=saveA3;
  RETURN;
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _OpenPrisonDoors(void) //TAG01f47a
{//(void)
  static dReg D7;
  static aReg A0, A1, A3;
  static pnt LOCAL_4, LOCAL_8;
  static int i;
  RESTARTMAP
    RESTART(1)
  END_RESTARTMAP
  A3 = (pnt)&d.Pointer22952; // Prison door left
  LOCAL_4 = d.LogicalScreenBase + 4800;
  LOCAL_8 = pntGear(A3 + 36);
  for (D7W = 1; D7W != 32; D7W++)
  {
    //UI_Sleep(60);
    if ((D7W % 3) ==1)
    {
      StartSound(d.Pointer22964, 145, 1); //Graphic #535 // Start sound
    };
    MemMove(pntGear(A3+32), pntGear(A3+36), 20608); // Dungeon interior.
// I commented this out.. I could see no change.
    TAG0088b2(d.pViewportBMP,  // left part of interior.
              pntGear(A3+36),
              (RectPos *)d.Word60,
              0,
              0,
              112,
              128,
              -1);
    if (((RectPos *)d.Word68)->w.x2 >= 0)
    {
      TAG0088b2(pntGear(A3+(D7W&3)*4), // Left Door
                pntGear(A3+36),
                (RectPos *)d.Word68,
                (D7W&0xfc)*4,
                0,
                64,
                128,
                -1);
      ((RectPos *)d.Word68)->w.x2 -= 4;
    };
    TAG0088b2(pntGear(((D7W&3)+4)*4+A3), // Right side door
              pntGear(A3+36),
              (RectPos *)d.Word76,
              (D7W&3)*4,
              0,
              64,
              128,
              -1);
    ((RectPos *)d.Word76)->w.x1 += 4;
    for (i=0; i<6; i++) wvbl(_1_);
    A0 = LOCAL_4;
    A1 = LOCAL_8;
    for (i32 i=0; i<161; i++)
    {
      memmove(A0, A1, 112); // Move image to physbase
      A1 += 112;
      *(A0+112) = *(A1+0);
      *(A0+114) = *(A1+2);
      *(A0+116) = *(A1+4);
      *(A0+118) = *(A1+6);
      A0 += 160;
      A1 += 16;
    };
  }; // for (D7W)
  //RestoreRegs(0x0880);
  RETURN;
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01f746(void)
{//(void)
  static aReg A0;
  static RectPos dstPos;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01f746 4e56 fff8                LINK     A6,#-8
//01f74a 48e7 0700                MOVEM.L  D567,-(A7)
  static dReg D6,D7;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
    RESTART(4)
    RESTART(5)
    RESTART(6)
  END_RESTARTMAP
  //i32 D5;
//01f74e 41ec bddc                LEA      -16932(A4),A0
//01f752 2948 be18                MOVE.L   A0,-16872(A4)
  d.PrimaryButtonList = d.Buttons16932;
  d.SecondaryButtonList = NULL;
  d.pKeyXlate2 = NULL;
  d.pKeyXlate1 = NULL;
//01f756 42ac be14                CLR.L    -16876(A4)
//01f75a 42ac be10                CLR.L    -16880(A4)
//01f75e 42ac be0c                CLR.L    -16884(A4)
//01f762 4247                     CLR.W    D7
//01f764 601c                     BRA      $+30 (=0x01f782)
  for (D7W=0; D7W<8; D7W++)
  {
//01f766 4267                     CLR.W    -(A7)
//01f768 2f3c 0000 2840           MOVE.L   #10304,-(A7)
//01f76e 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//01f772 5c8f                     ADDQ.L   #6,A7
    A0 = allocateMemory(10304, 0); // temporary memory
//01f774 3207                     MOVE.W   D7,D1
//01f776 e581                     ASL.L    #2,D1
//01f778 41ec a658                LEA      -22952(A4),A0
//01f77c d0c1                     ADD.W    D1,A0
//01f77e 2080                     MOVE.L   D0,(A0)
    (&d.Pointer22952)[D7W] = A0;
//01f780 5247                     ADDQ.W   #1,D7
//
//01f782 0c47 0008                CMP.W    #8,D7
//01f786 65de                     BCS      $-32 (=0x01f766)
  };
//01f788 4267                     CLR.W    -(A7)
//01f78a 2f3c 0000 5080           MOVE.L   #20608,-(A7)
//01f790 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//01f794 5c8f                     ADDQ.L   #6,A7
//01f796 2940 a678                MOVE.L   D0,-22920(A4)
  d.Pointer22920 = allocateMemory(20608,0);
//01f79a 4267                     CLR.W    -(A7)
//01f79c 2f3c 0000 5080           MOVE.L   #20608,-(A7)
//01f7a2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//01f7a6 5c8f                     ADDQ.L   #6,A7
//01f7a8 2940 a67c                MOVE.L   D0,-22916(A4)
  d.Pointer22916 = allocateMemory(20608,0);
//01f7ac 4267                     CLR.W    -(A7)
//01f7ae 4267                     CLR.W    -(A7)
//01f7b0 2f2c a668                MOVE.L   -22936(A4),-(A7)
//01f7b4 3f3c 0003                MOVE.W   #3,-(A7)
//01f7b8 4ead 0606                JSR      1542(A5)    (=0x022b86)
//01f7bc defc 000a                ADD.W    #10,A7
  ReadAndExpandGraphic(3,d.Pointer22936,0,0);
//01f7c0 4267                     CLR.W    -(A7)
//01f7c2 4267                     CLR.W    -(A7)
//01f7c4 2f2c a658                MOVE.L   -22952(A4),-(A7)
//01f7c8 3f3c 0002                MOVE.W   #2,-(A7)
//01f7cc 4ead 0606                JSR      1542(A5)    (=0x022b86)
//01f7d0 defc 000a                ADD.W    #10,A7
  ReadAndExpandGraphic(2,d.Pointer22952,0,0);
//01f7d4 486c a654                PEA      -22956(A4)
//01f7d8 3f3c 0004                MOVE.W   #4,-(A7)
//01f7dc 4eba ff20                JSR      $-222 (=0x01f6fe)
//01f7e0 5c8f                     ADDQ.L   #6,A7
  D6L = AllocateExpandGraphic(4,&d.Pointer22956);
//01f7e2 2c00                     MOVE.L   D0,D6
//01f7e4 486c a650                PEA      -22960(A4)
//01f7e8 3f3c 0005                MOVE.W   #5,-(A7)
//01f7ec 4eba ff10                JSR      $-238 (=0x01f6fe)
//01f7f0 5c8f                     ADDQ.L   #6,A7
//01f7f2 dc80                     ADD.L    D0,D6
  D6L += AllocateExpandGraphic(5, &d.Pointer22960);
//01f7f4 486c a64c                PEA      -22964(A4)
//01f7f8 3f3c 0217                MOVE.W   #535,-(A7)
//01f7fc 4eba ff00                JSR      $-254 (=0x01f6fe)
//01f800 5c8f                     ADDQ.L   #6,A7
//01f802 dc80                     ADD.L    D0,D6
  D6L += AllocateExpandGraphic(535, &d.Pointer22964);
        // Open prison door sound
//01f804 486c a648                PEA      -22968(A4)
//01f808 3f3c 0216                MOVE.W   #534,-(A7)
//01f80c 4eba fef0                JSR      $-270 (=0x01f6fe)
//01f810 5c8f                     ADDQ.L   #6,A7
//01f812 dc80                     ADD.L    D0,D6
  D6L =+ AllocateExpandGraphic(534, &d.Pointer22968);
//01f814 4ead 0600                JSR      1536(A5)    (=0x021d76)
  closeGraphicsFile();
//01f818 426c a608                CLR.W    -23032(A4)
//01f81c 426e fff8                CLR.W    -8(A6)
  d.UseByteCoordinates = 0;
  dstPos.w.x1 = 0;
  dstPos.w.x2 = 100;
  dstPos.w.y1 = 0;
  dstPos.w.y2 = 160;
//01f820 3d7c 0064 fffa           MOVE.W   #100,-6(A6)
//01f826 426e fffc                CLR.W    -4(A6)
//01f82a 3d7c 00a0 fffe           MOVE.W   #160,-2(A6)
//01f830 3e3c 0001                MOVE.W   #1,D7
//01f834 6034                     BRA      $+54 (=0x01f86a)
  for (D7W=1; D7W<4; D7W++)
  {
//01f836 2f3c 0040 ffff           MOVE.L   #4259839,-(A7)
//01f83c 3f3c 0040                MOVE.W   #64,-(A7)
//01f840 4267                     CLR.W    -(A7)
//01f842 3007                     MOVE.W   D7,D0
//01f844 e540                     ASL.W    #2,D0
//01f846 3f00                     MOVE.W   D0,-(A7)
//01f848 486e fff8                PEA      -8(A6)
//01f84c 3007                     MOVE.W   D7,D0
//01f84e e580                     ASL.L    #2,D0
//01f850 41ec a658                LEA      -22952(A4),A0
//01f854 d0c0                     ADD.W    D0,A0
//01f856 2f10                     MOVE.L   (A0),-(A7)
//01f858 2f2c a658                MOVE.L   -22952(A4),-(A7)
//01f85c 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//01f860 defc 0016                ADD.W    #22,A7
    TAG0088b2(d.Pointer22952,
              (&d.Pointer22952)[D7W],
              &dstPos,
              D7W*4,0,64,64,-1);
    dstPos.w.x2 -= 4;
//01f864 596e fffa                SUBQ.W   #4,-6(A6)
//01f868 5247                     ADDQ.W   #1,D7

//01f86a 0c47 0004                CMP.W    #4,D7
//01f86e 65c6                     BCS      $-56 (=0x01f836)
  };
//01f870 3d7c 007f fffa           MOVE.W   #127,-6(A6)
  dstPos.w.x2 = 127;
//01f876 3e3c 0005                MOVE.W   #5,D7
//01f87a 6030                     BRA      $+50 (=0x01f8ac)
  for (D7W=5; D7W<8; D7W++)
  {
//01f87c 586e fff8                ADDQ.W   #4,-8(A6)
    dstPos.w.x1 += 4;
//01f880 2f3c 0040 ffff           MOVE.L   #4259839,-(A7)
//01f886 3f3c 0040                MOVE.W   #64,-(A7)
//01f88a 4267                     CLR.W    -(A7)
//01f88c 4267                     CLR.W    -(A7)
//01f88e 486e fff8                PEA      -8(A6)
//01f892 3007                     MOVE.W   D7,D0
//01f894 e580                     ASL.L    #2,D0
//01f896 41ec a658                LEA      -22952(A4),A0
//01f89a d0c0                     ADD.W    D0,A0
//01f89c 2f10                     MOVE.L   (A0),-(A7)
//01f89e 2f2c a668                MOVE.L   -22936(A4),-(A7)
//01f8a2 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//01f8a6 defc 0016                ADD.W    #22,A7
  TAG0088b2(d.Pointer22936,
            (&d.Pointer22952)[D7W],
            &dstPos,
            0,0,64,64,-1);
//01f8aa 5247                     ADDQ.W   #1,D7

//01f8ac 0c47 0008                CMP.W    #8,D7
//01f8b0 65ca                     BCS      $-52 (=0x01f87c)
  };
  do
  {
//01f8b2 4eba fd36                JSR      $-712 (=0x01f5ea)
  TAG01f5ea(_1_); //Remove "CHAOS" display, Show Prison door.
//01f8b6 4ead 00a8                JSR      168(A5)    (=0x003026)
  STShowCursor(); //TAG003026
  UI_BeginRecordOK(true);
//01f8ba 4ead 052e                JSR      1326(A5)    (=0x01953a)
  TAG01953a();
//01f8be 397c 0063 d25a           MOVE.W   #99,-11686(A4)
  d.Word11686 = 99;
    do
    {
//01f8c4 3f3c 0025                MOVE.W   #37,-(A7)
//01f8c8 4ead 0138                JSR      312(A5)    (=0x001ce8)
//01f8cc 548f                     ADDQ.L   #2,A7
      wvbl(_2_);
//01f8ce 4ead 051c                JSR      1308(A5)    (=0x01a7b2)
      HandleMouseEvents(_4_,0); //TAG01a7b2();
      if (DiskMenuNeeded) DisplayDiskMenu(_6_);
//01f8d2 0c6c 0063 d25a           CMP.W    #99,-11686(A4)
//01f8d8 67ea                     BEQ      $-20 (=0x01f8c4)
    } while (d.Word11686 == 99);
//01f8da 0c6c 00ca d25a           CMP.W    #202,-11686(A4)
//01f8e0 67d0                     BEQ      $-46 (=0x01f8b2)
  } while (d.Word11686 == 202);
//01f8e2 2f3c 0070 0001           MOVE.L   #7340033,-(A7)
//01f8e8 2f2c a648                MOVE.L   -22968(A4),-(A7)
//01f8ec 4ead 00b4                JSR      180(A5)    (=0x001d16)
//01f8f0 508f                     ADDQ.L   #0,A7
  UI_BeginRecordOK(false);
  StartSound(d.Pointer22968, 112, 1); // Start sound
//01f8f2 3f3c 0014                MOVE.W   #20,-(A7)
//01f8f6 4ead 019e                JSR      414(A5)    (=0x000fb2)
//01f8fa 548f                     ADDQ.L   #2,A7
  VBLDelay(_3_,20); // 20 vertical blanks
//01f8fc 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();//TAG002fd2
//01f900 302c d25a                MOVE.W   -11686(A4),D0
//01f904 6704                     BEQ      $+6 (=0x01f90a)
  if (d.Word11686 != 0)
  {
//01f906 4eba fb72                JSR      $-1164 (=0x01f47a)
    OpenPrisonDoors(_5_);
  };
//01f90a 2006                     MOVE.L   D6,D0
//01f90c d0bc 0001 4200           ADD.L    #82432,D0
//01f912 d0bc 0000 a100           ADD.L    #41216,D0
//01f918 2f00                     MOVE.L   D0,-(A7)
//01f91a 4ead 05be                JSR      1470(A5)    (=0x0217d8)
//01f91e 588f                     ADDQ.L   #4,A7
  FreeTempMemory(D6L + 82432 + 41216);
//01f920 4cdf 00e0                MOVEM.L  (A7)+,D567
//01f924 4e5e                     UNLK     A6
//01f926 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01f5ea(void)
{//(void)
  static dReg D7;
  static aReg A0;
  static LEVELDESC LOCAL_170;
  static i8  LOCAL_154[5][5];
  static CELLFLAG *LOCAL_128[5];
  RESTARTMAP
    RESTART(1)
    RESTART(2)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01f5ea 4e56 ff56                LINK     A6,#-170
//01f5ee 3f07                     MOVE.W   D7,-(A7)
  //i32 saveD7=D7;
//01f5f0 397c 00ff d244           MOVE.W   #255,-11708(A4)
//01f5f6 397c 0001 d25c           MOVE.W   #1,-11684(A4)
//01f5fc 397c 0005 d744           MOVE.W   #5,-10428(A4)
//01f602 397c 0005 d742           MOVE.W   #5,-10430(A4)
  d.partyLevel = 255;
  d.Word11684 = 1;
  d.width  = 5;
  d.height = 5;
//01f608 41ee ff80                LEA      -128(A6),A0
//01f60c 2948 d748                MOVE.L   A0,-10424(A4)
  d.LevelCellFlags = LOCAL_128;
//01f610 41ee ff56                LEA      -170(A6),A0
//01f614 2948 d750                MOVE.L   A0,-10416(A4)
  d.pCurLevelDesc = &LOCAL_170;
//01f618 3f3c 0001                MOVE.W   #1,-(A7)
//01f61c 4267                     CLR.W    -(A7)
//01f61e 3f3c 0019                MOVE.W   #25,-(A7)
//01f622 486e ff66                PEA      -154(A6)
//01f626 4ead 003c                JSR      60(A5)    (=0x000ac0)
//01f62a defc 000a                ADD.W    #10,A7
  fillWithByte(LOCAL_154[0], 25, 0, 1);
//01f62e 4247                     CLR.W    D7
//01f630 602a                     BRA      $+44 (=0x01f65c)
  for (D7W=0; D7W<5; D7W++)
  {
//01f632 3007                     MOVE.W   D7,D0
//01f634 c0fc 0005                MULU     #5,D0
//01f638 41ee ff66                LEA      -154(A6),A0
//01f63c d0c0                     ADD.W    D0,A0
//01f63e 41d0                     LEA      (A0),A0
//01f640 2008                     MOVE.L   A0,D0
    A0 = LOCAL_154[0] + 5*D7W;
//01f642 3207                     MOVE.W   D7,D1
//01f644 e581                     ASL.L    #2,D1
//01f646 41ee ff80                LEA      -128(A6),A0
//01f64a d0c1                     ADD.W    D1,A0
//01f64c 2080                     MOVE.L   D0,(A0)
    LOCAL_128[D7W] = (CELLFLAG *)A0;
//01f64e 3007                     MOVE.W   D7,D0
//01f650 41ee ff70                LEA      -144(A6),A0
//01f654 d0c0                     ADD.W    D0,A0
//01f656 10bc 0020                MOVE.B   #32,(A0)
    LOCAL_154[2][D7W] = 32;
//01f65a 5247                     ADDQ.W   #1,D7

//01f65c 0c47 0005                CMP.W    #5,D7
//01f660 65d0                     BCS      $-46 (=0x01f632)
  };
//01f662 1d7c 0020 ff6d           MOVE.B   #32,-147(A6)
  LOCAL_154[1][2] = 32;
//01f668 486c d176                PEA      -11914(A4)
//01f66c 4eba f9e0                JSR      $-1566 (=0x01f04e)
//01f670 588f                     ADDQ.L   #4,A7
  FadeToPalette(_1_,&d.Palette11914);
//01f672 4267                     CLR.W    -(A7)
//01f674 4267                     CLR.W    -(A7)
//01f676 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01f67a 2f2c a654                MOVE.L   -22956(A4),-(A7)
//01f67e 4ead 05d0                JSR      1488(A5)    (=0x0213ec)
//01f682 defc 000c                ADD.W    #12,A7
  //
  // Entrance to prison without door.
  ExpandGraphic(d.Pointer22956, d.LogicalScreenBase, 0, 0);
//01f686 4267                     CLR.W    -(A7)
//01f688 2f3c 0002 0002           MOVE.L   #131074,-(A7)
//01f68e 4ead 01e6                JSR      486(A5)    (=0x00803a)
//01f692 5c8f                     ADDQ.L   #6,A7
  DrawViewport(2, 2, 0); //facing south, x=y=0.
//01f694 426c d222                CLR.W    -11742(A4)
  d.ViewportUpdated = 0;
//01f698 303c 0000                MOVE.W   #0,D0
//01f69c 3940 a608                MOVE.W   D0,-23032(A4)
  d.UseByteCoordinates = 0;
//01f6a0 2f3c 0080 ffff           MOVE.L   #8454143,-(A7)
//01f6a6 2f3c 001e 00a0           MOVE.L   #1966240,-(A7)
//01f6ac 4267                     CLR.W    -(A7)
//01f6ae 486c ffac                PEA      -84(A4)
//01f6b2 2f2c a678                MOVE.L   -22920(A4),-(A7)
//01f6b6 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01f6ba 4ead 01e0                JSR      480(A5)    (=0x0088b2)
//01f6be defc 0016                ADD.W    #22,A7
  //
  //
  TAG0088b2(d.LogicalScreenBase, // Save interior prison entrance
            d.Pointer22920,
            (RectPos *)d.Word84,
            0,   // src x
            30,  // src y
            160, // src bytes/line
            128, // dst bytes/line
            -1);
//01f6c2 2f3c 0040 ffff           MOVE.L   #4259839,-(A7)
//01f6c8 486c ffa4                PEA      -92(A4)
//01f6cc 2f2c a658                MOVE.L   -22952(A4),-(A7)
//01f6d0 4ead 00a2                JSR      162(A5)    (=0x000f7e)
//01f6d4 defc 000c                ADD.W    #12,A7
  //
  //Put left half of door in opening
  BLT2Screen(d.Pointer22952, (RectPos *)d.Word92, 64, -1);
//01f6d8 2f3c 0040 ffff           MOVE.L   #4259839,-(A7)
//01f6de 486c ff9c                PEA      -100(A4)
//01f6e2 2f2c a668                MOVE.L   -22936(A4),-(A7)
//01f6e6 4ead 00a2                JSR      162(A5)    (=0x000f7e)
//01f6ea defc 000c                ADD.W    #12,A7
  //
  // Put right half of door in opening
  BLT2Screen(d.Pointer22936, (RectPos *)d.Word100, 64, -1);
//01f6ee 486c fe98                PEA      -360(A4)
//01f6f2 4eba f95a                JSR      $-1700 (=0x01f04e)
//01f6f6 588f                     ADDQ.L   #4,A7
  //
  // Display prison entrance.
  FadeToPalette(_2_,&d.Palette360);
//01f6f8 3e1f                     MOVE.W   (A7)+,D7
//01f6fa 4e5e                     UNLK     A6
//01f6fc 4e75                     RTS
  //D7=saveD7;
  RETURN;
}

//*********************************************************
//
//*********************************************************
i32 AllocateExpandGraphic(i16 graphicNum, pnt *address) // TAG01f6fe
{ // Returns size of resulting graphic
  dReg D0;
  aReg A0;
  i32 graphicSize;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//01f6fe 4e56 fffc                LINK     A6,#-4
//01f702 4267                     CLR.W    -(A7)
//01f704 4267                     CLR.W    -(A7)
//01f706 4267                     CLR.W    -(A7)
//01f708 3f2e 0008                MOVE.W   8(A6),-(A7)
//01f70c 4ead 05dc                JSR      1500(A5)    (=0x022d7a)
//01f710 548f                     ADDQ.L   #2,A7
//01f712 c0bc 0000 ffff           AND.L    #65535,D0
//01f718 2d40 fffc                MOVE.L   D0,-4(A6)
  D0L = GetGraphicExpSize(graphicNum);
  graphicSize = D0L;
//01f71c 2f00                     MOVE.L   D0,-(A7)
//01f71e 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//01f722 5c8f                     ADDQ.L   #6,A7
  A0 = allocateMemory(D0L, 0);
//01f724 206e 000a                MOVE.L   10(A6),A0
//01f728 2080                     MOVE.L   D0,(A0)
  *address = A0;
//01f72a 2f00                     MOVE.L   D0,-(A7)
//01f72c 302e 0008                MOVE.W   8(A6),D0
//01f730 807c 8000                OR.W     #32768,D0
//01f734 3f00                     MOVE.W   D0,-(A7)
//01f736 4ead 0606                JSR      1542(A5)    (=0x022b86)
//01f73a defc 000a                ADD.W    #10,A7
  ReadAndExpandGraphic(graphicNum | 0x8000,A0, 0, 0);
  return graphicSize;
//01f73e 202e fffc                MOVE.L   -4(A6),D0
//01f742 4e5e                     UNLK     A6
//01f744 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG01f928(void)
{//(void)
  static dReg D0;
  static i16 LOCAL_2=0;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
  END_RESTARTMAP
//01f92c 3d7c 0708 fffe           MOVE.W   #1800,-2(A6)
//01f932 4ead 0024                JSR      36(A5)    (=0x002fd2)
  STHideCursor();//TAG002fd2
//01f936 486c d176                PEA      -11914(A4)
//01f93a 4eba f712                JSR      $-2284 (=0x01f04e)
//01f93e 588f                     ADDQ.L   #4,A7
  FadeToPalette(_1_,&d.Palette11914); //
//01f940 4267                     CLR.W    -(A7)
//01f942 4267                     CLR.W    -(A7)
//01f944 2f2c d132                MOVE.L   -11982(A4),-(A7)
//01f948 2f2c a650                MOVE.L   -22960(A4),-(A7)
//01f94c 4ead 05d0                JSR      1488(A5)    (=0x0213ec)
//01f950 defc 000c                ADD.W    #12,A7
  ExpandGraphic(d.Pointer22960,d.LogicalScreenBase,0,0);
//01f954 486c feb8                PEA      -328(A4)
//01f958 4eba f6f4                JSR      $-2314 (=0x01f04e)
//01f95c 588f                     ADDQ.L   #4,A7
  FadeToPalette(_2_,&d.Palette328);
  do
  {
//01f95e 3f3c 0025                MOVE.W   #37,-(A7)
//01f962 4ead 0138                JSR      312(A5)    (=0x001ce8)
//01f966 548f                     ADDQ.L   #2,A7
    wvbl(_3_);
//01f968 536e fffe                SUBQ.W   #1,-2(A6)
//01f96c 302e fffe                MOVE.W   -2(A6),D0
  D0W = --LOCAL_2;
//01f970 6714                     BEQ      $+22 (=0x01f986)
    if (D0W == 0) break;
//01f972 302c a5b0                MOVE.W   -23120(A4),D0
//01f976 660e                     BNE      $+16 (=0x01f986)
    if (d.MouseSwitches != 0) break;
//01f978 3f3c 000b                MOVE.W   #11,-(A7)
//01f97c 4ead 0096                JSR      150(A5)    (=0x001cf8)
//01f980 548f                     ADDQ.L   #2,A7
//01f982 4a40                     TST.W    D0
//01f984 67d8                     BEQ      $-38 (=0x01f95e)
  } while (UI_CONSTAT()==0); // keyboard
//01f986 397c 00ca d25a           MOVE.W   #202,-11686(A4)
  d.Word11686 = 202;
//01f98c 4e5e                     UNLK     A6
//01f98e 4e75                     RTS
  RETURN;
}

class ATARIMEM
{ // We put this in a class so that the destructor
  // will happily release the memory.  Keeps debug
  // aids quiet.
  char *m_mem;
public:
  ATARIMEM(void) {m_mem=NULL;};
  ~ATARIMEM(void) {if (m_mem!=NULL) free (m_mem);};
  char *mem(i32 size)
  {
    if (m_mem!=NULL) free(m_mem);
    m_mem = (char *)malloc(size);
    return m_mem;
  };
};

ATARIMEM AtariMem;

//   TAG020286
void InitializeHeap(void)
{
  dReg D0, D5,D6,D7;
  aReg A0, A1;
  // i32 D4;
  DisableCursor();
  firstMemoryBlock = NULL;
  D0L = GetAvailMemory();
  D7L = D0L-20;
  SetSupervisorMode();
  A1 = (pnt)0x5000; // System start address
  D6L = 0x19c00; // System end address;
  A0 = (pnt)0x19bf4; // ???
  D5L = 0x19c00; // What is 4(a0) // End of available memory
  D5L -= D6L; // say there is no space above OS
  ClearSupervisorMode();
  InitializeCaches(AtariMem.mem(D7L), D7L, NULL, D5L);
}

void TAG020466(void)
{
  dReg D7;
  //i32 saveD6=D6, saveD7=D7;
  SetSupervisorMode();
  D7W = sw(NumberFloppyDrives);
  ClearSupervisorMode();
  if (D7W == 2)
  {
    d.Word22992 = 1;
  }
  else
  {
    d.Word22970 = 1;
  };
  d.Pointer22984 = "DRIVE.";
  d.Pointer22988 = "DRIVE.";
  //D6=saveD6; D7=saveD7;
}

i8 data2052e[] = {0x05, 0x05, 0x11, 0x1e, 0x2b};
i8 data20577[] = "CSBGAMEx.BAK";

//*********************************************************
//
//*********************************************************
//   TAG0204bc
RESTARTABLE _SelectSaveGame(const i32 P1)
{//(void)
  static dReg D0;
  static aReg A0, A1;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
  END_RESTARTMAP
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0204bc 48e7 fffe                MOVEM.L  D01234567A0123456,-(A7)
//0204c0 3f00                     MOVE.W   D0,-(A7)
//0204c2 3f00                     MOVE.W   D0,-(A7)
//0204c4 3f00                     MOVE.W   D0,-(A7)
//0204c6 487a 0091                PEA      $+147 (=0x020559)
//0204ca 487a 0080                PEA      $+130 (=0x02054c)
//0204ce 487a 006f                PEA      $+113 (=0x02053f)
//0204d2 487a 005f                PEA      $+97 (=0x020533)
//0204d6 487a 008e                PEA      $+144 (=0x020566)
//0204da 42a7                     CLR.L    -(A7)
//0204dc 4eba d19a                JSR      $-11876 (=0x01d678)
//0204e0 4fef 001e                LEA      30(A7),A7
  DoMenu(_2_,
         NULL, //TAG01d678
         "SELECT GAME FILE",
         "CSBGAME.DAT",
         "CSBGAME2.DAT",
         "CSBGAME3.DAT",
         "CSBGAME4.DAT",
         P1,
         P1,
         P1);

//0204e4 2f3c 0002 0001           MOVE.L   #131073,-(A7)
//0204ea 3f2f 0006                MOVE.W   6(A7),-(A7)
//0204ee 3f3c 0004                MOVE.W   #4,-(A7)
//0204f2 4eba ccd8                JSR      $-13094 (=0x01d1cc)
//0204f6 504f                     ADDQ.W   #0,A7
  WaitForMenuSelect(_1_, 4,P1,2,1) ;
  D0W = i16Result;
//0204f8 41fa 0034                LEA      $+54 (=0x02052e),A0
//0204fc 1030 0000                MOVE.B   0(A0,D0W),D0
//020500 d0c0                     ADD.W    D0,A0
//020502 2948 a640                MOVE.L   A0,-22976(A4)
  switch (D0W)
  {
  case 1: A0 = "CSBGAME.DAT"; break;
  case 2: A0 = "CSBGAME2.DAT"; break;
  case 3: A0 = "CSBGAME3.DAT"; break;
  case 4: A0 = "CSBGAME4.DAT"; break;
  default: A0 = "Huh"; NotImplemented(0x20502);
  }
  d.SaveGameFilename = A0;
//020506 43fa 006f                LEA      $+113 (=0x020577),A1
  A1 = data20577;
  do
  {
//02050a 12d0                     MOVE.B   (A0),(A1)+
    *(A1++) = *A0;
//02050c 0c18 002e                CMP.B    #46,(A0)+
//020510 66f8                     BNE      $-6 (=0x02050a)
  } while ( *(A0++) != 46); // until period
//020512 12fc 0042                MOVE.B   #66,(A1)+
//020516 12fc 0041                MOVE.B   #65,(A1)+
//02051a 12fc 004b                MOVE.B   #75,(A1)+
  *(A1++) = 66;
  *(A1++) = 65;
  *(A1++) = 75;
  *(A1) = 0;
//02051e 4211                     CLR.B    (A1)
//020520 43fa 0055                LEA      $+87 (=0x020577),A1
//020524 2949 a63c                MOVE.L   A1,-22980(A4)
  d.Pointer22980 = data20577;
//020528 4cdf 7fff                MOVEM.L  (A7)+,D01234567A0123456
//02052c 4e75                     RTS
  RETURN;
}

//*********************************************************
//
//*********************************************************
void getBits(i32 numbits, dReg& D0, dReg& D1, aReg& A3)
// internal subroutine for TAG0206f8
{ // Bits added to D1U and swapped to D1W
  // D0 keeps track of number of bits remaining in D1W
//020752 b042                     CMP.W    D2,D0
//020754 6a08                     BPL      $+10 (=0x02075e)
  if (D0W < numbits)
  {
//020756 9440                     SUB.W    D0,D2
    numbits -= D0W;
//020758 e1a9                     LSL.L    D0,D1
    D1L <<= D0W;
//02075a 321b                     MOVE.W   (A3)+,D1
//02075c 7010                     MOVEQ.L  #16,D0
    D1W = LE16(wordGear(A3));
    A3 += 2;
    D0L = 16; // Number of bits remaining in D1W
  };
//02075e 9042                     SUB.W    D2,D0
  D0W = sw(D0W - numbits);
//020760 e5a9                     LSL.L    D2,D1
  D1L <<= numbits;
//020762 4841                     SWAP     D1
  // do this in calling routine SWAP(D1); //D1 = (D1U & 0xffff)|((D1W&0xffff)<<16);
//020764 4e75                     RTS
}


void ExpandData(pnt src,pnt dst,i32 dstSize) //TAG0206f8
{
  // Data file consists of variable length fields.
  // There are 20 bytes at the beginning that are
  // lookup tables---the first is 4 bytes long
  // and is used to decode data of the form
  // ( in binary ) of 0xx.  xx is the index into the
  // table.
  // The next table is 16 bytes long and is used to
  // decode data of the form 10yyyy where yyyy is used
  // as an index into the table.
  // The only remaining case is data of the form
  // 11zzzzzzzz where zzzzzzzz is a literal byte.
  // Summary:
  // 0xx        index into small table
  // 10yyyy     index into large table
  // 11zzzzzzzz literal data
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  dReg D0, D1, D7;
  aReg A0, A2, A3;
//0206f8 4e56 0000                LINK     A6,#0
//0206fc 48e7 0130                MOVEM.L  D7A23,-(A7)
  //SaveRegs(0x0130);
//020700 266e 0008                MOVE.L   8(A6),A3
//020704 246e 000c                MOVE.L   12(A6),A2
//020708 2e2e 0010                MOVE.L   16(A6),D7
  A3 = src;
  A2 = dst;
  D7L = dstSize;
//02070c 204b                     MOVE.L   A3,A0
//02070e 47eb 0014                LEA      20(A3),A3
  A0 = A3;
  A3 = A3 + 20;
//020712 7200                     MOVEQ.L  #0,D1
//020714 321b                     MOVE.W   (A3)+,D1
  D1H1 = 0;
tag020714: // Start new 16-bit input word.
  D1W = LE16(wordGear(A3));
  A3 += 2;
//020716 7010                     MOVEQ.L  #16,D0
  D0L = 16; // # bits in D1W;
//020718 0801 000f                BTST     15,D1
//02071c 671e                     BEQ      $+32 (=0x02073c)
tag020718: // Is next bit a '1'?
  if ((D1W & 0x8000) == 0) goto tag02073c;
//02071e 7402                     MOVEQ.L  #2,D2
  getBits(2, D0, D1, A3);
  SWAP(D1);
//020720 6130                     BSR      $+50 (=0x020752)
//020722 5541                     SUBQ.W   #2,D1
  D1W -= 2;
//020724 660c                     BNE      $+14 (=0x020732)
  if (D1W != 0) goto tag020732;
//020726 4841                     SWAP     D1
  SWAP(D1);
//020728 7404                     MOVEQ.L  #4,D2
//02072a 6126                     BSR      $+40 (=0x020752)
  getBits(4, D0, D1, A3);
  SWAP(D1);
//02072c 14f0 1004                MOVE.B   4(A0,D1W),(A2)+
//020730 6012                     BRA      $+20 (=0x020744)
  *A2 = *(A0 + D1W + 4);
  A2++;
  goto tag020744;
tag020732:
//020732 4841                     SWAP     D1
  SWAP(D1);//D1L = D1UH[1] | (D1UH[0] << 16);
//020734 7408                     MOVEQ.L  #8,D2
//020736 611a                     BSR      $+28 (=0x020752)
  getBits(8, D0, D1, A3);
  SWAP(D1);//D1L = D1UH[1] | (D1UH[0] << 16);
//020738 14c1                     MOVE.B   D1,(A2)+
//02073a 6008                     BRA      $+10 (=0x020744)
  *A2 = D1B;
  A2++;
  goto tag020744;
tag02073c:
//02073c 7403                     MOVEQ.L  #3,D2
//02073e 6112                     BSR      $+20 (=0x020752)
  getBits(3, D0, D1, A3);
  SWAP(D1);//D1L = D1UH[1] | (D1UH[0] << 16);
//020740 14f0 1000                MOVE.B   0(A0,D1W),(A2)+
  *A2 = *(A0 + D1W);
  A2++;
tag020744:
//020744 5387                     SUBQ.L   #1,D7
  D7W--;
  if (D7W == 0) goto tag020766;
//020746 671e                     BEQ      $+32 (=0x020766)
//020748 4241                     CLR.W    D1
//02074a 4841                     SWAP     D1
  D1H2 = D1H1;
  D1H1 = 0;
//02074c 4a40                     TST.W    D0
//02074e 66c8                     BNE      $-54 (=0x020718)
  if (D0W != 0) goto tag020718;
//020750 60c2                     BRA      $-60 (=0x020714)
  goto tag020714;
tag020766:
//020766 4cdf 0c80                MOVEM.L  (A7)+,D7A23
  //RestoreRegs(0x0c80);
//02076a 4e5e                     UNLK     A6
//02076c 4e75                     RTS
  return;
}

//*********************************************************
//
//*********************************************************
void TAG0207cc(void)
{//(void)
  dReg D7;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0207cc 4e56 0000                LINK     A6,#0
//0207d0 3f07                     MOVE.W   D7,-(A7)
//0207d2 302c d256                MOVE.W   -11690(A4),D0
//0207d6 670e                     BEQ      $+16 (=0x0207e6)
  if (d.PartySleeping)
  {
//0207d8 4ead 0540                JSR      1344(A5)    (=0x01a77c)
    DisplaySleepScreen();
//0207dc 4267                     CLR.W    -(A7)
//0207de 4ead 01ec                JSR      492(A5)    (=0x0042da)
//0207e2 548f                     ADDQ.L   #2,A7
    MarkViewportUpdated(0);
//0207e4 6046                     BRA      $+72 (=0x02082c)
  }
  else
  {
//0207e6 3e2c b0d6                MOVE.W   -20266(A4),D7
    D7W = d.MagicCaster;
//0207ea 397c ffff b0d6           MOVE.W   #65535,-20266(A4)
    d.MagicCaster = -1;
//0207f0 3f07                     MOVE.W   D7,-(A7)
//0207f2 4ead 0534                JSR      1332(A5)    (=0x01b726)
//0207f6 548f                     ADDQ.L   #2,A7
    SelectMagicCaster(D7W);
//0207f8 302c b0ea                MOVE.W   -20246(A4),D0
//0207fc 6606                     BNE      $+8 (=0x020804)
    if (d.AttackingCharacterOrdinal == 0)
    {
//0207fe 397c 0001 b0e4           MOVE.W   #1,-20252(A4)
      d.Word20252 = 1;
    };
//020804 4ead 0552                JSR      1362(A5)    (=0x01b098)
    TAG01b098();
//020808 3e2c bf1a                MOVE.W   -16614(A4),D7
    D7W = d.SelectedCharacterOrdinal;
//02080c 6712                     BEQ      $+20 (=0x020820)
    if (d.SelectedCharacterOrdinal)
    {
//02080e 426c bf1a                CLR.W    -16614(A4)
      d.SelectedCharacterOrdinal = 0;
//020812 3007                     MOVE.W   D7,D0
//020814 5340                     SUBQ.W   #1,D0
//020816 3f00                     MOVE.W   D0,-(A7)
//020818 4ead 044a                JSR      1098(A5)    (=0x0192f4)
//02081c 548f                     ADDQ.L   #2,A7
      TAG0192f4(D7W-1);
//02081e 6008                     BRA      $+10 (=0x020828)
    }
    else
    {
//020820 4ead 01c2                JSR      450(A5)    (=0x004308)
      FloorAndCeilingOnly();
//020824 4ead 0528                JSR      1320(A5)    (=0x01b880)
      DrawMovementButtons();
    };
//020828 4ead 0192                JSR      402(A5)    (=0x002572)
    TAG002572();
  };
//02082c 426c d21a                CLR.W    -11750(A4)
  d.Word11750 = 0;
//020830 3e1f                     MOVE.W   (A7)+,D7
//020832 4e5e                     UNLK     A6
//020834 4e75                     RTS
}

void TAG020836(i8 *P1)
{
  dReg D0, D7;
  //i32 D6;
  aReg A3;
  i16 index;
  A3=d.Pointer4+128;
  D7W=0; D7B=*A3; A3++;
  index=0;
  for (index=0; index< D7W; index++)
  {
    P1[index] = A3[index];
  };
  D0W=index;
  index++;
  D0W=index;
  P1[index]=0;
  D0W=D7W;
}

i16 TAG020880(i16 width,i16 height,i16 scale)
{ //width is in bytes (should be multiple of 8?)
  //scale is in 32ths.   eg:  16 means 1/2
  dReg D0, D1;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//020880 4e56 0000                LINK     A6,#0
//  ui32 n;
//020884 302e 0008                MOVE.W   8(A6),D0
//020888 c1ee 000c                MULU     12(A6),D0
//02088c 322e 000c                MOVE.W   12(A6),D1
//020890 e241                     ASR.W    #1,D1
//020892 d041                     ADD.W    D1,D0
//020894 ea40                     ASR.W    #5,D0
//020896 5e40                     ADDQ.W   #7,D0
    D0W = sw((width * scale + scale/2) / 32);
    D0W += 7; // Round to multiple of 8
//020898 c07c fff8                AND.W    #65528,D0
    D0W &= 0xfff8;
//02089c 322e 000a                MOVE.W   10(A6),D1
//0208a0 c3ee 000c                MULU     12(A6),D1
//0208a4 342e 000c                MOVE.W   12(A6),D2
//0208a8 e242                     ASR.W    #1,D2
//0208aa d242                     ADD.W    D2,D1
//0208ac ea41                     ASR.W    #5,D1
    D1L = (height * scale + scale/2) / 32;
//0208ae c1c1                     MULU     D1,D0
    D0L = D0UW * D1UW;
//0208b0 4e5e                     UNLK     A6
//0208b2 4e75                     RTS
    return D0W;
}

void swap4words(i16 *words)
{
  words[0]=LE16(words[0]);
  words[1]=LE16(words[1]);
  words[2]=LE16(words[2]);
  words[3]=LE16(words[3]);
}

void swapNwords(i16 *words, i32 n)
{
  for (i32 i=0; i<n; i++)
  {
    words[i]=LE16(words[i]);
  };
}

void SwapKeyXlate(pnt p)
{
  while (wordGear(p)!=0)
  {
    wordGear(p) = LE16(wordGear(p));
    p+=2;
    wordGear(p) = LE16(wordGear(p));
    p+=2;
  };
}

void swapRectPos(RectPos *rect)
{
  swap4words((i16 *)rect);
}

void fixSpell(SPELL *p)
{
  p->spellID = LE32(p->spellID);
  p->word6 = LE16(p->word6);
}

void fixItem26(ITEM26 *pItem)
{
  pItem->word2  = LE16(pItem->word2);
  pItem->word4  = LE16(pItem->word4);
  pItem->word14 = LE16(pItem->word14);
  pItem->word16 = LE16(pItem->word16);
  pItem->word18 = LE16(pItem->word18);
  pItem->word20 = LE16(pItem->word20);
};

void SwapGraphic0x231(void)
{
  SwapWordsInButtonList(d.Buttons17796); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17760); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons16932); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons16956); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17304); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17196); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17832); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17892); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons17952); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18060); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18216); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18324); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18372); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18660); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18624); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18792); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18768); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18708); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18876); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18828); // Fix littleEndian problem
  SwapWordsInButtonList(d.Buttons18936); // Fix littleEndian problem
  SwapKeyXlate(d.Byte18448);
  SwapKeyXlate(d.Byte18440);
  SwapKeyXlate((pnt)d.Byte18428);
  SwapKeyXlate(d.Byte18400);
  swapNwords((i16 *)d.MoveButn18496,4*4); // 4 directions
  swapNwords(d.Word18552,28);
  swapNwords(d.Word18568,4); // deltax
  swapNwords(d.Word18560,4); // deltay
  swapNwords((i16 *)d.Byte18600,4); // RectPos
  swapNwords((i16 *)d.Byte18592,4); // RectPos
  swapNwords((i16 *)d.Byte18584,4); // RectPos
  swapNwords((i16 *)d.Byte18576,4); // RectPos
};

void TAG0208b4(void)
{
  dReg D0, D1, D3, D4, D5, D6, D7;
  aReg A0;
  ITEM6 *psA3;
  pnt LOCAL_4;
  pnt LOCAL_8;
  i16 LOCAL_10, LOCAL_12;
  i32 i;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0208b4 4e56 fff4                LINK     A6,#-12
//0208b8 48e7 0f30                MOVEM.L  D4567A23,-(A7)
  //i32 saveD4=D4, saveD5=D5, saveD6=D6, saveD7=D7;
  //pnt saveA2=A2, saveA3=A3;
//0208bc 3f3c 0001                MOVE.W   #1,-(A7)
//0208c0 2f3c 0000 0cb0           MOVE.L   #3248,-(A7)
//0208c6 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0208ca 5c8f                     ADDQ.L   #6,A7
//0208cc 2940 f7e6                MOVE.L   D0,-2074(A4)
  d.Pointer2074 = allocateMemory(3248, 1);
//0208d0 3f3c 0001                MOVE.W   #1,-(A7)
//0208d4 2f3c 0000 1ea0           MOVE.L   #7840,-(A7)
//0208da 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0208de 5c8f                     ADDQ.L   #6,A7
//0208e0 2940 f7ea                MOVE.L   D0,-2070(A4)
  d.Pointer2070 = allocateMemory(7840, 1);
//0208e4 3f3c 0001                MOVE.W   #1,-(A7)
//0208e8 2f3c 0000 0188           MOVE.L   #392,-(A7)
//0208ee 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0208f2 5c8f                     ADDQ.L   #6,A7
//0208f4 2940 f7d6                MOVE.L   D0,-2090(A4)
  d.Pointer2090 = allocateMemory(392, 1);
//0208f8 3f3c 0001                MOVE.W   #1,-(A7)
//0208fc 2f3c 0000 0188           MOVE.L   #392,-(A7)
//020902 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020906 5c8f                     ADDQ.L   #6,A7
//020908 2940 f7da                MOVE.L   D0,-2086(A4)
  d.Pointer2086 = allocateMemory(392, 1);
//02090c 3f3c 0001                MOVE.W   #1,-(A7)
//020910 2f3c 0000 0cc0           MOVE.L   #3264,-(A7)
//020916 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02091a 5c8f                     ADDQ.L   #6,A7
//02091c 2940 f7d2                MOVE.L   D0,-2094(A4)
  d.Pointer2094 = allocateMemory(3264, 1);
//020920 3f3c 0001                MOVE.W   #1,-(A7)
//020924 2f3c 0000 13f8           MOVE.L   #5112,-(A7)
//02092a 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02092e 5c8f                     ADDQ.L   #6,A7
//020930 2940 f7ce                MOVE.L   D0,-2098(A4)
  d.Pointer2098 = allocateMemory(5112, 1);
//020934 3f3c 0001                MOVE.W   #1,-(A7)
//020938 2f3c 0000 3780           MOVE.L   #14208,-(A7)
//02093e 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020942 5c8f                     ADDQ.L   #6,A7
//020944 2940 f7ca                MOVE.L   D0,-2102(A4)
  d.Pointer2102 = allocateMemory(14208, 1);
//020948 3f3c 0001                MOVE.W   #1,-(A7)
//02094c 2f3c 0000 0880           MOVE.L   #2176,-(A7)
//020952 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020956 5c8f                     ADDQ.L   #6,A7
//020958 2940 f7c6                MOVE.L   D0,-2106(A4)
  d.Pointer2106 = allocateMemory(2176, 1);
//02095c 3f3c 0001                MOVE.W   #1,-(A7)
//020960 2f3c 0000 0880           MOVE.L   #2176,-(A7)
//020966 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02096a 5c8f                     ADDQ.L   #6,A7
//02096c 2940 f7c2                MOVE.L   D0,-2110(A4)
  d.Pointer2110 = allocateMemory(2176, 1);
//020970 3f3c 0001                MOVE.W   #1,-(A7)
//020974 2f3c 0000 0090           MOVE.L   #144,-(A7)
//02097a 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02097e 5c8f                     ADDQ.L   #6,A7
//020980 2940 f7be                MOVE.L   D0,-2114(A4)
  d.Pointer2114 = allocateMemory(144, 1);
//020984 3f3c 0001                MOVE.W   #1,-(A7)
//020988 2f3c 0000 0100           MOVE.L   #256,-(A7)
//02098e 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020992 5c8f                     ADDQ.L   #6,A7
//020994 2940 f7ba                MOVE.L   D0,-2118(A4)
  d.Pointer2118 = allocateMemory(256, 1);
//020998 3f3c 0001                MOVE.W   #1,-(A7)
//02099c 2f3c 0000 02b0           MOVE.L   #688,-(A7)
//0209a2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0209a6 5c8f                     ADDQ.L   #6,A7
//0209a8 2940 f7b6                MOVE.L   D0,-2122(A4)
  d.Pointer2122 = allocateMemory(688, 1);
//0209ac 3f3c 0001                MOVE.W   #1,-(A7)
//0209b0 2f3c 0000 02c0           MOVE.L   #704,-(A7)
//0209b6 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0209ba 5c8f                     ADDQ.L   #6,A7
//0209bc 2940 f7b2                MOVE.L   D0,-2126(A4)
  d.Pointer2126 = allocateMemory(704, 1);
//0209c0 3f3c 0001                MOVE.W   #1,-(A7)
//0209c4 2f3c 0000 0618           MOVE.L   #1560,-(A7)
//0209ca 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0209ce 5c8f                     ADDQ.L   #6,A7
//0209d0 2940 f7ae                MOVE.L   D0,-2130(A4)
  d.Pointer2130 = allocateMemory(1560, 1);
//0209d4 3f3c 0001                MOVE.W   #1,-(A7)
//0209d8 2f3c 0000 05e0           MOVE.L   #1504,-(A7)
//0209de 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0209e2 5c8f                     ADDQ.L   #6,A7
//0209e4 2940 f7aa                MOVE.L   D0,-2134(A4)
  d.Pointer2134 = allocateMemory(1504, 1);
//0209e8 3f3c 0001                MOVE.W   #1,-(A7)
//0209ec 2f3c 0000 05e0           MOVE.L   #1504,-(A7)
//0209f2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//0209f6 5c8f                     ADDQ.L   #6,A7
//0209f8 2940 f7a2                MOVE.L   D0,-2142(A4)
  d.Pointer2142 = allocateMemory(1504, 1);
//0209fc 3f3c 0001                MOVE.W   #1,-(A7)
//020a00 2f3c 0000 07b0           MOVE.L   #1968,-(A7)
//020a06 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020a0a 5c8f                     ADDQ.L   #6,A7
//020a0c 2940 f7a6                MOVE.L   D0,-2138(A4)
  d.Pointer2138 = allocateMemory(1968, 1);
//020a10 3f3c 0001                MOVE.W   #1,-(A7)
//020a14 2f3c 0000 3b80           MOVE.L   #15232,-(A7)
//020a1a 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020a1e 5c8f                     ADDQ.L   #6,A7
//020a20 2940 d25e                MOVE.L   D0,-11682(A4)
  d.pViewportBMP = allocateMemory(15232, 1);
//020a24 202c d25e                MOVE.L   -11682(A4),D0
//020a28 d0bc 0000 0cb0           ADD.L    #3248,D0
//020a2e 2940 f7e2                MOVE.L   D0,-2078(A4)
  d.Pointer2078 = d.pViewportBMP + 3248;
//020a32 202c f7e2                MOVE.L   -2078(A4),D0
//020a36 d0bc 0000 1030           ADD.L    #4144,D0
//020a3c 2940 f7de                MOVE.L   D0,-2082(A4)
  d.Pointer2082 = d.Pointer2078 + 4144;
//020a40 4267                     CLR.W    -(A7)
//020a42 4267                     CLR.W    -(A7)
//020a44 41ec e3b0                LEA      -7248(A4),A0
//020a48 2008                     MOVE.L   A0,D0
//020a4a 5480                     ADDQ.L   #2,D0
//020a4c 2f00                     MOVE.L   D0,-(A7)
//020a4e 3f3c 822e                MOVE.W   #33326,-(A7)
//020a52 4ead 0606                JSR      1542(A5)    (=0x022b86)
//020a56 defc 000a                ADD.W    #10,A7
  ASSERT(d.GraphicExpSizes[0x22e] == 0x1270);
  ReadAndExpandGraphic(0x8000|0x22e, d.Byte7248+2, 0,0); //TAG022b86
  for (i=0; i<27; i++)
  {
    d.Item6414[i].word0 = LE16(d.Item6414[i].word0);
  };
  d.Word4040[0] = LE16(d.Word4040[0]);
  swapNwords(d.Word7246, 12);
  swapNwords(d.Word7222, 30);
  swapNwords(d.Word7162, 60);
  swapNwords(d.Word7042, 30);
//020a5a 4267                     CLR.W    -(A7)
//020a5c 4267                     CLR.W    -(A7)
//020a5e 41ec b606                LEA      -18938(A4),A0
//020a62 2008                     MOVE.L   A0,D0
//020a64 5480                     ADDQ.L   #2,D0
//020a66 2f00                     MOVE.L   D0,-(A7)
//020a68 3f3c 8231                MOVE.W   #33329,-(A7)
//020a6c 4ead 0606                JSR      1542(A5)    (=0x022b86)
//020a70 defc 000a                ADD.W    #10,A7
  ASSERT(d.GraphicExpSizes[0x231] == 0x804);
  ReadAndExpandGraphic(0x8000|0x231,d.Byte18938+2, 0, 0);
  SwapGraphic0x231();
//020a74 4267                     CLR.W    -(A7)
//020a76 4267                     CLR.W    -(A7)
//020a78 41ec b0ec                LEA      -20244(A4),A0
//020a7c 2008                     MOVE.L   A0,D0
//020a7e 5480                     ADDQ.L   #2,D0
//020a80 2f00                     MOVE.L   D0,-(A7)
//020a82 3f3c 8230                MOVE.W   #33328,-(A7)
//020a86 4ead 0606                JSR      1542(A5)    (=0x022b86)
//020a8a defc 000a                ADD.W    #10,A7
  ASSERT(d.GraphicExpSizes[0x230] == 0x4e8);
  ReadAndExpandGraphic(0x8000|0x230, d.Byte20244+2, 0, 0);
  swapRectPos(&d.wRectPos20202);
  swapRectPos(&d.wRectPos20210);
  swapRectPos(&d.wRectPos20218);
  swapRectPos(&d.wRectPos20226);
  swapRectPos(&d.wRectPos20234);
  for (i=0; i<25; i++) fixSpell(&d.Spells[i]);
//020a8e 4267                     CLR.W    -(A7)
//020a90 4267                     CLR.W    -(A7)
//020a92 41ec d79a                LEA      -10342(A4),A0
//020a96 2008                     MOVE.L   A0,D0
//020a98 5480                     ADDQ.L   #2,D0
//020a9a 2f00                     MOVE.L   D0,-(A7)
//020a9c 3f3c 822f                MOVE.W   #33327,-(A7)
//020aa0 4ead 0606                JSR      1542(A5)    (=0x022b86)
//020aa4 defc 000a                ADD.W    #10,A7
  ASSERT(d.GraphicExpSizes[0x22f] == 0xc0e);
  ReadAndExpandGraphic(0x8000|0x22f, d.Byte10342+2, 0,0);
  swapNwords(d.Word9760, 44);
  for (i=0; i<4; i++)
  {
    d.DeltaX[i] = LE16(d.DeltaX[i]);
    d.DeltaY[i] = LE16(d.DeltaY[i]);
  };
  for (i=0; i<27; i++) fixItem26(&d.MonsterDescriptor[i]);
  for (i=0; i<8; i++)d.FoodValue[i]=LE16(d.FoodValue[i]);
  for (i=0; i<180; i++)
  {
    d.ObjDesc[i].littleEndian();
    //d.ObjDesc[i].objectType = LE16(d.ObjDesc[i].objectType);
    //d.ObjDesc[i].word4 = LE16(d.ObjDesc[i].word4);
  };
  for (i=0; i<46; i++)
  {
    d.weapons[i].word4 = LE16(d.weapons[i].word4);
  };
//020aa8 4267                     CLR.W    -(A7)
//020aaa 4267                     CLR.W    -(A7)
//020aac 41ec f8d8                LEA      -1832(A4),A0
//020ab0 2008                     MOVE.L   A0,D0
//020ab2 5480                     ADDQ.L   #2,D0
//020ab4 2f00                     MOVE.L   D0,-(A7)
//020ab6 3f3c 8232                MOVE.W   #33330,-(A7)
//020aba 4ead 0606                JSR      1542(A5)    (=0x022b86)
//020abe defc 000a                ADD.W    #10,A7
  ASSERT(d.GraphicExpSizes[0x232] == 0x722);
  ReadAndExpandGraphic(0x8000|0x232, d.Byte1832+2, 0,0);
  swap4words(d.Word20242);
  swap4words((i16 *)&d.wRectPos1780);
  swap4words((i16 *)&d.wRectPos1788);
  swap4words((i16 *)&d.wRectPos1796);
  for (i=0; i<22; i++)
  {
    d.sound1772[i].word0 = LE16(d.sound1772[i].word0);
  };
  swapNwords(d.DropOrder,30);
  swapNwords(d.Word1526[0],12);
  swapNwords(d.Word1502[0],24);
  for (i=0; i<4; i++) swapNwords((i16 *)&d.wRectPos1454[i],4);
  swapNwords(d.Word1406,8);
  swapNwords((i16 *)d.Byte1094,4);
  swapNwords((i16 *)d.PaletteBrightness,6);
  swapNwords((i16 *)d.Word1074,16);
  swapNwords((i16 *)&d.CarryLocation[0],38);
  swapNwords((i16 *)&d.wRectPos966,4);
  swapNwords((i16 *)&d.wRectPos958,4);
  swapNwords((i16 *)&d.wRectPos950,4);
  swapNwords((i16 *)&d.wRectPos942,4);
  swapNwords((i16 *)&d.wRectPos934,4);
  swapNwords((i16 *)&d.wRectPos926,4);
  swapNwords((i16 *)&d.IconDisplay, 3*46);
  swap4words((i16 *)&d.wRectPos624);
  swapNwords(d.Word612,7);
  swapNwords(&d.Word554,1);
  for (i=0; i<6; i++) swapNwords((i16 *)&d.Palette552[i],16);
  swapNwords((i16 *)&d.Palette360,16);
  swapNwords((i16 *)&d.Palette328,16);
  swap4words(d.Word140);
  swap4words(d.Word132);
  swap4words(d.Word124);
  swap4words(d.Word116);
  swap4words(d.Word108);
  swap4words(d.Word100);
  swap4words(d.Word92);
  swap4words(d.Word84);
  swap4words(d.Word76);
  swap4words(d.Word68);
  swap4words(d.Word60);
  swap4words(d.Word52);
  swap4words(d.Word44);
  swap4words(d.Word36);
  swap4words(d.Word28);
  swap4words(d.Word20);
  swap4words(d.Word12);
  for (i32 j=0; j<70; j++)
    d.uw296[j] = LE16(d.uw296[j]);
//020ac2 206c a516                MOVE.L   -23274(A4),A0
//020ac6 30bc 3b80                MOVE.W   #15232,(A0)
  d.pwPointer23274[0] = 15232;
//020aca 206c a516                MOVE.L   -23274(A4),A0
//020ace 317c 11d0 0002           MOVE.W   #4560,2(A0)
  d.pwPointer23274[1] = 4560;
//020ad4 206c a516                MOVE.L   -23274(A4),A0
//020ad8 317c 04a0 0004           MOVE.W   #1184,4(A0)
  d.pwPointer23274[2] = 1184;
//020ade 206c a516                MOVE.L   -23274(A4),A0
//020ae2 317c 0378 0006           MOVE.W   #888,6(A0)
  d.pwPointer23274[3] = 888;
//020ae8 3e3c 000f                MOVE.W   #15,D7
//020aec 604a                     BRA      $+76 (=0x020b38)
  for (D7W=15; D7W<=16; D7W++)
  {
//020aee 3007                     MOVE.W   D7,D0
//020af0 d07c 011e                ADD.W    #286,D0
//020af4 3207                     MOVE.W   D7,D1
//020af6 e581                     ASL.L    #2,D1
    D0W = sw(D7W + 286);
    //D1W = D7W << 2;
//020af8 41ec f62e                LEA      -2514(A4),A0
//020afc d0c1                     ADD.W    D1,A0
    //A0 = d.Byte2514 + D1W;
//020afe 3080                     MOVE.W   D0,(A0)
    //wordGear(A0) = D0W;
    d.Word2514[D7W][0] = D0W;
//020b00 3007                     MOVE.W   D7,D0
//020b02 e580                     ASL.L    #2,D0
    //D0W = D7W << 2;
//020b04 41ec f630                LEA      -2512(A4),A0
//020b08 d0c0                     ADD.W    D0,A0
    //A0 = d.Byte2512 + D0W;
//020b0a 30bc 0001                MOVE.W   #1,(A0)
    //wordGear(A0) = 1;
    d.Word2514[D7W][1] = 1;
//020b0e 3007                     MOVE.W   D7,D0
//020b10 c1fc 0002                MULS     #2,D0
//020b14 e380                     ASL.L    #1,D0
    //D0 = D7W*2;
//020b16 206c a516                MOVE.L   -23274(A4),A0
//020b1a d0c0                     ADD.W    D0,A0
    //A0 = d.pwPointer23274 + D7W;
//020b1c 317c 03d8 0088           MOVE.W   #984,136(A0)
    d.pwPointer23274[68 + D7W*2] = 984;
//020b22 3007                     MOVE.W   D7,D0
//020b24 c1fc 0002                MULS     #2,D0
//020b28 e380                     ASL.L    #1,D0
    //D0 = D7W * 2;
    //D0 <<= 1;
//020b2a 206c a516                MOVE.L   -23274(A4),A0
    //A0 = d.Pointer23274 + D0;
//020b2e d0c0                     ADD.W    D0,A0
//020b30 317c 07a0 008a           MOVE.W   #1952,138(A0)
    d.pwPointer23274[69 + D7W*2] = 1952;
//020b36 5247                     ADDQ.W   #1,D7
  };
//020b38 0c47 0010                CMP.W    #16,D7
//020b3c 6fb0                     BLE      $-78 (=0x020aee)
//020b3e 397c 00f1 f6ae           MOVE.W   #241,-2386(A4)
  d.Word2386 = 241;
//020b44 397c 0001 f6b0           MOVE.W   #1,-2384(A4)
  d.Word2384 = 1;
//020b4a 303c 0000                MOVE.W   #0,D0
//020b4e 3e00                     MOVE.W   D0,D7
//020b50 41ec ea1c                LEA      -5604(A4),A0
//020b54 2008                     MOVE.L   A0,D0
//020b56 2640                     MOVE.L   D0,A3
  psA3 = d.s5604;
//020b58 6000 00ec                BRA      $+238 (=0x020c46)
  for (D7W=0; D7W<85; D7W++)
  {
//020b5c 3a3c 0068                MOVE.W   #104,D5
//020b60 4243                     CLR.W    D3
    D5W = sw(104 + psA3->uByte1);
//020b62 162b 0001                MOVE.B   1(A3),D3
//    D3W=(UI8)(psA3->byte1);
//020b66 da43                     ADD.W    D3,D5
//    D5W += D3W
//020b68 3f3c 0010                MOVE.W   #16,-(A7)
//020b6c 4240                     CLR.W    D0
//020b6e 102b 0003                MOVE.B   3(A3),D0
//020b72 3f00                     MOVE.W   D0,-(A7)
//020b74 4240                     CLR.W    D0
//020b76 102b 0002                MOVE.B   2(A3),D0
//020b7a 3f00                     MOVE.W   D0,-(A7)
//020b7c 4eba fd02                JSR      $-764 (=0x020880)
//020b80 5c8f                     ADDQ.L   #6,A7
    D0W = TAG020880((UI8)(psA3->byte2), (UI8)(psA3->byte3), 16);
//020b82 3205                     MOVE.W   D5,D1
//020b84 5245                     ADDQ.W   #1,D5
//020b86 e381                     ASL.L    #1,D1
//020b88 206c a516                MOVE.L   -23274(A4),A0
//020b8c d0c1                     ADD.W    D1,A0
//020b8e 3080                     MOVE.W   D0,(A0)
    d.pwPointer23274[D5W++] = D0W;
//020b90 3f3c 0014                MOVE.W   #20,-(A7)
//020b94 4240                     CLR.W    D0
//020b96 102b 0003                MOVE.B   3(A3),D0
//020b9a 3f00                     MOVE.W   D0,-(A7)
//020b9c 4240                     CLR.W    D0
//020b9e 102b 0002                MOVE.B   2(A3),D0
//020ba2 3f00                     MOVE.W   D0,-(A7)
//020ba4 4eba fcda                JSR      $-804 (=0x020880)
//020ba8 5c8f                     ADDQ.L   #6,A7
    D0W = TAG020880((UI8)(psA3->byte2), (UI8)(psA3->byte3), 20);
//020baa 3205                     MOVE.W   D5,D1
//020bac 5245                     ADDQ.W   #1,D5
//020bae e381                     ASL.L    #1,D1
//020bb0 206c a516                MOVE.L   -23274(A4),A0
//020bb4 d0c1                     ADD.W    D1,A0
//020bb6 3080                     MOVE.W   D0,(A0)
    d.pwPointer23274[D5W++] = D0W;
//020bb8 4240                     CLR.W    D0
//020bba 102b 0004                MOVE.B   4(A3),D0
//020bbe c07c 0001                AND.W    #1,D0
//020bc2 6738                     BEQ      $+58 (=0x020bfc)
    if (psA3->byte4 & 1)  // Only three or four have this bit set
    {
//020bc4 3005                     MOVE.W   D5,D0
//020bc6 e380                     ASL.L    #1,D0
//020bc8 206c a516                MOVE.L   -23274(A4),A0
//020bcc d0c0                     ADD.W    D0,A0
//020bce 3028 fffc                MOVE.W   -4(A0),D0
      D0W = d.pwPointer23274[D5W - 2];
//020bd2 3205                     MOVE.W   D5,D1
//020bd4 e381                     ASL.L    #1,D1
//020bd6 206c a516                MOVE.L   -23274(A4),A0
//020bda d0c1                     ADD.W    D1,A0
//020bdc 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
//020bde 5245                     ADDQ.W   #1,D5
      D5W++;
//020be0 3005                     MOVE.W   D5,D0
//020be2 e380                     ASL.L    #1,D0
//020be4 206c a516                MOVE.L   -23274(A4),A0
//020be8 d0c0                     ADD.W    D0,A0
//020bea 3028 fffc                MOVE.W   -4(A0),D0
      D0W = d.pwPointer23274[D5W - 2];
//020bee 3205                     MOVE.W   D5,D1
//020bf0 e381                     ASL.L    #1,D1
//020bf2 206c a516                MOVE.L   -23274(A4),A0
//020bf6 d0c1                     ADD.W    D1,A0
//020bf8 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
//020bfa 5245                     ADDQ.W   #1,D5
      D5W++;
    };
//020bfc 4240                     CLR.W    D0
//020bfe 102b 0004                MOVE.B   4(A3),D0
//020c02 c07c 0010                AND.W    #16,D0
//020c06 6736                     BEQ      $+56 (=0x020c3e)
    if (psA3->byte4 & 16) // Only the first one has this bit
    {
//020c08 3005                     MOVE.W   D5,D0
//020c0a e380                     ASL.L    #1,D0
//020c0c 206c a516                MOVE.L   -23274(A4),A0
//020c10 d0c0                     ADD.W    D0,A0
//020c12 3028 fffc                MOVE.W   -4(A0),D0
      D0W = d.pwPointer23274[D5W - 2];
//020c16 3205                     MOVE.W   D5,D1
//020c18 e381                     ASL.L    #1,D1
//020c1a 206c a516                MOVE.L   -23274(A4),A0
//020c1e d0c1                     ADD.W    D1,A0
//020c20 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
//020c22 5245                     ADDQ.W   #1,D5
      D5W++;
//020c24 3005                     MOVE.W   D5,D0
//020c26 e380                     ASL.L    #1,D0
//020c28 206c a516                MOVE.L   -23274(A4),A0
//020c2c d0c0                     ADD.W    D0,A0
//020c2e 3028 fffc                MOVE.W   -4(A0),D0
      D0W = d.pwPointer23274[D5W - 2];
//020c32 3205                     MOVE.W   D5,D1
//020c34 e381                     ASL.L    #1,D1
//020c36 206c a516                MOVE.L   -23274(A4),A0
//020c3a d0c1                     ADD.W    D1,A0
//020c3c 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
    };
//020c3e 3007                     MOVE.W   D7,D0
//020c40 5247                     ADDQ.W   #1,D7
//020c42 200b                     MOVE.L   A3,D0
//020c44 5c8b                     ADDQ.L   #6,A3
    psA3++;
  };
//020c46 0c47 0055                CMP.W    #85,D7
//020c4a 6d00 ff10                BLT      $-238 (=0x020b5c)
//020c4e 303c 0000                MOVE.W   #0,D0
//020c52 3e00                     MOVE.W   D0,D7
//020c54 41ec e9c8                LEA      -5688(A4),A0
//020c58 2008                     MOVE.L   A0,D0
//020c5a 2d40 fffc                MOVE.L   D0,-4(A6)
  LOCAL_4 = d.Byte5688;
//020c5e 6000 00bc                BRA      $+190 (=0x020d1c)
  for (D7W=0; D7W<14; D7W++)
  {
//020c62 206e fffc                MOVE.L   -4(A6),A0
//020c66 3028 0004                MOVE.W   4(A0),D0
//020c6a c07c 0100                AND.W    #256,D0
    if ((LE16(wordGear(LOCAL_4 + 4)) & 256) ==0)
    {
//020c6e 6600 00a0                BNE      $+162 (=0x020d10)
//020c72 3a3c 011a                MOVE.W   #282,D5
      D5W = 282;
//020c76 206e fffc                MOVE.L   -4(A6),A0
      A0 = LOCAL_4;
//020c7a 4243                     CLR.W    D3
      D3W = ((ui8 *)A0)[1];
//020c7c 1628 0001                MOVE.B   1(A0),D3
//020c80 da43                     ADD.W    D3,D5
      D5W = sw(D5W + D3W);
//020c82 4246                     CLR.W    D6
//020c84 6000 0082                BRA      $+132 (=0x020d08)
      for (D6W=0; D6W<6; D6W++)
      {
//020c88 41ec e988                LEA      -5752(A4),A0
//020c8c d0c6                     ADD.W    D6,A0
        A0 = d.Byte5752+D6W;
//020c8e 4240                     CLR.W    D0
//020c90 1010                     MOVE.B   (A0),D0
//020c92 3f00                     MOVE.W   D0,-(A7)
//020c94 206e fffc                MOVE.L   -4(A6),A0
//020c98 4240                     CLR.W    D0
//020c9a 1028 0003                MOVE.B   3(A0),D0
//020c9e 3f00                     MOVE.W   D0,-(A7)
//020ca0 206e fffc                MOVE.L   -4(A6),A0
//020ca4 4240                     CLR.W    D0
//020ca6 1028 0002                MOVE.B   2(A0),D0
//020caa 3f00                     MOVE.W   D0,-(A7)
//020cac 4eba fbd2                JSR      $-1068 (=0x020880)
//020cb0 5c8f                     ADDQ.L   #6,A7
        D4W = TAG020880((UI8)(LOCAL_4[2]),
                         (UI8)(LOCAL_4[3]),
                         (UI8)(A0[0]));
//020cb2 3800                     MOVE.W   D0,D4
//
//020cb4 3205                     MOVE.W   D5,D1
//020cb6 e381                     ASL.L    #1,D1
//020cb8 206c a516                MOVE.L   -23274(A4),A0
//020cbc d0c1                     ADD.W    D1,A0
//020cbe 3080                     MOVE.W   D0,(A0)
        d.pwPointer23274[D5W] = D4W;
//020cc0 206e fffc                MOVE.L   -4(A6),A0
//020cc4 3028 0004                MOVE.W   4(A0),D0
//020cc8 c07c 0003                AND.W    #3,D0
        D0W = (I16)(LE16(wordGear(LOCAL_4 + 4)) & 3);
//020ccc 0c40 0003                CMP.W    #3,D0
//020cd0 672e                     BEQ      $+48 (=0x020d00)
        if (D0W != 3)
        {
//020cd2 3005                     MOVE.W   D5,D0
//020cd4 e380                     ASL.L    #1,D0
//020cd6 206c a516                MOVE.L   -23274(A4),A0
//020cda d0c0                     ADD.W    D0,A0
//020cdc 3144 000c                MOVE.W   D4,12(A0)
          d.pwPointer23274[D5W + 6] = D4W;
//020ce0 206e fffc                MOVE.L   -4(A6),A0
//020ce4 3028 0004                MOVE.W   4(A0),D0
//020ce8 c07c 0003                AND.W    #3,D0
//020cec 0c40 0002                CMP.W    #2,D0
          D0W = (I16)(LE16(wordGear(LOCAL_4 + 4)) & 3);
//020cf0 670e                     BEQ      $+16 (=0x020d00)
          if (D0W != 2)
          {
//020cf2 3005                     MOVE.W   D5,D0
//020cf4 e380                     ASL.L    #1,D0
//020cf6 206c a516                MOVE.L   -23274(A4),A0
//020cfa d0c0                     ADD.W    D0,A0
//020cfc 3144 0018                MOVE.W   D4,24(A0)
            d.pwPointer23274[D5W +12] = D4W;
          };
        };
//020d00 3006                     MOVE.W   D6,D0
//020d02 5246                     ADDQ.W   #1,D6
//020d04 3005                     MOVE.W   D5,D0
//020d06 5245                     ADDQ.W   #1,D5
        D5W++;
      };
//020d08 0c46 0006                CMP.W    #6,D6
//020d0c 6d00 ff7a                BLT      $-132 (=0x020c88)
    };
//020d10 3007                     MOVE.W   D7,D0
//020d12 5247                     ADDQ.W   #1,D7
//020d14 202e fffc                MOVE.L   -4(A6),D0
//020d18 5cae fffc                ADDQ.L   #6,-4(A6)
    LOCAL_4 +=6;
  };
//020d1c 0c47 000e                CMP.W    #14,D7
//020d20 6d00 ff40                BLT      $-190 (=0x020c62)

//020d24 41ec e9a0                LEA      -5728(A4),A0
//020d28 2948 f8b8                MOVE.L   A0,-1864(A4)
//020d2c 41ec e990                LEA      -5744(A4),A0
//020d30 2948 f8bc                MOVE.L   A0,-1860(A4)
//020d34 41ec ff64                LEA      -156(A4),A0
//020d38 2008                     MOVE.L   A0,D0
  A0 = d.Byte5728; d.Pointer1864[0] = A0;
  A0 = d.Byte5744; d.Pointer1860 = A0;
  A0 = d.Byte156;  d.Pointer1852 = A0; d.Pointer1856 = A0;
//020d3a 2940 f8c4                MOVE.L   D0,-1852(A4)
//020d3e 2940 f8c0                MOVE.L   D0,-1856(A4)
//020d42 3a3c 01b6                MOVE.W   #438,D5
  D5W = 438;
//020d46 303c 0000                MOVE.W   #0,D0
//020d4a 3e00                     MOVE.W   D0,D7
//020d4c 41ec e9c0                LEA      -5696(A4),A0
//020d50 2008                     MOVE.L   A0,D0
//020d52 2d40 fff8                MOVE.L   D0,-8(A6)
  LOCAL_8 = d.Byte5696;
//020d56 6070                     BRA      $+114 (=0x020dc8)
  for (D7W=0; D7W<4; D7W++)
  {
//020d58 3c3c 0004                MOVE.W   #4,D6
//020d5c 602e                     BRA      $+48 (=0x020d8c)
    for (D6W=4; D6W<32; D6W+=2)
    {
//020d5e 3f06                     MOVE.W   D6,-(A7)
//020d60 206e fff8                MOVE.L   -8(A6),A0
//020d64 4240                     CLR.W    D0
//020d66 1028 0001                MOVE.B   1(A0),D0
//020d6a 3f00                     MOVE.W   D0,-(A7)
//020d6c 206e fff8                MOVE.L   -8(A6),A0
//020d70 4240                     CLR.W    D0
//020d72 1010                     MOVE.B   (A0),D0
//020d74 3f00                     MOVE.W   D0,-(A7)
//020d76 4eba fb08                JSR      $-1270 (=0x020880)
//020d7a 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880((UI8)(LOCAL_8[0]),(UI8)(LOCAL_8[1]), D6W);
//020d7c 3205                     MOVE.W   D5,D1
//020d7e 5245                     ADDQ.W   #1,D5
//020d80 e381                     ASL.L    #1,D1
//020d82 206c a516                MOVE.L   -23274(A4),A0
//020d86 d0c1                     ADD.W    D1,A0
//020d88 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020d8a 5446                     ADDQ.W   #2,D6
    };
//020d8c 0c46 0020                CMP.W    #32,D6
//020d90 6dcc                     BLT      $-50 (=0x020d5e)
//020d92 0c47 0003                CMP.W    #3,D7
//020d96 6624                     BNE      $+38 (=0x020dbc)
    if (D7W == 3)
    {
//020d98 206e fff8                MOVE.L   -8(A6),A0
//020d9c 1010                     MOVE.B   (A0),D0
      D0W = (UI8)(LOCAL_8[0]);
//020d9e 206e fff8                MOVE.L   -8(A6),A0
//020da2 4241                     CLR.W    D1
//020da4 1228 0001                MOVE.B   1(A0),D1
      D1W = (UI8)(LOCAL_8[1]);
//020da8 c07c 00ff                AND.W    #255,D0
//020dac c1c1                     MULU     D1,D0
      D0L = D0UW*D1UW;
//020dae 3205                     MOVE.W   D5,D1   //020db0 5245                     ADDQ.W   #1,D5
//020db2 e381                     ASL.L    #1,D1
//020db4 206c a516                MOVE.L   -23274(A4),A0
//020db8 d0c1                     ADD.W    D1,A0
//020dba 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
    };
//020dbc 3007                     MOVE.W   D7,D0
//020dbe 5247                     ADDQ.W   #1,D7
//020dc0 202e fff8                MOVE.L   -8(A6),D0
//020dc4 54ae fff8                ADDQ.L   #2,-8(A6)
    LOCAL_8+=2;
  };
//020dc8 0c47 0004                CMP.W    #4,D7
//020dcc 6d8a                     BLT      $-116 (=0x020d58)
//020dce 3a3c 01ef                MOVE.W   #495,D5
  D5W = 495;
//020dd2 4246                     CLR.W    D6
//020dd4 6000 01d6                BRA      $+472 (=0x020fac)
  for (D6W=0; D6W<27; D6W++)
  {
//020dd8 3006                     MOVE.W   D6,D0
//020dda c1fc 000c                MULS     #12,D0
//020dde 41ec e6f2                LEA      -6414(A4),A0
//020de2 d0c0                     ADD.W    D0,A0
//020de4 41d0                     LEA      (A0),A0
//020de6 2448                     MOVE.L   A0,A2
    //A2 = d.Byte6414 + 12*D6W;
//020de8 3e06                     MOVE.W   D6,D7
//020dea cffc 001a                MULS     #26,D7
//020dee 41ec da44                LEA      -9660(A4),A0
//020df2 d0c7                     ADD.W    D7,A0
    //qA0 = d.Byte9660 + D6W*26;
//020df4 3e10                     MOVE.W   (A0),D7
//020df6 3545 0002                MOVE.W   D5,2(A2)
    D7W = d.MonsterDescriptor[D6W].word4;
    d.Item6414[D6W].word2 = D5W;
//020dfa 3f3c 0010                MOVE.W   #16,-(A7)
//020dfe 4240                     CLR.W    D0
//020e00 102a 0005                MOVE.B   5(A2),D0
//020e04 3f00                     MOVE.W   D0,-(A7)
//020e06 4240                     CLR.W    D0
//020e08 102a 0004                MOVE.B   4(A2),D0
//020e0c 3f00                     MOVE.W   D0,-(A7)
//020e0e 4eba fa70                JSR      $-1422 (=0x020880)
    D0W = TAG020880(d.Item6414[D6W].uByte4,
                    d.Item6414[D6W].uByte5,
                    16);
//020e12 5c8f                     ADDQ.L   #6,A7
//020e14 3d40 fff4                MOVE.W   D0,-12(A6)
    LOCAL_12 = D0W;
//020e18 3205                     MOVE.W   D5,D1
//020e1a 5245                     ADDQ.W   #1,D5
//020e1c e381                     ASL.L    #1,D1
//020e1e 206c a516                MOVE.L   -23274(A4),A0
//020e22 d0c1                     ADD.W    D1,A0
//020e24 3080                     MOVE.W   D0,(A0)
    d.pwPointer23274[D5W++] = D0W;
//020e26 3f3c 0014                MOVE.W   #20,-(A7)
//020e2a 4240                     CLR.W    D0
//020e2c 102a 0005                MOVE.B   5(A2),D0
//020e30 3f00                     MOVE.W   D0,-(A7)
//020e32 4240                     CLR.W    D0
//020e34 102a 0004                MOVE.B   4(A2),D0
//020e38 3f00                     MOVE.W   D0,-(A7)
//020e3a 4eba fa44                JSR      $-1466 (=0x020880)
//020e3e 5c8f                     ADDQ.L   #6,A7
    D0W = TAG020880(d.Item6414[D6W].uByte4,
                     d.Item6414[D6W].uByte5,
                     20);
//020e40 3d40 fff6                MOVE.W   D0,-10(A6)
    LOCAL_10 = D0W;
//020e44 3205                     MOVE.W   D5,D1
//020e46 5245                     ADDQ.W   #1,D5
//020e48 e381                     ASL.L    #1,D1
//020e4a 206c a516                MOVE.L   -23274(A4),A0
//020e4e d0c1                     ADD.W    D1,A0
//020e50 3080                     MOVE.W   D0,(A0)
    d.pwPointer23274[D5W] = D0W;
    D5W++;
//020e52 3007                     MOVE.W   D7,D0
//020e54 c07c 0008                AND.W    #8,D0
//020e58 6750                     BEQ      $+82 (=0x020eaa)
    if(D7W & 8)
    {
//020e5a 3f3c 0010                MOVE.W   #16,-(A7)
//020e5e 4240                     CLR.W    D0
//020e60 102a 0007                MOVE.B   7(A2),D0
//020e64 3f00                     MOVE.W   D0,-(A7)
//020e66 4240                     CLR.W    D0
//020e68 102a 0006                MOVE.B   6(A2),D0
//020e6c 3f00                     MOVE.W   D0,-(A7)
//020e6e 4eba fa10                JSR      $-1518 (=0x020880)
//020e72 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte6,
                       d.Item6414[D6W].uByte7,
                       16);
//020e74 3205                     MOVE.W   D5,D1
//020e76 5245                     ADDQ.W   #1,D5
//020e78 e381                     ASL.L    #1,D1
//020e7a 206c a516                MOVE.L   -23274(A4),A0
//020e7e d0c1                     ADD.W    D1,A0
//020e80 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020e82 3f3c 0014                MOVE.W   #20,-(A7)
//020e86 4240                     CLR.W    D0
//020e88 102a 0007                MOVE.B   7(A2),D0
//020e8c 3f00                     MOVE.W   D0,-(A7)
//020e8e 4240                     CLR.W    D0
//020e90 102a 0006                MOVE.B   6(A2),D0
//020e94 3f00                     MOVE.W   D0,-(A7)
//020e96 4eba f9e8                JSR      $-1558 (=0x020880)
//020e9a 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte6,
                       d.Item6414[D6W].uByte7,
                       20);
//020e9c 3205                     MOVE.W   D5,D1
//020e9e 5245                     ADDQ.W   #1,D5
//020ea0 e381                     ASL.L    #1,D1
//020ea2 206c a516                MOVE.L   -23274(A4),A0
//020ea6 d0c1                     ADD.W    D1,A0
//020ea8 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
    };
//020eaa 3007                     MOVE.W   D7,D0
//020eac c07c 0010                AND.W    #16,D0
//020eb0 6720                     BEQ      $+34 (=0x020ed2)
    if(D7W & 16)
    {
//020eb2 3005                     MOVE.W   D5,D0
//020eb4 5245                     ADDQ.W   #1,D5
//020eb6 e380                     ASL.L    #1,D0
//020eb8 206c a516                MOVE.L   -23274(A4),A0
//020ebc d0c0                     ADD.W    D0,A0
//020ebe 30ae fff4                MOVE.W   -12(A6),(A0)
      d.pwPointer23274[D5W] = LOCAL_12;
      D5W++;
//020ec2 3005                     MOVE.W   D5,D0
//020ec4 5245                     ADDQ.W   #1,D5
//020ec6 e380                     ASL.L    #1,D0
//020ec8 206c a516                MOVE.L   -23274(A4),A0
//020ecc d0c0                     ADD.W    D0,A0
//020ece 30ae fff6                MOVE.W   -10(A6),(A0)
      d.pwPointer23274[D5W] = LOCAL_10;
      D5W++;
    };
//020ed2 3007                     MOVE.W   D7,D0
//020ed4 c07c 0020                AND.W    #32,D0
//020ed8 6750                     BEQ      $+82 (=0x020f2a)
    if(D7W & 32)
    {
//020eda 3f3c 0010                MOVE.W   #16,-(A7)
//020ede 4240                     CLR.W    D0
//020ee0 102a 0009                MOVE.B   9(A2),D0
//020ee4 3f00                     MOVE.W   D0,-(A7)
//020ee6 4240                     CLR.W    D0
//020ee8 102a 0008                MOVE.B   8(A2),D0
//020eec 3f00                     MOVE.W   D0,-(A7)
//020eee 4eba f990                JSR      $-1646 (=0x020880)
//020ef2 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte8,
                       d.Item6414[D6W].uByte9,
                       16);
//020ef4 3205                     MOVE.W   D5,D1
//020ef6 5245                     ADDQ.W   #1,D5
//020ef8 e381                     ASL.L    #1,D1
//020efa 206c a516                MOVE.L   -23274(A4),A0
//020efe d0c1                     ADD.W    D1,A0
//020f00 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020f02 3f3c 0014                MOVE.W   #20,-(A7)
//020f06 4240                     CLR.W    D0
//020f08 102a 0009                MOVE.B   9(A2),D0
//020f0c 3f00                     MOVE.W   D0,-(A7)
//020f0e 4240                     CLR.W    D0
//020f10 102a 0008                MOVE.B   8(A2),D0
//020f14 3f00                     MOVE.W   D0,-(A7)
//020f16 4eba f968                JSR      $-1686 (=0x020880)
//020f1a 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte8,
                       d.Item6414[D6W].uByte9,
                       20);
//020f1c 3205                     MOVE.W   D5,D1
//020f1e 5245                     ADDQ.W   #1,D5
//020f20 e381                     ASL.L    #1,D1
//020f22 206c a516                MOVE.L   -23274(A4),A0
//020f26 d0c1                     ADD.W    D1,A0
//020f28 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
    };
//020f2a 3007                     MOVE.W   D7,D0
//020f2c c07c 0003                AND.W    #3,D0
//020f30 3d40 fff4                MOVE.W   D0,-12(A6)
     LOCAL_12 = (I16)(D7W & 3);
//020f34 6774                     BEQ      $+118 (=0x020faa)
    for (LOCAL_12=(I16)(D7W&3); LOCAL_12 != 0; LOCAL_12--)
    {
//020f36 102a 0004                MOVE.B   4(A2),D0
//020f3a 4241                     CLR.W    D1
//020f3c 122a 0005                MOVE.B   5(A2),D1
//020f40 c07c 00ff                AND.W    #255,D0
      D0W = d.Item6414[D6W].uByte4;
      D1W = d.Item6414[D6W].uByte5;
//020f44 c1c1                     MULU     D1,D0
      D0L = D0UW*D1UW;
//020f46 3205                     MOVE.W   D5,D1
//020f48 5245                     ADDQ.W   #1,D5
//020f4a e381                     ASL.L    #1,D1
//020f4c 206c a516                MOVE.L   -23274(A4),A0
//020f50 d0c1                     ADD.W    D1,A0
//020f52 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020f54 3f3c 0010                MOVE.W   #16,-(A7)
//020f58 4240                     CLR.W    D0
//020f5a 102a 0005                MOVE.B   5(A2),D0
//020f5e 3f00                     MOVE.W   D0,-(A7)
//020f60 4240                     CLR.W    D0
//020f62 102a 0004                MOVE.B   4(A2),D0
//020f66 3f00                     MOVE.W   D0,-(A7)
//020f68 4eba f916                JSR      $-1768 (=0x020880)
//020f6c 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte4,
                     d.Item6414[D6W].uByte5,
                     16);
//020f6e 3205                     MOVE.W   D5,D1
//020f70 5245                     ADDQ.W   #1,D5
//020f72 e381                     ASL.L    #1,D1
//020f74 206c a516                MOVE.L   -23274(A4),A0
//020f78 d0c1                     ADD.W    D1,A0
//020f7a 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020f7c 3f3c 0014                MOVE.W   #20,-(A7)
//020f80 4240                     CLR.W    D0
//020f82 102a 0005                MOVE.B   5(A2),D0
//020f86 3f00                     MOVE.W   D0,-(A7)
//020f88 4240                     CLR.W    D0
//020f8a 102a 0004                MOVE.B   4(A2),D0
//020f8e 3f00                     MOVE.W   D0,-(A7)
//020f90 4eba f8ee                JSR      $-1808 (=0x020880)
//020f94 5c8f                     ADDQ.L   #6,A7
      D0W = TAG020880(d.Item6414[D6W].uByte4,
                     d.Item6414[D6W].uByte5,
                     20);
//020f96 3205                     MOVE.W   D5,D1
//020f98 5245                     ADDQ.W   #1,D5
//020f9a e381                     ASL.L    #1,D1
//020f9c 206c a516                MOVE.L   -23274(A4),A0
//020fa0 d0c1                     ADD.W    D1,A0
//020fa2 3080                     MOVE.W   D0,(A0)
      d.pwPointer23274[D5W] = D0W;
      D5W++;
//020fa4 536e fff4                SUBQ.W   #1,-12(A6)
//020fa8 668c                     BNE      $-114 (=0x020f36)
    };
//020faa 5246                     ADDQ.W   #1,D6
  };
//020fac 0c46 001b                CMP.W    #27,D6
//020fb0 6d00 fe26                BLT      $-472 (=0x020dd8)
//020fb4 4cdf 0cf0                MOVEM.L  (A7)+,D4567A23
  //D4=saveD4;D5=saveD5;D6=saveD6;D7=saveD7;A2=saveA2;A3=saveA3;
//020fb8 4e5e                     UNLK     A6
//020fba 4e75                     RTS
}

void TAG020fbc(void)
{
//  dReg D6;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//020fbc 4e56 0000                LINK     A6,#0
//020fc0 3f3c 0001                MOVE.W   #1,-(A7)
//020fc4 2f3c 0000 0cc0           MOVE.L   #3264,-(A7)
//020fca 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020fce 5c8f                     ADDQ.L   #6,A7
//020fd0 2940 f75a                MOVE.L   D0,-2214(A4)
  d.Pointer2214 = allocateMemory(3264,1);
//020fd4 3f3c 0001                MOVE.W   #1,-(A7)
//020fd8 2f3c 0000 13f8           MOVE.L   #5112,-(A7)
//020fde 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020fe2 5c8f                     ADDQ.L   #6,A7
//020fe4 2940 f756                MOVE.L   D0,-2218(A4)
  d.Pointer2218 = allocateMemory(5112, 1);
//020fe8 3f3c 0001                MOVE.W   #1,-(A7)
//020fec 2f3c 0000 3780           MOVE.L   #14208,-(A7)
//020ff2 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//020ff6 5c8f                     ADDQ.L   #6,A7
//020ff8 2940 f752                MOVE.L   D0,-2222(A4)
  d.Pointer2222 = allocateMemory(14208,1);
//020ffc 3f3c 0001                MOVE.W   #1,-(A7)
//021000 2f3c 0000 0880           MOVE.L   #2176,-(A7)
//021006 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02100a 5c8f                     ADDQ.L   #6,A7
//02100c 2940 f74e                MOVE.L   D0,-2226(A4)
  d.Pointer2226 = allocateMemory(2176,1);
//021010 3f3c 0001                MOVE.W   #1,-(A7)
//021014 2f3c 0000 0880           MOVE.L   #2176,-(A7)
//02101a 4ead 05d6                JSR      1494(A5)    (=0x02175c)
//02101e 5c8f                     ADDQ.L   #6,A7
//021020 2940 f74a                MOVE.L   D0,-2230(A4)
  d.Pointer2230 = allocateMemory(2176,1);
//021024 4e5e                     UNLK     A6
//021026 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _TAG021028(void)
{ //(void)
  static dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//021028 4e56 fffe                LINK     A6,#-2
  static i16 LOCAL_2;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
    RESTART(4)
    RESTART(5)
    RESTART(6)
  END_RESTARTMAP
//02102c 426c d214                CLR.W    -11756(A4)
//021030 426c d212                CLR.W    -11758(A4)
//021034 426c d210                CLR.W    -11760(A4)
//021038 426c d20e                CLR.W    -11762(A4)
//02103c 426c d202                CLR.W    -11774(A4)
//021040 426c d200                CLR.W    -11776(A4)
//021044 426c d256                CLR.W    -11690(A4)
  d.Word11756 = 0;
  d.Word11758 = 0;
  d.Word11760 = 0;
  d.Word11762 = 0;
  d.FlashButnActive = 0;
  d.Word11776 = 0;
  d.PartySleeping = 0;
//021048 397c 000f f8d4           MOVE.W   #15,-1836(A4)
//02104e 426c f8d2                CLR.W    -1838(A4)
//021052 426c f8d0                CLR.W    -1840(A4)
//021056 426c b0ea                CLR.W    -20246(A4)
  d.Word1836 = 15;
  d.Word1838 = 0;
  d.Word1840 = 0;
  d.AttackingCharacterOrdinal = 0;
//02105a 397c 0001 b0e4           MOVE.W   #1,-20252(A4)
  d.Word20252 = 1;
//021060 426c a598                CLR.W    -23144(A4)
  d.Word23144 = 0;
//021064 297c 00ff ffff cd32      MOVE.L   #16777215,-13006(A4)
  d.WatchdogTime = 0xffffff;
//02106c 41ec bcd4                LEA      -17196(A4),A0
//021070 2948 be18                MOVE.L   A0,-16872(A4)
  d.PrimaryButtonList = d.Buttons17196; // characters, spells, movement
//021074 41ec bc68                LEA      -17304(A4),A0
//021078 2948 be14                MOVE.L   A0,-16876(A4)
  d.SecondaryButtonList = d.Buttons17304;
//02107c 41ec b820                LEA      -18400(A4),A0
//021080 2948 be10                MOVE.L   A0,-16880(A4)
  d.pKeyXlate2 = (KeyXlate *)d.Byte18400;
//021084 41ec b804                LEA      -18428(A4),A0
//021088 2948 be0c                MOVE.L   A0,-16884(A4)
  d.pKeyXlate1 = d.Byte18428;
//02108c 3f2c d244                MOVE.W   -11708(A4),-(A7)
//021090 4ead 01b0                JSR      432(A5)    (=0x000850)
//021094 548f                     ADDQ.L   #2,A7
//021096 302c d25a                MOVE.W   -11686(A4),D0
  LoadPartyLevel(_1_,d.partyLevel);
  D0W = d.Word11686;
//02109a 6600 0094                BNE      $+150 (=0x021130)
  if (d.Word11686 == 0)
  {
//02109e 3f3c 7d00                MOVE.W   #32000,-(A7)
//0210a2 2f2c d132                MOVE.L   -11982(A4),-(A7)
//0210a6 4ead 0054                JSR      84(A5)    (=0x000a84)
//0210aa 5c8f                     ADDQ.L   #6,A7
    //ClearMemory(d.LogicalScreenBase, 32000);
//0210ac 486c d136                PEA      -11978(A4)
//0210b0 4eba df9c                JSR      $-8290 (=0x01f04e)
//0210b4 588f                     ADDQ.L   #4,A7
    //FadeToPalette(&d.Palette11978);
//0210b6 2f3c 0001 0001           MOVE.L   #65537,-(A7)
//0210bc 3f3c 0001                MOVE.W   #1,-(A7)
//0210c0 42a7                     CLR.L    -(A7)
//0210c2 42a7                     CLR.L    -(A7)
//0210c4 42a7                     CLR.L    -(A7)
//0210c6 2f2c a684                MOVE.L   -22908(A4),-(A7)
//0210ca 2f2c a6e4                MOVE.L   -22812(A4),-(A7)
//0210ce 42a7                     CLR.L    -(A7)
//0210d0 4eba c5a6                JSR      $-14936 (=0x01d678)
//0210d4 defc 001e                ADD.W    #30,A7
    for (;;)
    {
      DoMenu(_6_,
             NULL,
             d.Pointer22812,
             "PLAY GAME",
             "ASCII DUMP",
             NULL,
             NULL,
             1,
             1,
             1);
//0210d8 4267                     CLR.W    -(A7)
//0210da 4267                     CLR.W    -(A7)
//0210dc 2f3c 0001 0001           MOVE.L   #65537,-(A7)
//0210e2 4eba c0e8                JSR      $-16150 (=0x01d1cc)
//0210e6 508f                     ADDQ.L   #0,A7
      //if (1 == TAG01d1cc(2,1,0,0)) break;
      WaitForMenuSelect(_2_, 2,1,0,0);
      if (i16Result == 1) break;
      AsciiDump();
    } ;
//0210e8 3d6c a534 fffe           MOVE.W   -23244(A4),-2(A6)
//0210ee 670a                     BEQ      $+12 (=0x0210fa)
    LOCAL_2 = d.Word23244;
    if (LOCAL_2 != 0)
    {
//0210f0 397c 0001 a534           MOVE.W   #1,-23244(A4)
      d.Word23244 = 1;
//0210f6 4ead 0600                JSR      1536(A5)    (=0x021d76)
      closeGraphicsFile();
    };
//0210fa 3f3c 0001                MOVE.W   #1,-(A7)
//0210fe 4267                     CLR.W    -(A7)
//021100 4eba c9a6                JSR      $-13912 (=0x01daa8)
//021104 588f                     ADDQ.L   #4,A7
    TAG01daa8(_3_, 0,1);
//021106 302e fffe                MOVE.W   -2(A6),D0
//02110a 670a                     BEQ      $+12 (=0x021116)
    if (LOCAL_2 != 0)
    {
//02110c 4ead 05fa                JSR      1530(A5)    (=0x021d36)
      openGraphicsFile();
//021110 396e fffe a534           MOVE.W   -2(A6),-23244(A4)
      d.Word23244 = LOCAL_2;
    };
//021116 3f3c 0025                MOVE.W   #37,-(A7)
//02111a 4ead 0138                JSR      312(A5)    (=0x001ce8)
//02111e 548f                     ADDQ.L   #2,A7
    wvbl(_4_);
//021120 3f3c 7d00                MOVE.W   #32000,-(A7)
//021124 2f2c d132                MOVE.L   -11982(A4),-(A7)
//021128 4ead 0054                JSR      84(A5)    (=0x000a84)
//02112c 5c8f                     ADDQ.L   #6,A7
    ClearMemory(d.LogicalScreenBase, 32000);
//02112e 6050                     BRA      $+82 (=0x021180)
  }
  else
  {
//021130 426c a608                CLR.W    -23032(A4)
    d.UseByteCoordinates = 0;
//021134 3f3c 0025                MOVE.W   #37,-(A7)
//021138 4ead 0138                JSR      312(A5)    (=0x001ce8)
//02113c 548f                     ADDQ.L   #2,A7
    wvbl(_5_);
//02113e 3f3c 00a0                MOVE.W   #160,-(A7)
//021142 4267                     CLR.W    -(A7)
//021144 486c f90c                PEA      -1780(A4)
//021148 2f2c d132                MOVE.L   -11982(A4),-(A7)
//02114c 4ead 01ce                JSR      462(A5)    (=0x008e60)
//021150 defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,&d.wRectPos1780,0,160); //Top of screen
//021154 3f3c 00a0                MOVE.W   #160,-(A7)
//021158 4267                     CLR.W    -(A7)
//02115a 486c f904                PEA      -1788(A4)
//02115e 2f2c d132                MOVE.L   -11982(A4),-(A7)
//021162 4ead 01ce                JSR      462(A5)    (=0x008e60)
//021166 defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,&d.wRectPos1788,0,160);//Right side of screen
//02116a 3f3c 00a0                MOVE.W   #160,-(A7)
//02116e 4267                     CLR.W    -(A7)
//021170 486c f8fc                PEA      -1796(A4)
//021174 2f2c d132                MOVE.L   -11982(A4),-(A7)
//021178 4ead 01ce                JSR      462(A5)    (=0x008e60)
//02117c defc 000c                ADD.W    #12,A7
    FillRectangle(d.LogicalScreenBase,&d.wRectPos1796,0,160);//Scrolling text area
  };
//021180 486c fdd8                PEA      -552(A4)
//021184 3f3c 0006                MOVE.W   #6,-(A7)
//021188 4ead 0138                JSR      312(A5)    (=0x001ce8)
//02118c 5c8f                     ADDQ.L   #6,A7
  setpalette(&d.Palette552[0]);
//02118e 4ead 0528                JSR      1320(A5)    (=0x01b880)
//021192 4ead 0498                JSR      1176(A5)    (=0x0138ec)
  DrawMovementButtons();
  TAG0138ec();
//021196 397c 0001 d254           MOVE.W   #1,-11692(A4)
  d.ClockRunning = 1;
//02119c 4e5e                     UNLK     A6
//02119e 4e75                     RTS
  RETURN
}

//*********************************************************
//
//*********************************************************
//   TAG0211a0
RESTARTABLE _ShowPrisonDoor(void)
{//(void)
  static dReg D7;
  RESTARTMAP
    //RESTART(1)
    RESTART(2)
    RESTART(3)
    RESTART(4)
    RESTART(5)
    RESTART(6)
  END_RESTARTMAP
  //i16 saveD7 = D7W;
  //i8 LOCAL_130[130];
////0211a6 486d 0156                PEA      342(A5)   ; (=0x001d0c)
////0211aa 3f3c 0026                MOVE.W   #38,-(A7) ; SuperExec
////0211ae 4ead 0138                JSR      312(A5)   ; (=0x001ce8) Trap14SaveA2
////0211b2 5c8f                     ADDQ.L   #6,A7     ;
////  TAG001d0c(); // Set critical error handler.
  //TAG020836(LOCAL_130); //??Copy some part of Program Descriptor?? // Never used!
  TAG020466();
  InitializeHeap();//TAG020286
  D7W = 1;
  ReadGraphicsIndex(); // TAG021d9a
  TAG0208b4();
  HangIfZero(D7W); // (=0x020276)
  //DisplayChaosStrikesBack(_1_);
  TAG00381a(0);
  TAG00386a(0);
////////021230 6102                     BSR      $+4 (=0x021234)
////////021232 601c                     BRA      $+30 (=0x021250)
////////021234 2f2d ffc0                MOVE.L   -64(A5),-(A7)
////////021238 4e75                     RTS
      HopefullyNotNeeded();
  TAG001c6e();
  TAG0010ae();
    //D0 = GetAbsFuncAddr(276); // TAG00dd78(276);
    //D0 = TAG00306c;
  d.pFunc23232 = TAG00306c;
    //D0 = TAG00dd78(&TAG00b152)
  d.pFunc23236 = TAG00b152;
  //  MemMove((pnt)palette,(pnt)&d.Palette11946, 32);
  //  MemMove((pnt)palette,(pnt)&d.Palette11978, 32);
  TAG0023b0();
  TAG000ec6(); // In supervisor mode
  jdisint(5);  // TRAP 14 (#26,#5);
  TAG0020ca();
  wvbl(_2_);
  MemMove((pnt)d.Palette552, (pnt)&d.Palette11946, 32); // Initialize palette
  MemMove((pnt)d.Palette552, (pnt)&d.Palette11978, 32); // Initialize palette
  TAG01f746(_3_);
  for (;;)
  {
    TAG01eb18(_4_);
    if (i16Result == 1) break;
    openGraphicsFile(); // TAG021d36
    TAG01f746(_5_);
  };
  openGraphicsFile();// TAG021d36
  TAG01b8ae();
  TAG021cd8();
  TAG021028(_6_);
  closeGraphicsFile();
  if (d.Word11686 != 0)
  {
    MoveObject(RN(RNnul), -1, 0, d.partyX, d.partyY);
  };
  STShowCursor(); //TAG003026
  TAG01953a();
  //D7W = saveD7;
  RETURN;
}

//*********************************************************
//
//*********************************************************
void TAG02134a(pnt)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//02134a 397c 0088 fc6a           MOVE.W   #136,-918(A4)
  d.Word918 = 136;
//021350 397c 0088 f8d2           MOVE.W   #136,-1838(A4)
  d.Word1838 = 136;
//021356 397c 0000 d218           MOVE.W   #0,-11752(A4)
  d.Word11752 = 0;
//02135c 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
//   TAG021822
void RemoveFromFreeList(ITEMQ *This)
{
  ITEMQ *Prev, *Next;
  VerifyGraphicFreeList();
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//021822 4e56 fffc                LINK     A6,#-4
//021826 48e7 0030                MOVEM.L  A23,-(A7)
//02182a 266e 0008                MOVE.L   8(A6),A3
  //pgA3 = pQ;
//02182e 200b                     MOVE.L   A3,D0
//021830 5880                     ADDQ.L   #4,D0
//021832 2040                     MOVE.L   D0,A0
//021834 2450                     MOVE.L   (A0),A2
  Prev = This->pgPrev();
//021836 200b                     MOVE.L   A3,D0
//021838 5080                     ADDQ.L   #0,D0
//02183a 2040                     MOVE.L   D0,A0
//02183c 2d50 fffc                MOVE.L   (A0),-4(A6)
  Next = This->pgNext();
//021840 b5fc 0000 0000           CMP.L    #0,A2
//021846 661e                     BNE      $+32 (=0x021866)
  if (Prev == NULL)
  {
//021848 4aae fffc                TST.L    -4(A6)
//02184c 6606                     BNE      $+8 (=0x021854)
    if (Next == NULL)
    {
//02184e 42ac a4d4                CLR.L    -23340(A4)
      d.pgUnused = NULL;
//021852 6010                     BRA      $+18 (=0x021864)
    }
    else
    {
//021854 296e fffc a4d4           MOVE.L   -4(A6),-23340(A4)
      d.pgUnused = Next;
//02185a 202e fffc                MOVE.L   -4(A6),D0
//02185e 5880                     ADDQ.L   #4,D0
//021860 2040                     MOVE.L   D0,A0
//021862 4290                     CLR.L    (A0)
      Next->pgPrev(NULL);
    };
//021864 601a                     BRA      $+28 (=0x021880)
  }
  else
  {
//021866 200a                     MOVE.L   A2,D0
//021868 5080                     ADDQ.L   #0,D0
//02186a 2040                     MOVE.L   D0,A0
//02186c 20ae fffc                MOVE.L   -4(A6),(A0)
    Prev->pgNext(Next);
//021870 4aae fffc                TST.L    -4(A6)
//021874 670a                     BEQ      $+12 (=0x021880)
    if (Next != NULL)
    {
//021876 202e fffc                MOVE.L   -4(A6),D0
//02187a 5880                     ADDQ.L   #4,D0
//02187c 2040                     MOVE.L   D0,A0
//02187e 208a                     MOVE.L   A2,(A0)
      Next->pgPrev(Prev);
    };
  };
//021880 4cdf 0c00                MOVEM.L  (A7)+,A23
//021884 4e5e                     UNLK     A6
//021886 4e75                     RTS
  VerifyGraphicFreeList();
}


ui8 long02160a[] = {
 0x00,0x01,0x00,0x01,
 0x00,0x03,0x00,0x03,
 0x00,0x07,0x00,0x07,
 0x00,0x0f,0x00,0x0f,
 0x00,0x1f,0x00,0x1f,
 0x00,0x3f,0x00,0x3f,
 0x00,0x7f,0x00,0x7f,
 0x00,0xff,0x00,0xff,
 0x01,0xff,0x01,0xff,
 0x03,0xff,0x03,0xff,
 0x07,0xff,0x07,0xff,
 0x0f,0xff,0x0f,0xff,
 0x1f,0xff,0x1f,0xff,
 0x3f,0xff,0x3f,0xff,
 0x7f,0xff,0x7f,0xff,
 0xff,0xff,0xff,0xff
};

ui8 long02164a[] = {
  0xff,0xfe,0xff,0xfe,
  0xff,0xfc,0xff,0xfc,
  0xff,0xf8,0xff,0xf8,
  0xff,0xf0,0xff,0xf0,
  0xff,0xe0,0xff,0xe0,
  0xff,0xc0,0xff,0xc0,
  0xff,0x80,0xff,0x80,
  0xff,0x00,0xff,0x00,
  0xfe,0x00,0xfe,0x00,
  0xfc,0x00,0xfc,0x00,
  0xf8,0x00,0xf8,0x00,
  0xf0,0x00,0xf0,0x00,
  0xe0,0x00,0xe0,0x00,
  0xc0,0x00,0xc0,0x00,
  0x80,0x00,0x80,0x00,
  0x00,0x00,0x00,0x00
};

ui8 long02168a[] = {
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0xff,0xff,0x00,0x00,0x00,0x00,
 0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0xff,0xff,0x00,0x00,
 0xff,0xff,0x00,0x00,0xff,0xff,0x00,0x00,
 0x00,0x00,0xff,0xff,0xff,0xff,0x00,0x00,
 0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,
 0xff,0xff,0x00,0x00,0x00,0x00,0xff,0xff,
 0x00,0x00,0xff,0xff,0x00,0x00,0xff,0xff,
 0xff,0xff,0xff,0xff,0x00,0x00,0xff,0xff,
 0x00,0x00,0x00,0x00,0xff,0xff,0xff,0xff,
 0xff,0xff,0x00,0x00,0xff,0xff,0xff,0xff,
 0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
 0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
};

enum BSRRETURNS {
  Tag02148a = 1,
  Tag0214aa,
  Tag0214e6,
  Tag0214f6,
  Tag02155e,
  Tag021564,
  Tag021568
};

#define BSR(to,from)              \
  retstack[--retindex]=Tag##from; \
  goto tag##to;                   \
tag##from:

#define RTS goto tagReturn;

#define POP retindex++;

void ExpandGraphic(pnt src,pnt dest,i16 P3,i16 P4)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0213ec 4e56 0000                LINK     A6,#0
//0213f0 48e7 0f3e                MOVEM.L  D4567A23456,-(A7)
  dReg D0, D1, D2, D3, D4, D5, D6, D7;
  aReg A0, A1, A2=NULL, A3, A4, A5, A6;
  //i32 saveD4=D4,saveD5=D5,saveD6=D6,saveD7=D7;
  //pnt saveA2=A2,saveA3=A3,saveA4=A4,saveA5=A5,saveA6=A6;
  i32 retindex=10;
  i32 C;
  BSRRETURNS retstack[10];
//0213f4 206e 0008                MOVE.L   8(A6),A0
//0213f8 226e 000c                MOVE.L   12(A6),A1
  D1L = 0;
  D7L = 0;
  A0 = src;
  A1 = dest;
//0213fc 3a2e 0010                MOVE.W   16(A6),D5
//021400 382e 0012                MOVE.W   18(A6),D4
  D5W = P3;
  D4W = P4;
//021404 3e18                     MOVE.W   (A0)+,D7
//021406 3c18                     MOVE.W   (A0)+,D6
  D7W = LE16(wordGear(A0)); A0+=2; // GraphicWidth
  D6W = LE16(wordGear(A0)); A0+=2; // GraphicHeight
//021408 de7c 000f                ADD.W    #15,D7
//02140c e24f                     LSR.W    #1,D7
//02140e ce7c fff8                AND.W    #65528,D7
  D7W = (I16)(((D7W+15)>>1)&0x7ff8); // Number of bytes/GraphicLine
//021412 b3fc 0000 0000           CMP.L    #0,A1
//021418 6638                     BNE      $+58 (=0x021452)
  if((A1==0) && (A1=physbase(),D7W!=160)) // If destination not specified
  {
//02141a 2f08                     MOVE.L   A0,-(A7)
//02141c 3f3c 0002                MOVE.W   #2,-(A7)
//021420 4e4e                     TRAP     #14
//021422 548f                     ADDQ.L   #2,A7
//021424 2240                     MOVE.L   D0,A1
//021426 205f                     MOVE.L   (A7)+,A0
//021428 303c 00a0                MOVE.W   #160,D0
//02142c be40                     CMP.W    D0,D7
//02142e 6722                     BEQ      $+36 (=0x021452)
    D5W = (I16)((D5W>>1)&0x7ff8);
    A1 += D5W;
//021430 e24d                     LSR.W    #1,D5
//021432 ca7c fff8                AND.W    #65528,D5
//021436 d2c5                     ADD.W    D5,A1
    D4L = D4UW * 160; //(ui16)D0W;
//021438 c8c0                     MULU     D0,D4
//02143a d2c4                     ADD.W    D4,A1
    A1 += D4W;
    A3 = A1 + D7W;
//02143c 2649                     MOVE.L   A1,A3
//02143e d6c7                     ADD.W    D7,A3
//021440 5346                     SUBQ.W   #1,D6
    D6W--;
//021442 ccc0                     MULU     D0,D6
    D6L = D6UW * 160; //(ui16)D0W; // Total number bytes
//021444 dc8b                     ADD.L    A3,D6
    D6L += (i32)A3;
//021446 3800                     MOVE.W   D0,D4
    D4W = sw(160 - D7W);
//021448 9847                     SUB.W    D7,D4
//02144a 3e04                     MOVE.W   D4,D7
//    D7W = D4W;
//02144c 4847                     SWAP     D7
//02144e 3e00                     MOVE.W   D0,D7
    D7W = 160;
//021450 6008                     BRA      $+10 (=0x02145a)
  }
  else
  {
    D6L = D6UW * (ui16)D7W; // Total number bytes
//021452 ccc7                     MULU     D7,D6
//021454 2649                     MOVE.L   A1,A3
//021456 d7c6                     ADD.L    D6,A3
    A3 = A1 + D6L;//Destination LWA+1
//021458 2c0b                     MOVE.L   A3,D6
    D6L = (i32)A3;//Destination LWA+1
  };
//02145a 4291                     CLR.L    (A1)
//02145c 42a9 0004                CLR.L    4(A1)
  longGear(A1)   = 0;
  longGear(A1+4) = 0;
//021460 700f                     MOVEQ.L  #15,D0
  D0L = 15; //#pixels in this group-1
//021462 7800                     MOVEQ.L  #0,D4
  D4L = 0;  //pixel# in this group
//021464 49fa 01a4                LEA      $+422 (=0x02160a),A4
//021468 4bfa 01e0                LEA      $+482 (=0x02164a),A5
//02146c 4dfa 021c                LEA      $+542 (=0x02168a),A6
  A4 = (pnt)long02160a;
  A5 = (pnt)long02164a;
  A6 = (pnt)long02168a;
tag021470:
//021470 1618                     MOVE.B   (A0)+,D3
//021472 1403                     MOVE.B   D3,D2
  D2B = D3B = *(A0++);
//021474 6b48                     BMI      $+74 (=0x0214be)
  if (D2B < 0) goto tag0214be;
//021476 e80b                     LSR.B    #4,D3
//021478 6778                     BEQ      $+122 (=0x0214f2)
  D3B = (i8)((D3UB)>>4);
  if (D3B == 0) goto tag0214f2;
//02147a c47c 000f                AND.W    #15,D2
  D2W &= 0xf;
//02147e 7201                     MOVEQ.L  #1,D1
  D1L = 1;
//021480 d203                     ADD.B    D3,D1
  D1B = (i8)(D1B + D3B);
//021482 e74a                     LSL.W    #3,D2
tag021482:
  D2W <<= 3;
//021484 244e                     MOVE.L   A6,A2
  A2 = A6 + D2W;
//021486 d4c2                     ADD.W    D2,A2
tag021488:
//021488 6170                     BSR      $+114 (=0x0214fa)
  BSR (0214fa,02148a);//place D1 pixels of color *A2
//02148a 4a41                     TST.W    D1
//02148c 67e2                     BEQ      $-28 (=0x021470)
  if (D1W == 0) goto tag021470;
//02148e 0c41 0010                CMP.W    #16,D1
//021492 65f4                     BCS      $-10 (=0x021488)
  if ( (ui16)D1W < 16) goto tag021488;
//021494 2612                     MOVE.L   (A2),D3
//021496 2a2a 0004                MOVE.L   4(A2),D5
  D3L = LE32(longGear(A2));
  D5L = LE32(longGear(A2+4));
//02149a 0441 0010                SUB.W    #16,D1
  D1W -= 16;
tag02149e:
//02149e 22c3                     MOVE.L   D3,(A1)+
//0214a0 22c5                     MOVE.L   D5,(A1)+
  longGear(A1) = LE32(D3L);
  A1+=4;
  longGear(A1) = LE32(D5L);
  A1+=4;
//0214a2 b7c9                     CMP.L    A1,A3
//0214a4 6604                     BNE      $+6 (=0x0214aa)
  if (A1 != A3) goto tag0214aa;
//0214a6 6100 0094                BSR      $+150 (=0x02153c)
  BSR(02153c,0214aa);
//0214aa 0441 0010                SUB.W    #16,D1
  D1W -= 16;
//0214ae 64ee                     BCC      $-16 (=0x02149e)
  if ( (ui16)D1W < 65536-16) goto tag02149e;
//0214b0 4291                     CLR.L    (A1)
//0214b2 42a9 0004                CLR.L    4(A1)
  longGear(A1) = 0;
  longGear(A1+4) = 0;
//0214b6 0641 0010                ADD.W    #16,D1
  D1W +=16;
//0214ba 66cc                     BNE      $-50 (=0x021488)
  if (D1W != 0) goto tag021488;
  goto tag021470;
//0214bc 60b2                     BRA      $-76 (=0x021470)
tag0214be:
//0214be 0242 000f                AND.W    #15,D2
  D2W &= 15;
//0214c2 7200                     MOVEQ.L  #0,D1
  D1L = 0;
//0214c4 1218                     MOVE.B   (A0)+,D1
  D1B = *(A0++);
//0214c6 0803 0006                BTST     6,D3
//0214ca 6704                     BEQ      $+6 (=0x0214d0)
  if ((D3W & 0x40)==0) goto tag0214d0;
//0214cc e141                     ASL.W    #8,D1
  D1W <<= 8;
//0214ce 1218                     MOVE.B   (A0)+,D1
  D1B = *(A0++);
//0214d0 5241                     ADDQ.W   #1,D1
tag0214d0:
  D1W += 1;
//0214d2 0803 0004                BTST     4,D3
//0214d6 67aa                     BEQ      $-84 (=0x021482)
  if ((D3W&0x10)==0) goto tag021482;
//0214d8 0803 0005                BTST     5,D3
//0214dc 6700 007a                BEQ      $+124 (=0x021558)
  if ((D3W&0x20)==0) goto tag021558;
//0214e0 2449                     MOVE.L   A1,A2
//0214e2 94c7                     SUB.W    D7,A2
  A2 = A1-D7W;
//0214e4 6114                     BSR      $+22 (=0x0214fa)
tag0214e4:
  BSR(0214fa,0214e6);
//0214e6 4a85                     TST.L    D5
//0214e8 6702                     BEQ      $+4 (=0x0214ec)
  if (D5L == 0) goto tag0214ec;
//0214ea 2445                     MOVE.L   D5,A2
  A2 = (pnt)D5L;
//0214ec 508a                     ADDQ.L   #0,A2
tag0214ec:
  A2 += 8;
//2114ee 4a41                     TST.W    D1
//2114f0 66f2                     BNE      $-12 (=0x0214e4)
  if (D1W != 0) goto tag0214e4;
tag0214f2:
//0214f2 6100 007c                BSR      $+126 (=0x021570)
  BSR(021570,0214f6);
//0214f6 6000 ff78                BRA      $-134 (=0x021470)
  goto tag021470;
tag0214fa: // D1=repeat, D0=#-1 pixels remain in group *A2 = color
//0214fa 1800                     MOVE.B   D0,D4
  D4B = D0B;
//0214fc d844                     ADD.W    D4,D4
//0214fe d844                     ADD.W    D4,D4
  D4W = sw(D4W + D4W);
  D4W = sw(D4W + D4W);
//021500 2634 4000                MOVE.L   0(A4,D4W),D3
  D3L = LE32(longGear(A4+D4W)); //mask of remaining pixels in group
//021504 b081                     CMP.L    D1,D0
//021506 6b10                     BMI      $+18 (=0x021518)
  if (D0L < D1L) goto tag021518;
//021508 9041                     SUB.W    D1,D0
  D0W = sw(D0W - D1W); //decrement pixels remaining in group
//02150a 1800                     MOVE.B   D0,D4
  D4B = D0B;
//02150c d844                     ADD.W    D4,D4
//02150e d844                     ADD.W    D4,D4
  D4W = sw(D4W + D4W + D4W + D4W);
//021510 c6b5 4000                AND.L    0(A5,D4W),D3
  D3L &= LE32(longGear(A5 + D4W)); //data mask
//021514 7200                     MOVEQ.L  #0,D1
  D1L = 0;
//021516 6006                     BRA      $+8 (=0x02151e)
  goto tag02151e;
//021518 9240                     SUB.W    D0,D1
tag021518:
  D1W = sw(D1W - D0W);
//02151a 5341                     SUBQ.W   #1,D1
  D1W--;
//02151c 70ff                     MOVEQ.L  #-1,D0
  D0L = -1;
//02151e 2a12                     MOVE.L   (A2),D5
//021520 ca83                     AND.L    D3,D5
tag02151e:
  D5L = LE32(longGear(A2));
  D5L &= D3L;
//021522 8b99                     OR.L     D5,(A1)+
  longGear(A1) |= LE32(D5L);
  //D5L = 0;
  A1 += 4;
//021524 2a2a 0004                MOVE.L   4(A2),D5
//021528 ca83                     AND.L    D3,D5
  D5L = LE32(longGear(A2+4));
  D5L &= D3L;
//02152a 8b99                     OR.L     D5,(A1)+
  longGear(A1) |= LE32(D5L);
  A1 += 4;
//02152c 7a00                     MOVEQ.L  #0,D5
  D5L = 0;
//02152e 4a40                     TST.W    D0
//021530 6b06                     BMI      $+8 (=0x021538)
  if (D0W < 0) goto tag021538;
//021532 5189                     SUBQ.L   #0,A1
  A1 -= 8;
  RTS;
//021534 4e75                     RTS
tag021536:
//021536 5089                     ADDQ.L   #0,A1
  A1 += 8;
//021538 b7c9                     CMP.L    A1,A3
//02153a 6612                     BNE      $+20 (=0x02154e)
tag021538:
  if (A1 != A3) goto tag02154e;
tag02153c:
//02153c bc89                     CMP.L    A1,D6
//02153e 6700 01ca                BEQ      $+460 (=0x02170a)
  if (A1 == (pnt)D6L) goto tag02170a;
//021542 d6c7                     ADD.W    D7,A3
  A3 += D7W;
//021544 4847                     SWAP     D7
  SWAP(D7);//D7 = ((D7<<16)&0xffff0000) | ((D7>>16)&0xffff);
//021546 d2c7                     ADD.W    D7,A1
  A1 += D7W;
//021548 2a0a                     MOVE.L   A2,D5
//02154a da47                     ADD.W    D7,D5
  D5L = (i32)A2;
  D5W = sw(D5W + D7W);
//02154c 4847                     SWAP     D7
  SWAP(D7);// = ((D7<<16)&0xffff0000) | ((D7>>16)&0xffff);
//02154e 700f                     MOVEQ.L  #15,D0
tag02154e:
  D0L = 15;
//021550 4291                     CLR.L    (A1)
//021552 42a9 0004                CLR.L    4(A1)
  longGear(A1) = 0;
  longGear(A1+4) = 0;
  RTS;
//021556 4e75                     RTS

//021558 e249                     LSR.W    #1,D1
tag021558:
  C = D1W&1;
  D1W >>= 1;
//02155a 6402                     BCC      $+4 (=0x02155e)
  if (C==0) goto tag02155e;
//02155c 6112                     BSR      $+20 (=0x021570)
  BSR(021570,02155e)
//02155e 1410                     MOVE.B   (A0),D2
  D2B = *(A0);
//021560 e80a                     LSR.B    #4,D2
  D2B >>= 4;
//021562 610c                     BSR      $+14 (=0x021570)
  BSR(021570,021564);
//021564 1418                     MOVE.B   (A0)+,D2
  D2B = *(A0++);
//021566 6108                     BSR      $+10 (=0x021570)
  BSR(021570,021568);
//021568 5341                     SUBQ.W   #1,D1
  D1W--;
//02156a 66f2                     BNE      $-12 (=0x02155e)
  if (D1W != 0) goto tag02155e;
//02156c 6000 ff02                BRA      $-252 (=0x021470)
  goto tag021470;
tag021570://D0=#pixels left in group -1; place one pixel color D2
//021570 4243                     CLR.W    D3
  D3W = 0;
//021572 01c3                     BSET     D0,D3
  D3L |= 1<<D0B;
//021574 0242 000f                AND.W    #15,D2
//  D2W &= 15;
//021578 d442                     ADD.W    D2,D2
//  D2W += D2W
//  switch (D2W)
//  {
  if (D2W & 0x8) wordGear(A1+6) |= LE16((D3W));
  if (D2W & 0x4) wordGear(A1+4) |= LE16((D3W));
  if (D2W & 0x2) wordGear(A1+2) |= LE16((D3W));
  if (D2W & 0x1) wordGear(A1+0) |= LE16((D3W));
  D0W--;
  if (D0W < 0) goto tag021536;
  RTS;
//02157a 4efb 2002                JMP      2(PC,D2W)
//02157e 602a                     BRA      $+44 (=0x0215aa)
//  case 0: goto tag0215aa;
//021580 6026                     BRA      $+40 (=0x0215a8)
//  case 1: goto tag0215a8;
//021582 6048                     BRA      $+74 (=0x0215cc)
//  case 2: goto tag0215cc;
//021584 601e                     BRA      $+32 (=0x0215a4)
//  case 3: goto tag0215a4;
//021586 6054                     BRA      $+86 (=0x0215dc)
//  case 4: goto tag0215dc;
//021588 602c                     BRA      $+46 (=0x0215b6)
//  case 5: goto tag0215b6;
//02158a 603c                     BRA      $+62 (=0x0215c8)
//  case 6: goto tag0215c8;
//02158c 6012                     BRA      $+20 (=0x0215a0)
//  case 7: goto tag0215a0;
//02158e 605e                     BRA      $+96 (=0x0215ee)
//  case 8: goto tag0215ee;
//021590 605a                     BRA      $+92 (=0x0215ec)
//  case 9: goto tag0215ec;
//021592 6066                     BRA      $+104 (=0x0215fa)
//  case 10: goto tag0215fa;
//021594 6052                     BRA      $+84 (=0x0215e8)
//  case 11: goto tag0215e8;
//021596 6040                     BRA      $+66 (=0x0215d8)
//  case 12: goto tag0215d8;
//021598 6018                     BRA      $+26 (=0x0215b2)
//  case 13: goto tag0215b2;
//02159a 6028                     BRA      $+42 (=0x0215c4)
//  case 14: goto tag0215c4;
//  case 15:
//02159c 8769 0006                OR.W     D3,6(A1)
//    wordGear(A1+6) |= LE16(D3W);
//0215a0 8769 0004                OR.W     D3,4(A1)
//    wordGear(A1+4) |= LE16(D3W);
//0215a4 8769 0002                OR.W     D3,2(A1)
//    wordGear(A1+2) |= LE16(D3W);
//0215a8 8751                     OR.W     D3,(A1)
//    wordGear(A1) |= LE16(D3W);
//0215aa 5340                     SUBQ.W   #1,D0
//    D0W--;\
//0215ac 6b00 ff88                BMI      $-118 (=0x021536)
//    if (D0W < 0) goto tag021536;
//0215b0 4e75                     RTS
//    RTS;
//tag0215b2:
//0215b2 8769 0006                OR.W     D3,6(A1)
//0215b6 8769 0004                OR.W     D3,4(A1)
//0215ba 8751                     OR.W     D3,(A1)
//0215bc 5340                     SUBQ.W   #1,D0
//0215be 6b00 ff76                BMI      $-136 (=0x021536)
//0215c2 4e75                     RTS
//0215c4 8769 0006                OR.W     D3,6(A1)
//0215c8 8769 0004                OR.W     D3,4(A1)
//0215cc 8769 0002                OR.W     D3,2(A1)
//0215d0 5340                     SUBQ.W   #1,D0
//0215d2 6b00 ff62                BMI      $-156 (=0x021536)
//0215d6 4e75                     RTS
//0215d8 8769 0006                OR.W     D3,6(A1)
//0215dc 8769 0004                OR.W     D3,4(A1)
//0215e0 5340                     SUBQ.W   #1,D0
//0215e2 6b00 ff52                BMI      $-172 (=0x021536)
//0215e6 4e75                     RTS
//0215e8 8769 0002                OR.W     D3,2(A1)
//0215ec 8751                     OR.W     D3,(A1)
//0215ee 8769 0006                OR.W     D3,6(A1)
//0215f2 5340                     SUBQ.W   #1,D0
//0215f4 6b00 ff40                BMI      $-190 (=0x021536)
//0215f8 4e75                     RTS
//0215fa 8769 0006                OR.W     D3,6(A1)
//0215fe 8769 0002                OR.W     D3,2(A1)
//021602 5340                     SUBQ.W   #1,D0
//021604 6b00 ff30                BMI      $-206 (=0x021536)
//021608 4e75                     RTS
//02170a 588f                     ADDQ.L   #4,A7
tag02170a:
  POP; // discard return address
//02170c 4cdf 7cf0                MOVEM.L  (A7)+,D4567A23456
  //D4=saveD4;D5=saveD5;D6=saveD6;D7=saveD7;
  //A2=saveA2;A3=saveA3;A4=saveA4;A5=saveA5;A6=saveA6;
//021710 4e5e                     UNLK     A6
//021712 4e75                     RTS
  ASSERT(retindex==10);
  return;
tagReturn:
  switch (retstack[retindex++])
  {
  case Tag02148a: goto tag02148a;
  case Tag0214e6: goto tag0214e6;
  case Tag0214f6: goto tag0214f6;
  case Tag02155e: goto tag02155e;
  case Tag021564: goto tag021564;
  case Tag021568: goto tag021568;
  default: NotImplemented(0x21712);
  };
}




void checkMemory(void)
{
  pnt prevblk=NULL;
  pnt blk = firstMemoryBlock;
  //return;
  while (blk != NULL)
  {
    ASSERT(longGear(blk+8) == memDebugFlag);
    prevblk=blk;
    blk = pntGear(blk);
  };
}


pnt allocateMemory(i32 sizeNeeded,i16 type) //TAG02175c
{ // Allocate memory.
  // Type =1 --> permanent allocation.
  // Type =0 --> temporary allocation
  // Type =2 --> temporary allocation of as much as possible
  dReg D7;
  aReg A0, A3;
  ASSERT(sizeNeeded != 0);
  D7L = sizeNeeded;
  if (sizeNeeded&1) sizeNeeded++; // Make the size even
  if ((type==1) && (d.Long23326 >= sizeNeeded+12))
  {  // Allocate static memory from low addresses
    A3 = d.Pointer23322;
    d.Pointer23322 += sizeNeeded+12; // room for debug list
    d.Long23326 -= sizeNeeded+12;
    pntGear(A3) = firstMemoryBlock;
    firstMemoryBlock = A3;
    longGear(A3+8) = memDebugFlag;
    longGear(A3+4) = sizeNeeded-12;
    A3 += 12;
  }
  else
  { // Not enough space or temporary allocation
    if (sizeNeeded+12 > d.Long23306)
    { // die, I think.
      die(40);
    };
    if (type == 2)
    {
      // Allocate all that's left from end
      A3 = d.Pointer23298 - d.Long23306;
      d.Long23306 -= sizeNeeded;
    }
    else if (type==1)
    {
      // Allocate what's needed from the high end.
      d.Pointer23298 = A3 = d.Pointer23298-sizeNeeded-12;
      pntGear(A3) = firstMemoryBlock;
      firstMemoryBlock = A3;
      longGear(A3+4) = sizeNeeded;
      longGear(A3+8) = memDebugFlag;
      A3 += 12;
      d.Long23306 -= sizeNeeded+12;
    }
    else
    { // Temporary allocation
      d.Pointer23298 = A3 = d.Pointer23298-sizeNeeded;
      d.Long23306 -= sizeNeeded;
    }
  };
  A0 = A3;
  memset(A0, 0xba, sizeNeeded);
  //D7 = saveD7;
  //A3 = saveA3;
  return A0;
}


//   TAG0217d8
void FreeTempMemory(i32 size)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//0217d8 4e56 0000                LINK     A6,#0
//0217dc 2f07                     MOVE.L   D7,-(A7)
//0217de 2e2e 0008                MOVE.L   8(A6),D7
//0217e2 2007                     MOVE.L   D7,D0
//0217e4 c0bc 0000 0001           AND.L    #1,D0
//0217ea 6702                     BEQ      $+4 (=0x0217ee)
  if (size & 1)
  {
    size++;
//0217ec 5287                     ADDQ.L   #1,D7
  };
//0217ee 2007                     MOVE.L   D7,D0
//0217f0 d1ac a4f6                ADD.L    D0,-23306(A4)
  d.Long23306 += size;
//0217f4 2007                     MOVE.L   D7,D0
//0217f6 d1ac a4fe                ADD.L    D0,-23298(A4)
  d.Pointer23298 += size;
//0217fa 2e1f                     MOVE.L   (A7)+,D7
//0217fc 4e5e                     UNLK     A6
//0217fe 4e75                     RTS

}

void TAG021800(i32 num)
{
  if (num & 1) num++;
  d.Long23306 += num;
}

//   TAG021c08
void InitializeCaches(pnt memStart, i32 memSize, pnt P3, i32 P4)
{ // P4 is (0x19bf8-endOS)=space above Operating System
  dReg D7;
  aReg A3;
  A3 = memStart;
  D7L = memSize;
  d.Pointer23280 = memStart;
  d.Pointer23318 = memStart;
  d.Pointer23294 = d.Pointer23298 = A3+D7L;
  d.Long23306 = D7L;
  d.Pointer23322 = P3;
  d.Long23326 = P4;
  d.ClusterCache = allocateMemory(1024,1); //TAG02175c
  d.ppqBuffers = (ITEMQ **)allocateMemory(5100,1); //TAG02175c

  ClearMemory((pnt)d.ppqBuffers, 5100); // TAG000a84
  d.GraphicIndex = (i16 *)allocateMemory(1460, 1); //TAG02175c
  fillMemory(d.GraphicIndex, 730, -1, 2);
  d.pwPointer23274 = (i16 *)allocateMemory(1460, 1); //TAG02175c
  d.pgUnused = NULL;
  d.pqFirstGraphic2 = NULL;
  d.pqLastGraphic = NULL;
  d.pqFirstGraphic = NULL;
  d.Long23358 = -1;
  //D7 = saveD7;
  //A3 = saveA3;
}

void TAG021cd8(void)
{
  dReg D0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//021cd8 4e56 0000                LINK     A6,#0
//021cdc 202c a4fe                MOVE.L   -23298(A4),D0
//021ce0 222c a510                MOVE.L   -23280(A4),D1
//021ce4 9081                     SUB.L    D1,D0
  D0L = d.Pointer23298 - d.Pointer23280;
//021ce6 0c80 0005 a550           CMP.L    #370000,D0
//021cec 5ec0                     SGT      D0
//021cee c07c 0001                AND.W    #1,D0
  D0L = D0L > 370000 ? 1 : 0;
//021cf2 3940 a4c6                MOVE.W   D0,-23354(A4)
  d.Word23354 = D0W;
//021cf6 6704                     BEQ      $+6 (=0x021cfc)
  if (D0W != 0)
  {
//021cf8 4ead 059a                JSR      1434(A5)    (=0x020fbc)
    TAG020fbc();
  };
//021cfc 202c a4fe                MOVE.L   -23298(A4),D0
//021d00 90bc 0000 2710           SUB.L    #10000,D0
  d.Pointer23284 = d.Pointer23298 - 10000;
//021d06 2940 a50c                MOVE.L   D0,-23284(A4)
//021d0a 397c 0001 a4e0           MOVE.W   #1,-23328(A4)
  d.Word23328 = 1;
//021d10 202c a50c                MOVE.L   -23284(A4),D0
//021d14 2940 a4ee                MOVE.L   D0,-23314(A4)
  d.Pointer23310 = d.Pointer23314 = d.Pointer23284;
//021d18 2940 a4f2                MOVE.L   D0,-23310(A4)
//021d1c 297c 0000 2710 a4f6      MOVE.L   #10000,-23306(A4)
  d.Long23306 = 10000;
//021d24 202c a50c                MOVE.L   -23284(A4),D0
//021d28 222c a510                MOVE.L   -23280(A4),D1
//021d2c 9081                     SUB.L    D1,D0
  D0L = d.Pointer23284 - d.Pointer23280;
//021d2e 2940 a4d8                MOVE.L   D0,-23336(A4)
  d.Long23336 = D0L;
//021d32 4e5e                     UNLK     A6
//021d34 4e75                     RTS
}



i16 TAG022c90(i16 P1)
{
  dReg D0, D5, D6, D7;
  ITEMQ *pqA3;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//022c90 4e56 0000                LINK     A6,#0
//022c94 48e7 0710                MOVEM.L  D567A3,-(A7)
//022c98 3e2e 0008                MOVE.W   8(A6),D7
  D7W = P1;
//022c9c 202c d23a                MOVE.L   -11718(A4),D0
//022ca0 b0ac a4c2                CMP.L    -23358(A4),D0
//022ca4 6704                     BEQ      $+6 (=0x022caa)
  if (d.Time != d.Long23358)
  {
//022ca6 4eba fa78                JSR      $-1414 (=0x022720)
    ClearGraphicList2();
  };
//022caa 3007                     MOVE.W   D7,D0
//022cac e380                     ASL.L    #1,D0
//022cae 206c a51a                MOVE.L   -23270(A4),A0
//022cb2 d0c0                     ADD.W    D0,A0
//022cb4 3010                     MOVE.W   (A0),D0
  D6W = d.GraphicIndex[D7W];
//022cb6 3c00                     MOVE.W   D0,D6
//022cb8 0c40 ffff                CMP.W    #65535,D0
//022cbc 670e                     BEQ      $+16 (=0x022ccc)
  if (D6W != -1)
  {
//022cbe 3f06                     MOVE.W   D6,-(A7)
//022cc0 4eba fbd8                JSR      $-1062 (=0x02289a)
//022cc4 548f                     ADDQ.L   #2,A7
    TAG02289a(D6W);
//022cc6 303c 0001                MOVE.W   #1,D0
//022cca 6064                     BRA      $+102 (=0x022d30)
    D0W = 1;
  }
  else
  {
//022ccc 3a3c 0010                MOVE.W   #16,D5
//022cd0 3207                     MOVE.W   D7,D1
//022cd2 e381                     ASL.L    #1,D1
//022cd4 206c a516                MOVE.L   -23274(A4),A0
//022cd8 d0c1                     ADD.W    D1,A0
//022cda da50                     ADD.W    (A0),D5
    D5L = (ui16)(d.pwPointer23274[D7W] + 16);
//022cdc cabc 0000 ffff           AND.L    #65535,D5
//022ce2 2f05                     MOVE.L   D5,-(A7)
//022ce4 4eba f510                JSR      $-2798 (=0x0221f6)
//022ce8 588f                     ADDQ.L   #4,A7
//022cea 2640                     MOVE.L   D0,A3
    pqA3 = TAG0221f6(D5L);
//022cec 4246                     CLR.W    D6
    D6W = 0;
//022cee 3006                     MOVE.W   D6,D0
//022cf0 5246                     ADDQ.W   #1,D6
//022cf2 e580                     ASL.L    #2,D0
//022cf4 206c a4dc                MOVE.L   -23332(A4),A0
//022cf8 d0c0                     ADD.W    D0,A0
//022cfa 4a90                     TST.L    (A0)
//022cfc 66f0                     BNE      $-14 (=0x022cee)
    while (d.ppqBuffers[D6W++] != 0) {};
//022cfe 5346                     SUBQ.W   #1,D6
    D6W--;
//022d00 3006                     MOVE.W   D6,D0
//022d02 3207                     MOVE.W   D7,D1
//022d04 e381                     ASL.L    #1,D1
//022d06 206c a51a                MOVE.L   -23270(A4),A0
//022d0a d0c1                     ADD.W    D1,A0
//022d0c 3080                     MOVE.W   D0,(A0)
    d.GraphicIndex[D7W] = D6W;
//022d0e 3006                     MOVE.W   D6,D0
//022d10 e580                     ASL.L    #2,D0
//022d12 206c a4dc                MOVE.L   -23332(A4),A0
//022d16 d0c0                     ADD.W    D0,A0
//022d18 208b                     MOVE.L   A3,(A0)
    d.ppqBuffers[D6W] = pqA3;
//022d1a 3007                     MOVE.W   D7,D0
//022d1c 807c 8000                OR.W     #32768,D0
    D0W = UI16(D7W | 0x8000);
//022d20 220b                     MOVE.L   A3,D1
//022d22 d2bc 0000 000a           ADD.L    #10,D1
//022d28 2041                     MOVE.L   D1,A0
//022d2a 3080                     MOVE.W   D0,(A0)
    pqA3->graphicNum = D0W;
//022d2c 303c 0000                MOVE.W   #0,D0
    D0W = 0;
  };
//022d30 4cdf 08e0                MOVEM.L  (A7)+,D567A3
//022d34 4e5e                     UNLK     A6
//022d36 4e75                     RTS
#ifdef _VERIFYPPQ
  verifyppq();
#endif
  return D0W;
}

pnt TAG022d38(i16 P1)
{
  dReg D0;
  aReg A0;
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//022d38 4e56 0000                LINK     A6,#0
//022d3c 302e 0008                MOVE.W   8(A6),D0
//022d40 e380                     ASL.L    #1,D0
//022d42 206c a51a                MOVE.L   -23270(A4),A0
//022d46 d0c0                     ADD.W    D0,A0
//022d48 3010                     MOVE.W   (A0),D0
  D0W = d.GraphicIndex[P1];
//022d4a e580                     ASL.L    #2,D0
//022d4c 206c a4dc                MOVE.L   -23332(A4),A0
//022d50 d0c0                     ADD.W    D0,A0
//022d52 2010                     MOVE.L   (A0),D0
  A0 = (pnt)d.ppqBuffers[D0W];
//022d54 d0bc 0000 000c           ADD.L    #12,D0
//022d5a 4e5e                     UNLK     A6
//022d5c 4e75                     RTS
  return A0+12;
}

void TAG022d5e(i16 P1)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//022d5e 4e56 0000                LINK     A6,#0
//022d62 302e 0008                MOVE.W   8(A6),D0
//022d66 e380                     ASL.L    #1,D0
//022d68 206c a51a                MOVE.L   -23270(A4),A0
//022d6c d0c0                     ADD.W    D0,A0
//022d6e 3f10                     MOVE.W   (A0),-(A7)
//022d70 4eba fa06                JSR      $-1528 (=0x022778)
//022d74 548f                     ADDQ.L   #2,A7
  TAG022778(d.GraphicIndex[P1]);
//022d76 4e5e                     UNLK     A6
//022d78 4e75                     RTS
}

i16 GetGraphicExpSize(i32 P1) //(022d7a)
{
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//022d7a 4e56 0000                LINK     A6,#0
//022d7e 302e 0008                MOVE.W   8(A6),D0
//022d82 e380                     ASL.L    #1,D0
//022d84 206c a526                MOVE.L   -23258(A4),A0
//022d88 d0c0                     ADD.W    D0,A0
//022d8a 3010                     MOVE.W   (A0),D0
  return d.GraphicExpSizes[P1];
//022d8c 4e5e                     UNLK     A6
//022d8e 4e75                     RTS
}

//*********************************************************
//
//*********************************************************
RESTARTABLE _StartCSB(const CSB_UI_MESSAGE * /*msg*/) //
{//(void)
  static aReg A4, A5;
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
  END_RESTARTMAP
//  i32 dBankLen=programDescriptor[5]+programDescriptor[7];
  A4 = (char *) (&d);
  A4 += programDescriptor[7]; // uninitialized data,a4,initialized data
  A5=NULL; // Should stay that way.
  // Clear uninitialized memory.
  memset (A4-programDescriptor[7],0,sizeof (d));

  InitializeDBank(); // Pointers and initial values
  d.Pointer4 = NULL; // pointer to I-Bank (A5 in 68000 code).
  ShowPrisonDoor(_1_);
  GameIsComplete = false;
  //To get dump of prison   AsciiDump();
  for (;;)
  {
    MainLoop(_2_); //TAG00068e
    d.DynamicPaletteSwitching = 0;
    TAG01f9e6(_3_,d.Word11696); //Show Credits
    if (GameIsComplete) break;
  };
  RETURN;
}

RESTARTABLE _AskWhatToDo(void)
{//(i32)
  RESTARTMAP
    RESTART(1)
    RESTART(2)
    RESTART(3)
    RESTART(4)
    RESTART(5)
    RESTART(6)
    RESTART(7)
    RESTART(8)
    RESTART(9)
    RESTART(10)
  END_RESTARTMAP
  InitializeDBank(); // Pointers and initial values
  TAG020466();
  InitializeHeap();//TAG020286
  ReadGraphicsIndex(); // TAG021d9a
  TAG0208b4();
  DisplayChaosStrikesBack(_2_);
  TAG00381a(0);
  TAG00386a(0);
////////021230 6102                     BSR      $+4 (=0x021234)
////////021232 601c                     BRA      $+30 (=0x021250)
////////021234 2f2d ffc0                MOVE.L   -64(A5),-(A7)
////////021238 4e75                     RTS
      HopefullyNotNeeded();
  TAG001c6e();
/*
  TAG0010ae();
    //D0 = GetAbsFuncAddr(276); // TAG00dd78(276);
    //D0 = TAG00306c;
  d.pFunc23232 = TAG00306c;
    //D0 = TAG00dd78(&TAG00b152)
  d.pFunc23236 = TAG00b152;
  //  MemMove((pnt)palette,(pnt)&d.Palette11946, 32);
  //  MemMove((pnt)palette,(pnt)&d.Palette11978, 32);
  TAG0023b0();
  TAG000ec6(); // In supervisor mode
  jdisint(5);  // TRAP 14 (#26,#5);
  TAG0020ca();
  wvbl(_2_);
  MemMove((pnt)d.Palette552, (pnt)&d.Palette11946, 32); // Initialize palette
  MemMove((pnt)d.Palette552, (pnt)&d.Palette11978, 32); // Initialize palette
  TAG01f746(_3_);
  */
  setscreen(d.LogicalScreenBase,d.LogicalScreenBase,0);
  MemMove((pnt)d.Palette552, (pnt)&d.Palette11978, 32); // Initialize palette
  DoMenu(_1_,  //build the menu
         "I AM READY, MASTER",
         "WHAT IS YOUR WISH?",
         "QUIT",
         "DUNGEON",
         "UTILITY",
         "HINT",
         1,
         1,
         1);
  if (PlaybackCommandOption)
  {
    GameMode = 1;
  }
  else
  {
    WaitForMenuSelect(_3_, 4,1,0,0);
    GameMode=i16Result-1;
  };
  switch (GameMode)
  {
  case 0:
    UI_PushMessage(UIM_TERMINATE);
    WAITFORMESSAGE(_4_); //Should never come back!
    ASSERT(0);
    break;
  case 1:
    do
    {
      StartCSB(_5_);
    } while (RepeatGame);
    UI_PushMessage(UIM_TERMINATE);
    WAITFORMESSAGE(_6_); //Should never come back!
    ASSERT(0);
    break;
  case 2:
    StartChaos(_7_);
    UI_PushMessage(UIM_TERMINATE);
    WAITFORMESSAGE(_8_); //Should never come back!
    ASSERT(0);
    break;
    break;
  case 3:
    StartHint(_9_);
    UI_PushMessage(UIM_TERMINATE);
    WAITFORMESSAGE(_10_); //Should never come back!
    ASSERT(0);
    break;
    break;
  };
  RETURN;
}

